/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2014 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include "JNpp_common.hpp"
#include "JNppi.hpp"


//=== Auto-generated part: ===================================================

/**
 * <pre>
 * 8-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_18u_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_8u_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_8u_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp8u* jniResult_native = nppiMalloc_8u_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 8-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_18u_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_8u_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_8u_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp8u* jniResult_native = nppiMalloc_8u_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 8-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_18u_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_8u_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_8u_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp8u* jniResult_native = nppiMalloc_8u_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_18u_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_8u_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_8u_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp8u* jniResult_native = nppiMalloc_8u_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 16-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116u_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16u_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16u_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16u* jniResult_native = nppiMalloc_16u_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 16-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116u_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16u_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16u_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16u* jniResult_native = nppiMalloc_16u_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 16-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116u_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16u_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16u_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16u* jniResult_native = nppiMalloc_16u_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116u_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16u_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16u_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16u* jniResult_native = nppiMalloc_16u_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 16-bit signed image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116s_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16s_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16s_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16s* jniResult_native = nppiMalloc_16s_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 16-bit signed image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116s_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16s_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16s_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16s* jniResult_native = nppiMalloc_16s_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 16-bit signed image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116s_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16s_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16s_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16s* jniResult_native = nppiMalloc_16s_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 1 channel 16-bit signed complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116sc_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16sc_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16sc_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16sc* jniResult_native = nppiMalloc_16sc_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 16-bit signed complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116sc_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16sc_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16sc_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16sc* jniResult_native = nppiMalloc_16sc_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 16-bit signed complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116sc_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16sc_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16sc_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16sc* jniResult_native = nppiMalloc_16sc_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 16-bit signed complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116sc_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16sc_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16sc_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16sc* jniResult_native = nppiMalloc_16sc_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 32-bit signed image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132s_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32s_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32s_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32s* jniResult_native = nppiMalloc_32s_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 32-bit signed image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132s_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32s_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32s_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32s* jniResult_native = nppiMalloc_32s_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 32-bit signed image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132s_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32s_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32s_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32s* jniResult_native = nppiMalloc_32s_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132sc_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32sc_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32sc_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32sc* jniResult_native = nppiMalloc_32sc_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132sc_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32sc_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32sc_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32sc* jniResult_native = nppiMalloc_32sc_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 32-bit integer complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132sc_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32sc_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32sc_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32sc* jniResult_native = nppiMalloc_32sc_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 32-bit integer complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132sc_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32sc_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32sc_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32sc* jniResult_native = nppiMalloc_32sc_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 32-bit floating point image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132f_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32f_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32f_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32f* jniResult_native = nppiMalloc_32f_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 32-bit floating point image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132f_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32f_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32f_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32f* jniResult_native = nppiMalloc_32f_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 32-bit floating point image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132f_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32f_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32f_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32f* jniResult_native = nppiMalloc_32f_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 32-bit floating point image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132f_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32f_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32f_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32f* jniResult_native = nppiMalloc_32f_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132fc_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32fc_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32fc_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32fc* jniResult_native = nppiMalloc_32fc_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 32-bit float complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132fc_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32fc_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32fc_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32fc* jniResult_native = nppiMalloc_32fc_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 32-bit float complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132fc_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32fc_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32fc_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32fc* jniResult_native = nppiMalloc_32fc_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 32-bit float complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132fc_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32fc_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32fc_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32fc* jniResult_native = nppiMalloc_32fc_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * Free method for any 2D allocated memory.
 * This method should be used to free memory allocated with any of the nppiMalloc_<modifier> methods.
 * @param pData A pointer to memory allocated using nppiMalloc_<modifier>.
 * </pre>
 */
JNIEXPORT void JNICALL Java_jcuda_jnpp_JNppi_nppiFreeNative(JNIEnv *env, jclass cls, jobject pData)
{
    // Null-checks for non-primitive arguments
    if (pData == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pData' is null for nppiFree");
        return;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFree(pData=%p)\n",
        pData);

    // Native variable declarations
    void* pData_native;

    // Obtain native variable values
    PointerData *pData_pointerData = initPointerData(env, pData);
    if (pData_pointerData == NULL)
    {
        return;
    }
    pData_native = (void*)pData_pointerData->getPointer(env);

    // Native function call
    nppiFree(pData_native);

    // Write back native variable values
    if (!releasePointerData(env, pData_pointerData, 0)) return;

}

/**
 * <pre>
 * 8-bit image set.
 * @param nValue The pixel value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18s_1C1RNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8s_C1R(nValue=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s nValue_native;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp8s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8s_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit two-channel image set.
 * @param aValue The pixel value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18s_1C2RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8s_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s aValue_native[2];
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8s_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18s_1C3RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8s_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s aValue_native[3];
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8s_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18s_1C4RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8s_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s aValue_native[4];
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8s_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit four-channel image set ignoring alpha channel.
 * @param aValue The pixel value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18s_1AC4RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8s_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s aValue_native[3];
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8s_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C1RNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C1R(nValue=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned image set.
 * @param aValue The pixel value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C2RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aValue_native[2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned image set.
 * @param aValue The pixel value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C3RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aValue_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C4RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aValue_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image set method, not affecting Alpha channel.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1AC4RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aValue_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C1RNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C1R(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 16-bit unsigned image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C2RNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aValue_native[2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C3RNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aValue_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C4RNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aValue_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image set method, not affecting Alpha channel.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1AC4RNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aValue_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C1RNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C1R(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 16-bit image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C2RNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s aValue_native[2];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C3RNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s aValue_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C4RNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s aValue_native[4];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit image set method, not affecting Alpha channel.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1AC4RNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s aValue_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex integer image set.
 * @param oValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject oValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (oValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oValue' is null for nppiSet_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16sc_C1R(oValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        oValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc oValue_native;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, oValue, oValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16sc_C1R(oValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, oValue_native, oValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex integer two-channel image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116sc_1C2RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16sc_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc aValue_native[2];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16sc_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex integer three-channel image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116sc_1C3RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16sc_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc aValue_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16sc_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex integer four-channel image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116sc_1C4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16sc_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc aValue_native[4];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16sc_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex integer four-channel image set ignoring alpha.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116sc_1AC4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16sc_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc aValue_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16sc_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit image set.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C1RNative(JNIEnv *env, jclass cls, jint nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C1R(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 32-bit image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C2RNative(JNIEnv *env, jclass cls, jintArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aValue_native[2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C3RNative(JNIEnv *env, jclass cls, jintArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aValue_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C4RNative(JNIEnv *env, jclass cls, jintArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aValue_native[4];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit image set method, not affecting Alpha channel.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1AC4RNative(JNIEnv *env, jclass cls, jintArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aValue_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132u_1C1RNative(JNIEnv *env, jclass cls, jint nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32u_C1R(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nValue_native;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp32u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32u_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 32-bit unsigned image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132u_1C2RNative(JNIEnv *env, jclass cls, jintArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32u_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aValue_native[2];
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32u_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit unsigned image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132u_1C3RNative(JNIEnv *env, jclass cls, jintArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32u_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aValue_native[3];
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32u_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132u_1C4RNative(JNIEnv *env, jclass cls, jintArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32u_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aValue_native[4];
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32u_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit unsigned image set method, not affecting Alpha channel.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132u_1AC4RNative(JNIEnv *env, jclass cls, jintArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32u_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aValue_native[3];
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32u_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit complex integer image set.
 * @param oValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject oValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (oValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oValue' is null for nppiSet_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32sc_C1R(oValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        oValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc oValue_native;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, oValue, oValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32sc_C1R(oValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, oValue_native, oValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two channel 32-bit complex integer image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132sc_1C2RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32sc_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc aValue_native[2];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32sc_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit complex integer image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132sc_1C3RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32sc_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc aValue_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32sc_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit complex integer image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132sc_1C4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32sc_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc aValue_native[4];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32sc_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132sc_1AC4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32sc_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc aValue_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32sc_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C1RNative(JNIEnv *env, jclass cls, jfloat nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C1R(nValue=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C2RNative(JNIEnv *env, jclass cls, jfloatArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aValue_native[2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C3RNative(JNIEnv *env, jclass cls, jfloatArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aValue_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C4RNative(JNIEnv *env, jclass cls, jfloatArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aValue_native[4];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image set method, not affecting Alpha channel.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1AC4RNative(JNIEnv *env, jclass cls, jfloatArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aValue_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit complex image set.
 * @param oValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject oValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (oValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oValue' is null for nppiSet_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32fc_C1R(oValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        oValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc oValue_native;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, oValue, oValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32fc_C1R(oValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, oValue_native, oValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132fc_1C2RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32fc_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aValue_native[2];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32fc_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit complex image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132fc_1C3RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32fc_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aValue_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32fc_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit complex image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132fc_1C4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32fc_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aValue_native[4];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32fc_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex four-channel image set ignoring alpha.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32fc_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aValue_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32fc_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 8-bit unsigned image set. 
 * @param nValue The pixel value to be set.
 * @param pDst Pointer \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C1MRNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C1MR(nValue=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C1MR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 3 channel 8-bit unsigned image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C3MRNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C3MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u aValue_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C3MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 8-bit unsigned image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C4MRNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C4MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u aValue_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C4MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 8-bit unsigned image set method, not affecting Alpha channel.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1AC4MRNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_AC4MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u aValue_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_AC4MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C1MRNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C1MR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C1MR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C3MRNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C3MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u aValue_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C3MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 16-bit unsigned image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C4MRNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C4MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u aValue_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C4MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 16-bit unsigned image set method, not affecting Alpha channel.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1AC4MRNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_AC4MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u aValue_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_AC4MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 16-bit image set.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C1MRNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C1MR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C1MR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 3 channel 16-bit image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C3MRNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C3MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s aValue_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C3MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C4MRNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C4MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s aValue_native[4];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C4MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 16-bit image set method, not affecting Alpha channel.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1AC4MRNative(JNIEnv *env, jclass cls, jshortArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_AC4MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s aValue_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_AC4MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 32-bit image set.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C1MRNative(JNIEnv *env, jclass cls, jint nValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C1MR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C1MR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 3 channel 32-bit image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C3MRNative(JNIEnv *env, jclass cls, jintArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C3MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s aValue_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C3MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 32-bit image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C4MRNative(JNIEnv *env, jclass cls, jintArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C4MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s aValue_native[4];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C4MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 16-bit image set method, not affecting Alpha channel.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1AC4MRNative(JNIEnv *env, jclass cls, jintArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_AC4MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s aValue_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_AC4MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 32-bit floating point image set.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C1MRNative(JNIEnv *env, jclass cls, jfloat nValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C1MR(nValue=%f, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C1MR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 3 channel 32-bit floating point image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C3MRNative(JNIEnv *env, jclass cls, jfloatArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C3MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f aValue_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C3MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 32-bit floating point image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C4MRNative(JNIEnv *env, jclass cls, jfloatArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C4MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f aValue_native[4];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C4MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 32-bit floating point image set method, not affecting Alpha channel.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1AC4MRNative(JNIEnv *env, jclass cls, jfloatArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_AC4MR(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f aValue_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_AC4MR(aValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned image set affecting only single channel.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C3CRNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C3CR(nValue=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C3CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image set affecting only single channel.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C4CRNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C4CR(nValue=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C4CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned image set affecting only single channel.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C3CRNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C3CR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C3CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image set affecting only single channel.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C4CRNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C4CR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C4CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed image set affecting only single channel.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C3CRNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C3CR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C3CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed image set affecting only single channel.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C4CRNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C4CR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C4CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C3CRNative(JNIEnv *env, jclass cls, jint nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C3CR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C3CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit unsigned image set affecting only single channel.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C4CRNative(JNIEnv *env, jclass cls, jint nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C4CR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C4CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point image set affecting only single channel.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C3CRNative(JNIEnv *env, jclass cls, jfloat nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C3CR(nValue=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C3CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image set affecting only single channel.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C4CRNative(JNIEnv *env, jclass cls, jfloat nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C4CR(nValue=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C4CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 8-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8s_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8s_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit image copy, ignoring alpha channel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image copy, not affecting Alpha channel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image copy, not affecting Alpha channel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit image copy, not affecting Alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16sc_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16sc_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16sc_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16sc_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16sc_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16sc_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16sc_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16sc_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit complex image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116sc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16sc_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16sc_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit image copy, not affecting Alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32sc_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32sc_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32sc_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32sc_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32sc_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32sc_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32sc_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32sc_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132sc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32sc_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32sc_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating point image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image copy, not affecting Alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating-point complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32fc_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32fc_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit floating-point complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132fc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32fc_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32fc_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32fc_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32fc_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32fc_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32fc_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point complex image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32fc_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32fc_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation 8-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C1MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C1MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C3MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C3MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C3MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 8-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 8-bit unsigned image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1AC4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_AC4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_AC4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C1MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C1MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation three channel 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C3MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C3MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C3MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 16-bit unsigned image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1AC4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_AC4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_AC4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation 16-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C1MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C1MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation three channel 16-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C3MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C3MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C3MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 16-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 16-bit signed image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1AC4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_AC4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_AC4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation 32-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C1MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C1MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation three channel 32-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C3MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C3MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C3MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 32-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 32-bit signed image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1AC4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_AC4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_AC4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation 32-bit float image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C1MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C1MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation three channel 32-bit float image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C3MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C3MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C3MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 32-bit float image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 32-bit float image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1AC4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_AC4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_AC4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 8-bit unsigned image copy for three-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C3CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C3CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 8-bit unsigned image copy for four-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C4CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C4CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C4CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 16-bit signed image copy for three-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C3CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C3CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 16-bit signed image copy for four-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C4CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C4CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C4CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 16-bit unsigned image copy for three-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C3CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C3CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 16-bit unsigned image copy for four-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C4CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C4CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C4CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 32-bit signed image copy for three-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C3CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C3CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 32-bit signed image copy for four-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C4CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C4CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C4CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 32-bit float image copy for three-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C3CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C3CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 32-bit float image copy for four-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C4CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C4CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C4CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel to single-channel 8-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel to single-channel 8-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel to single-channel 16-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel to single-channel 16-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel to single-channel 16-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel to single-channel 16-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel to single-channel 32-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel to single-channel 32-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel to single-channel 32-bit float image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel to single-channel 32-bit float image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to four-channel 8-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to three-channel 16-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to four-channel 16-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to four-channel 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to three-channel 32-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to four-channel 32-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to three-channel 32-bit float image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to four-channel 32-bit float image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C3P3R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* aDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C3P3R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_8u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C4P4R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* aDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C4P4R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_16s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C3P3R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* aDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C3P3R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_16s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C4P4R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* aDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C4P4R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_16u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C3P3R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* aDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C3P3R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_16u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C4P4R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* aDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C4P4R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_32s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C3P3R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* aDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C3P3R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_32s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C4P4R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* aDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C4P4R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_32f_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C3P3R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* aDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C3P3R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit float packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_32f_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C4P4R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* aDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C4P4R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned planar to packed image copy.
 * @param aSrc Planar \ref source_image_pointer.
 * @param nSrcStep \ref source_planar_image_pointer_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_P3C3R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* aSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_P3C3R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned planar to packed image copy.
 * @param aSrc Planar \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1P4C4RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_8u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_P4C4R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* aSrc_native[4];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_P4C4R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned planar to packed image copy.
 * @param aSrc Planar \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_16u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_P3C3R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp16u* aSrc_native[3];
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_P3C3R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned planar to packed image copy.
 * @param aSrc Planar \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1P4C4RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_16u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_P4C4R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp16u* aSrc_native[4];
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_P4C4R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed planar to packed image copy.
 * @param aSrc Planar \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_16s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_P3C3R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp16s* aSrc_native[3];
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_P3C3R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed planar to packed image copy.
 * @param aSrc Planar \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1P4C4RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_16s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_P4C4R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp16s* aSrc_native[4];
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_P4C4R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed planar to packed image copy.
 * @param aSrc Planar \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_32s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_P3C3R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp32s* aSrc_native[3];
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_P3C3R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1P4C4RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_32s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_P4C4R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp32s* aSrc_native[4];
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_P4C4R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit float planar to packed image copy.
 * @param aSrc Planar \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_32f_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_P3C3R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp32f* aSrc_native[3];
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_P3C3R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit float planar to packed image copy.
 * @param aSrc Planar \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1P4C4RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_32f_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_P4C4R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp32f* aSrc_native[4];
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_P4C4R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit unsigned conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned to 16-bit unsigned  conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 16-bit unsigned  conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 16-bit unsigned conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned to 16-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 16-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 16-bit signed conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u32s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u32s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u32s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 32-bit signed conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u32s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 32-bit floating-point conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s32s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit signed to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s32s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit signed to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s32s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit signed to 32-bit signed conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s32s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit signed to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit signed to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit signed to 32-bit floating-point conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u32s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u32s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u32s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned to 32-bit signed conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u32s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s32s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s32s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s32s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed to 32-bit signed conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s32s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed to 32-bit floating-point conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 8-bit unsigned conversion with saturation.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s8u_1C1RsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s8u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s8u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s8u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s8u_C1Rs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s8u_C1Rs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit unsigned conversion with saturation.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s16u_1C1RsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s16u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s16u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s16u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s16u_C1Rs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s16u_C1Rs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 32-bit unsigned conversion with saturation.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18s32u_1C1RsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8s32u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8s32u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8s32u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8s32u_C1Rs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8s32u_C1Rs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed to 16-bit unsigned conversion with saturation.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s16u_1C1RsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s16u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s16u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s16u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s16u_C1Rs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s16u_C1Rs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed to 32-bit unsigned conversion with saturation.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s32u_1C1RsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s32u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s32u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s32u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s32u_C1Rs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s32u_C1Rs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned to 32-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u32u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u32u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u32u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit signed to 32-bit unsigned conversion with saturation.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s32u_1C1RsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s32u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s32u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s32u_C1Rs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s32u_C1Rs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s32u_C1Rs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit signed to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u8u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u8u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned to 8-bit unsigned conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u8u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s8u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s8u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed to 8-bit unsigned conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s8u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s8u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s8u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit signed to 8-bit unsigned conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s8u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit signed to 8-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s8s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s8s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s8s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit signed to 8-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s8s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s8s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s8s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s8s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s8s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s8s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit signed to 8-bit signed conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s8s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s8s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s8s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u8s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u8s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u8s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned to 8-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode \ref rounding_mode_parameter.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u8s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u8s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u8s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed to 8-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode \ref rounding_mode_parameter.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s8s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s8s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s8s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u16s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit unsigned to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode \ref rounding_mode_parameter.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132u8u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32u8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32u8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32u8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32u8u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32u8u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132u8s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32u8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32u8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32u8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32u8s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32u8s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit unsigned to 16-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode \ref rounding_mode_parameter.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132u16u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32u16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32u16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32u16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32u16u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32u16u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit unsigned to 16-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode \ref rounding_mode_parameter.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132u16s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32u16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32u16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32u16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32u16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32u16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit unsigned to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode \ref rounding_mode_parameter.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132u32s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32u32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32u32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32u32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32u32s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32u32s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit unsigned to 16-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode \ref rounding_mode_parameter.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s16u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s16u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s16u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit unsigned to 16-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode \ref rounding_mode_parameter.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132s16s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32s16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32s16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32s16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32s16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32s16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating point to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating point to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f8u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating point to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f8u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating point to 8-bit unsigned conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f8u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f8s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f8s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f8s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating point to 8-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f8s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f8s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f8s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating point to 8-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f8s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f8s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f8s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating point to 8-bit signed conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f8s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f8s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f8s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating point to 16-bit unsigned conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating point to 16-bit unsigned conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating point to 16-bit unsigned conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating point to 16-bit unsigned conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating point to 16-bit signed conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating point to 16-bit signed conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating point to 16-bit signed conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating point to 16-bit signed conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating point to 8-bit unsigned conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f8u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f8u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f8u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating point to 8-bit signed conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f8s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f8s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f8s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f8s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating point to 16-bit unsigned conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating point to 16-bit signed conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f32u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f32u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f32u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f32u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f32u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f32u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating point to 32-bit signed conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f32s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f32s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f32s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit unsigned conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u16u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned to 16-bit unsigned  conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u16u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u16u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 16-bit unsigned conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u16u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u16s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 16-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u16s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 16-bit signed conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u16s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u32s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u32s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 32-bit signed conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u32s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u32s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nMin specifies the minimum saturation value to which every output value will be clamped.
 * @param nMax specifies the maximum saturation value to which every output value will be clamped.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nMin, jfloat nMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMin is primitive
    // nMax is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nMin=%f, nMax=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nMin_native;
    Npp32f nMax_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMin_native = (Npp32f)nMin;
    nMax_native = (Npp32f)nMax;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nMin_native, nMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nMin is primitive
    // nMax is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nMin specifies the minimum saturation value to which every output value will be clamped.
 * @param nMax specifies the maximum saturation value to which every output value will be clamped.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nMin, jfloat nMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMin is primitive
    // nMax is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nMin=%f, nMax=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nMin_native;
    Npp32f nMax_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMin_native = (Npp32f)nMin;
    nMax_native = (Npp32f)nMax;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nMin_native, nMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nMin is primitive
    // nMax is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 32-bit floating-point conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nMin specifies the minimum saturation value to which every output value will be clamped.
 * @param nMax specifies the maximum saturation value to which every output value will be clamped.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nMin, jfloat nMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMin is primitive
    // nMax is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nMin=%f, nMax=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nMin_native;
    Npp32f nMax_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMin_native = (Npp32f)nMin;
    nMax_native = (Npp32f)nMax;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nMin_native, nMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nMin is primitive
    // nMax is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 32-bit floating-point conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nMin specifies the minimum saturation value to which every output value will be clamped.
 * @param nMax specifies the maximum saturation value to which every output value will be clamped.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nMin, jfloat nMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMin is primitive
    // nMax is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_8u32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nMin=%f, nMax=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nMin_native;
    Npp32f nMax_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMin_native = (Npp32f)nMin;
    nMax_native = (Npp32f)nMax;

    // Native function call
    NppStatus jniResult_native = nppiScale_8u32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nMin_native, nMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nMin is primitive
    // nMax is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_116u8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_16u8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_16u8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_116u8u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_16u8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_16u8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_16u8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_16u8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_16u8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_116u8u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_16u8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_16u8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_16u8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_16u8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_16u8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned to 8-bit unsigned conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_116u8u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_16u8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_16u8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_16u8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_16u8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_16u8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_116s8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_16s8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_16s8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_116s8u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_16s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_16s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_16s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_16s8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_16s8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_116s8u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_16s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_16s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_16s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_16s8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_16s8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed to 8-bit unsigned conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_116s8u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_16s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_16s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_16s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_16s8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_16s8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_132s8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_32s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_32s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_32s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_32s8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_32s8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_132s8u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_32s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_32s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_32s8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_32s8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_32s8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit signed to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_132s8u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_32s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_32s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_32s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_32s8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_32s8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit signed to 8-bit unsigned conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param hint algorithm performance or accuracy selector, currently ignored
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_132s8u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint hint)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_32s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_32s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_32s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // hint is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_32s8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, hint=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, hint);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppHintAlgorithm hint_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    hint_native = (NppHintAlgorithm)hint;

    // Native function call
    NppStatus jniResult_native = nppiScale_32s8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, hint_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // hint is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_132f8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nMin, jfloat nMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_32f8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_32f8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_32f8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMin is primitive
    // nMax is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_32f8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nMin=%f, nMax=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nMin_native;
    Npp32f nMax_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMin_native = (Npp32f)nMin;
    nMax_native = (Npp32f)nMax;

    // Native function call
    NppStatus jniResult_native = nppiScale_32f8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nMin_native, nMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nMin is primitive
    // nMax is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating point to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nMin specifies the minimum saturation value to which every output value will be clamped.
 * @param nMax specifies the maximum saturation value to which every output value will be clamped.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_132f8u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nMin, jfloat nMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_32f8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_32f8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_32f8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMin is primitive
    // nMax is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_32f8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nMin=%f, nMax=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nMin_native;
    Npp32f nMax_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMin_native = (Npp32f)nMin;
    nMax_native = (Npp32f)nMax;

    // Native function call
    NppStatus jniResult_native = nppiScale_32f8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nMin_native, nMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nMin is primitive
    // nMax is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating point to 8-bit unsigned conversion.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nMin specifies the minimum saturation value to which every output value will be clamped.
 * @param nMax specifies the maximum saturation value to which every output value will be clamped.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_132f8u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nMin, jfloat nMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_32f8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_32f8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_32f8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMin is primitive
    // nMax is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_32f8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nMin=%f, nMax=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nMin_native;
    Npp32f nMax_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMin_native = (Npp32f)nMin;
    nMax_native = (Npp32f)nMax;

    // Native function call
    NppStatus jniResult_native = nppiScale_32f8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nMin_native, nMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nMin is primitive
    // nMax is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating point to 8-bit unsigned conversion, not affecting Alpha.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nMin specifies the minimum saturation value to which every output value will be clamped.
 * @param nMax specifies the maximum saturation value to which every output value will be clamped.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, ::NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiScale_132f8u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nMin, jfloat nMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiScale_32f8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiScale_32f8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiScale_32f8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMin is primitive
    // nMax is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiScale_32f8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nMin=%f, nMax=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nMin, nMax);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nMin_native;
    Npp32f nMax_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMin_native = (Npp32f)nMin;
    nMax_native = (Npp32f)nMax;

    // Native function call
    NppStatus jniResult_native = nppiScale_32f8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nMin_native, nMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nMin is primitive
    // nMax is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned integer image copy with constant border color.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and constant border color (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the constant border color.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @param nValue The pixel value to be set for border pixels.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jbyte nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, nValue=%c)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp8u nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    nValue_native = (Npp8u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_8u_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned integer image copy with constant border color.
 * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGBA values of the border pixels to be set.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jbyteArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp8u aValue_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_8u_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer image copy with constant border color.
 * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGBA values of the border pixels to be set.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jbyteArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp8u aValue_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_8u_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer image copy with constant border color with alpha channel unaffected.
 * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGB values of the border pixels. Because this method does not
 *      affect the destination image's alpha channel, only three components of the border color
 *      are needed.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jbyteArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp8u aValue_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned integer image copy with constant border color.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and constant border color (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the constant border color.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @param nValue The pixel value to be set for border pixels.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_16u_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, nValue=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp16u nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    nValue_native = (Npp16u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_16u_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned integer image copy with constant border color.
 * See nppiCopyConstBorder_16u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGBA values of the border pixels to be set.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jshortArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_16u_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp16u aValue_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_16u_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned integer image copy with constant border color.
 * See nppiCopyConstBorder_16u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGBA values of the border pixels to be set.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jshortArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_16u_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp16u aValue_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_16u_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned integer image copy with constant border color with alpha channel unaffected.
 * See nppiCopyConstBorder_16u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGB values of the border pixels. Because this method does not
 *      affect the destination image's alpha channel, only three components of the border color
 *      are needed.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jshortArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_16u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp16u aValue_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_16u_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed integer image copy with constant border color.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and constant border color (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the constant border color.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @param nValue The pixel value to be set for border pixels.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, nValue=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp16s nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    nValue_native = (Npp16s)nValue;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_16s_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed integer image copy with constant border color.
 * See nppiCopyConstBorder_16s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGBA values of the border pixels to be set.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jshortArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_16s_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp16s aValue_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_16s_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed integer image copy with constant border color.
 * See nppiCopyConstBorder_16s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGBA values of the border pixels to be set.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jshortArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_16s_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp16s aValue_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_16s_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed integer image copy with constant border color with alpha channel unaffected.
 * See nppiCopyConstBorder_16s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGB values of the border pixels. Because this method does not
 *      affect the destination image's alpha channel, only three components of the border color
 *      are needed.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jshortArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_16s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp16s aValue_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_16s_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit signed integer image copy with constant border color.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and constant border color (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the constant border color.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @param nValue The pixel value to be set for border pixels.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jint nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_32s_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, nValue=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp32s nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    nValue_native = (Npp32s)nValue;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_32s_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit signed integer image copy with constant border color.
 * See nppiCopyConstBorder_32s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGBA values of the border pixels to be set.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jintArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_32s_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp32s aValue_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_32s_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed integer image copy with constant border color.
 * See nppiCopyConstBorder_32s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGBA values of the border pixels to be set.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jintArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_32s_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp32s aValue_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_32s_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed integer image copy with constant border color with alpha channel unaffected.
 * See nppiCopyConstBorder_32s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGB values of the border pixels. Because this method does not
 *      affect the destination image's alpha channel, only three components of the border color
 *      are needed.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jintArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_32s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp32s aValue_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_32s_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point image copy with constant border color.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and constant border color (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the constant border color.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @param nValue The pixel value to be set for border pixels.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jfloat nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, nValue=%f)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp32f nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    nValue_native = (Npp32f)nValue;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jfloatArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp32f aValue_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_32f_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jfloatArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp32f aValue_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_32f_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image copy with constant border color with alpha channel unaffected.
 * See nppiCopyConstBorder_32f_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGB values of the border pixels. Because this method does not
 *      affect the destination image's alpha channel, only three components of the border color
 *      are needed.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jfloatArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp32f aValue_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_32f_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned integer image copy with nearest source image pixel color.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and nearest source image pixel color (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the nearest source image pixel color.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_8u_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned integer image copy with nearest source image pixel color.
 * See nppiCopyReplicateBorder_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_8u_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer image copy with nearest source image pixel color.
 * See nppiCopyReplicateBorder_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_8u_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer image copy with nearest source image pixel color with alpha channel unaffected.
 * See nppiCopyReplicateBorder_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_16u_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_16u_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned integer image copy with nearest source image pixel color.
 * See nppiCopyReplicateBorder_16u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_16u_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_16u_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned integer image copy with nearest source image pixel color.
 * See nppiCopyReplicateBorder_16u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_16u_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_16u_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image copy with nearest source image pixel color with alpha channel unaffected.
 * See nppiCopyReplicateBorder_16u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_16u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_16u_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed integer image copy with nearest source image pixel color.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and nearest source image pixel color (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the nearest source image pixel color.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_16s_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed integer image copy with nearest source image pixel color.
 * See nppiCopyReplicateBorder_16s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_16s_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_16s_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed integer image copy with nearest source image pixel color.
 * See nppiCopyReplicateBorder_16s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_16s_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_16s_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed integer image copy with nearest source image pixel color with alpha channel unaffected.
 * See nppiCopyReplicateBorder_16s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_16s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_16s_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit signed integer image copy with nearest source image pixel color.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and nearest source image pixel color (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the nearest source image pixel color.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_32s_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_32s_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit signed image copy with nearest source image pixel color.
 * See nppiCopyReplicateBorder_32s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_32s_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_32s_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed integer image copy with nearest source image pixel color.
 * See nppiCopyReplicateBorder_32s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_32s_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_32s_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed integer image copy with nearest source image pixel color with alpha channel unaffected.
 * See nppiCopyReplicateBorder_32s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_32s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_32s_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point image copy with nearest source image pixel color.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and nearest source image pixel color (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the nearest source image pixel color.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point image copy with nearest source image pixel color.
 * See nppiCopyReplicateBorder_32f_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_32f_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image copy with nearest source image pixel color.
 * See nppiCopyReplicateBorder_32f_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_32f_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image copy with nearest source image pixel color with alpha channel unaffected.
 * See nppiCopyReplicateBorder_32f_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyReplicateBorder_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyReplicateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyReplicateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyReplicateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyReplicateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyReplicateBorder_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyReplicateBorder_32f_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and a border consisting of wrapped replication of the source image pixel colors (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the wrapped replication of the corresponding column of source image pixels colors.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_8u_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_8u_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors.
 * See nppiCopyWrapBorder_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_8u_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors with alpha channel unaffected.
 * See nppiCopyWrapBorder_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and a border consisting of wrapped replication of the source image pixel colors (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the wrapped replication of the corresponding column of source image pixels colors.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_16u_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_16u_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors.
 * See nppiCopyWrapBorder_16u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_16u_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_16u_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors.
 * See nppiCopyWrapBorder_16u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_16u_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_16u_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned integer image copy with the borders wrapped by replication of source image pixel colors with alpha channel unaffected.
 * See nppiCopyWrapBorder_16u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_16u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_16u_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed integer image copy with the borders wrapped by replication of source image pixel colors.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and a border consisting of wrapped replication of the source image pixel colors (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the wrapped replication of the corresponding column of source image pixels colors.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_16s_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed integer image copy with the borders wrapped by replication of source image pixel colors.
 * See nppiCopyWrapBorder_16s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_16s_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_16s_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_16s_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_16s_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed integer image copy with the borders wrapped by replication of source image pixel colors with alpha channel unaffected.
 * See nppiCopyWrapBorder_16s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_16s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_16s_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_32s_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_32s_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit signed integer image copy with the borders wrapped by replication of source image pixel colors.
 * See nppiCopyWrapBorder_32s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_32s_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_32s_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed integer image copy with the borders wrapped by replication of source image pixel colors.
 * See nppiCopyWrapBorder_32s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_32s_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_32s_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_32s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_32s_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point image copy with the borders wrapped by replication of source image pixel colors.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and a border consisting of wrapped replication of the source image pixel colors (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the wrapped replication of the corresponding column of source image pixels colors.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point image copy with the borders wrapped by replication of source image pixel colors.
 * See nppiCopyWrapBorder_32f_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_32f_C3R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image copy with the borders wrapped by replication of source image pixel colors.
 * See nppiCopyWrapBorder_32f_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_32f_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point image copy with the borders wrapped by replication of source image pixel colors with alpha channel unaffected.
 * See nppiCopyWrapBorder_32f_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyWrapBorder_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyWrapBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyWrapBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyWrapBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyWrapBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyWrapBorder_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;

    // Native function call
    NppStatus jniResult_native = nppiCopyWrapBorder_32f_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned integer linearly interpolated source image subpixel coordinate color copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image, source image ROI is assumed to be same as destination image ROI.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned integer linearly interpolated source image subpixel coordinate color copy.
 * See nppiCopySubpix_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer linearly interpolated source image subpixel coordinate color copy.
 * See nppiCopySubpix_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer linearly interpolated source image subpixel coordinate color copy with alpha channel unaffected.
 * See nppiCopySubpix_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned integer linearly interpolated source image subpixel coordinate color copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image, source image ROI is assumed to be same as destination image ROI.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned integer linearly interpolated source image subpixel coordinate color copy.
 * See nppiCopySubpix_16u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned linearly interpolated source image subpixel coordinate color copy with alpha channel unaffected.
 * See nppiCopySubpix_16u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed integer linearly interpolated source image subpixel coordinate color copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image, source image ROI is assumed to be same as destination image ROI.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed integer linearly interpolated source image subpixel coordinate color copy.
 * See nppiCopySubpix_16s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed integer linearly interpolated source image subpixel coordinate color copy.
 * See nppiCopySubpix_16s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed integer linearly interpolated source image subpixel coordinate color copy with alpha channel unaffected.
 * See nppiCopySubpix_16s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit signed integer linearly interpolated source image subpixel coordinate color copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image, source image ROI is assumed to be same as destination image ROI.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit signed linearly interpolated source image subpixel coordinate color copy.
 * See nppiCopySubpix_32s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed integer linearly interpolated source image subpixel coordinate color copy.
 * See nppiCopySubpix_32s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed integer linearly interpolated source image subpixel coordinate color copy with alpha channel unaffected.
 * See nppiCopySubpix_32s_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point linearly interpolated source image subpixel coordinate color copy.
 * See nppiCopySubpix_32f_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point linearly interpolated source image subpixel coordinate color copy.
 * See nppiCopySubpix_32f_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point linearly interpolated source image subpixel coordinate color copy with alpha channel unaffected.
 * See nppiCopySubpix_32f_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nDx Fractional part of source image X coordinate.
 * @param nDy Fractional part of source image Y coordinate.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopySubpix_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jfloat nDx, jfloat nDy)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopySubpix_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopySubpix_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopySubpix_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDx is primitive
    // nDy is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopySubpix_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nDx=%f, nDy=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI, nDx, nDy);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    Npp32f nDx_native;
    Npp32f nDy_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDx_native = (Npp32f)nDx;
    nDy_native = (Npp32f)nDy;

    // Native function call
    NppStatus jniResult_native = nppiCopySubpix_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, nDx_native, nDy_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nDx is primitive
    // nDy is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned integer source image duplicated in all 3 channels of destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image, source image ROI is assumed to be same as destination image ROI.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_18u_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_8u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_8u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_8u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_8u_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_8u_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned integer source image duplicated in all 4 channels of destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_18u_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_8u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_8u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_8u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_8u_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_8u_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned integer source image duplicated in 3 channels of 4 channel destination image with alpha channel unaffected.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_18u_1C1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_8u_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_8u_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_8u_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_8u_C1AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_8u_C1AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned integer source image duplicated in all 3 channels of destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image, source image ROI is assumed to be same as destination image ROI.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_116u_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_16u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_16u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_16u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_16u_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_16u_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned integer source image duplicated in all 4 channels of destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_116u_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_16u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_16u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_16u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_16u_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_16u_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned integer source image duplicated in 3 channels of 4 channel destination image with alpha channel unaffected.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_116u_1C1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_16u_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_16u_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_16u_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_16u_C1AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_16u_C1AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed integer source image duplicated in all 3 channels of destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image, source image ROI is assumed to be same as destination image ROI.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_116s_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_16s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_16s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_16s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_16s_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_16s_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed integer source image duplicated in all 4 channels of destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_116s_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_16s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_16s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_16s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_16s_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_16s_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed integer source image duplicated in 3 channels of 4 channel destination image with alpha channel unaffected.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_116s_1C1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_16s_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_16s_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_16s_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_16s_C1AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_16s_C1AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit signed integer source image duplicated in all 3 channels of destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image, source image ROI is assumed to be same as destination image ROI.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_132s_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_32s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_32s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_32s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_32s_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_32s_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit signed integer source image duplicated in all 4 channels of destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_132s_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_32s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_32s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_32s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_32s_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_32s_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit signed integer source image duplicated in 3 channels of 4 channel destination image with alpha channel unaffected.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_132s_1C1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_32s_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_32s_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_32s_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_32s_C1AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_32s_C1AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point source image duplicated in all 3 channels of destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image, source image ROI is assumed to be same as destination image ROI.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_132f_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_32f_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_32f_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_32f_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_32f_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_32f_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point source image duplicated in all 4 channels of destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_132f_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_32f_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_32f_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_32f_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_32f_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_32f_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point source image duplicated in 3 channels of 4 channel destination image with alpha channel unaffected.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDup_132f_1C1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oDstSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDup_32f_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDup_32f_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiDup_32f_C1AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDup_32f_C1AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oDstSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDup_32f_C1AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oDstSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned int image transpose.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned int image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned int image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned int image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned int image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed int image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed int image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed int image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit signed int image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit signed int image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed int image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSrcROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiTranspose_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSrcROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSrcROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSrcROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSrcROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSrcROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned integer source image to 3 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned integer in place image.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSwapChannels_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer source image to 3 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to a 3 channel BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_18u_1C4C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_8u_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_8u_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_8u_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_8u_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_8u_C4C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_8u_C4C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer source image to 4 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer in place image.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSwapChannels_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_8u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_8u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned integer source image to 4 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
 *      channel order.
 * @param nValue (V) Single channel constant value that can be replicated in one or more of the 4 destination channels.
 *      nValue is either written or not written to a particular channel depending on the aDstOrder entry for that destination
 *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
 *      particular destination channel value unmodified.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_18u_1C3C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder, jbyte nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_8u_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_8u_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_8u_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_8u_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_8u_C3C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p, nValue=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder, nValue);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];
    Npp8u nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nValue_native = (Npp8u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_8u_C3C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned integer source image to 4 channel destination image with destination alpha channel unaffected.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
 *      channel order.
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to BGRA
 *      channel order. In the AC4R case, the alpha channel is always assumed to be channel 3.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned integer source image to 3 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSwapChannels_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned integer source image to 3 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to a 3 channel BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116u_1C4C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_16u_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_16u_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16u_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16u_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16u_C4C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16u_C4C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned integer source image to 4 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned integer in place image.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSwapChannels_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned integer source image to 4 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
 *      channel order.
 * @param nValue (V) Single channel constant value that can be replicated in one or more of the 4 destination channels.
 *      nValue is either written or not written to a particular channel depending on the aDstOrder entry for that destination
 *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
 *      particular destination channel value unmodified.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116u_1C3C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_16u_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_16u_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16u_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16u_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16u_C3C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p, nValue=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder, nValue);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];
    Npp16u nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nValue_native = (Npp16u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16u_C3C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned integer source image to 4 channel destination image with destination alpha channel unaffected.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to BGRA
 *      channel order. In the AC4R case, the alpha channel is always assumed to be channel 3.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed integer source image to 3 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed integer in place image.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSwapChannels_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116s_1C4C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_16s_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_16s_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16s_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16s_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16s_C4C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16s_C4C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed integer source image to 4 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSwapChannels_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16s_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16s_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed integer source image to 4 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
 *      channel order.
 * @param nValue (V) Single channel constant value that can be replicated in one or more of the 4 destination channels.
 *      nValue is either written or not written to a particular channel depending on the aDstOrder entry for that destination
 *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
 *      particular destination channel value unmodified.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116s_1C3C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_16s_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_16s_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16s_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16s_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16s_C3C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p, nValue=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder, nValue);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];
    Npp16s nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nValue_native = (Npp16s)nValue;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16s_C3C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed integer source image to 4 channel destination image with destination alpha channel unaffected.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to BGRA
 *      channel order. In the AC4R case, the alpha channel is always assumed to be channel 3.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit signed integer source image to 3 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit signed integer in place image.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSwapChannels_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed integer source image to 3 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to a 3 channel BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132s_1C4C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_32s_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_32s_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32s_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32s_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32s_C4C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32s_C4C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed integer source image to 4 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed integer in place image.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSwapChannels_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32s_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32s_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit signed integer source image to 4 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
 *      channel order.
 * @param nValue (V) Single channel constant value that can be replicated in one or more of the 4 destination channels.
 *      nValue is either written or not written to a particular channel depending on the aDstOrder entry for that destination
 *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
 *      particular destination channel value unmodified.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132s_1C3C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder, jint nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_32s_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_32s_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32s_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32s_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32s_C3C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p, nValue=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder, nValue);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];
    Npp32s nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nValue_native = (Npp32s)nValue;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32s_C3C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed integer source image to 4 channel destination image with destination alpha channel unaffected.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to BGRA
 *      channel order. In the AC4R case, the alpha channel is always assumed to be channel 3.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point source image to 3 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point in place image.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSwapChannels_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point source image to 3 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to a 3 channel BGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132f_1C4C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_32f_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_32f_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32f_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32f_C4C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32f_C4C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32f_C4C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point source image to 4 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point in place image.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an ARGB image, aDstOrder = [3,2,1,0] converts this to BGRA
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSwapChannels_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point source image to 4 channel destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
 *      channel order.
 * @param nValue (V) Single channel constant value that can be replicated in one or more of the 4 destination channels.
 *      nValue is either written or not written to a particular channel depending on the aDstOrder entry for that destination
 *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
 *      particular destination channel value unmodified.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132f_1C3C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder, jfloat nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_32f_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_32f_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32f_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32f_C3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32f_C3C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p, nValue=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder, nValue);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];
    Npp32f nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nValue_native = (Npp32f)nValue;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32f_C3C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point source image to 4 channel destination image with destination alpha channel unaffected.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGBA image, aDstOrder = [2,1,0] converts this to BGRA
 *      channel order. In the AC4R case, the alpha channel is always assumed to be channel 3.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSwapChannels_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSwapChannels_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C1IRSfs(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel..
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel 8-bit unsigned char in place image add constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel..
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel..
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image add constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel..
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image add constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[4];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image add constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[4];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s nConstant_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image add constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s nConstant_native;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image add constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[3];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image add constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[3];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[4];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image add constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[4];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc nConstant_native;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image add constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc nConstant_native;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc aConstants_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image add constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc aConstants_native[3];
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc aConstants_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc aConstants_native[3];
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image add constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32s_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image add constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32s_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32s_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc nConstant_native;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image add constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc nConstant_native;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc aConstants_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc aConstants_native[3];
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc aConstants_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image add constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc aConstants_native[3];
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloat nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f nConstant_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32f)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image add constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C1IRNative(JNIEnv *env, jclass cls, jfloat nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C1IR(nConstant=%f, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nConstant_native;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32f)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image add constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C3IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[3];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image add constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1AC4IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[3];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[4];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image add constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C4IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[4];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc nConstant_native;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image add constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C1IR(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc nConstant_native;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[3];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha in place image add constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[3];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[4];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image add constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAddC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[4];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C1IRSfs(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel 8-bit unsigned char in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[4];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[4];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s nConstant_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s nConstant_native;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[3];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[3];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[4];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[4];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc nConstant_native;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc nConstant_native;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc aConstants_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc aConstants_native[3];
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc aConstants_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc aConstants_native[3];
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32s_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32s_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32s_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc nConstant_native;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc nConstant_native;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc aConstants_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc aConstants_native[3];
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc aConstants_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc aConstants_native[3];
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloat nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f nConstant_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32f)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image multiply by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C1IRNative(JNIEnv *env, jclass cls, jfloat nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C1IR(nConstant=%f, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nConstant_native;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32f)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image multiply by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C3IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[3];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image multiply by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1AC4IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[3];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[4];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image multiply by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C4IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[4];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc nConstant_native;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C1IR(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc nConstant_native;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image multiply by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[3];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[3];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[4];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image multiply by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[4];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image multiply by constant and scale by max bit width value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C1IRNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C1IR(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel 8-bit unsigned char in place image multiply by constant and scale by max bit width value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C3IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image multiply by constant, scale and scale by max bit width value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1AC4IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image multiply by constant and scale by max bit width value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C4IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[4];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C1IRNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image multiply by constant and scale by max bit width value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C3IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image multiply by constant and scale by max bit width value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1AC4IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image multiply by constant and scale by max bit width value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C4IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiMulCScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[4];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C1IRSfs(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel 8-bit unsigned char in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[4];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[4];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s nConstant_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s nConstant_native;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[3];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[3];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[4];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[4];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc nConstant_native;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc nConstant_native;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc aConstants_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc aConstants_native[3];
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc aConstants_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc aConstants_native[3];
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32s_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32s_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32s_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc nConstant_native;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc nConstant_native;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc aConstants_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc aConstants_native[3];
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc aConstants_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc aConstants_native[3];
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloat nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f nConstant_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32f)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image subtract constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C1IRNative(JNIEnv *env, jclass cls, jfloat nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C1IR(nConstant=%f, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nConstant_native;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32f)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image subtract constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C3IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[3];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image subtract constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1AC4IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[3];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[4];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image subtract constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C4IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[4];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc nConstant_native;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image subtract constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C1IR(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc nConstant_native;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image subtract constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[3];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[3];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[4];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image subtract constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiSubC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[4];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C1IRSfs(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel 8-bit unsigned char in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u aConstants_native[4];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u aConstants_native[4];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s nConstant_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s nConstant_native;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[3];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[3];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s aConstants_native[4];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s aConstants_native[4];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc nConstant_native;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc nConstant_native;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc aConstants_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc aConstants_native[3];
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc aConstants_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc aConstants_native[3];
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image divided by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32s_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32s_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32s_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc nConstant_native;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc nConstant_native;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc aConstants_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_C3IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc aConstants_native[3];
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_C3IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc aConstants_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_AC4IRSfs(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc aConstants_native[3];
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_AC4IRSfs(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloat nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f nConstant_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32f)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C1IRNative(JNIEnv *env, jclass cls, jfloat nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C1IR(nConstant=%f, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nConstant_native;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32f)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image divided by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C3IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[3];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image divided by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1AC4IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[3];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloatArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f aConstants_native[4];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C4IRNative(JNIEnv *env, jclass cls, jfloatArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aConstants_native[4];
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc nConstant_native;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image divided by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C1IR(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc nConstant_native;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image divided by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[3];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha in place image divided by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[3];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobjectArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc aConstants_native[4];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobjectArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiDivC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aConstants_native[4];
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image absolute difference with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiffC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nConstant)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiffC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiffC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiffC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nConstant is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiffC_8u_C1R(pSrc1=%p, nSrc1Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nConstant=%c)\n",
        pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI, nConstant);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nConstant_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nConstant_native = (Npp8u)nConstant;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiffC_8u_C1R(pSrc1_native, nSrc1Step_native, pDst_native, nDstStep_native, oSizeROI_native, nConstant_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nConstant is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image absolute difference with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiffC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nConstant)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiffC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiffC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiffC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nConstant is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiffC_16u_C1R(pSrc1=%p, nSrc1Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nConstant=%d)\n",
        pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI, nConstant);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nConstant_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nConstant_native = (Npp16u)nConstant;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiffC_16u_C1R(pSrc1_native, nSrc1Step_native, pDst_native, nDstStep_native, oSizeROI_native, nConstant_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nConstant is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiffC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nConstant)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiffC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiffC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiffC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nConstant is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiffC_32f_C1R(pSrc1=%p, nSrc1Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nConstant=%f)\n",
        pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI, nConstant);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nConstant_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nConstant_native = (Npp32f)nConstant;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiffC_32f_C1R(pSrc1_native, nSrc1Step_native, pDst_native, nDstStep_native, oSizeROI_native, nConstant_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nConstant is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead. 
 * 32-bit image add.
 * Add the pixel values of corresponding pixels in the ROI and write them to the output image.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image squared then added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_18u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddSquare_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_8u32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_8u32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image squared then added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_18u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_8u32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_8u32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_116u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddSquare_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_16u32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_16u32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image squared then added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_116u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_16u32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_16u32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image squared then added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_132f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddSquare_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image squared then added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_18u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddProduct_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_8u32f_C1IMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_8u32f_C1IMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image product added to in place floating point destination image.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_18u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_8u32f_C1IR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_8u32f_C1IR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_116u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddProduct_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_16u32f_C1IMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_16u32f_C1IMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image product added to in place floating point destination image.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_116u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_16u32f_C1IR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_16u32f_C1IR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image product added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_132f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddProduct_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_32f_C1IMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_32f_C1IMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image product added to in place floating point destination image.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_32f_C1IR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_32f_C1IR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel alpha weighted image added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_18u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddWeighted_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_8u32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_8u32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel alpha weighted image added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_18u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_8u32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_8u32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel alpha weighted image added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_116u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddWeighted_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_16u32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_16u32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel alpha weighted image added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_116u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_16u32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_16u32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_132f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddWeighted_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel alpha weighted image added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead.
 * 1 channel 32-bit image multiplication.
 * Multiply corresponding pixels in ROI. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead. 
 * 32-bit image subtraction.
 * Subtract pSrc1's pixels from corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead. 
 * 32-bit image division.
 * Divide pixels in pSrc2 by pSrc1's pixels. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image absolute value with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image absolute value with unmodified alpha.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image absolute value with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image absolute value with unmodified alpha.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel absolute difference of image1 minus image2.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiff_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiff_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAbsDiff_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiff_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiff_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiff_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiff_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channels absolute difference of image1 minus image2.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiff_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiff_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAbsDiff_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiff_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiff_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiff_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiff_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channels absolute difference of image1 minus image2.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiff_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiff_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAbsDiff_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiff_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiff_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiff_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiff_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel absolute difference of image1 minus image2.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiff_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiff_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAbsDiff_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiff_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiff_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiff_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiff_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel absolute difference of image1 minus image2.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiff_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiff_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAbsDiff_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiff_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiff_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiff_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiff_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image squared.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image squared.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image squared.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image squared with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image squared with unmodified alpha.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image squared.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image squared.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image square root.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image square root.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image square root.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image square root.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image square root with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image square root with unmodified alpha.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image square root.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image square root.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_8u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_8u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_8u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_8u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_8u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_8u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_8u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_8u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16s_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16s_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16s_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16s_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16s_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16s_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image natural logarithm.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLn_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image natural logarithm.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLn_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image natural logarithm.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLn_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image natural logarithm.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLn_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_8u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_8u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_8u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_8u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_8u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_8u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_8u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_8u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16s_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16s_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16s_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16s_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16s_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16s_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image exponential.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiExp_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image exponential.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiExp_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image exponential.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiExp_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image exponential.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiExp_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical and with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C1IRNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C1IR(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical and with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C3IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical and with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical and with constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical and with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C4IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[4];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical and with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C1IRNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical and with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C3IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical and with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical and with constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical and with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C4IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[4];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical and with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical and with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C3IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical and with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical and with constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1AC4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[4];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical and with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiAndC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aConstants_native[4];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C1IRNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C1IR(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C3IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical or with constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C4IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[4];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical or with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C1IRNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C3IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical or with constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C4IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[4];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical or with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C3IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical or with constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1AC4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[4];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiOrC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aConstants_native[4];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical exclusive or with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C1IRNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C1IR(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical exclusive or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C3IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical exclusive or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical exclusive or with constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyteArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u aConstants_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical exclusive or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C4IRNative(JNIEnv *env, jclass cls, jbyteArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aConstants_native[4];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical exclusive or with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C1IRNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical exclusive or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C3IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical exclusive or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical exclusive or with constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshortArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u aConstants_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical exclusive or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C4IRNative(JNIEnv *env, jclass cls, jshortArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aConstants_native[4];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical exclusive or with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical exclusive or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C3IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical exclusive or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical exclusive or with constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1AC4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s aConstants_native[4];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical exclusive or with constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiXorC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aConstants_native[4];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image right shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image right shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C3IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image right shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image right shift by constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image right shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[4];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit signed char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit signed char channel in place image right shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit signed char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit signed char channel in place image right shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C3IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit signed char channel image right shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit signed char channel in place image right shift by constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1AC4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit signed char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[4];
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit signed char channel in place image right shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_8s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[4];
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image right shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image right shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C3IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image right shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image right shift by constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image right shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[4];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image right shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image right shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C3IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image right shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image right shift by constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1AC4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[4];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image right shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[4];
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image right shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image right shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C3IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image right shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image right shift by constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1AC4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[4];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image right shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiRShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[4];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} image_rshiftc */
/**
 * <pre>
 * @defgroup image_lshiftc LShiftC
 *
 * Pixel by pixel left shift of an image by a constant value.
 *
 * @{
 * </pre>
 */
/**
 * <pre>
 * One 8-bit unsigned char channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image left shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image left shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C3IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image left shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image left shift by constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image left shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[4];
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image left shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image left shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C3IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image left shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image left shift by constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image left shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[4];
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image left shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C3R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C3R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image left shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C3IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C3IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C3IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image left shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_AC4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_AC4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image left shift by constant with unmodified alpha.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1AC4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_AC4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[3];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_AC4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jintArray aConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C4R(pSrc1=%p, nSrc1Step=%d, aConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, aConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u aConstants_native[4];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C4R(pSrc1_native, nSrc1Step_native, aConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image left shift by constant.
 * @param aConstants fixed size array of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C4IRNative(JNIEnv *env, jclass cls, jintArray aConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiLShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C4IR(aConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        aConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u aConstants_native[4];
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C4IR(aConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical and with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical and with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical and with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical and with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical and with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical and with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical exclusive or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical exclusive or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical exclusive or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical exclusive or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical exclusive or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical exclusive or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image logical not.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNot_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiNot_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical not.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiNot_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical not.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNot_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiNot_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical not.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiNot_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNot_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiNot_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical not with unmodified alpha.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiNot_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical not.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNot_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiNot_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical not.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiNot_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pSrc2, jint nSrc2Step, jbyte nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pSrc2=%p, nSrc2Step=%d, nAlpha2=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u nAlpha2_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp8u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_8u_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pSrc2, jint nSrc2Step, jbyte nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_8u_C3R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pSrc2=%p, nSrc2Step=%d, nAlpha2=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u nAlpha2_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp8u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_8u_C3R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pSrc2, jint nSrc2Step, jbyte nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_8u_C4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pSrc2=%p, nSrc2Step=%d, nAlpha2=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u nAlpha2_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp8u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_8u_C4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image composition with alpha using constant source alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pSrc2, jint nSrc2Step, jbyte nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pSrc2=%p, nSrc2Step=%d, nAlpha2=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u nAlpha2_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp8u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_8u_AC4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit signed char channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pSrc2, jint nSrc2Step, jbyte nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_8s_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pSrc2=%p, nSrc2Step=%d, nAlpha2=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s nAlpha1_native;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8s nAlpha2_native;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8s)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp8s)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_8s_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pSrc2, jint nSrc2Step, jshort nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u nAlpha2_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp16u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_16u_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pSrc2, jint nSrc2Step, jshort nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_16u_C3R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u nAlpha2_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp16u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_16u_C3R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pSrc2, jint nSrc2Step, jshort nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_16u_C4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u nAlpha2_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp16u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_16u_C4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image composition with alpha using constant source alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pSrc2, jint nSrc2Step, jshort nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u nAlpha2_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp16u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_16u_AC4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pSrc2, jint nSrc2Step, jshort nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_16s_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s nAlpha1_native;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s nAlpha2_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16s)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp16s)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_16s_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit unsigned integer channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_132u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nAlpha1, jobject pSrc2, jint nSrc2Step, jint nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_32u_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nAlpha1_native;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32u nAlpha2_native;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp32u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp32u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_32u_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nAlpha1, jobject pSrc2, jint nSrc2Step, jint nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nAlpha1_native;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s nAlpha2_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp32s)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp32s)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_32s_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0.0 - 1.0).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0.0 - 1.0).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloat nAlpha1, jobject pSrc2, jint nSrc2Step, jfloat nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_32f_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%f, pSrc2=%p, nSrc2Step=%d, nAlpha2=%f, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f nAlpha1_native;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f nAlpha2_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp32f)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp32f)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_32f_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C1IRNative(JNIEnv *env, jclass cls, jbyte nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C1IR(nAlpha1=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nAlpha1_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C1IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C3R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C3R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C3IRNative(JNIEnv *env, jclass cls, jbyte nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C3IR(nAlpha1=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nAlpha1_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C3IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C4IRNative(JNIEnv *env, jclass cls, jbyte nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C4IR(nAlpha1=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nAlpha1_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C4IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image premultiplication with alpha using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_AC4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image premultiplication with alpha using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jbyte nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_AC4IR(nAlpha1=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nAlpha1_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_AC4IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C1IRNative(JNIEnv *env, jclass cls, jshort nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C1IR(nAlpha1=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nAlpha1_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C1IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C3R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C3R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C3IRNative(JNIEnv *env, jclass cls, jshort nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C3IR(nAlpha1=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nAlpha1_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C3IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C4IRNative(JNIEnv *env, jclass cls, jshort nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C4IR(nAlpha1=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nAlpha1_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C4IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image premultiplication with alpha using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_AC4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image premultiplication with alpha using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jshort nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_AC4IR(nAlpha1=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nAlpha1_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_AC4IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_18u_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_8u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_8u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_8u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_8u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_8u_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_8u_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit signed char channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_18s_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_8s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_8s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_8s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_8s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_8s_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_8s_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_116u_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_16u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_16u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_16u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_16u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_16u_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_16u_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_116s_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_16s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_16s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_16s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_16s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_16s_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_16s_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit unsigned integer channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132u_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32u_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32u_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit unsigned integer channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132s_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32s_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32s_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image composition using image alpha values (0.0 - 1.0).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132f_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32f_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32f_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32f_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32f_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32f_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32f_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image composition using image alpha values (0.0 - 1.0).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image premultiplication with pixel alpha (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremul_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremul_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremul_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremul_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremul_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremul_8u_AC4R(pSrc1_native, nSrc1Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image premultiplication with pixel alpha (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremul_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremul_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremul_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremul_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremul_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremul_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremul_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremul_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremul_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremul_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremul_16u_AC4R(pSrc1_native, nSrc1Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image premultiplication with pixel alpha (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremul_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremul_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremul_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremul_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremul_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned packed YUV color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYUV_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYUV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYUV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYUV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYUV_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYUV_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed YUV color conversion with alpha, not affecting alpha.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYUV_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYUV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYUV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYUV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYUV_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYUV_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar RGB to 3 channel 8-bit unsigned planar YUV color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYUV_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYUV_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYUV_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYUV_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYUV_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYUV_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned planar YUV color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYUV_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYUV_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYUV_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYUV_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYUV_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYUV_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned planar YUV color conversion with alpha.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYUV_18u_1AC4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYUV_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYUV_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYUV_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYUV_8u_AC4P4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYUV_8u_AC4P4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYUV_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYUV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYUV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYUV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYUV_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYUV_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYUV_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYUV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYUV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYUV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYUV_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYUV_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar BGR to 3 channel 8-bit unsigned planar YUV color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYUV_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYUV_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYUV_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYUV_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYUV_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYUV_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYUV_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYUV_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYUV_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYUV_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYUV_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYUV_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYUV_18u_1AC4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYUV_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYUV_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYUV_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYUV_8u_AC4P4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYUV_8u_AC4P4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed YUV to 3 channel 8-bit unsigned packed RGB color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUVToRGB_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUVToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUVToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUVToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUVToRGB_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUVToRGB_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUVToRGB_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUVToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUVToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUVToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUVToRGB_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUVToRGB_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV to 3 channel 8-bit unsigned planar RGB color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUVToRGB_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUVToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUVToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUVToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUVToRGB_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUVToRGB_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV to 3 channel 8-bit unsigned packed RGB color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUVToRGB_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUVToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUVToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUVToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUVToRGB_8u_P3C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUVToRGB_8u_P3C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed YUV to 3 channel 8-bit unsigned packed BGR color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUVToBGR_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUVToBGR_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUVToBGR_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUVToBGR_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUVToBGR_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUVToBGR_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUVToBGR_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUVToBGR_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUVToBGR_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUVToBGR_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUVToBGR_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUVToBGR_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV to 3 channel 8-bit unsigned planar BGR color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUVToBGR_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUVToBGR_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUVToBGR_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUVToBGR_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUVToBGR_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUVToBGR_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV to 3 channel 8-bit unsigned packed BGR color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUVToBGR_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUVToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUVToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUVToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUVToBGR_8u_P3C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUVToBGR_8u_P3C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 2 channel 8-bit unsigned packed YUV422 color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYUV422_18u_1C3C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYUV422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYUV422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYUV422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYUV422_8u_C3C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYUV422_8u_C3C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar RGB to 3 channel 8-bit unsigned planar YUV422 color conversion.
 * images.
 *                         
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYUV422_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYUV422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYUV422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiRGBToYUV422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYUV422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYUV422_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYUV422_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned planar YUV422 color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYUV422_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYUV422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYUV422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiRGBToYUV422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYUV422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYUV422_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYUV422_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YUV422 to 3 channel 8-bit unsigned packed RGB color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUV422ToRGB_18u_1C2C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUV422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUV422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUV422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUV422ToRGB_8u_C2C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUV422ToRGB_8u_C2C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV422 to 3 channel 8-bit unsigned planar RGB color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUV422ToRGB_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUV422ToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYUV422ToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUV422ToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUV422ToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUV422ToRGB_8u_P3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUV422ToRGB_8u_P3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV422 to 3 channel 8-bit unsigned packed RGB color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUV422ToRGB_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUV422ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYUV422ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUV422ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUV422ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUV422ToRGB_8u_P3C3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUV422ToRGB_8u_P3C3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV422 to 4 channel 8-bit unsigned packed RGB color conversion with alpha.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUV422ToRGB_18u_1P3AC4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUV422ToRGB_8u_P3AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYUV422ToRGB_8u_P3AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUV422ToRGB_8u_P3AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUV422ToRGB_8u_P3AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUV422ToRGB_8u_P3AC4R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUV422ToRGB_8u_P3AC4R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYUV420_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYUV420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYUV420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiRGBToYUV420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYUV420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYUV420_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYUV420_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYUV420_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYUV420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYUV420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiRGBToYUV420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYUV420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYUV420_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYUV420_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV420 to 3 channel 8-bit unsigned planar RGB color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUV420ToRGB_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUV420ToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYUV420ToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUV420ToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUV420ToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUV420ToRGB_8u_P3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUV420ToRGB_8u_P3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV420 to 3 channel 8-bit unsigned packed RGB color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUV420ToRGB_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUV420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYUV420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUV420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUV420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUV420ToRGB_8u_P3C3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUV420ToRGB_8u_P3C3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV420 to 4 channel 8-bit unsigned packed RGB color conversion with constant alpha (0xFF).
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUV420ToRGB_18u_1P3C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUV420ToRGB_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYUV420ToRGB_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUV420ToRGB_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUV420ToRGB_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUV420ToRGB_8u_P3C4R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUV420ToRGB_8u_P3C4R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV420 to 4 channel 8-bit unsigned packed RGB color conversion with alpha.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUV420ToRGB_18u_1P3AC4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUV420ToRGB_8u_P3AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYUV420ToRGB_8u_P3AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUV420ToRGB_8u_P3AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUV420ToRGB_8u_P3AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUV420ToRGB_8u_P3AC4R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUV420ToRGB_8u_P3AC4R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 *  2 channel 8-bit unsigned planar NV21 to 4 channel 8-bit unsigned packed ARGB color conversion with constant alpha (0xFF).
 *
 * @param pSrc \ref source_planar_image_pointer_array (one for Y plane, one for VU plane).
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNV21ToRGB_18u_1P2C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNV21ToRGB_8u_P2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiNV21ToRGB_8u_P2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNV21ToRGB_8u_P2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNV21ToRGB_8u_P2C4R(pSrc=%p, rSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[2];
    int rSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    rSrcStep_native = (int)rSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNV21ToRGB_8u_P2C4R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // rSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYUV420_18u_1AC4P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYUV420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYUV420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYUV420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYUV420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYUV420_8u_AC4P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYUV420_8u_AC4P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUV420ToBGR_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUV420ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYUV420ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUV420ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUV420ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUV420ToBGR_8u_P3C3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUV420ToBGR_8u_P3C3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YUV420 to 4 channel 8-bit unsigned packed BGR color conversion with constant alpha (0xFF).
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYUV420ToBGR_18u_1P3C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYUV420ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYUV420ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYUV420ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYUV420ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYUV420ToBGR_8u_P3C4R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYUV420ToBGR_8u_P3C4R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 *  2 channel 8-bit unsigned planar NV21 to 4 channel 8-bit unsigned packed BGRA color conversion with constant alpha (0xFF).
 *
 * @param pSrc \ref source_planar_image_pointer_array (one for Y plane, one for VU plane).
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNV21ToBGR_18u_1P2C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNV21ToBGR_8u_P2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiNV21ToBGR_8u_P2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNV21ToBGR_8u_P2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNV21ToBGR_8u_P2C4R(pSrc=%p, rSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[2];
    int rSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    rSrcStep_native = (int)rSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNV21ToBGR_8u_P2C4R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // rSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel unsigned 8-bit packed YCbCr with alpha color conversion, not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 3 channel unsigned 8-bit planar YCbCr color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr_18u_1AC4P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr_8u_AC4P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr_8u_AC4P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToRGB_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToRGB_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToRGB_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed YCbCr with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion, not affecting alpha.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToRGB_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToRGB_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToRGB_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr to 3 channel 8-bit unsigned planar RGB color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToRGB_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToRGB_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToRGB_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr to 3 channel 8-bit unsigned packed RGB color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToRGB_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToRGB_8u_P3C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToRGB_8u_P3C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr to 4 channel 8-bit unsigned packed RGB color conversion with constant alpha.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAval 8-bit unsigned alpha constant.                                         
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToRGB_18u_1P3C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nAval)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToRGB_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToRGB_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToRGB_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAval is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToRGB_8u_P3C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nAval=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nAval_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAval_native = (Npp8u)nAval;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToRGB_8u_P3C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nAval_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nAval is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr to 3 channel 8-bit unsigned packed BGR color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToBGR_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToBGR_8u_P3C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToBGR_8u_P3C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToBGR_18u_1P3C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nAval)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAval is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToBGR_8u_P3C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nAval=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nAval_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAval_native = (Npp8u)nAval;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToBGR_8u_P3C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nAval_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nAval is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr to 3 channel 8-bit unsigned packed BGR_709CSC color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToBGR_1709CSC_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToBGR_709CSC_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToBGR_709CSC_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToBGR_709CSC_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToBGR_709CSC_8u_P3C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToBGR_709CSC_8u_P3C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToBGR_1709CSC_18u_1P3C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nAval)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToBGR_709CSC_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToBGR_709CSC_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToBGR_709CSC_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAval is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToBGR_709CSC_8u_P3C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nAval=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nAval_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAval_native = (Npp8u)nAval;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToBGR_709CSC_8u_P3C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nAval_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nAval is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 2 channel 8-bit unsigned packed YCbCr422 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr422_18u_1C3C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr422_8u_C3C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr422_8u_C3C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned planar YCbCr422 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr422_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiRGBToYCbCr422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr422_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr422_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar RGB to 2 channel 8-bit unsigned packed YCbCr422 color conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr422_18u_1P3C2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr422_8u_P3C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr422_8u_P3C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned packed RGB color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToRGB_18u_1C2C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToRGB_8u_C2C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToRGB_8u_C2C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned planar RGB color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToRGB_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToRGB_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToRGB_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToRGB_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToRGB_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToRGB_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToRGB_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr422ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToRGB_8u_P3C3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToRGB_8u_P3C3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 2 channel 8-bit unsigned packed YCrCb422 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCrCb422_18u_1C3C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCrCb422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCrCb422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCrCb422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCrCb422_8u_C3C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCrCb422_8u_C3C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCrCb422_18u_1P3C2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCrCb422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCrCb422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCrCb422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCrCb422_8u_P3C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCrCb422_8u_P3C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCrCb422 to 3 channel 8-bit unsigned packed RGB color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCrCb422ToRGB_18u_1C2C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCrCb422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCrCb422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCrCb422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCrCb422ToRGB_8u_C2C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCrCb422ToRGB_8u_C2C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCrCb422 to 3 channel 8-bit unsigned planar RGB color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCrCb422ToRGB_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCrCb422ToRGB_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCrCb422ToRGB_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCrCb422ToRGB_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCrCb422ToRGB_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCrCb422ToRGB_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed BGR to 2 channel 8-bit unsigned packed YCrCb422 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr422_18u_1C3C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr422_8u_C3C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr422_8u_C3C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr422_18u_1AC4C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr422_8u_AC4C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr422_8u_AC4C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr422_8u_AC4C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr422_8u_AC4C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr422_8u_AC4C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCbCr422 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr422_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCbCr422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr422_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr422_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr422_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCbCr422 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr422_18u_1AC4P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr422_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr422_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCbCr422_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr422_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr422_8u_AC4P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr422_8u_AC4P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCrCb422 to 3 channel 8-bit unsigned packed BGR color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToBGR_18u_1C2C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToBGR_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToBGR_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToBGR_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToBGR_8u_C2C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToBGR_8u_C2C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCrCb422 to 4 channel 8-bit unsigned packed BGR color conversion with constant alpha.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAval 8-bit unsigned alpha constant.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToBGR_18u_1C2C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nAval)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToBGR_8u_C2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToBGR_8u_C2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToBGR_8u_C2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAval is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToBGR_8u_C2C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nAval=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nAval_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAval_native = (Npp8u)nAval;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToBGR_8u_C2C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nAval_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nAval is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr422 to 3 channel 8-bit unsigned packed BGR color conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToBGR_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr422ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToBGR_8u_P3C3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToBGR_8u_P3C3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 2 channel 8-bit unsigned packed CbYCr422 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToCbYCr422_18u_1C3C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToCbYCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToCbYCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToCbYCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToCbYCr422_8u_C3C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToCbYCr422_8u_C3C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB first gets forward gamma corrected then converted to 2 channel 8-bit unsigned packed CbYCr422 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToCbYCr422Gamma_18u_1C3C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToCbYCr422Gamma_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToCbYCr422Gamma_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToCbYCr422Gamma_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToCbYCr422Gamma_8u_C3C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToCbYCr422Gamma_8u_C3C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed CbYCrC22 to 3 channel 8-bit unsigned packed RGB color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCbYCr422ToRGB_18u_1C2C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCbYCr422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCbYCr422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCbYCr422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCbYCr422ToRGB_8u_C2C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCbYCr422ToRGB_8u_C2C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 2 channel 8-bit unsigned packed CbYCr422 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToCbYCr422_18u_1AC4C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToCbYCr422_8u_AC4C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToCbYCr422_8u_AC4C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToCbYCr422_8u_AC4C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToCbYCr422_8u_AC4C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToCbYCr422_8u_AC4C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed BGR to 2 channel 8-bit unsigned packed CbYCr422_709HDTV color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToCbYCr422_1709HDTV_18u_1C3C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToCbYCr422_709HDTV_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToCbYCr422_709HDTV_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToCbYCr422_709HDTV_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToCbYCr422_709HDTV_8u_C3C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToCbYCr422_709HDTV_8u_C3C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 2 channel 8-bit unsigned packed CbYCr422_709HDTV color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToCbYCr422_1709HDTV_18u_1AC4C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToCbYCr422_709HDTV_8u_AC4C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToCbYCr422_709HDTV_8u_AC4C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToCbYCr422_709HDTV_8u_AC4C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToCbYCr422_709HDTV_8u_AC4C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToCbYCr422_709HDTV_8u_AC4C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCbYCr422ToBGR_18u_1C2C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nAval)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCbYCr422ToBGR_8u_C2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCbYCr422ToBGR_8u_C2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCbYCr422ToBGR_8u_C2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAval is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCbYCr422ToBGR_8u_C2C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nAval=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nAval_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAval_native = (Npp8u)nAval;

    // Native function call
    NppStatus jniResult_native = nppiCbYCr422ToBGR_8u_C2C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nAval_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nAval is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCbYCr422ToBGR_1709HDTV_18u_1C2C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCbYCr422ToBGR_709HDTV_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCbYCr422ToBGR_709HDTV_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCbYCr422ToBGR_709HDTV_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCbYCr422ToBGR_709HDTV_8u_C2C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCbYCr422ToBGR_709HDTV_8u_C2C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed CbYCr422 to 4 channel 8-bit unsigned packed BGR_709HDTV color conversion with constant alpha.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAval 8-bit unsigned alpha constant.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCbYCr422ToBGR_1709HDTV_18u_1C2C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nAval)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCbYCr422ToBGR_709HDTV_8u_C2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCbYCr422ToBGR_709HDTV_8u_C2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCbYCr422ToBGR_709HDTV_8u_C2C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAval is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCbYCr422ToBGR_709HDTV_8u_C2C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nAval=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nAval);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nAval_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAval_native = (Npp8u)nAval;

    // Native function call
    NppStatus jniResult_native = nppiCbYCr422ToBGR_709HDTV_8u_C2C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nAval_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nAval is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned planar YCbCr420 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr420_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiRGBToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr420_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr420_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToRGB_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToRGB_8u_P3C3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToRGB_8u_P3C3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed RGB with alpha to 3 channel 8-bit unsigned planar YCrCb420 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCrCb420_18u_1AC4P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCrCb420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCrCb420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiRGBToYCrCb420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCrCb420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCrCb420_8u_AC4P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCrCb420_8u_AC4P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCrCb420 to 4 channel 8-bit unsigned packed RGB color conversion with constant alpha.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAval 8-bit unsigned alpha constant.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCrCb420ToRGB_18u_1P3C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nAval)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCrCb420ToRGB_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCrCb420ToRGB_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCrCb420ToRGB_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCrCb420ToRGB_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAval is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCrCb420ToRGB_8u_P3C4R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nAval=%c)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI, nAval);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nAval_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAval_native = (Npp8u)nAval;

    // Native function call
    NppStatus jniResult_native = nppiYCrCb420ToRGB_8u_P3C4R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nAval_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nAval is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCbCr420 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr420_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr420_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr420_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCbCr420 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr420_18u_1AC4P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCbCr420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr420_8u_AC4P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr420_8u_AC4P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCbCr420_709CSC color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr420_1709CSC_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr420_709CSC_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr420_709CSC_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCbCr420_709CSC_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr420_709CSC_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr420_709CSC_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr420_709CSC_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCbCr420_709CSC color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr420_1709CSC_18u_1AC4P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr420_709CSC_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr420_709CSC_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCbCr420_709CSC_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr420_709CSC_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr420_709CSC_8u_AC4P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr420_709CSC_8u_AC4P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCbCr420_709HDTV color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr420_1709HDTV_18u_1AC4P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr420_709HDTV_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr420_709HDTV_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCbCr420_709HDTV_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr420_709HDTV_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr420_709HDTV_8u_AC4P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr420_709HDTV_8u_AC4P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCrCb420_709CSC color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCrCb420_1709CSC_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCrCb420_709CSC_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCrCb420_709CSC_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCrCb420_709CSC_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCrCb420_709CSC_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCrCb420_709CSC_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCrCb420_709CSC_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCrCb420_709CSC color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCrCb420_1709CSC_18u_1AC4P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCrCb420_709CSC_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCrCb420_709CSC_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCrCb420_709CSC_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCrCb420_709CSC_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCrCb420_709CSC_8u_AC4P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCrCb420_709CSC_8u_AC4P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToBGR_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr420ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr420ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToBGR_8u_P3C3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToBGR_8u_P3C3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToBGR_18u_1P3C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nAval)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr420ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr420ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAval is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToBGR_8u_P3C4R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nAval=%c)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI, nAval);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nAval_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAval_native = (Npp8u)nAval;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToBGR_8u_P3C4R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nAval_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nAval is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr420 to 3 channel 8-bit unsigned packed BGR_709CSC color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToBGR_1709CSC_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr420ToBGR_709CSC_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr420ToBGR_709CSC_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToBGR_709CSC_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToBGR_709CSC_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToBGR_709CSC_8u_P3C3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToBGR_709CSC_8u_P3C3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr420 to 4 channel 8-bit unsigned packed BGR_709HDTV color conversion with constant alpha.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAval 8-bit unsigned alpha constant.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToBGR_1709HDTV_18u_1P3C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nAval)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr420ToBGR_709HDTV_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr420ToBGR_709HDTV_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToBGR_709HDTV_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToBGR_709HDTV_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAval is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToBGR_709HDTV_8u_P3C4R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nAval=%c)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI, nAval);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nAval_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAval_native = (Npp8u)nAval;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToBGR_709HDTV_8u_P3C4R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nAval_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nAval is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCrCb420 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCrCb420_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCrCb420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCrCb420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCrCb420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCrCb420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCrCb420_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCrCb420_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCrCb420 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCrCb420_18u_1AC4P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCrCb420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCrCb420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCrCb420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCrCb420_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCrCb420_8u_AC4P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCrCb420_8u_AC4P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCbCr411 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr411_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr411_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr411_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCbCr411_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr411_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr411_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr411_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr411_18u_1AC4P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr411_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr411_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiBGRToYCbCr411_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr411_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr411_8u_AC4P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr411_8u_AC4P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar YCbCr color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 3 channel 8-bit unsigned planar YCbCr color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr_18u_1AC4P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr_8u_AC4P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr_8u_AC4P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr_8u_AC4P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 4 channel 8-bit unsigned planar YCbCr color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToYCbCr_18u_1AC4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToYCbCr_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToYCbCr_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToYCbCr_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToYCbCr_8u_AC4P4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToYCbCr_8u_AC4P4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToBGR_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr411ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr411ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToBGR_8u_P3C3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToBGR_8u_P3C3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToBGR_18u_1P3C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nAval)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr411ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr411ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToBGR_8u_P3C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAval is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToBGR_8u_P3C4R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nAval=%c)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI, nAval);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nAval_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAval_native = (Npp8u)nAval;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToBGR_8u_P3C4R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nAval_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nAval is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToXYZ_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToXYZ_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToXYZ_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToXYZ_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToXYZ_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToXYZ_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed XYZ with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToXYZ_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToXYZ_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToXYZ_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToXYZ_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToXYZ_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToXYZ_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed XYZ to 3 channel 8-bit unsigned packed RGB color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXYZToRGB_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXYZToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXYZToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXYZToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXYZToRGB_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXYZToRGB_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed XYZ with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXYZToRGB_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXYZToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXYZToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXYZToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXYZToRGB_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXYZToRGB_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned packed LUV color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToLUV_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToLUV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToLUV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToLUV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToLUV_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToLUV_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed LUV with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToLUV_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToLUV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToLUV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToLUV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToLUV_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToLUV_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed LUV to 3 channel 8-bit unsigned packed RGB color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUVToRGB_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUVToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUVToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUVToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUVToRGB_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUVToRGB_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed LUV with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUVToRGB_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUVToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUVToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUVToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUVToRGB_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUVToRGB_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToLab_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToLab_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToLab_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToLab_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToLab_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToLab_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed Lab to 3 channel 8-bit unsigned packed BGR color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLabToBGR_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLabToBGR_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLabToBGR_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLabToBGR_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLabToBGR_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLabToBGR_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned packed YCC color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCC_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCC_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed YCC with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCC_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCC_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCCToRGB_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCCToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCCToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCCToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCCToRGB_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCCToRGB_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed YCC with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCCToRGB_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCCToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCCToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCCToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCCToRGB_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCCToRGB_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 3 channel 8-bit unsigned packed HLS color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToHLS_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToHLS_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToHLS_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToHLS_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToHLS_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToHLS_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed HLS with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToHLS_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToHLS_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToHLS_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToHLS_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToHLS_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToHLS_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed HLS to 3 channel 8-bit unsigned packed RGB color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHLSToRGB_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHLSToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiHLSToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHLSToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHLSToRGB_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiHLSToRGB_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed HLS with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHLSToRGB_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHLSToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiHLSToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHLSToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHLSToRGB_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiHLSToRGB_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 4 channel 8-bit unsigned packed HLS with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToHLS_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToHLS_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToHLS_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToHLS_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToHLS_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToHLS_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed BGR to 3 channel 8-bit unsigned planar HLS color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToHLS_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToHLS_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToHLS_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToHLS_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToHLS_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToHLS_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed BGR with alpha to 4 channel 8-bit unsigned planar HLS with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToHLS_18u_1AC4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToHLS_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToHLS_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToHLS_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToHLS_8u_AC4P4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToHLS_8u_AC4P4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToHLS_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToHLS_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToHLS_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToHLS_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToHLS_8u_P3C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToHLS_8u_P3C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned planar BGR with alpha to 4 channel 8-bit unsigned packed HLS with alpha color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToHLS_18u_1AP4C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToHLS_8u_AP4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToHLS_8u_AP4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToHLS_8u_AP4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToHLS_8u_AP4C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToHLS_8u_AP4C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar BGR to 3 channel 8-bit unsigned planar HLS color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToHLS_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToHLS_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToHLS_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToHLS_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToHLS_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToHLS_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned planar BGR with alpha to 4 channel 8-bit unsigned planar HLS with alpha color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiBGRToHLS_18u_1AP4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiBGRToHLS_8u_AP4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiBGRToHLS_8u_AP4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiBGRToHLS_8u_AP4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiBGRToHLS_8u_AP4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiBGRToHLS_8u_AP4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed HLS to 3 channel 8-bit unsigned planar BGR color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHLSToBGR_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHLSToBGR_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiHLSToBGR_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHLSToBGR_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHLSToBGR_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiHLSToBGR_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed HLS with alpha to 4 channel 8-bit unsigned planar BGR with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHLSToBGR_18u_1AC4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHLSToBGR_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiHLSToBGR_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHLSToBGR_8u_AC4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHLSToBGR_8u_AC4P4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiHLSToBGR_8u_AC4P4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHLSToBGR_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHLSToBGR_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiHLSToBGR_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHLSToBGR_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHLSToBGR_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiHLSToBGR_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHLSToBGR_18u_1AP4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHLSToBGR_8u_AP4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiHLSToBGR_8u_AP4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHLSToBGR_8u_AP4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHLSToBGR_8u_AP4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiHLSToBGR_8u_AP4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed HLS with alpha to 4 channel 8-bit unsigned packed BGR with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHLSToBGR_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHLSToBGR_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiHLSToBGR_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHLSToBGR_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHLSToBGR_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar HLS to 3 channel 8-bit unsigned packed BGR color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHLSToBGR_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHLSToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiHLSToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHLSToBGR_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHLSToBGR_8u_P3C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiHLSToBGR_8u_P3C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned planar HLS with alpha to 4 channel 8-bit unsigned packed BGR with alpha color conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHLSToBGR_18u_1AP4C4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHLSToBGR_8u_AP4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiHLSToBGR_8u_AP4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHLSToBGR_8u_AP4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHLSToBGR_8u_AP4C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiHLSToBGR_8u_AP4C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToHSV_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToHSV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToHSV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToHSV_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToHSV_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToHSV_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed RGB with alpha to 4 channel 8-bit unsigned packed HSV with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToHSV_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToHSV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToHSV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToHSV_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToHSV_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToHSV_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHSVToRGB_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHSVToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiHSVToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHSVToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHSVToRGB_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiHSVToRGB_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed HSV with alpha to 4 channel 8-bit unsigned packed RGB with alpha color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHSVToRGB_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHSVToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiHSVToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHSVToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHSVToRGB_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiHSVToRGB_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to 1 channel 8-bit unsigned packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToGray_18u_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToGray_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToGray_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToGray_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToGray_8u_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToGray_8u_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed RGB with alpha to 1 channel 8-bit unsigned packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToGray_18u_1AC4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToGray_8u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToGray_8u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToGray_8u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToGray_8u_AC4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToGray_8u_AC4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned packed RGB to 1 channel 16-bit unsigned packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToGray_116u_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToGray_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToGray_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToGray_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToGray_16u_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToGray_16u_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned packed RGB with alpha to 1 channel 16-bit unsigned packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToGray_116u_1AC4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToGray_16u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToGray_16u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToGray_16u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToGray_16u_AC4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToGray_16u_AC4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed packed RGB to 1 channel 16-bit signed packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToGray_116s_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToGray_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToGray_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToGray_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToGray_16s_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToGray_16s_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed packed RGB with alpha to 1 channel 16-bit signed packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToGray_116s_1AC4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToGray_16s_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToGray_16s_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToGray_16s_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToGray_16s_AC4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToGray_16s_AC4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToGray_132f_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToGray_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToGray_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToGray_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToGray_32f_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToGray_32f_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point packed RGB with alpha to 1 channel 32-bit floating point packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToGray_132f_1AC4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToGray_32f_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToGray_32f_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToGray_32f_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToGray_32f_AC4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToGray_32f_AC4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorToGray_18u_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorToGray_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorToGray_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorToGray_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiColorToGray_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorToGray_8u_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aCoeffs=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aCoeffs_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorToGray_8u_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aCoeffs_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed RGB with alpha to 1 channel 8-bit unsigned packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorToGray_18u_1AC4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorToGray_8u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorToGray_8u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorToGray_8u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiColorToGray_8u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorToGray_8u_AC4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aCoeffs=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aCoeffs_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorToGray_8u_AC4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aCoeffs_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned packed RGB to 1 channel 16-bit unsigned packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorToGray_116u_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorToGray_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorToGray_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorToGray_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiColorToGray_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorToGray_16u_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aCoeffs=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aCoeffs_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorToGray_16u_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aCoeffs_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned packed RGB with alpha to 1 channel 16-bit unsigned packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorToGray_116u_1AC4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorToGray_16u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorToGray_16u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorToGray_16u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiColorToGray_16u_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorToGray_16u_AC4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aCoeffs=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aCoeffs_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorToGray_16u_AC4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aCoeffs_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed packed RGB to 1 channel 16-bit signed packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorToGray_116s_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorToGray_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorToGray_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorToGray_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiColorToGray_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorToGray_16s_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aCoeffs=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aCoeffs_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorToGray_16s_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aCoeffs_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed packed RGB with alpha to 1 channel 16-bit signed packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorToGray_116s_1AC4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorToGray_16s_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorToGray_16s_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorToGray_16s_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiColorToGray_16s_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorToGray_16s_AC4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aCoeffs=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aCoeffs_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorToGray_16s_AC4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aCoeffs_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point packed RGB to 1 channel 32-bit floating point packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorToGray_132f_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorToGray_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorToGray_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorToGray_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiColorToGray_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorToGray_32f_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aCoeffs=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aCoeffs_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorToGray_32f_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aCoeffs_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point packed RGB with alpha to 1 channel 32-bit floating point packed Gray conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aCoeffs fixed size array of constant floating point conversion coefficient values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorToGray_132f_1AC4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorToGray_32f_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorToGray_32f_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorToGray_32f_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiColorToGray_32f_AC4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorToGray_32f_AC4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aCoeffs=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aCoeffs);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aCoeffs_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorToGray_32f_AC4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aCoeffs_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr420 to 2 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDstY \ref destination_planar_image_pointer.
 * @param nDstYStep \ref destination_planar_image_line_step.
 * @param pDstCbCr \ref destination_planar_image_pointer.
 * @param nDstCbCrStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToYCbCr411_18u_1P3P2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiYCbCr420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiYCbCr420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToYCbCr411_8u_P3P2R(pSrc=%p, rSrcStep=%p, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToYCbCr411_8u_P3P2R(pSrc_native, rSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned planar YCbCr420 to 3 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
 *
 * @param pSrcY \ref source_planar_image_pointer.
 * @param nSrcYStep \ref source_planar_image_line_step.
 * @param pSrcCbCr \ref source_planar_image_pointer.
 * @param nSrcCbCrStep \ref source_planar_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToYCbCr411_18u_1P2P3RNative(JNIEnv *env, jclass cls, jobject pSrcY, jint nSrcYStep, jobject pSrcCbCr, jint nSrcCbCrStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcY' is null for nppiYCbCr420ToYCbCr411_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcYStep is primitive
    if (pSrcCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcCbCr' is null for nppiYCbCr420ToYCbCr411_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcCbCrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToYCbCr411_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr420ToYCbCr411_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToYCbCr411_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToYCbCr411_8u_P2P3R(pSrcY=%p, nSrcYStep=%d, pSrcCbCr=%p, nSrcCbCrStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcY_native = NULL;
    int nSrcYStep_native = 0;
    Npp8u* pSrcCbCr_native = NULL;
    int nSrcCbCrStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcY_pointerData = initPointerData(env, pSrcY);
    if (pSrcY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcY_native = (Npp8u*)pSrcY_pointerData->getPointer(env);
    nSrcYStep_native = (int)nSrcYStep;
    PointerData *pSrcCbCr_pointerData = initPointerData(env, pSrcCbCr);
    if (pSrcCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcCbCr_native = (Npp8u*)pSrcCbCr_pointerData->getPointer(env);
    nSrcCbCrStep_native = (int)nSrcCbCrStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToYCbCr411_8u_P2P3R(pSrcY_native, nSrcYStep_native, pSrcCbCr_native, nSrcCbCrStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcYStep is primitive
    if (!releasePointerData(env, pSrcCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcCbCrStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr422 to 2 channel 8-bit unsigned packed YCbCr422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422_18u_1P3C2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422_8u_P3C2R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422_8u_P3C2R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCbCr422 to 2 channel 8-bit unsigned packed YCrCb422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCrCb422_18u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCrCb422_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToYCrCb422_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCrCb422_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCrCb422_8u_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCrCb422_8u_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr422 to 2 channel 8-bit unsigned packed YCrCb422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCrCb422_18u_1P3C2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCrCb422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr422ToYCrCb422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToYCrCb422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCrCb422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCrCb422_8u_P3C2R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCrCb422_8u_P3C2R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCbCr422 to 2 channel 8-bit unsigned packed CbYCr422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToCbYCr422_18u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToCbYCr422_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToCbYCr422_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToCbYCr422_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToCbYCr422_8u_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToCbYCr422_8u_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCbYCr422ToYCbCr411_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCbYCr422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCbYCr422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiCbYCr422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCbYCr422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCbYCr422ToYCbCr411_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCbYCr422ToYCbCr411_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr422 to 3 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCbCr420_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobjectArray pDst, jintArray nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr422ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nDstStep' is null for nppiYCbCr422ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCbCr420_8u_P3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%p, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native[3];
    int nDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nDstStep, nDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCbCr420_8u_P3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    if (!releaseNative(env, nDstStep_native, nDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr422 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDstY \ref destination_planar_image_pointer.
 * @param nDstYStep \ref destination_planar_image_line_step.
 * @param pDstCbCr \ref destination_planar_image_pointer.
 * @param nDstCbCrStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCbCr420_18u_1P3P2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr422ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiYCbCr422ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiYCbCr422ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCbCr420_8u_P3P2R(pSrc=%p, rSrcStep=%p, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCbCr420_8u_P3P2R(pSrc_native, rSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCbCr420_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCbCr420_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCbCr420_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCbCr422 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDstY \ref destination_planar_image_pointer.
 * @param nDstYStep \ref destination_planar_image_line_step.
 * @param pDstCbCr \ref destination_planar_image_pointer.
 * @param nDstCbCrStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCbCr420_18u_1C2P2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCbCr420_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiYCbCr422ToYCbCr420_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiYCbCr422ToYCbCr420_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCbCr420_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCbCr420_8u_C2P2R(pSrc=%p, nSrcStep=%d, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCbCr420_8u_C2P2R(pSrc_native, nSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCrCb420 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCrCb420ToYCbCr422_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCrCb420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCrCb420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCrCb420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCrCb420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCrCb420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCrCb420ToYCbCr422_8u_P3R(pSrc=%p, rSrcStep=%p, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCrCb420ToYCbCr422_8u_P3R(pSrc_native, rSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCrCb420 to 2 channel 8-bit unsigned packed YCbCr422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCrCb420ToYCbCr422_18u_1P3C2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCrCb420ToYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCrCb420ToYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCrCb420ToYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCrCb420ToYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCrCb420ToYCbCr422_8u_P3C2R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCrCb420ToYCbCr422_8u_P3C2R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned planar YCrCb420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCrCb420_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCrCb420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToYCrCb420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr422ToYCrCb420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCrCb420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCrCb420_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCrCb420_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCbCr411_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCbCr411_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr422ToYCbCr411_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToYCbCr411_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr422ToYCbCr411_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCbCr411_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCbCr411_8u_P3R(pSrc=%p, rSrcStep=%p, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCbCr411_8u_P3R(pSrc_native, rSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr422 to 2 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDstY \ref destination_planar_image_pointer.
 * @param nDstYStep \ref destination_planar_image_line_step.
 * @param pDstCbCr \ref destination_planar_image_pointer.
 * @param nDstCbCrStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCbCr411_18u_1P3P2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr422ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiYCbCr422ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiYCbCr422ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCbCr411_8u_P3P2R(pSrc=%p, rSrcStep=%p, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCbCr411_8u_P3P2R(pSrc_native, rSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCbCr422 to 3 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCbCr411_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCbCr411_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCbCr411_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCbCr422 to 2 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDstY \ref destination_planar_image_pointer.
 * @param nDstYStep \ref destination_planar_image_line_step.
 * @param pDstCbCr \ref destination_planar_image_pointer.
 * @param nDstCbCrStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCbCr411_18u_1C2P2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCbCr411_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiYCbCr422ToYCbCr411_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiYCbCr422ToYCbCr411_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCbCr411_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCbCr411_8u_C2P2R(pSrc=%p, nSrcStep=%d, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCbCr411_8u_C2P2R(pSrc_native, nSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCrCb422 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCrCb422ToYCbCr422_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCrCb422ToYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCrCb422ToYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCrCb422ToYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCrCb422ToYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCrCb422ToYCbCr422_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCrCb422ToYCbCr422_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed YCrCb422 to 3 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCrCb422ToYCbCr420_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCrCb422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCrCb422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCrCb422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCrCb422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCrCb422ToYCbCr420_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCrCb422ToYCbCr420_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCrCb422ToYCbCr411_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCrCb422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCrCb422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCrCb422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCrCb422ToYCbCr411_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCrCb422ToYCbCr411_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCrCb422ToYCbCr411_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed CbYCr422 to 2 channel 8-bit unsigned packed YCbCr422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCbYCr422ToYCbCr422_18u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCbYCr422ToYCbCr422_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCbYCr422ToYCbCr422_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCbYCr422ToYCbCr422_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCbYCr422ToYCbCr422_8u_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCbYCr422ToYCbCr422_8u_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed CbYCr422 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCbYCr422ToYCbCr422_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCbYCr422ToYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCbYCr422ToYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiCbYCr422ToYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCbYCr422ToYCbCr422_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCbYCr422ToYCbCr422_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCbYCr422ToYCbCr422_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed CbYCr422 to 3 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCbYCr422ToYCbCr420_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCbYCr422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCbYCr422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiCbYCr422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCbYCr422ToYCbCr420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCbYCr422ToYCbCr420_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCbYCr422ToYCbCr420_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed CbYCr422 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDstY \ref destination_planar_image_pointer.
 * @param nDstYStep \ref destination_planar_image_line_step.
 * @param pDstCbCr \ref destination_planar_image_pointer.
 * @param nDstCbCrStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCbYCr422ToYCbCr420_18u_1C2P2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCbYCr422ToYCbCr420_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiCbYCr422ToYCbCr420_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiCbYCr422ToYCbCr420_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCbYCr422ToYCbCr420_8u_C2P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCbYCr422ToYCbCr420_8u_C2P2R(pSrc=%p, nSrcStep=%d, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCbYCr422ToYCbCr420_8u_C2P2R(pSrc_native, nSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned packed CbYCr422 to 3 channel 8-bit unsigned planar YCrCb420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCbYCr422ToYCrCb420_18u_1C2P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCbYCr422ToYCrCb420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCbYCr422ToYCrCb420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiCbYCr422ToYCrCb420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCbYCr422ToYCrCb420_8u_C2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCbYCr422ToYCrCb420_8u_C2P3R(pSrc=%p, nSrcStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCbYCr422ToYCrCb420_8u_C2P3R(pSrc_native, nSrcStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr420 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDstY \ref destination_planar_image_pointer.
 * @param nDstYStep \ref destination_planar_image_line_step.
 * @param pDstCbCr \ref destination_planar_image_pointer.
 * @param nDstCbCrStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420_18u_1P3P2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420_8u_P3P2R(pSrc=%p, rSrcStep=%p, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420_8u_P3P2R(pSrc_native, rSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420_18u_1P2P3RNative(JNIEnv *env, jclass cls, jobject pSrcY, jint nSrcYStep, jobject pSrcCbCr, jint nSrcCbCrStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcY' is null for nppiYCbCr420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcYStep is primitive
    if (pSrcCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcCbCr' is null for nppiYCbCr420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcCbCrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420_8u_P2P3R(pSrcY=%p, nSrcYStep=%d, pSrcCbCr=%p, nSrcCbCrStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcY_native = NULL;
    int nSrcYStep_native = 0;
    Npp8u* pSrcCbCr_native = NULL;
    int nSrcCbCrStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcY_pointerData = initPointerData(env, pSrcY);
    if (pSrcY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcY_native = (Npp8u*)pSrcY_pointerData->getPointer(env);
    nSrcYStep_native = (int)nSrcYStep;
    PointerData *pSrcCbCr_pointerData = initPointerData(env, pSrcCbCr);
    if (pSrcCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcCbCr_native = (Npp8u*)pSrcCbCr_pointerData->getPointer(env);
    nSrcCbCrStep_native = (int)nSrcCbCrStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420_8u_P2P3R(pSrcY_native, nSrcYStep_native, pSrcCbCr_native, nSrcCbCrStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcYStep is primitive
    if (!releasePointerData(env, pSrcCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcCbCrStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr420 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToYCbCr422_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobjectArray pDst, jintArray nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nDstStep' is null for nppiYCbCr420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToYCbCr422_8u_P3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%p, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native[3];
    int nDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nDstStep, nDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToYCbCr422_8u_P3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    if (!releaseNative(env, nDstStep_native, nDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned planar YCbCr420 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
 *
 * @param pSrcY \ref source_planar_image_pointer.
 * @param nSrcYStep \ref source_planar_image_line_step.
 * @param pSrcCbCr \ref source_planar_image_pointer.
 * @param nSrcCbCrStep \ref source_planar_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToYCbCr422_18u_1P2P3RNative(JNIEnv *env, jclass cls, jobject pSrcY, jint nSrcYStep, jobject pSrcCbCr, jint nSrcCbCrStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcY' is null for nppiYCbCr420ToYCbCr422_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcYStep is primitive
    if (pSrcCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcCbCr' is null for nppiYCbCr420ToYCbCr422_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcCbCrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToYCbCr422_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr420ToYCbCr422_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToYCbCr422_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToYCbCr422_8u_P2P3R(pSrcY=%p, nSrcYStep=%d, pSrcCbCr=%p, nSrcCbCrStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcY_native = NULL;
    int nSrcYStep_native = 0;
    Npp8u* pSrcCbCr_native = NULL;
    int nSrcCbCrStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcY_pointerData = initPointerData(env, pSrcY);
    if (pSrcY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcY_native = (Npp8u*)pSrcY_pointerData->getPointer(env);
    nSrcYStep_native = (int)nSrcYStep;
    PointerData *pSrcCbCr_pointerData = initPointerData(env, pSrcCbCr);
    if (pSrcCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcCbCr_native = (Npp8u*)pSrcCbCr_pointerData->getPointer(env);
    nSrcCbCrStep_native = (int)nSrcCbCrStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToYCbCr422_8u_P2P3R(pSrcY_native, nSrcYStep_native, pSrcCbCr_native, nSrcCbCrStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcYStep is primitive
    if (!releasePointerData(env, pSrcCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcCbCrStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned planar YCbCr420 to 2 channel 8-bit unsigned packed YCbCr422 sampling format conversion.
 *
 * @param pSrcY \ref source_planar_image_pointer.
 * @param nSrcYStep \ref source_planar_image_line_step.
 * @param pSrcCbCr \ref source_planar_image_pointer.
 * @param nSrcCbCrStep \ref source_planar_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToYCbCr422_18u_1P2C2RNative(JNIEnv *env, jclass cls, jobject pSrcY, jint nSrcYStep, jobject pSrcCbCr, jint nSrcCbCrStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcY' is null for nppiYCbCr420ToYCbCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcYStep is primitive
    if (pSrcCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcCbCr' is null for nppiYCbCr420ToYCbCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcCbCrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToYCbCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToYCbCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToYCbCr422_8u_P2C2R(pSrcY=%p, nSrcYStep=%d, pSrcCbCr=%p, nSrcCbCrStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcY_native = NULL;
    int nSrcYStep_native = 0;
    Npp8u* pSrcCbCr_native = NULL;
    int nSrcCbCrStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcY_pointerData = initPointerData(env, pSrcY);
    if (pSrcY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcY_native = (Npp8u*)pSrcY_pointerData->getPointer(env);
    nSrcYStep_native = (int)nSrcYStep;
    PointerData *pSrcCbCr_pointerData = initPointerData(env, pSrcCbCr);
    if (pSrcCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcCbCr_native = (Npp8u*)pSrcCbCr_pointerData->getPointer(env);
    nSrcCbCrStep_native = (int)nSrcCbCrStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToYCbCr422_8u_P2C2R(pSrcY_native, nSrcYStep_native, pSrcCbCr_native, nSrcCbCrStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcYStep is primitive
    if (!releasePointerData(env, pSrcCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcCbCrStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned planar YCbCr420 to 2 channel 8-bit unsigned packed CbYCr422 sampling format conversion.
 *
 * @param pSrcY \ref source_planar_image_pointer.
 * @param nSrcYStep \ref source_planar_image_line_step.
 * @param pSrcCbCr \ref source_planar_image_pointer.
 * @param nSrcCbCrStep \ref source_planar_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToCbYCr422_18u_1P2C2RNative(JNIEnv *env, jclass cls, jobject pSrcY, jint nSrcYStep, jobject pSrcCbCr, jint nSrcCbCrStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcY' is null for nppiYCbCr420ToCbYCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcYStep is primitive
    if (pSrcCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcCbCr' is null for nppiYCbCr420ToCbYCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcCbCrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToCbYCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToCbYCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToCbYCr422_8u_P2C2R(pSrcY=%p, nSrcYStep=%d, pSrcCbCr=%p, nSrcCbCrStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcY_native = NULL;
    int nSrcYStep_native = 0;
    Npp8u* pSrcCbCr_native = NULL;
    int nSrcCbCrStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcY_pointerData = initPointerData(env, pSrcY);
    if (pSrcY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcY_native = (Npp8u*)pSrcY_pointerData->getPointer(env);
    nSrcYStep_native = (int)nSrcYStep;
    PointerData *pSrcCbCr_pointerData = initPointerData(env, pSrcCbCr);
    if (pSrcCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcCbCr_native = (Npp8u*)pSrcCbCr_pointerData->getPointer(env);
    nSrcCbCrStep_native = (int)nSrcCbCrStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToCbYCr422_8u_P2C2R(pSrcY_native, nSrcYStep_native, pSrcCbCr_native, nSrcCbCrStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcYStep is primitive
    if (!releasePointerData(env, pSrcCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcCbCrStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToYCrCb420_18u_1P2P3RNative(JNIEnv *env, jclass cls, jobject pSrcY, jint nSrcYStep, jobject pSrcCbCr, jint nSrcCbCrStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcY' is null for nppiYCbCr420ToYCrCb420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcYStep is primitive
    if (pSrcCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcCbCr' is null for nppiYCbCr420ToYCrCb420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcCbCrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToYCrCb420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr420ToYCrCb420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToYCrCb420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToYCrCb420_8u_P2P3R(pSrcY=%p, nSrcYStep=%d, pSrcCbCr=%p, nSrcCbCrStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcY_native = NULL;
    int nSrcYStep_native = 0;
    Npp8u* pSrcCbCr_native = NULL;
    int nSrcCbCrStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcY_pointerData = initPointerData(env, pSrcY);
    if (pSrcY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcY_native = (Npp8u*)pSrcY_pointerData->getPointer(env);
    nSrcYStep_native = (int)nSrcYStep;
    PointerData *pSrcCbCr_pointerData = initPointerData(env, pSrcCbCr);
    if (pSrcCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcCbCr_native = (Npp8u*)pSrcCbCr_pointerData->getPointer(env);
    nSrcCbCrStep_native = (int)nSrcCbCrStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToYCrCb420_8u_P2P3R(pSrcY_native, nSrcYStep_native, pSrcCbCr_native, nSrcCbCrStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcYStep is primitive
    if (!releasePointerData(env, pSrcCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcCbCrStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCrCb420 to 2 channel 8-bit unsigned packed CbYCr422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCrCb420ToCbYCr422_18u_1P3C2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCrCb420ToCbYCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCrCb420ToCbYCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCrCb420ToCbYCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCrCb420ToCbYCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCrCb420ToCbYCr422_8u_P3C2R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCrCb420ToCbYCr422_8u_P3C2R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCrCb420 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDstY \ref destination_planar_image_pointer.
 * @param nDstYStep \ref destination_planar_image_line_step.
 * @param pDstCbCr \ref destination_planar_image_pointer.
 * @param nDstCbCrStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCrCb420ToYCbCr420_18u_1P3P2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCrCb420ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCrCb420ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiYCrCb420ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiYCrCb420ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCrCb420ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCrCb420ToYCbCr420_8u_P3P2R(pSrc=%p, rSrcStep=%p, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCrCb420ToYCbCr420_8u_P3P2R(pSrc_native, rSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCrCb420 to 2 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDstY \ref destination_planar_image_pointer.
 * @param nDstYStep \ref destination_planar_image_line_step.
 * @param pDstCbCr \ref destination_planar_image_pointer.
 * @param nDstCbCrStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCrCb420ToYCbCr411_18u_1P3P2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCrCb420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCrCb420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiYCrCb420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiYCrCb420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCrCb420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCrCb420ToYCbCr411_8u_P3P2R(pSrc=%p, rSrcStep=%p, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCrCb420ToYCbCr411_8u_P3P2R(pSrc_native, rSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr411 to 2 channel 8-bit unsigned planar YCbCr411 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDstY \ref destination_planar_image_pointer.
 * @param nDstYStep \ref destination_planar_image_line_step.
 * @param pDstCbCr \ref destination_planar_image_pointer.
 * @param nDstCbCrStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411_18u_1P3P2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411_8u_P3P2R(pSrc=%p, rSrcStep=%p, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411_8u_P3P2R(pSrc_native, rSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411_18u_1P2P3RNative(JNIEnv *env, jclass cls, jobject pSrcY, jint nSrcYStep, jobject pSrcCbCr, jint nSrcCbCrStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcY' is null for nppiYCbCr411_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcYStep is primitive
    if (pSrcCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcCbCr' is null for nppiYCbCr411_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcCbCrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr411_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411_8u_P2P3R(pSrcY=%p, nSrcYStep=%d, pSrcCbCr=%p, nSrcCbCrStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcY_native = NULL;
    int nSrcYStep_native = 0;
    Npp8u* pSrcCbCr_native = NULL;
    int nSrcCbCrStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcY_pointerData = initPointerData(env, pSrcY);
    if (pSrcY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcY_native = (Npp8u*)pSrcY_pointerData->getPointer(env);
    nSrcYStep_native = (int)nSrcYStep;
    PointerData *pSrcCbCr_pointerData = initPointerData(env, pSrcCbCr);
    if (pSrcCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcCbCr_native = (Npp8u*)pSrcCbCr_pointerData->getPointer(env);
    nSrcCbCrStep_native = (int)nSrcCbCrStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411_8u_P2P3R(pSrcY_native, nSrcYStep_native, pSrcCbCr_native, nSrcCbCrStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcYStep is primitive
    if (!releasePointerData(env, pSrcCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcCbCrStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr411 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToYCbCr422_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobjectArray pDst, jintArray nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr411ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr411ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nDstStep' is null for nppiYCbCr411ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToYCbCr422_8u_P3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%p, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native[3];
    int nDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nDstStep, nDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToYCbCr422_8u_P3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    if (!releaseNative(env, nDstStep_native, nDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToYCbCr422_18u_1P3C2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr411ToYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr411ToYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411ToYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToYCbCr422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToYCbCr422_8u_P3C2R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToYCbCr422_8u_P3C2R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned planar YCbCr411 to 3 channel 8-bit unsigned planar YCbCr422 sampling format conversion.
 *
 * @param pSrcY \ref source_planar_image_pointer.
 * @param nSrcYStep \ref source_planar_image_line_step.
 * @param pSrcCbCr \ref source_planar_image_pointer.
 * @param nSrcCbCrStep \ref source_planar_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToYCbCr422_18u_1P2P3RNative(JNIEnv *env, jclass cls, jobject pSrcY, jint nSrcYStep, jobject pSrcCbCr, jint nSrcCbCrStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcY' is null for nppiYCbCr411ToYCbCr422_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcYStep is primitive
    if (pSrcCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcCbCr' is null for nppiYCbCr411ToYCbCr422_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcCbCrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411ToYCbCr422_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr411ToYCbCr422_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToYCbCr422_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToYCbCr422_8u_P2P3R(pSrcY=%p, nSrcYStep=%d, pSrcCbCr=%p, nSrcCbCrStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcY_native = NULL;
    int nSrcYStep_native = 0;
    Npp8u* pSrcCbCr_native = NULL;
    int nSrcCbCrStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcY_pointerData = initPointerData(env, pSrcY);
    if (pSrcY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcY_native = (Npp8u*)pSrcY_pointerData->getPointer(env);
    nSrcYStep_native = (int)nSrcYStep;
    PointerData *pSrcCbCr_pointerData = initPointerData(env, pSrcCbCr);
    if (pSrcCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcCbCr_native = (Npp8u*)pSrcCbCr_pointerData->getPointer(env);
    nSrcCbCrStep_native = (int)nSrcCbCrStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToYCbCr422_8u_P2P3R(pSrcY_native, nSrcYStep_native, pSrcCbCr_native, nSrcCbCrStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcYStep is primitive
    if (!releasePointerData(env, pSrcCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcCbCrStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned planar YCbCr411 to 2 channel 8-bit unsigned packed YCbCr422 sampling format conversion.
 *
 * @param pSrcY \ref source_planar_image_pointer.
 * @param nSrcYStep \ref source_planar_image_line_step.
 * @param pSrcCbCr \ref source_planar_image_pointer.
 * @param nSrcCbCrStep \ref source_planar_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToYCbCr422_18u_1P2C2RNative(JNIEnv *env, jclass cls, jobject pSrcY, jint nSrcYStep, jobject pSrcCbCr, jint nSrcCbCrStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcY' is null for nppiYCbCr411ToYCbCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcYStep is primitive
    if (pSrcCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcCbCr' is null for nppiYCbCr411ToYCbCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcCbCrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411ToYCbCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToYCbCr422_8u_P2C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToYCbCr422_8u_P2C2R(pSrcY=%p, nSrcYStep=%d, pSrcCbCr=%p, nSrcCbCrStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcY_native = NULL;
    int nSrcYStep_native = 0;
    Npp8u* pSrcCbCr_native = NULL;
    int nSrcCbCrStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcY_pointerData = initPointerData(env, pSrcY);
    if (pSrcY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcY_native = (Npp8u*)pSrcY_pointerData->getPointer(env);
    nSrcYStep_native = (int)nSrcYStep;
    PointerData *pSrcCbCr_pointerData = initPointerData(env, pSrcCbCr);
    if (pSrcCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcCbCr_native = (Npp8u*)pSrcCbCr_pointerData->getPointer(env);
    nSrcCbCrStep_native = (int)nSrcCbCrStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToYCbCr422_8u_P2C2R(pSrcY_native, nSrcYStep_native, pSrcCbCr_native, nSrcCbCrStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcYStep is primitive
    if (!releasePointerData(env, pSrcCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcCbCrStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr411 to 3 channel 8-bit unsigned planar YCrCb422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToYCrCb422_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobjectArray pDst, jintArray nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr411ToYCrCb422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr411ToYCrCb422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411ToYCrCb422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nDstStep' is null for nppiYCbCr411ToYCrCb422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToYCrCb422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToYCrCb422_8u_P3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%p, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native[3];
    int nDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nDstStep, nDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToYCrCb422_8u_P3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    if (!releaseNative(env, nDstStep_native, nDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr411 to 2 channel 8-bit unsigned packed YCrCb422 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToYCrCb422_18u_1P3C2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr411ToYCrCb422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr411ToYCrCb422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411ToYCrCb422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToYCrCb422_8u_P3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToYCrCb422_8u_P3C2R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToYCrCb422_8u_P3C2R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToYCbCr420_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobjectArray pDst, jintArray nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr411ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr411ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nDstStep' is null for nppiYCbCr411ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToYCbCr420_8u_P3R(pSrc=%p, rSrcStep=%p, pDst=%p, nDstStep=%p, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDst_native[3];
    int nDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nDstStep, nDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToYCbCr420_8u_P3R(pSrc_native, rSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    if (!releaseNative(env, nDstStep_native, nDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr411 to 2 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
 * images.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param rSrcStep \ref source_planar_image_line_step_array.
 * @param pDstY \ref destination_planar_image_pointer.
 * @param nDstYStep \ref destination_planar_image_line_step.
 * @param pDstCbCr \ref destination_planar_image_pointer.
 * @param nDstCbCrStep \ref destination_planar_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToYCbCr420_18u_1P3P2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray rSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr411ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rSrcStep' is null for nppiYCbCr411ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiYCbCr411ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiYCbCr411ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToYCbCr420_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToYCbCr420_8u_P3P2R(pSrc=%p, rSrcStep=%p, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, rSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int rSrcStep_native[3];
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rSrcStep, rSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToYCbCr420_8u_P3P2R(pSrc_native, rSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, rSrcStep_native, rSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned planar YCbCr411 to 3 channel 8-bit unsigned planar YCbCr420 sampling format conversion.
 *
 * @param pSrcY \ref source_planar_image_pointer.
 * @param nSrcYStep \ref source_planar_image_line_step.
 * @param pSrcCbCr \ref source_planar_image_pointer.
 * @param nSrcCbCrStep \ref source_planar_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToYCbCr420_18u_1P2P3RNative(JNIEnv *env, jclass cls, jobject pSrcY, jint nSrcYStep, jobject pSrcCbCr, jint nSrcCbCrStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcY' is null for nppiYCbCr411ToYCbCr420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcYStep is primitive
    if (pSrcCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcCbCr' is null for nppiYCbCr411ToYCbCr420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcCbCrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411ToYCbCr420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr411ToYCbCr420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToYCbCr420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToYCbCr420_8u_P2P3R(pSrcY=%p, nSrcYStep=%d, pSrcCbCr=%p, nSrcCbCrStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcY_native = NULL;
    int nSrcYStep_native = 0;
    Npp8u* pSrcCbCr_native = NULL;
    int nSrcCbCrStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcY_pointerData = initPointerData(env, pSrcY);
    if (pSrcY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcY_native = (Npp8u*)pSrcY_pointerData->getPointer(env);
    nSrcYStep_native = (int)nSrcYStep;
    PointerData *pSrcCbCr_pointerData = initPointerData(env, pSrcCbCr);
    if (pSrcCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcCbCr_native = (Npp8u*)pSrcCbCr_pointerData->getPointer(env);
    nSrcCbCrStep_native = (int)nSrcCbCrStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToYCbCr420_8u_P2P3R(pSrcY_native, nSrcYStep_native, pSrcCbCr_native, nSrcCbCrStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcYStep is primitive
    if (!releasePointerData(env, pSrcCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcCbCrStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned planar YCbCr411 to 3 channel 8-bit unsigned planar YCrCb420 sampling format conversion.
 *
 * @param pSrcY \ref source_planar_image_pointer.
 * @param nSrcYStep \ref source_planar_image_line_step.
 * @param pSrcCbCr \ref source_planar_image_pointer.
 * @param nSrcCbCrStep \ref source_planar_image_line_step.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param rDstStep \ref destination_planar_image_line_step_array.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr411ToYCrCb420_18u_1P2P3RNative(JNIEnv *env, jclass cls, jobject pSrcY, jint nSrcYStep, jobject pSrcCbCr, jint nSrcCbCrStep, jobjectArray pDst, jintArray rDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcY' is null for nppiYCbCr411ToYCrCb420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcYStep is primitive
    if (pSrcCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcCbCr' is null for nppiYCbCr411ToYCrCb420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcCbCrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr411ToYCrCb420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rDstStep' is null for nppiYCbCr411ToYCrCb420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr411ToYCrCb420_8u_P2P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr411ToYCrCb420_8u_P2P3R(pSrcY=%p, nSrcYStep=%d, pSrcCbCr=%p, nSrcCbCrStep=%d, pDst=%p, rDstStep=%p, oSizeROI=%p)\n",
        pSrcY, nSrcYStep, pSrcCbCr, nSrcCbCrStep, pDst, rDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcY_native = NULL;
    int nSrcYStep_native = 0;
    Npp8u* pSrcCbCr_native = NULL;
    int nSrcCbCrStep_native = 0;
    Npp8u* pDst_native[3];
    int rDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcY_pointerData = initPointerData(env, pSrcY);
    if (pSrcY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcY_native = (Npp8u*)pSrcY_pointerData->getPointer(env);
    nSrcYStep_native = (int)nSrcYStep;
    PointerData *pSrcCbCr_pointerData = initPointerData(env, pSrcCbCr);
    if (pSrcCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcCbCr_native = (Npp8u*)pSrcCbCr_pointerData->getPointer(env);
    nSrcCbCrStep_native = (int)nSrcCbCrStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rDstStep, rDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr411ToYCrCb420_8u_P2P3R(pSrcY_native, nSrcYStep_native, pSrcCbCr_native, nSrcCbCrStep_native, pDst_native, rDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcYStep is primitive
    if (!releasePointerData(env, pSrcCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcCbCrStep is primitive
    // pDst is an array of native pointers
    if (!releaseNative(env, rDstStep_native, rDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed color not in place forward gamma correction.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaFwd_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiGammaFwd_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiGammaFwd_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaFwd_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaFwd_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaFwd_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed color in place forward gamma correction.
 *
 * @param pSrcDst in place packed pixel image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaFwd_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiGammaFwd_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaFwd_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaFwd_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaFwd_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed color with alpha not in place forward gamma correction.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaFwd_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiGammaFwd_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiGammaFwd_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaFwd_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaFwd_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaFwd_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaFwd_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiGammaFwd_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaFwd_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaFwd_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaFwd_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar color not in place forward gamma correction.
 *
 * @param pSrc source planar pixel format image pointer array.
 * @param nSrcStep source planar pixel format image line step.
 * @param pDst destination planar pixel format image pointer array.
 * @param nDstStep destination planar pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaFwd_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiGammaFwd_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiGammaFwd_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaFwd_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaFwd_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaFwd_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar color in place forward gamma correction.
 *
 * @param pSrcDst in place planar pixel format image pointer array.
 * @param nSrcDstStep in place planar pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaFwd_18u_1IP3RNative(JNIEnv *env, jclass cls, jobjectArray pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiGammaFwd_8u_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaFwd_8u_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaFwd_8u_IP3R(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native[3];
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrcDst, pSrcDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaFwd_8u_IP3R(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrcDst is an array of native pointers
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaInv_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiGammaInv_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiGammaInv_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaInv_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaInv_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaInv_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed color in place inverse gamma correction.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaInv_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiGammaInv_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaInv_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaInv_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaInv_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaInv_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiGammaInv_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiGammaInv_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaInv_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaInv_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaInv_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed color with alpha in place inverse gamma correction.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaInv_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiGammaInv_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaInv_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaInv_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaInv_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar color not in place inverse gamma correction.
 *
 * @param pSrc source planar pixel format image pointer array.
 * @param nSrcStep source planar pixel format image line step.
 * @param pDst destination planar pixel format image pointer array.
 * @param nDstStep destination planar pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaInv_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiGammaInv_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiGammaInv_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaInv_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaInv_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaInv_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar color in place inverse gamma correction.
 *
 * @param pSrcDst in place planar pixel format image pointer array.
 * @param nSrcDstStep in place planar pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGammaInv_18u_1IP3RNative(JNIEnv *env, jclass cls, jobjectArray pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiGammaInv_8u_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiGammaInv_8u_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGammaInv_8u_IP3R(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native[3];
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, pSrcDst, pSrcDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGammaInv_8u_IP3R(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // pSrcDst is an array of native pointers
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned packed color complement color key replacement of source image 1 by source image 2.
 *
 * @param pSrc1 source1 packed pixel format image pointer.
 * @param nSrc1Step source1 packed pixel format image line step.
 * @param pSrc2 source2 packed pixel format image pointer.
 * @param nSrc2Step source2 packed pixel format image line step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nColorKeyConst color key constant
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompColorKey_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nColorKeyConst)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompColorKey_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompColorKey_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompColorKey_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompColorKey_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nColorKeyConst is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompColorKey_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nColorKeyConst=%c)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nColorKeyConst);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nColorKeyConst_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nColorKeyConst_native = (Npp8u)nColorKeyConst;

    // Native function call
    NppStatus jniResult_native = nppiCompColorKey_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nColorKeyConst_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nColorKeyConst is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed color complement color key replacement of source image 1 by source image 2.
 *
 * @param pSrc1 source1 packed pixel format image pointer.
 * @param nSrc1Step source1 packed pixel format image line step.
 * @param pSrc2 source2 packed pixel format image pointer.
 * @param nSrc2Step source2 packed pixel format image line step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nColorKeyConst color key constant array
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompColorKey_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray nColorKeyConst)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompColorKey_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompColorKey_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompColorKey_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompColorKey_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nColorKeyConst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nColorKeyConst' is null for nppiCompColorKey_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompColorKey_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nColorKeyConst=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nColorKeyConst);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nColorKeyConst_native[3];

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nColorKeyConst, nColorKeyConst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCompColorKey_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nColorKeyConst_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nColorKeyConst_native, nColorKeyConst, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed color complement color key replacement of source image 1 by source image 2.
 *
 * @param pSrc1 source1 packed pixel format image pointer.
 * @param nSrc1Step source1 packed pixel format image line step.
 * @param pSrc2 source2 packed pixel format image pointer.
 * @param nSrc2Step source2 packed pixel format image line step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nColorKeyConst color key constant array
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompColorKey_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray nColorKeyConst)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompColorKey_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompColorKey_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompColorKey_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompColorKey_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nColorKeyConst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nColorKeyConst' is null for nppiCompColorKey_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompColorKey_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nColorKeyConst=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nColorKeyConst);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nColorKeyConst_native[4];

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nColorKeyConst, nColorKeyConst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCompColorKey_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nColorKeyConst_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nColorKeyConst_native, nColorKeyConst, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed color complement color key replacement of source image 1 by source image 2 with alpha blending.
 *
 * @param pSrc1 source1 packed pixel format image pointer.
 * @param nSrc1Step source1 packed pixel format image line step.
 * @param nAlpha1 source1 image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 source2 packed pixel format image pointer.
 * @param nSrc2Step source2 packed pixel format image line step.
 * @param nAlpha2 source2 image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nColorKeyConst color key constant array
 * @param nppAlphaOp NppiAlphaOp alpha compositing operation selector  (excluding premul ops).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompColorKey_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pSrc2, jint nSrc2Step, jbyte nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray nColorKeyConst, jint nppAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompColorKey_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompColorKey_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompColorKey_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompColorKey_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nColorKeyConst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nColorKeyConst' is null for nppiAlphaCompColorKey_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nppAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompColorKey_8u_AC4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pSrc2=%p, nSrc2Step=%d, nAlpha2=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, nColorKeyConst=%p, nppAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, nColorKeyConst, nppAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u nAlpha2_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nColorKeyConst_native[4];
    NppiAlphaOp nppAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp8u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nColorKeyConst, nColorKeyConst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nppAlphaOp_native = (NppiAlphaOp)nppAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompColorKey_8u_AC4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, nColorKeyConst_native, nppAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nColorKeyConst_native, nColorKeyConst, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // nppAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1C2IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8u_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_C2IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_C2IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned color twist, with alpha copy.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is copied unmodified from the source pixel to the destination pixel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned in place color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is unmodified.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned in place color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned color twist with 4x4 matrix and constant vector addition.
 *
 * An input 4x4 color twist matrix with floating-point coefficient values with an additional constant vector addition
 * is applied within ROI.  For this particular version of the function the result is generated as shown below.
 *
 *  \code
 *      dst[0] = aTwist[0][0] * src[0] + aTwist[0][1] * src[1] + aTwist[0][2] * src[2] + aTwist[0][3] * src[3] + aConstants[0]
 *      dst[1] = aTwist[1][0] * src[0] + aTwist[1][1] * src[1] + aTwist[1][2] * src[2] + aTwist[1][3] * src[3] + aConstants[1]
 *      dst[2] = aTwist[2][0] * src[0] + aTwist[2][1] * src[1] + aTwist[2][2] * src[2] + aTwist[2][3] * src[3] + aConstants[2]
 *      dst[3] = aTwist[3][0] * src[0] + aTwist[3][1] * src[1] + aTwist[3][2] * src[2] + aTwist[3][3] * src[3] + aConstants[3]
 *  \endcode
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @param aConstants fixed size array of constant values, one per channel..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32fC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist, jfloatArray aConstants)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32fC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32fC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32fC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32fC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiColorTwist32fC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32fC_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p, aConstants=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist, aConstants);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[4][4];
    Npp32f aConstants_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32fC_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native, aConstants_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32fC_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist, jfloatArray aConstants)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32fC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32fC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32fC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiColorTwist32fC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32fC_8u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p, aConstants=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist, aConstants);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[4][4];
    Npp32f aConstants_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32fC_8u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native, aConstants_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar color twist.
 *
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place planar pixel format image pointer array, one pointer per plane.
 * @param nSrcDstStep in place planar pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1IP3RNative(JNIEnv *env, jclass cls, jobjectArray pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8u_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8u_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_IP3R(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8u* pSrcDst_native[3];
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    if (!initNative(env, pSrcDst, pSrcDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_IP3R(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    // pSrcDst is an array of native pointers
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit signed color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit signed in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit signed color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit signed in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1C2IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8s_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_C2IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_C2IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit signed color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit signed in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit signed color twist, with alpha copy.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is copied unmodified from the source pixel to the destination pixel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit signed in place color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is unmodified.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit signed color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit signed in place color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit signed planar color twist.
 *
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    const Npp8s* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp8s* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit signed planar in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place planar pixel format image pointer array, one pointer per plane.
 * @param nSrcDstStep in place planar pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18s_1IP3RNative(JNIEnv *env, jclass cls, jobjectArray pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_8s_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8s_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_8s_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8s_IP3R(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp8s* pSrcDst_native[3];
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    if (!initNative(env, pSrcDst, pSrcDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8s_IP3R(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    // pSrcDst is an array of native pointers
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16u_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16u_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 16-bit unsigned in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116u_1C2IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_16u_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16u_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16u_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16u_C2IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16u_C2IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned in place color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned planar color twist.
 *
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned planar in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place planar pixel format image pointer array, one pointer per plane.
 * @param nSrcDstStep in place planar pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116u_1IP3RNative(JNIEnv *env, jclass cls, jobjectArray pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_16u_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16u_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16u_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16u_IP3R(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16u* pSrcDst_native[3];
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    if (!initNative(env, pSrcDst, pSrcDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16u_IP3R(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    // pSrcDst is an array of native pointers
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 16-bit signed color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16s_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16s_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 16-bit signed in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116s_1C2IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_16s_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16s_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16s_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16s_C2IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16s_C2IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed in place color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed planar color twist.
 *
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16s_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    const Npp16s* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp16s* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16s_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed planar in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place planar pixel format image pointer array, one pointer per plane.
 * @param nSrcDstStep in place planar pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_116s_1IP3RNative(JNIEnv *env, jclass cls, jobjectArray pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist32f_16s_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_16s_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist32f_16s_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_16s_IP3R(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp16s* pSrcDst_native[3];
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    if (!initNative(env, pSrcDst, pSrcDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_16s_IP3R(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    // pSrcDst is an array of native pointers
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 32-bit floating point color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 32-bit floating point in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1C2IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist_32f_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_C2IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_C2IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_C2IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point in place color twist.
 * 
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point color twist, with alpha copy.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is copied unmodified from the source pixel to the destination pixel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point in place color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is not modified.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point in place color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point coefficient values is applied with
 * in ROI.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point color twist with 4x4 matrix and constant vector addition.
 *
 * An input 4x4 color twist matrix with floating-point coefficient values with an additional constant vector addition
 * is applied within ROI.  For this particular version of the function the result is generated as shown below.
 *
 *  \code
 *      dst[0] = aTwist[0][0] * src[0] + aTwist[0][1] * src[1] + aTwist[0][2] * src[2] + aTwist[0][3] * src[3] + aConstants[0]
 *      dst[1] = aTwist[1][0] * src[0] + aTwist[1][1] * src[1] + aTwist[1][2] * src[2] + aTwist[1][3] * src[3] + aConstants[1]
 *      dst[2] = aTwist[2][0] * src[0] + aTwist[2][1] * src[1] + aTwist[2][2] * src[2] + aTwist[2][3] * src[3] + aConstants[2]
 *      dst[3] = aTwist[3][0] * src[0] + aTwist[3][1] * src[1] + aTwist[3][2] * src[2] + aTwist[3][3] * src[3] + aConstants[3]
 *  \endcode
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @param aConstants fixed size array of constant values, one per channel..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132fC_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist, jfloatArray aConstants)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist_32fC_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist_32fC_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32fC_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32fC_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiColorTwist_32fC_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32fC_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p, aConstants=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist, aConstants);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[4][4];
    Npp32f aConstants_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32fC_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native, aConstants_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point in place color twist with 4x4 matrix and an additional constant vector addition.
 *
 * An input 4x4 color twist matrix with floating-point coefficient values with an additional constant vector addition
 * is applied within ROI.  For this particular version of the function the result is generated as shown below.
 *
 *  \code
 *      dst[0] = aTwist[0][0] * src[0] + aTwist[0][1] * src[1] + aTwist[0][2] * src[2] + aTwist[0][3] * src[3] + aConstants[0]
 *      dst[1] = aTwist[1][0] * src[0] + aTwist[1][1] * src[1] + aTwist[1][2] * src[2] + aTwist[1][3] * src[3] + aConstants[1]
 *      dst[2] = aTwist[2][0] * src[0] + aTwist[2][1] * src[1] + aTwist[2][2] * src[2] + aTwist[2][3] * src[3] + aConstants[2]
 *      dst[3] = aTwist[3][0] * src[0] + aTwist[3][1] * src[1] + aTwist[3][2] * src[2] + aTwist[3][3] * src[3] + aConstants[3]
 *  \endcode
 *
 * @param pSrcDst in place packed pixel format image pointer.
 * @param nSrcDstStep in place packed pixel format image line step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @param aConstants fixed size array of constant values, one per channel..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132fC_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist, jfloatArray aConstants)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist_32fC_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32fC_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32fC_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aConstants' is null for nppiColorTwist_32fC_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32fC_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p, aConstants=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist, aConstants);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[4][4];
    Npp32f aConstants_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aConstants, aConstants_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32fC_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native, aConstants_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aConstants_native, aConstants, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point planar color twist.
 *
 * An input color twist matrix with floating-point coefficient values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aTwist The color twist matrix with floating-point coefficient values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aTwist);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    int nSrcStep_native = 0;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // nSrcStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist_132f_1IP3RNative(JNIEnv *env, jclass cls, jobjectArray pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray aTwist)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiColorTwist_32f_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist_32f_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aTwist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aTwist' is null for nppiColorTwist_32f_IP3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist_32f_IP3R(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aTwist=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aTwist);

    // Native variable declarations
    Npp32f* pSrcDst_native[3];
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aTwist_native[3][4];

    // Obtain native variable values
    if (!initNative(env, pSrcDst, pSrcDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aTwist, aTwist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist_32f_IP3R(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aTwist_native);

    // Write back native variable values
    // pSrcDst is an array of native pointers
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aTwist_native, aTwist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned look-up-table color conversion.
 *
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned look-up-table in place color conversion.
 *
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_8u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_8u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned look-up-table in place color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned look-up-table in place color conversion.
 *
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table in place color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed look-up-table color conversion.
 *
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed look-up-table in place color conversion.
 *
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16s_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16s_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed look-up-table in place color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point look-up-table color conversion.
 *
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pValues_native = NULL;
    Npp32f* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32f*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32f*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point look-up-table in place color conversion.
 *
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pValues_native = NULL;
    Npp32f* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32f*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32f*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points with no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[4];
    const Npp32f* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[4];
    const Npp32f* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point look-up-table in place color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned linear interpolated look-up-table color conversion.
 *
 * The LUT is derived from a set of user defined mapping points through linear interpolation. 
 *
 * >>>>>>> ATTENTION ATTENTION <<<<<<<
 *
 * NOTE: As of the 5.0 release of NPP, the pValues and pLevels pointers need to be device memory pointers.
 *
 * >>>>>>>                     <<<<<<<
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is now a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is now a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned linear interpolated look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through linear interpolation. 
 *
 * >>>>>>> ATTENTION ATTENTION <<<<<<<
 *
 * NOTE: As of the 5.0 release of NPP, the pValues and pLevels pointers need to be host memory pointers to arrays of device memory pointers.
 *
 * >>>>>>>                     <<<<<<<
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned linear interpolated look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through linear interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned linear interpolated look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through linear interpolation. 
 *
 * >>>>>>> ATTENTION ATTENTION <<<<<<<
 *
 * NOTE: As of the 5.0 release of NPP, the pValues and pLevels pointers need to be host memory pointers to arrays of device memory pointers.
 *
 * >>>>>>>                     <<<<<<<
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned linear interpolated look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through linear interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_8u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_8u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned look-up-table in place color conversion.
 *
 * The LUT is derived from a set of user defined mapping points using linear interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using linear interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using linear interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table in place color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed look-up-table color conversion.
 *
 * The LUT is derived from a set of user defined mapping points using linear interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using linear interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using linear interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16s_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16s_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed look-up-table in place color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point look-up-table color conversion.
 *
 * The LUT is derived from a set of user defined mapping points using linear interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pValues_native = NULL;
    Npp32f* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32f*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32f*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point look-up-table in place color conversion.
 *
 * The LUT is derived from a set of user defined mapping points using linear interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pValues_native = NULL;
    Npp32f* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32f*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32f*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using linear interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[4];
    const Npp32f* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[4];
    const Npp32f* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point look-up-table in place color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Linear_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned cubic interpolated look-up-table in place color conversion.
 *
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned cubic interpolated look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned cubic interpolated look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned cubic interpolated look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned cubic interpolated look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_8u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_8u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned cubic interpolated look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned cubic interpolated look-up-table in place color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned look-up-table in place color conversion.
 *
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned look-up-table in place color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed look-up-table color conversion.
 *
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed look-up-table in place color conversion.
 *
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16s_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16s_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed look-up-table in place color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point look-up-table color conversion.
 *
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pValues_native = NULL;
    Npp32f* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32f*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32f*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point look-up-table in place color conversion.
 *
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values (this is a device memory pointer)
 * @param pLevels Pointer to an array of user defined INPUT values  (this is a device memory pointer)
 * @param nLevels Number of user defined number of input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pValues_native = NULL;
    Npp32f* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32f*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32f*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through cubic interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[4];
    const Npp32f* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point look-up-table in place color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 4 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[4];
    const Npp32f* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Cubic_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Cubic_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point look-up-table in place color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points using no interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT values.
 * @param pLevels Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined INPUT values.
 * @param nLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per color CHANNEL.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 1024 (the current size limit).
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Cubic_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Cubic_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Cubic_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Cubic_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Cubic_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Cubic_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Cubic_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32f* pValues_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Cubic_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned 3D trilinear interpolated look-up-table color conversion, with alpha copy.
 * Alpha channel is the last channel and is copied to the destination unmodified.
 *
 * The LUT is derived from a set of user defined mapping points through trilinear interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Device pointer to aLevels[2] number of contiguous 2D x,y planes of 4-byte packed RGBX values
 *        containing the user defined base OUTPUT values at that x,y, and z (R,G,B) level location. Each level must contain x * y 4-byte
 *        packed pixel values (4th byte is used for alignement only and is ignored) in row (x) order.
 * @param pLevels Host pointer to an array of 3 host pointers, one per cube edge, pointing to user defined INPUT level values.
 * @param aLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per 3D cube edge.
 *        aLevels[0] represents the number of x axis levels (Red), aLevels[1] represents the number of y axis levels (Green), 
 *        and aLevels[2] represets the number of z axis levels (Blue).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Trilinear_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobjectArray pLevels, jintArray aLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Trilinear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Trilinear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Trilinear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Trilinear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Trilinear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aLevels' is null for nppiLUT_Trilinear_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Trilinear_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, aLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, aLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32u* pValues_native = NULL;
    Npp8u* pLevels_native[3];
    int aLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32u*)pValues_pointerData->getPointer(env);
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aLevels, aLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Trilinear_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, aLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // pLevels is an array of native pointers
    if (!releaseNative(env, aLevels_native, aLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned 3D trilinear interpolated look-up-table color conversion, not affecting alpha.
 * Alpha channel is the last channel and is not processed.
 *
 * The LUT is derived from a set of user defined mapping points through trilinear interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Device pointer to aLevels[2] number of contiguous 2D x,y planes of 4-byte packed RGBX values
 *        containing the user defined base OUTPUT values at that x,y, and z (R,G,B) level location. Each level must contain x * y 4-byte
 *        packed pixel values (4th byte is used for alignement only and is ignored) in row (x) order.
 * @param pLevels Host pointer to an array of 3 host pointers, one per cube edge, pointing to user defined INPUT level values.
 * @param aLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per 3D cube edge.
 *        aLevels[0] represents the number of x axis levels (Red), aLevels[1] represents the number of y axis levels (Green), 
 *        and aLevels[2] represets the number of z axis levels (Blue).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Trilinear_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobjectArray pLevels, jintArray aLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Trilinear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Trilinear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Trilinear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Trilinear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Trilinear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aLevels' is null for nppiLUT_Trilinear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Trilinear_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, aLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, aLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32u* pValues_native = NULL;
    Npp8u* pLevels_native[3];
    int aLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32u*)pValues_pointerData->getPointer(env);
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aLevels, aLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Trilinear_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, aLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // pLevels is an array of native pointers
    if (!releaseNative(env, aLevels_native, aLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned 3D trilinear interpolated look-up-table in place color conversion, not affecting alpha.
 * Alpha channel is the last channel and is not processed.
 *
 * The LUT is derived from a set of user defined mapping points through trilinear interpolation. 
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Device pointer aLevels[2] number of contiguous 2D x,y planes of 4-byte packed RGBX values
 *        containing the user defined base OUTPUT values at that x,y, and z (R,G,B) level location. Each level must contain x * y 4-byte
 *        packed pixel values (4th byte is used for alignement only and is ignored) in row (x) order.
 * @param pLevels Host pointer to an array of 3 host pointers, one per cube edge, pointing to user defined INPUT level values.
 * @param aLevels Host pointer to an array of 3 user defined number of input/output mapping points, one per 3D cube edge.
 *        aLevels[0] represents the number of x axis levels (Red), aLevels[1] represents the number of y axis levels (Green), 
 *        and aLevels[2] represets the number of z axis levels (Blue).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than 2 or greater than 256.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Trilinear_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jobject pValues, jobjectArray pLevels, jintArray aLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLUT_Trilinear_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Trilinear_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Trilinear_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Trilinear_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aLevels' is null for nppiLUT_Trilinear_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Trilinear_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, aLevels=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, pValues, pLevels, aLevels);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32u* pValues_native = NULL;
    Npp8u* pLevels_native[3];
    int aLevels_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32u*)pValues_pointerData->getPointer(env);
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aLevels, aLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Trilinear_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, pValues_native, pLevels_native, aLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // pLevels is an array of native pointers
    if (!releaseNative(env, aLevels_native, aLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One channel 8-bit unsigned bit range restricted palette look-up-table color conversion.
 *
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
 * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pTable, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTable' is null for nppiLUTPalette_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTable=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pTable_native = NULL;
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTable_pointerData = initPointerData(env, pTable);
    if (pTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTable_native = (Npp8u*)pTable_pointerData->getPointer(env);
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTable_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One channel 8-bit unsigned bit range restricted 24-bit palette look-up-table color conversion with 24-bit destination output per pixel.
 *
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step (3 bytes per pixel).
 * @param oSizeROI \ref roi_specification.
 * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
 * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_18u24u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pTable, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_8u24u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_8u24u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_8u24u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTable' is null for nppiLUTPalette_8u24u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_8u24u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTable=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pTable_native = NULL;
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTable_pointerData = initPointerData(env, pTable);
    if (pTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTable_native = (Npp8u*)pTable_pointerData->getPointer(env);
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_8u24u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTable_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One channel 8-bit unsigned bit range restricted 32-bit palette look-up-table color conversion with 32-bit destination output per pixel.
 *
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step (4 bytes per pixel).
 * @param oSizeROI \ref roi_specification.
 * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
 * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_18u32u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pTable, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_8u32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_8u32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_8u32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTable' is null for nppiLUTPalette_8u32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_8u32u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTable=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32u* pTable_native = NULL;
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTable_pointerData = initPointerData(env, pTable);
    if (pTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTable_native = (Npp32u*)pTable_pointerData->getPointer(env);
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_8u32u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTable_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned bit range restricted palette look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
 * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pTables, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTables == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTables' is null for nppiLUTPalette_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTables=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp8u* pTables_native[3];
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pTables, pTables_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTables_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pTables is an array of native pointers
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned bit range restricted palette look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pTables Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
 * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pTables, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTables == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTables' is null for nppiLUTPalette_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTables=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp8u* pTables_native[4];
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pTables, pTables_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTables_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pTables is an array of native pointers
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned bit range restricted palette look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
 * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pTables, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTables == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTables' is null for nppiLUTPalette_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTables=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp8u* pTables_native[3];
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pTables, pTables_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTables_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pTables is an array of native pointers
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One channel 16-bit unsigned bit range restricted palette look-up-table color conversion.
 *
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
 * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pTable, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTable' is null for nppiLUTPalette_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTable=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u* pTable_native = NULL;
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTable_pointerData = initPointerData(env, pTable);
    if (pTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTable_native = (Npp16u*)pTable_pointerData->getPointer(env);
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTable_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One channel 16-bit unsigned bit range restricted 8-bit unsigned palette look-up-table color conversion with 8-bit unsigned destination output per pixel.
 *
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step (1 unsigned byte per pixel).
 * @param oSizeROI \ref roi_specification.
 * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
 * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_116u8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pTable, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTable' is null for nppiLUTPalette_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_16u8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTable=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pTable_native = NULL;
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTable_pointerData = initPointerData(env, pTable);
    if (pTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTable_native = (Npp8u*)pTable_pointerData->getPointer(env);
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_16u8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTable_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One channel 16-bit unsigned bit range restricted 24-bit unsigned palette look-up-table color conversion with 24-bit unsigned destination output per pixel.
 *
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step (3 unsigned bytes per pixel).
 * @param oSizeROI \ref roi_specification.
 * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
 * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_116u24u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pTable, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_16u24u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_16u24u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_16u24u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTable' is null for nppiLUTPalette_16u24u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_16u24u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTable=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pTable_native = NULL;
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTable_pointerData = initPointerData(env, pTable);
    if (pTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTable_native = (Npp8u*)pTable_pointerData->getPointer(env);
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_16u24u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTable_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One channel 16-bit unsigned bit range restricted 32-bit palette look-up-table color conversion with 32-bit unsigned destination output per pixel.
 *
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step (4 bytes per pixel).
 * @param oSizeROI \ref roi_specification.
 * @param pTable Pointer to an array of user defined OUTPUT palette values (this is a device memory pointer)
 * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_116u32u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pTable, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_16u32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_16u32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_16u32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTable' is null for nppiLUTPalette_16u32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_16u32u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTable=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTable, nBitSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32u* pTable_native = NULL;
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTable_pointerData = initPointerData(env, pTable);
    if (pTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTable_native = (Npp32u*)pTable_pointerData->getPointer(env);
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_16u32u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTable_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned bit range restricted palette look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
 * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pTables, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTables == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTables' is null for nppiLUTPalette_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTables=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp16u* pTables_native[3];
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pTables, pTables_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTables_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pTables is an array of native pointers
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned bit range restricted palette look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pTables Host pointer to an array of 4 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
 * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pTables, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTables == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTables' is null for nppiLUTPalette_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTables=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp16u* pTables_native[4];
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pTables, pTables_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTables_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pTables is an array of native pointers
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned bit range restricted palette look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
 * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPalette_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pTables, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPalette_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPalette_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPalette_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTables == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTables' is null for nppiLUTPalette_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPalette_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTables=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pTables, nBitSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp16u* pTables_native[3];
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pTables, pTables_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPalette_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pTables_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pTables is an array of native pointers
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned source bit range restricted palette look-up-table color conversion to four channel 8-bit unsigned destination output with alpha.
 *
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 * This function also reverses the source pixel channel order in the destination so the Alpha channel is the first channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step (3 bytes per pixel).
 * @param nAlphaValue Signed alpha value that will be used to initialize the pixel alpha channel position in all modified destination pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step (4 bytes per pixel with alpha).
 * @param oSizeROI \ref roi_specification.
 * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
 * Alpha values < 0 or > 255 will cause destination pixel alpha channel values to be unmodified.
 * @param nBitSize Number of least significant bits (must be > 0 and <= 8) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 8.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPaletteSwap_18u_1C3A0C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jint nAlphaValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pTables, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPaletteSwap_8u_C3A0C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    // nAlphaValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPaletteSwap_8u_C3A0C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPaletteSwap_8u_C3A0C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTables == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTables' is null for nppiLUTPaletteSwap_8u_C3A0C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPaletteSwap_8u_C3A0C4R(pSrc=%p, nSrcStep=%d, nAlphaValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTables=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, nAlphaValue, pDst, nDstStep, oSizeROI, pTables, nBitSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    int nAlphaValue_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp8u* pTables_native[3];
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    nAlphaValue_native = (int)nAlphaValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pTables, pTables_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPaletteSwap_8u_C3A0C4R(pSrc_native, nSrcStep_native, nAlphaValue_native, pDst_native, nDstStep_native, oSizeROI_native, pTables_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // nAlphaValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pTables is an array of native pointers
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned source bit range restricted palette look-up-table color conversion to four channel 16-bit unsigned destination output with alpha.
 *
 * The LUT is derived from a set of user defined mapping points in a palette and 
 * source pixels are then processed using a restricted bit range when looking up palette values.
 * This function also reverses the source pixel channel order in the destination so the Alpha channel is the first channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step (3 unsigned short integers per pixel).
 * @param nAlphaValue Signed alpha value that will be used to initialize the pixel alpha channel position in all modified destination pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step (4 unsigned short integers per pixel with alpha).
 * @param oSizeROI \ref roi_specification.
 * @param pTables Host pointer to an array of 3 device memory pointers, one per color CHANNEL, pointing to user defined OUTPUT palette values.
 * Alpha values < 0 or > 65535 will cause destination pixel alpha channel values to be unmodified.
 * @param nBitSize Number of least significant bits (must be > 0 and <= 16) of each source pixel value to use as index into palette table during conversion.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - ::NPP_LUT_PALETTE_BITSIZE_ERROR if nBitSize is < 1 or > 16.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUTPaletteSwap_116u_1C3A0C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jint nAlphaValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pTables, jint nBitSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUTPaletteSwap_16u_C3A0C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    // nAlphaValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUTPaletteSwap_16u_C3A0C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUTPaletteSwap_16u_C3A0C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTables == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTables' is null for nppiLUTPaletteSwap_16u_C3A0C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nBitSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUTPaletteSwap_16u_C3A0C4R(pSrc=%p, nSrcStep=%d, nAlphaValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pTables=%p, nBitSize=%d)\n",
        pSrc, nSrcStep, nAlphaValue, pDst, nDstStep, oSizeROI, pTables, nBitSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    int nAlphaValue_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp16u* pTables_native[3];
    int nBitSize_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    nAlphaValue_native = (int)nAlphaValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pTables, pTables_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nBitSize_native = (int)nBitSize;

    // Native function call
    NppStatus jniResult_native = nppiLUTPaletteSwap_16u_C3A0C4R(pSrc_native, nSrcStep_native, nAlphaValue_native, pDst_native, nDstStep_native, oSizeROI_native, pTables_native, nBitSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // nAlphaValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pTables is an array of native pointers
    // nBitSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nThreshold, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%c, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyte nThreshold, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%c, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nThreshold, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%f, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nThreshold, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%f, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed short threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed short in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char in place image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed short image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyte nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%c)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short in place threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point in place threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%f)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed short threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point in place threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GT_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GT_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GT_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GT_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GT_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GT_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyte nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%c)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point in place threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nThreshold)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%f)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed short threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point in place threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LT_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LT_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LT_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LT_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LT_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LT_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nThreshold, jbyte nValue, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%c, nValue=%c, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;
    Npp8u nValue_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;
    nValue_native = (Npp8u)nValue;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyte nThreshold, jbyte nValue, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%c, nValue=%c, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;
    Npp8u nValue_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;
    nValue_native = (Npp8u)nValue;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;
    Npp16u nValue_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;
    nValue_native = (Npp16u)nValue;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;
    Npp16u nValue_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;
    nValue_native = (Npp16u)nValue;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;
    Npp16s nValue_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;
    nValue_native = (Npp16s)nValue;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;
    Npp16s nValue_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;
    nValue_native = (Npp16s)nValue;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nThreshold, jfloat nValue, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%f, nValue=%f, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;
    Npp32f nValue_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;
    nValue_native = (Npp32f)nValue;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nThreshold, jfloat nValue, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%f, nValue=%f, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;
    Npp32f nValue_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;
    nValue_native = (Npp32f)nValue;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed short in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point in place threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char in place image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed short image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_Val_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_Val_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1Val_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_Val_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_Val_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_Val_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_Val_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_Val_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p, eComparisonOperation=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_Val_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nThreshold, jbyte nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%c, nValue=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;
    Npp8u nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;
    nValue_native = (Npp8u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyte nThreshold, jbyte nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%c, nValue=%c)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;
    Npp8u nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;
    nValue_native = (Npp8u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;
    Npp16u nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;
    nValue_native = (Npp16u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;
    Npp16u nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;
    nValue_native = (Npp16u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;
    Npp16s nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;
    nValue_native = (Npp16s)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short in place threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;
    Npp16s nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;
    nValue_native = (Npp16s)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nThreshold, jfloat nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%f, nValue=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;
    Npp32f nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;
    nValue_native = (Npp32f)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point in place threshold.
 * If for a comparison operations sourcePixel is greater than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nThreshold, jfloat nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%f, nValue=%f)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;
    Npp32f nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;
    nValue_native = (Npp32f)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char threshold.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short threshold.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point threshold.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point in place threshold.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed short image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_GTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_GTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is greater than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1GTVal_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_GTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_GTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_GTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_GTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_GTVal_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_GTVal_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nThreshold, jbyte nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%c, nValue=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;
    Npp8u nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;
    nValue_native = (Npp8u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyte nThreshold, jbyte nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%c, nValue=%c)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThreshold_native;
    Npp8u nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp8u)nThreshold;
    nValue_native = (Npp8u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;
    Npp16u nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;
    nValue_native = (Npp16u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThreshold_native;
    Npp16u nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16u)nThreshold;
    nValue_native = (Npp16u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;
    Npp16s nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;
    nValue_native = (Npp16s)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThreshold, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%d, nValue=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThreshold_native;
    Npp16s nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp16s)nThreshold;
    nValue_native = (Npp16s)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nThreshold, jfloat nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%f, nValue=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;
    Npp32f nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;
    nValue_native = (Npp32f)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point in place threshold.
 * If for a comparison operations sourcePixel is less than nThreshold is true, the pixel is set
 * to nValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param nValue The threshold replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nThreshold, jfloat nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThreshold=%f, nValue=%f)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThreshold, nValue);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;
    Npp32f nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;
    nValue_native = (Npp32f)nValue;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThreshold_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char threshold.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short threshold.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed short threshold.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed short in place threshold.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point threshold.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholds, jbyteArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholds_native[3];
    Npp8u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholds_native[3];
    Npp16u rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed short image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed short in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholds, jshortArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholds_native[3];
    Npp16s rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than rThreshold is true, the pixel is set
 * value is set to rValue, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholds The threshold values, one per color channel.
 * @param rValues The threshold replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTVal_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholds, jfloatArray rValues)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholds' is null for nppiThreshold_LTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValues' is null for nppiThreshold_LTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTVal_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholds=%p, rValues=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholds, rValues);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholds_native[3];
    Npp32f rValues_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholds, rThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValues, rValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTVal_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholds_native, rValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholds_native, rThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValues_native, rValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char threshold.
 * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
 * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThresholdLT The thresholdLT value.
 * @param nValueLT The thresholdLT replacement value.
 * @param nThresholdGT The thresholdGT value.
 * @param nValueGT The thresholdGT replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nThresholdLT, jbyte nValueLT, jbyte nThresholdGT, jbyte nValueGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThresholdLT=%c, nValueLT=%c, nThresholdGT=%c, nValueGT=%c)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThresholdLT_native;
    Npp8u nValueLT_native;
    Npp8u nThresholdGT_native;
    Npp8u nValueGT_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThresholdLT_native = (Npp8u)nThresholdLT;
    nValueLT_native = (Npp8u)nValueLT;
    nThresholdGT_native = (Npp8u)nThresholdGT;
    nValueGT_native = (Npp8u)nValueGT;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThresholdLT_native, nValueLT_native, nThresholdGT_native, nValueGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
 * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThresholdLT The thresholdLT value.
 * @param nValueLT The thresholdLT replacement value.
 * @param nThresholdGT The thresholdGT value.
 * @param nValueGT The thresholdGT replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyte nThresholdLT, jbyte nValueLT, jbyte nThresholdGT, jbyte nValueGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThresholdLT=%c, nValueLT=%c, nThresholdGT=%c, nValueGT=%c)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nThresholdLT_native;
    Npp8u nValueLT_native;
    Npp8u nThresholdGT_native;
    Npp8u nValueGT_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThresholdLT_native = (Npp8u)nThresholdLT;
    nValueLT_native = (Npp8u)nValueLT;
    nThresholdGT_native = (Npp8u)nThresholdGT;
    nValueGT_native = (Npp8u)nValueGT;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThresholdLT_native, nValueLT_native, nThresholdGT_native, nValueGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short threshold.
 * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
 * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThresholdLT The thresholdLT value.
 * @param nValueLT The thresholdLT replacement value.
 * @param nThresholdGT The thresholdGT value.
 * @param nValueGT The thresholdGT replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThresholdLT, jshort nValueLT, jshort nThresholdGT, jshort nValueGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThresholdLT=%d, nValueLT=%d, nThresholdGT=%d, nValueGT=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThresholdLT_native;
    Npp16u nValueLT_native;
    Npp16u nThresholdGT_native;
    Npp16u nValueGT_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThresholdLT_native = (Npp16u)nThresholdLT;
    nValueLT_native = (Npp16u)nValueLT;
    nThresholdGT_native = (Npp16u)nThresholdGT;
    nValueGT_native = (Npp16u)nValueGT;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThresholdLT_native, nValueLT_native, nThresholdGT_native, nValueGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
 * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThresholdLT The thresholdLT value.
 * @param nValueLT The thresholdLT replacement value.
 * @param nThresholdGT The thresholdGT value.
 * @param nValueGT The thresholdGT replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThresholdLT, jshort nValueLT, jshort nThresholdGT, jshort nValueGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThresholdLT=%d, nValueLT=%d, nThresholdGT=%d, nValueGT=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u nThresholdLT_native;
    Npp16u nValueLT_native;
    Npp16u nThresholdGT_native;
    Npp16u nValueGT_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThresholdLT_native = (Npp16u)nThresholdLT;
    nValueLT_native = (Npp16u)nValueLT;
    nThresholdGT_native = (Npp16u)nThresholdGT;
    nValueGT_native = (Npp16u)nValueGT;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThresholdLT_native, nValueLT_native, nThresholdGT_native, nValueGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short threshold.
 * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
 * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThresholdLT The thresholdLT value.
 * @param nValueLT The thresholdLT replacement value.
 * @param nThresholdGT The thresholdGT value.
 * @param nValueGT The thresholdGT replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshort nThresholdLT, jshort nValueLT, jshort nThresholdGT, jshort nValueGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThresholdLT=%d, nValueLT=%d, nThresholdGT=%d, nValueGT=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThresholdLT_native;
    Npp16s nValueLT_native;
    Npp16s nThresholdGT_native;
    Npp16s nValueGT_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThresholdLT_native = (Npp16s)nThresholdLT;
    nValueLT_native = (Npp16s)nValueLT;
    nThresholdGT_native = (Npp16s)nThresholdGT;
    nValueGT_native = (Npp16s)nValueGT;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThresholdLT_native, nValueLT_native, nThresholdGT_native, nValueGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short in place threshold.
 * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
 * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThresholdLT The thresholdLT value.
 * @param nValueLT The thresholdLT replacement value.
 * @param nThresholdGT The thresholdGT value.
 * @param nValueGT The thresholdGT replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshort nThresholdLT, jshort nValueLT, jshort nThresholdGT, jshort nValueGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThresholdLT=%d, nValueLT=%d, nThresholdGT=%d, nValueGT=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s nThresholdLT_native;
    Npp16s nValueLT_native;
    Npp16s nThresholdGT_native;
    Npp16s nValueGT_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThresholdLT_native = (Npp16s)nThresholdLT;
    nValueLT_native = (Npp16s)nValueLT;
    nThresholdGT_native = (Npp16s)nThresholdGT;
    nValueGT_native = (Npp16s)nValueGT;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThresholdLT_native, nValueLT_native, nThresholdGT_native, nValueGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point threshold.
 * If for a comparison operations sourcePixel is less than nThresholdLT is true, the pixel is set
 * to nValueLT, else if sourcePixel is greater than nThresholdGT the pixel is set to nValueGT, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThresholdLT The thresholdLT value.
 * @param nValueLT The thresholdLT replacement value.
 * @param nThresholdGT The thresholdGT value.
 * @param nValueGT The thresholdGT replacement value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nThresholdLT, jfloat nValueLT, jfloat nThresholdGT, jfloat nValueGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThresholdLT=%f, nValueLT=%f, nThresholdGT=%f, nValueGT=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThresholdLT_native;
    Npp32f nValueLT_native;
    Npp32f nThresholdGT_native;
    Npp32f nValueGT_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThresholdLT_native = (Npp32f)nThresholdLT;
    nValueLT_native = (Npp32f)nValueLT;
    nThresholdGT_native = (Npp32f)nThresholdGT;
    nValueGT_native = (Npp32f)nValueGT;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThresholdLT_native, nValueLT_native, nThresholdGT_native, nValueGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nThresholdLT, jfloat nValueLT, jfloat nThresholdGT, jfloat nValueGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nThresholdLT=%f, nValueLT=%f, nThresholdGT=%f, nValueGT=%f)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nThresholdLT, nValueLT, nThresholdGT, nValueGT);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThresholdLT_native;
    Npp32f nValueLT_native;
    Npp32f nThresholdGT_native;
    Npp32f nValueGT_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThresholdLT_native = (Npp32f)nThresholdLT;
    nValueLT_native = (Npp32f)nValueLT;
    nThresholdGT_native = (Npp32f)nThresholdGT;
    nValueGT_native = (Npp32f)nValueGT;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nThresholdLT_native, nValueLT_native, nThresholdGT_native, nValueGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nThresholdLT is primitive
    // nValueLT is primitive
    // nThresholdGT is primitive
    // nValueGT is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char threshold.
 * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
 * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholdsLT The thresholdLT values, one per color channel.
 * @param rValuesLT The thresholdLT replacement values, one per color channel.
 * @param rThresholdsGT The thresholdGT values, one per channel.
 * @param rValuesGT The thresholdGT replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholdsLT, jbyteArray rValuesLT, jbyteArray rThresholdsGT, jbyteArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholdsLT_native[3];
    Npp8u rValuesLT_native[3];
    Npp8u rThresholdsGT_native[3];
    Npp8u rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char in place threshold.
 * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
 * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref destination_image_pointer.
 * @param nSrcDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholdsLT The thresholdLT values, one per color channel.
 * @param rValuesLT The thresholdLT replacement values, one per color channel.
 * @param rThresholdsGT The thresholdGT values, one per channel.
 * @param rValuesGT The thresholdGT replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholdsLT, jbyteArray rValuesLT, jbyteArray rThresholdsGT, jbyteArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholdsLT_native[3];
    Npp8u rValuesLT_native[3];
    Npp8u rThresholdsGT_native[3];
    Npp8u rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short threshold.
 * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
 * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholdsLT The thresholdLT values, one per color channel.
 * @param rValuesLT The thresholdLT replacement values, one per color channel.
 * @param rThresholdsGT The thresholdGT values, one per channel.
 * @param rValuesGT The thresholdGT replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholdsLT, jshortArray rValuesLT, jshortArray rThresholdsGT, jshortArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholdsLT_native[3];
    Npp16u rValuesLT_native[3];
    Npp16u rThresholdsGT_native[3];
    Npp16u rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short in place threshold.
 * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
 * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholdsLT The thresholdLT values, one per color channel.
 * @param rValuesLT The thresholdLT replacement values, one per color channel.
 * @param rThresholdsGT The thresholdGT values, one per channel.
 * @param rValuesGT The thresholdGT replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholdsLT, jshortArray rValuesLT, jshortArray rThresholdsGT, jshortArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholdsLT_native[3];
    Npp16u rValuesLT_native[3];
    Npp16u rThresholdsGT_native[3];
    Npp16u rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed short threshold.
 * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
 * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholdsLT The thresholdLT values, one per color channel.
 * @param rValuesLT The thresholdLT replacement values, one per color channel.
 * @param rThresholdsGT The thresholdGT values, one per channel.
 * @param rValuesGT The thresholdGT replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholdsLT, jshortArray rValuesLT, jshortArray rThresholdsGT, jshortArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholdsLT_native[3];
    Npp16s rValuesLT_native[3];
    Npp16s rThresholdsGT_native[3];
    Npp16s rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholdsLT, jshortArray rValuesLT, jshortArray rThresholdsGT, jshortArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholdsLT_native[3];
    Npp16s rValuesLT_native[3];
    Npp16s rThresholdsGT_native[3];
    Npp16s rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point threshold.
 * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
 * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholdsLT The thresholdLT values, one per color channel.
 * @param rValuesLT The thresholdLT replacement values, one per color channel.
 * @param rThresholdsGT The thresholdGT values, one per channel.
 * @param rValuesGT The thresholdGT replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholdsLT, jfloatArray rValuesLT, jfloatArray rThresholdsGT, jfloatArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholdsLT_native[3];
    Npp32f rValuesLT_native[3];
    Npp32f rThresholdsGT_native[3];
    Npp32f rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point in place threshold.
 * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
 * to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholdsLT The thresholdLT values, one per color channel.
 * @param rValuesLT The thresholdLT replacement values, one per color channel.
 * @param rThresholdsGT The thresholdGT values, one per channel.
 * @param rValuesGT The thresholdGT replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholdsLT, jfloatArray rValuesLT, jfloatArray rThresholdsGT, jfloatArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholdsLT_native[3];
    Npp32f rValuesLT_native[3];
    Npp32f rThresholdsGT_native[3];
    Npp32f rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
 * value is set to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholdsLT The thresholdLT values, one per color channel.
 * @param rValuesLT The thresholdLT replacement values, one per color channel.
 * @param rThresholdsGT The thresholdGT values, one per channel.
 * @param rValuesGT The thresholdGT replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray rThresholdsLT, jbyteArray rValuesLT, jbyteArray rThresholdsGT, jbyteArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholdsLT_native[3];
    Npp8u rValuesLT_native[3];
    Npp8u rThresholdsGT_native[3];
    Npp8u rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
 * value is set to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholdsLT The thresholdLT values, one per color channel.
 * @param rValuesLT The thresholdLT replacement values, one per color channel.
 * @param rThresholdsGT The thresholdGT values, one per channel.
 * @param rValuesGT The thresholdGT replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jbyteArray rThresholdsLT, jbyteArray rValuesLT, jbyteArray rThresholdsGT, jbyteArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u rThresholdsLT_native[3];
    Npp8u rValuesLT_native[3];
    Npp8u rThresholdsGT_native[3];
    Npp8u rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholdsLT, jshortArray rValuesLT, jshortArray rThresholdsGT, jshortArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholdsLT_native[3];
    Npp16u rValuesLT_native[3];
    Npp16u rThresholdsGT_native[3];
    Npp16u rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholdsLT, jshortArray rValuesLT, jshortArray rThresholdsGT, jshortArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u rThresholdsLT_native[3];
    Npp16u rValuesLT_native[3];
    Npp16u rThresholdsGT_native[3];
    Npp16u rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jshortArray rThresholdsLT, jshortArray rValuesLT, jshortArray rThresholdsGT, jshortArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholdsLT_native[3];
    Npp16s rValuesLT_native[3];
    Npp16s rThresholdsGT_native[3];
    Npp16s rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jshortArray rThresholdsLT, jshortArray rValuesLT, jshortArray rThresholdsGT, jshortArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s rThresholdsLT_native[3];
    Npp16s rValuesLT_native[3];
    Npp16s rThresholdsGT_native[3];
    Npp16s rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
 * value is set to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholdsLT The thresholdLT values, one per color channel.
 * @param rValuesLT The thresholdLT replacement values, one per color channel.
 * @param rThresholdsGT The thresholdGT values, one per channel.
 * @param rValuesGT The thresholdGT replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloatArray rThresholdsLT, jfloatArray rValuesLT, jfloatArray rThresholdsGT, jfloatArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_LTValGTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_LTValGTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholdsLT_native[3];
    Npp32f rValuesLT_native[3];
    Npp32f rThresholdsGT_native[3];
    Npp32f rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point in place image threshold, not affecting Alpha.
 * If for a comparison operations sourcePixel is less than rThresholdLT is true, the pixel is set
 * value is set to rValueLT, else if sourcePixel is greater than rThresholdGT the pixel is set to rValueGT, otherwise it is set to sourcePixel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rThresholdsLT The thresholdLT values, one per color channel.
 * @param rValuesLT The thresholdLT replacement values, one per color channel.
 * @param rThresholdsGT The thresholdGT values, one per channel.
 * @param rValuesGT The thresholdGT replacement values, one per color channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_1LTValGTVal_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloatArray rThresholdsLT, jfloatArray rValuesLT, jfloatArray rThresholdsGT, jfloatArray rValuesGT)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiThreshold_LTValGTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_LTValGTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsLT' is null for nppiThreshold_LTValGTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesLT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesLT' is null for nppiThreshold_LTValGTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rThresholdsGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rThresholdsGT' is null for nppiThreshold_LTValGTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rValuesGT == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rValuesGT' is null for nppiThreshold_LTValGTVal_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_LTValGTVal_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rThresholdsLT=%p, rValuesLT=%p, rThresholdsGT=%p, rValuesGT=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, rThresholdsLT, rValuesLT, rThresholdsGT, rValuesGT);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f rThresholdsLT_native[3];
    Npp32f rValuesLT_native[3];
    Npp32f rThresholdsGT_native[3];
    Npp32f rValuesGT_native[3];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsLT, rThresholdsLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesLT, rValuesLT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rThresholdsGT, rThresholdsGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rValuesGT, rValuesGT_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_LTValGTVal_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rThresholdsLT_native, rValuesLT_native, rThresholdsGT_native, rValuesGT_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, rThresholdsLT_native, rThresholdsLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesLT_native, rValuesLT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rThresholdsGT_native, rThresholdsGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, rValuesGT_native, rValuesGT, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char image compare.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned char image compare, not affecting Alpha.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short image compare.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short image compare.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short image compare.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short image compare, not affecting Alpha.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed short image compare.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_16s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_16s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed short image compare.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_16s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_16s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_16s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_16s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed short image compare, not affecting Alpha.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_16s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_16s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point image compare.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image compare.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned char image compare with constant value.
 * Compare pSrc's pixels with constant value. 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param nConstant constant value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_8u_C1R(pSrc=%p, nSrcStep=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, nConstant, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_8u_C1R(pSrc_native, nSrcStep_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned char image compare with constant value.
 * Compare pSrc's pixels with constant value. 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per color channel..
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_8u_C3R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_8u_C3R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_8u_C4R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_8u_C4R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_8u_AC4R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_8u_AC4R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned short image compare with constant value.
 * Compare pSrc's pixels with constant value. 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param nConstant constant value
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_16u_C1R(pSrc=%p, nSrcStep=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, nConstant, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_16u_C1R(pSrc_native, nSrcStep_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned short image compare with constant value.
 * Compare pSrc's pixels with constant value. 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pConstants pointer to a list of constants, one per color channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_16u_C3R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_16u_C3R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_16u_C4R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_16u_C4R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned short image compare, not affecting Alpha.
 * Compare pSrc's pixels with constant value. 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pConstants pointer to a list of constants, one per color channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_16u_AC4R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_16u_AC4R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_16s_C1R(pSrc=%p, nSrcStep=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, nConstant, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    nConstant_native = (Npp16s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_16s_C1R(pSrc_native, nSrcStep_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed short image compare with constant value.
 * Compare pSrc's pixels with constant value. 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pConstants pointer to a list of constants, one per color channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_16s_C3R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_16s_C3R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_16s_C4R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_16s_C4R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_16s_AC4R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_16s_AC4R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jfloat nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_32f_C1R(pSrc=%p, nSrcStep=%d, nConstant=%f, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, nConstant, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    nConstant_native = (Npp32f)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_32f_C1R(pSrc_native, nSrcStep_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point image compare with constant value.
 * Compare pSrc's pixels with constant value. 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pConstants pointer to a list of constants, one per color channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_32f_C3R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_32f_C3R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image compare with constant value.
 * Compare pSrc's pixels with constant value. 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pConstants pointer to a list of constants, one per color channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_32f_C4R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_32f_C4R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareC_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareC_32f_AC4R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompareC_32f_AC4R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point image compare whether two images are equal within epsilon.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2 to determine whether they are equal with a difference of epsilon. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nEpsilon epsilon tolerance value to compare to pixel absolute differences
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareEqualEps_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nEpsilon)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompareEqualEps_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompareEqualEps_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareEqualEps_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareEqualEps_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nEpsilon is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareEqualEps_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nEpsilon=%f)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nEpsilon);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nEpsilon_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nEpsilon_native = (Npp32f)nEpsilon;

    // Native function call
    NppStatus jniResult_native = nppiCompareEqualEps_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nEpsilon_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nEpsilon is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point image compare whether two images are equal within epsilon.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2 to determine whether they are equal with a difference of epsilon. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nEpsilon epsilon tolerance value to compare to per color channel pixel absolute differences
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareEqualEps_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nEpsilon)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompareEqualEps_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompareEqualEps_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareEqualEps_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareEqualEps_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nEpsilon is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareEqualEps_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nEpsilon=%f)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nEpsilon);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nEpsilon_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nEpsilon_native = (Npp32f)nEpsilon;

    // Native function call
    NppStatus jniResult_native = nppiCompareEqualEps_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nEpsilon_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nEpsilon is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image compare whether two images are equal within epsilon.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2 to determine whether they are equal with a difference of epsilon. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nEpsilon epsilon tolerance value to compare to per color channel pixel absolute differences
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareEqualEps_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nEpsilon)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompareEqualEps_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompareEqualEps_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareEqualEps_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareEqualEps_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nEpsilon is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareEqualEps_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nEpsilon=%f)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nEpsilon);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nEpsilon_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nEpsilon_native = (Npp32f)nEpsilon;

    // Native function call
    NppStatus jniResult_native = nppiCompareEqualEps_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nEpsilon_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nEpsilon is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed floating point compare whether two images are equal within epsilon, not affecting Alpha.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2 to determine whether they are equal with a difference of epsilon. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nEpsilon epsilon tolerance value to compare to per color channel pixel absolute differences
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareEqualEps_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nEpsilon)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompareEqualEps_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompareEqualEps_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareEqualEps_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareEqualEps_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nEpsilon is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareEqualEps_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nEpsilon=%f)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nEpsilon);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nEpsilon_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nEpsilon_native = (Npp32f)nEpsilon;

    // Native function call
    NppStatus jniResult_native = nppiCompareEqualEps_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nEpsilon_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nEpsilon is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point image compare whether image and constant are equal within epsilon.
 * Compare pSrc's pixels with constant value to determine whether they are equal within a difference of epsilon. 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param nConstant constant value
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nEpsilon epsilon tolerance value to compare to pixel absolute differences
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareEqualEpsC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jfloat nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nEpsilon)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareEqualEpsC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareEqualEpsC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareEqualEpsC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nEpsilon is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareEqualEpsC_32f_C1R(pSrc=%p, nSrcStep=%d, nConstant=%f, pDst=%p, nDstStep=%d, oSizeROI=%p, nEpsilon=%f)\n",
        pSrc, nSrcStep, nConstant, pDst, nDstStep, oSizeROI, nEpsilon);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nEpsilon_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    nConstant_native = (Npp32f)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nEpsilon_native = (Npp32f)nEpsilon;

    // Native function call
    NppStatus jniResult_native = nppiCompareEqualEpsC_32f_C1R(pSrc_native, nSrcStep_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nEpsilon_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nEpsilon is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point image compare whether image and constant are equal within epsilon.
 * Compare pSrc's pixels with constant value to determine whether they are equal within a difference of epsilon. 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pConstants pointer to a list of constants, one per color channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nEpsilon epsilon tolerance value to compare to per color channel pixel absolute differences
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareEqualEpsC_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nEpsilon)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareEqualEpsC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareEqualEpsC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareEqualEpsC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareEqualEpsC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nEpsilon is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareEqualEpsC_32f_C3R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nEpsilon=%f)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, nEpsilon);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nEpsilon_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nEpsilon_native = (Npp32f)nEpsilon;

    // Native function call
    NppStatus jniResult_native = nppiCompareEqualEpsC_32f_C3R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nEpsilon_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nEpsilon is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image compare whether image and constant are equal within epsilon.
 * Compare pSrc's pixels with constant value to determine whether they are equal within a difference of epsilon. 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pConstants pointer to a list of constants, one per color channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nEpsilon epsilon tolerance value to compare to per color channel pixel absolute differences
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareEqualEpsC_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nEpsilon)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareEqualEpsC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareEqualEpsC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareEqualEpsC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareEqualEpsC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nEpsilon is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareEqualEpsC_32f_C4R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nEpsilon=%f)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, nEpsilon);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nEpsilon_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nEpsilon_native = (Npp32f)nEpsilon;

    // Native function call
    NppStatus jniResult_native = nppiCompareEqualEpsC_32f_C4R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nEpsilon_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nEpsilon is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompareEqualEpsC_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nEpsilon)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCompareEqualEpsC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiCompareEqualEpsC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompareEqualEpsC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompareEqualEpsC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nEpsilon is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompareEqualEpsC_32f_AC4R(pSrc=%p, nSrcStep=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nEpsilon=%f)\n",
        pSrc, nSrcStep, pConstants, pDst, nDstStep, oSizeROI, nEpsilon);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nEpsilon_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nEpsilon_native = (Npp32f)nEpsilon;

    // Native function call
    NppStatus jniResult_native = nppiCompareEqualEpsC_32f_AC4R(pSrc_native, nSrcStep_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nEpsilon_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nEpsilon is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 8-bit unsigned integer dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned integer dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer dilation, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point dilation, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_8u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_8u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_8u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer dilation with border control, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer dilation with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_16u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_16u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_16u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_16u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer dilation with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_16u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_16u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer dilation with border control, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_16u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_16u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating-point dilation with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_32f_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point dilation with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_32f_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilateBorder_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilateBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilateBorder_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilateBorder_32f_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned integer 3x3 dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer 3x3 dilation, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer 3x3 dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned integer 3x3 dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer 3x3 dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point 3x3 dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating-point 3x3 dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point 3x3 dilation.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point 3x3 dilation, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3_64f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp64f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3_64f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_8u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned integer 3x3 dilation with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_8u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer 3x3 dilation with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_8u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer 3x3 dilation with border control, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer 3x3 dilation with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_16u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_16u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned integer 3x3 dilation with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_16u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_16u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer 3x3 dilation with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_16u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_16u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer 3x3 dilation with border control, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_16u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_16u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point 3x3 dilation with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_32f_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point 3x3 dilation with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_32f_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate3x3Border_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate3x3Border_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiDilate3x3Border_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiDilate3x3Border_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate3x3Border_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate3x3Border_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate3x3Border_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiDilate3x3Border_32f_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 8-bit unsigned integer erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned integer erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer erosion, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned integer erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating-point erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point erosion, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 8-bit unsigned integer erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_8u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned integer erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_8u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_8u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer erosion with border control, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_16u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_16u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned integer erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_16u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_16u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_16u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_16u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer erosion with border control, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_16u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_16u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating-point erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_32f_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_32f_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErodeBorder_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErodeBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErodeBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErodeBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErodeBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErodeBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErodeBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErodeBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErodeBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErodeBorder_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErodeBorder_32f_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 8-bit unsigned integer 3x3 erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer 3x3 erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer 3x3 erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer 3x3 erosion, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point 3x3 erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating-point 3x3 erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point 3x3 erosion, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 64-bit floating-point 3x3 erosion.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3_64f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp64f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3_64f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 8-bit unsigned integer 3x3 erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_8u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned integer 3x3 erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_8u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer 3x3 erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_8u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer 3x3 erosion with border control, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer 3x3 erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_16u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_16u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned integer 3x3 erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_16u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_16u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer 3x3 erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_16u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_16u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer 3x3 erosion with border control, ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_16u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_16u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point 3x3 erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_32f_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point 3x3 erosion with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_32f_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode3x3Border_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode3x3Border_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiErode3x3Border_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiErode3x3Border_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode3x3Border_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode3x3Border_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode3x3Border_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiErode3x3Border_32f_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned single-channel 1D column convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned three-channel 1D column convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned four-channel 1D column convolution ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit single-channel 1D column convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit three-channel 1D column convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit four-channel 1D column convolution ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float three-channel 1D column convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float four-channel 1D column convolution ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_64f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp64f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp64f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp64f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_64f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned single-channel 1D column convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned four-channel 1D column convolution.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned three-channel 1D column convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned four-channel 1D column convolution.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned four-channel 1D column convolution ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit three-channel 1D column convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit four-channel 1D column convolution.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit four-channel 1D column convolution ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn32f_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn32f_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn32f_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned single-channel 1D row convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned three-channel 1D row convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned four-channel 1D row convolution.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned single-channel 1D row convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned four-channel 1D row convolution.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned four-channel 1D row convolution ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit single-channel 1D row convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit three-channel 1D row convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit four-channel 1D row convolution.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit four-channel 1D row convolution ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float single-channel 1D row convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float three-channel 1D row convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float four-channel 1D row convolution.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_64f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp64f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp64f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp64f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_64f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned 1D row convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel channel 8-bit unsigned 1D row convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned convolution 1D row filter with border control, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_16u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned 1D row convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_16u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel channel 16-bit 1D row unsigned convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_16u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned 1D row convolution filter with border control, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_16u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit 1D row convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_16s_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel channel 16-bit 1D row convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_16s_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit 1D row convolution filter with border control, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_16s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit float 1D row convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit float 1D row convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit float 1D row convolution filter with border control, ignoring alpha channel.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param nMaskSize Width of the kernel.
 * @param nAnchor X offset of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRowBorder_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRowBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterRowBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterRowBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRowBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRowBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRowBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRowBorder_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, nMaskSize, nAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned single-channel 1D row convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned three-channel 1D row convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned four-channel 1D row convolution.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit three-channel 1D row convolution.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit four-channel 1D row convolution.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit four-channel 1D row convolution ignoring alpha-channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *                 source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow32f_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow32f_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32f* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow32f_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowColumn_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowColumn_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowColumn_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowColumn_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowColumn_8u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowColumn_8u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned 1D (column) sum to 32f.
 *
 * Apply Column Window Summation filter over a 1D mask region around each
 * source pixel for 3-channel 8 bit/pixel input images with 32-bit floating point
 * output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring column pixel values in a mask region of the source image defined by
 * nMaskSize and nAnchor. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowColumn_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowColumn_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowColumn_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowColumn_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowColumn_8u32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowColumn_8u32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowColumn_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowColumn_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowColumn_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowColumn_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowColumn_8u32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowColumn_8u32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowColumn_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowColumn_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowColumn_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowColumn_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowColumn_16u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowColumn_16u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned 1D (column) sum to 32f.
 *
 * Apply Column Window Summation filter over a 1D mask region around each
 * source pixel for 3-channel 16 bit/pixel input images with 32-bit floating point
 * output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring column pixel values in a mask region of the source image defined by
 * nMaskSize and nAnchor. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowColumn_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowColumn_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowColumn_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowColumn_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowColumn_16u32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowColumn_16u32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned 1D (column) sum to 32f.
 *
 * Apply Column Window Summation filter over a 1D mask region around each
 * source pixel for 4-channel 16 bit/pixel input images with 32-bit floating point
 * output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring column pixel values in a mask region of the source image defined by
 * nMaskSize and nAnchor. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowColumn_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowColumn_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowColumn_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowColumn_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowColumn_16u32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowColumn_16u32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowColumn_116s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowColumn_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowColumn_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowColumn_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowColumn_16s32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowColumn_16s32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed 1D (column) sum to 32f.
 *
 * Apply Column Window Summation filter over a 1D mask region around each
 * source pixel for 1-channel 16 bit/pixel input images with 32-bit floating point
 * output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring column pixel values in a mask region of the source image defined by
 * nMaskSize and nAnchor. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowColumn_116s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowColumn_16s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowColumn_16s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowColumn_16s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowColumn_16s32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowColumn_16s32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed 1D (column) sum to 32f.
 *
 * Apply Column Window Summation filter over a 1D mask region around each
 * source pixel for 4-channel 16 bit/pixel input images with 32-bit floating point
 * output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring column pixel values in a mask region of the source image defined by
 * nMaskSize and nAnchor. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowColumn_116s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowColumn_16s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowColumn_16s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowColumn_16s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowColumn_16s32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowColumn_16s32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One channel 8-bit unsigned 1D (row) sum to 32f.
 *
 * Apply Row Window Summation filter over a 1D mask region around each source
 * pixel for 1-channel 8-bit pixel input images with 32-bit floating point output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring row pixel values in a mask region of the source image defined
 * by iKernelDim and iAnchorX. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowRow_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowRow_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowRow_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowRow_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowRow_8u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowRow_8u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned 1D (row) sum to 32f.
 *
 * Apply Row Window Summation filter over a 1D mask region around each source
 * pixel for 3-channel 8-bit pixel input images with 32-bit floating point output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring row pixel values in a mask region of the source image defined
 * by iKernelDim and iAnchorX. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowRow_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowRow_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowRow_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowRow_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowRow_8u32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowRow_8u32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned 1D (row) sum to 32f.
 *
 * Apply Row Window Summation filter over a 1D mask region around each source
 * pixel for 4-channel 8-bit pixel input images with 32-bit floating point output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring row pixel values in a mask region of the source image defined
 * by iKernelDim and iAnchorX. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowRow_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowRow_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowRow_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowRow_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowRow_8u32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowRow_8u32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowRow_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowRow_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowRow_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowRow_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowRow_16u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowRow_16u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned 1D (row) sum to 32f.
 *
 * Apply Row Window Summation filter over a 1D mask region around each source
 * pixel for 3-channel 16-bit pixel input images with 32-bit floating point output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring row pixel values in a mask region of the source image defined
 * by iKernelDim and iAnchorX. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowRow_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowRow_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowRow_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowRow_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowRow_16u32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowRow_16u32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned 1D (row) sum to 32f.
 *
 * Apply Row Window Summation filter over a 1D mask region around each source
 * pixel for 4-channel 16-bit pixel input images with 32-bit floating point output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring row pixel values in a mask region of the source image defined
 * by iKernelDim and iAnchorX. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowRow_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowRow_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowRow_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowRow_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowRow_16u32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowRow_16u32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One channel 16-bit signed 1D (row) sum to 32f.
 *
 * Apply Row Window Summation filter over a 1D mask region around each source
 * pixel for 1-channel 16-bit pixel input images with 32-bit floating point output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring row pixel values in a mask region of the source image defined
 * by iKernelDim and iAnchorX. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowRow_116s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowRow_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowRow_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowRow_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowRow_16s32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowRow_16s32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed 1D (row) sum to 32f.
 *
 * Apply Row Window Summation filter over a 1D mask region around each source
 * pixel for 3-channel 16-bit pixel input images with 32-bit floating point output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring row pixel values in a mask region of the source image defined
 * by iKernelDim and iAnchorX. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference relative to the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowRow_116s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowRow_16s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowRow_16s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowRow_16s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowRow_16s32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowRow_16s32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowRow_116s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowRow_16s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowRow_16s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowRow_16s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowRow_16s32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowRow_16s32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned convolution filter.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned convolution filter, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned convolution filter.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel channel 16-bit unsigned convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned convolution filter, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit convolution filter.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit convolution filter.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel channel 16-bit convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit convolution filter, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_132f_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_32f_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter_32f_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit float convolution filter.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit float convolution filter, ignoring alpha channel.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 64-bit float convolution filter.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_64f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp64f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp64f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp64f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter_64f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two channel 8-bit unsigned convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8u_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8u_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned convolution filter, ignorint alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8s_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8s_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit signed convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned convolution filter, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit convolution filter, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit convolution filter, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned to 16-bit signed convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18u16s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8u16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8u16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18u16s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8u16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8u16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 16-bit signed convolution filter, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18u16s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8u16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8u16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit to 16-bit signed convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8s16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8s16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit to 16-bit signed convolution filter.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18s16s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8s16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8s16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18s16s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8s16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8s16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit to 16-bit signed convolution filter, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter32f_18s16s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter32f_8s16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilter32f_8s16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_8u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_8u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel channel 8-bit unsigned convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_8u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned convolution filter with border control, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_16u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_16u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_16u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_16u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel channel 16-bit unsigned convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_16u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_16u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_16u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_16u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_16s_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_16s_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_16s_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_16s_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_16s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_16s_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_132f_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_32f_C2R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_32f_C2R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit float convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_32f_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit float convolution filter with border control.
 * 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_32f_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder_32f_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two channel 8-bit unsigned convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8u_C2R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8u_C2R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8s_C2R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8s_C2R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8s_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8s_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit signed convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8s_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8s_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit signed convolution filter with border control, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8s_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_16u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_16u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_16u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_16u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_16u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_16u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned convolution filter with border control, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_16u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_16u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_16s_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_16s_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_16s_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_16s_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit convolution filter with border control, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_16s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_16s_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_32s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_32s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_32s_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_32s_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_32s_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_32s_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit convolution filter with border control, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_32s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_32s_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8u16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8u16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned to 16-bit signed convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18u16s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8u16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8u16s_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8u16s_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 16-bit signed convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18u16s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8u16s_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8u16s_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned to 16-bit signed convolution filter with border control, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18u16s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8u16s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8u16s_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit to 16-bit signed convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8s16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8s16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit to 16-bit signed convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18s16s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8s16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8s16s_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8s16s_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit to 16-bit signed convolution filter with border control.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18s16s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8s16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8s16s_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8s16s_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit to 16-bit signed convolution filter with border control, ignoring alpha channel.
 * 
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBorder32f_18s16s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBorder32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterBorder32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterBorder32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBorder32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBorder32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterBorder32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilterBorder32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBorder32f_8s16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBorder32f_8s16s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32f*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterBorder32f_8s16s_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned box filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned box filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned box filter, ignorting alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned box filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned box filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned box filter, ignorting alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit box filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit box filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit box filter, ignorting alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point box filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point box filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point box filter, ignorting alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 64-bit floating-point box filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference relative to
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_64f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp64f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_64f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned maximum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned maximum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned maximum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned maximum filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned maximum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned maximum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned maximum filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed maximum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed maximum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed maximum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed maximum filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point maximum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point maximum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point maximum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point maximum filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned minimum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned minimum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned minimum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned minimum filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned minimum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned minimum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned minimum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned minimum filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed minimum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed minimum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed minimum filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point minimum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point minimum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point minimum filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned median filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned median filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned median filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned median filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned median filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned median filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed median filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed median filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed median filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point median filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point median filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point median filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point median filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Median operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        relative to the source pixel.
 * @param pBuffer Pointer to the user-allocated scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedian_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMedian_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMedian_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedian_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedian_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMedian_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiFilterMedian_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedian_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor, pBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedian_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned median filter scratch memory size.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_8u_C1R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_8u_C1R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned median filter scratch memory size.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_8u_C3R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_8u_C3R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned median filter scratch memory size.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_8u_C4R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_8u_C4R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned median filter, ignoring alpha channel.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_8u_AC4R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_8u_AC4R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned median filter scratch memory size.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_16u_C1R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_16u_C1R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned median filter scratch memory size.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_16u_C3R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_16u_C3R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned median filter scratch memory size.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_16u_C4R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_16u_C4R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned median filter, ignoring alpha channel.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_16u_AC4R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_16u_AC4R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed median filter scratch memory size.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_16s_C1R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_16s_C1R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed median filter scratch memory size.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_16s_C3R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_16s_C3R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed median filter scratch memory size.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_16s_C4R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_16s_C4R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed median filter, ignoring alpha channel.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_16s_AC4R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_16s_AC4R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_32f_C1R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_32f_C1R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point median filter scratch memory size.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_32f_C3R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_32f_C3R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_32f_C4R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_32f_C4R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point median filter, ignoring alpha channel.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local Median operation.
 * @param nBufferSize Pointer to the size of the scratch buffer required for the Median operation.
 * @return
 *  \ref image_data_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMedianGetBufferSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jobject oMaskSize, jobject nBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMedianGetBufferSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMedianGetBufferSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nBufferSize' is null for nppiFilterMedianGetBufferSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMedianGetBufferSize_32f_AC4R(oSizeROI=%p, oMaskSize=%p, nBufferSize=%p)\n",
        oSizeROI, oMaskSize, nBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    Npp32u* nBufferSize_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *nBufferSize_pointerData = initPointerData(env, nBufferSize);
    if (nBufferSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nBufferSize_native = (Npp32u*)nBufferSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiFilterMedianGetBufferSize_32f_AC4R(oSizeROI_native, oMaskSize_native, nBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    if (!releasePointerData(env, nBufferSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned horizontal Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned horizontal Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned horizontal Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed horizontal Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed horizontal Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed horizontal Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed horizontal Prewitt filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point horizontal Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point horizontal Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point horizontal Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point horizontal Prewitt filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittHoriz_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittHoriz_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittHoriz_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittHoriz_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittHoriz_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittHoriz_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned vertical Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned vertical Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned vertical Prewitt filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed vertical Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed vertical Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point vertical Prewitt filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point vertical Prewitt filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterPrewittVert_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterPrewittVert_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterPrewittVert_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterPrewittVert_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterPrewittVert_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterPrewittVert_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrHoriz_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrHoriz_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrHoriz_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrHoriz_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrHoriz_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrHoriz_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrHoriz_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrHoriz_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrHoriz_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrHoriz_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrHoriz_8s16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrHoriz_8s16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point horizontal Scharr filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrHoriz_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrHoriz_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrHoriz_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrHoriz_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrHoriz_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrHoriz_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed vertical Scharr filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrVert_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrVert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrVert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrVert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrVert_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrVert_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed vertical Scharr filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrVert_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrVert_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrVert_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrVert_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrVert_8s16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrVert_8s16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point vertical Scharr filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrVert_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrVert_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrVert_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrVert_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrVert_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrVert_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed horizontal Scharr filter kernel with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrHorizBorder_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrHorizBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterScharrHorizBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterScharrHorizBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrHorizBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrHorizBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrHorizBorder_8u16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrHorizBorder_8u16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrHorizBorder_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrHorizBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterScharrHorizBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterScharrHorizBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrHorizBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrHorizBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrHorizBorder_8s16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrHorizBorder_8s16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point horizontal Scharr filter kernel with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrHorizBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrHorizBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterScharrHorizBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterScharrHorizBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrHorizBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrHorizBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrHorizBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrHorizBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed vertical Scharr filter kernel with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrVertBorder_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrVertBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterScharrVertBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterScharrVertBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrVertBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrVertBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrVertBorder_8u16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrVertBorder_8u16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed vertical Scharr filter kernel with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrVertBorder_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrVertBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterScharrVertBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterScharrVertBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrVertBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrVertBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrVertBorder_8s16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrVertBorder_8s16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point vertical Scharr filter kernel with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterScharrVertBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterScharrVertBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterScharrVertBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterScharrVertBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterScharrVertBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterScharrVertBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterScharrVertBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterScharrVertBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed horizontal Sobel filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned horizontal Sobel filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point horizontal Sobel filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHoriz_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHoriz_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHoriz_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHoriz_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHoriz_8s16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHoriz_8s16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizMask_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizMask_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizMask_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizMask_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizMask_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizMask_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed vertical Sobel filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned vertical Sobel filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point vertical Sobel filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVert_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVert_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVert_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVert_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVert_8s16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVert_8s16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertMask_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertMask_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertMask_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertMask_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertMask_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertMask_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed second derivative, horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizSecond_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizSecond_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizSecond_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizSecond_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizSecond_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizSecond_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed second derivative, horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizSecond_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizSecond_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizSecond_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizSecond_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizSecond_8s16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizSecond_8s16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point second derivative, horizontal Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizSecond_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizSecond_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizSecond_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizSecond_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizSecond_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizSecond_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed second derivative, vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertSecond_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertSecond_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertSecond_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertSecond_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertSecond_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertSecond_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed second derivative, vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertSecond_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertSecond_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertSecond_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertSecond_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertSecond_8s16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertSecond_8s16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point second derivative, vertical Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertSecond_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertSecond_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertSecond_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertSecond_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertSecond_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertSecond_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed second cross derivative Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelCross_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelCross_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelCross_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelCross_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelCross_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelCross_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed second cross derivative Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelCross_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelCross_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelCross_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelCross_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelCross_8s16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelCross_8s16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point second cross derivative Sobel filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelCross_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelCross_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelCross_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelCross_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelCross_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelCross_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned horizontal Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_8u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned horizontal Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_8u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned horizontal Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_8u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed horizontal Sobel filter with border control, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed horizontal Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed horizontal Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_16s_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_16s_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed horizontal Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_16s_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_16s_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned horizontal Sobel filter with border control, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_16s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_16s_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point horizontal Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_32f_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point horizontal Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_32f_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point horizontal Sobel filter with border control, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_32f_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_8u16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_8u16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed horizontal Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizBorder_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizBorder_8s16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizBorder_8s16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizMaskBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizMaskBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizMaskBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizMaskBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizMaskBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizMaskBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizMaskBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizMaskBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_8u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_8u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_8u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed vertical Sobel filter with border control, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_16s_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_16s_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_16s_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_16s_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned vertical Sobel filter with border control, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_16s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_16s_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_32f_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_32f_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point vertical Sobel filter with border control, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_32f_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_8u16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_8u16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertBorder_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertBorder_8s16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertBorder_8s16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertMaskBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertMaskBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertMaskBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertMaskBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertMaskBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertMaskBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertMaskBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertMaskBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed second derivative, horizontal Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizSecondBorder_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizSecondBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizSecondBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizSecondBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizSecondBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizSecondBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizSecondBorder_8u16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizSecondBorder_8u16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizSecondBorder_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizSecondBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizSecondBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizSecondBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizSecondBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizSecondBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizSecondBorder_8s16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizSecondBorder_8s16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point second derivative, horizontal Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelHorizSecondBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelHorizSecondBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelHorizSecondBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelHorizSecondBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelHorizSecondBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelHorizSecondBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelHorizSecondBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelHorizSecondBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed second derivative, vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertSecondBorder_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertSecondBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertSecondBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertSecondBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertSecondBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertSecondBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertSecondBorder_8u16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertSecondBorder_8u16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed second derivative, vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertSecondBorder_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertSecondBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertSecondBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertSecondBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertSecondBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertSecondBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertSecondBorder_8s16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertSecondBorder_8s16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point second derivative, vertical Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelVertSecondBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelVertSecondBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelVertSecondBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelVertSecondBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelVertSecondBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelVertSecondBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelVertSecondBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelVertSecondBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed second cross derivative Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelCrossBorder_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelCrossBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelCrossBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelCrossBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelCrossBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelCrossBorder_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelCrossBorder_8u16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelCrossBorder_8u16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed second cross derivative Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelCrossBorder_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelCrossBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelCrossBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelCrossBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelCrossBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelCrossBorder_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelCrossBorder_8s16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelCrossBorder_8s16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point second cross derivative Sobel filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSobelCrossBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSobelCrossBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterSobelCrossBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterSobelCrossBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSobelCrossBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSobelCrossBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSobelCrossBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterSobelCrossBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * @name FilterRobertsDown
 *
 * Filters the image using a horizontal Roberts filter kernel:
 *
 * \f[
 *  \left( \begin{array}{rrr}
 *   0 & 0 &  0 \\
 *   0 & 1 &  0 \\
 *   0 & 0 & -1 \\
 *  \end{array} \right)
 * \f]
 *
 * @{
 *
 * </pre>
 */
/**
 * <pre>
 * Single channel 8-bit unsigned horizontal Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned horizontal Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned horizontal Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned horizontal Roberts filter, ignoring alpha-channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed horizontal Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed horizontal Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed horizontal Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed horizontal Roberts filter, ignoring alpha-channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point horizontal Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point horizontal Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point horizontal Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point horizontal Roberts filter, ignoring alpha-channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsDown_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsDown_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsDown_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsDown_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsDown_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsDown_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned vertical Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned vertical Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned vertical Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned vertical Roberts filter, ignoring alpha-channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed vertical Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed vertical Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed vertical Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed vertical Roberts filter, ignoring alpha-channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point vertical Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point vertical Roberts filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point vertical Roberts filter, ignoring alpha-channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRobertsUp_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRobertsUp_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRobertsUp_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterRobertsUp_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRobertsUp_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterRobertsUp_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned Laplace filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned Laplace filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned Laplace filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned Laplace filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed Laplace filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed Laplace filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed Laplace filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed Laplace filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point Laplace filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point Laplace filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point Laplace filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned to 16-bit signed Laplace filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit signed to 16-bit signed Laplace filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLaplace_18s16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLaplace_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLaplace_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLaplace_8s16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLaplace_8s16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLaplace_8s16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned Gauss filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned Gauss filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned Gauss filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned Gauss filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned Gauss filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned Gauss filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed Gauss filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed Gauss filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed Gauss filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed Gauss filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point Gauss filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point Gauss filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point Gauss filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point Gauss filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGauss_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGauss_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGauss_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGauss_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGauss_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterGauss_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned Gauss filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_8u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned Gauss filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_8u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_8u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned Gauss filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_8u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned Gauss filter with border control, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned Gauss filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_16u_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_16u_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned Gauss filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_16u_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_16u_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned Gauss filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_16u_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_16u_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned Gauss filter with border control, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_16u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_16u_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed Gauss filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_16s_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_16s_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed Gauss filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_16s_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_16s_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed Gauss filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_16s_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_16s_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_16s_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_16s_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point Gauss filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_32f_C1R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point Gauss filter with border control.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_32f_C3R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_32f_C4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point Gauss filter with border control, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Source image width and height in pixels relative to pSrc.
 * @param oSrcOffset Source image starting point relative to pSrc. 
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @param eBorderType The border type operation to be applied at source image border boundaries.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterGaussBorder_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSize, jobject oSrcOffset, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize, jint eBorderType)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterGaussBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiFilterGaussBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcOffset == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcOffset' is null for nppiFilterGaussBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterGaussBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterGaussBorder_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive
    // eBorderType is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterGaussBorder_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcSize=%p, oSrcOffset=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d, eBorderType=%d)\n",
        pSrc, nSrcStep, oSrcSize, oSrcOffset, pDst, nDstStep, oSizeROI, eMaskSize, eBorderType);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    NppiSize oSrcSize_native;
    NppiPoint oSrcOffset_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;
    NppiBorderType eBorderType_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcOffset, oSrcOffset_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;
    eBorderType_native = (NppiBorderType)eBorderType;

    // Native function call
    NppStatus jniResult_native = nppiFilterGaussBorder_32f_AC4R(pSrc_native, nSrcStep_native, oSrcSize_native, oSrcOffset_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native, eBorderType_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSize is a read-only structure
    // oSrcOffset is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive
    // eBorderType is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned high-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned high-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned high-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned high-pass filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned high-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned high-pass filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed high-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed high-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed high-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed high-pass filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point high-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point high-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point high-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point high-pass filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterHighPass_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterHighPass_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterHighPass_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterHighPass_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterHighPass_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterHighPass_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned low-pass filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned low-pass filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit signed low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed low-pass filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point low-pass filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point high-pass filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eMaskSize Enumeration value specifying the mask size.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterLowPass_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eMaskSize)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterLowPass_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterLowPass_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterLowPass_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eMaskSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterLowPass_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eMaskSize=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eMaskSize);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiMaskSize eMaskSize_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eMaskSize_native = (NppiMaskSize)eMaskSize;

    // Native function call
    NppStatus jniResult_native = nppiFilterLowPass_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eMaskSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eMaskSize is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 8-bit unsigned sharpening filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 8-bit unsigned sharpening filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned sharpening filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 8-bit unsigned sharpening filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 16-bit unsigned sharpening filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit unsigned sharpening filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned sharpening filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit unsigned sharpening filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 16-bit signed sharpening filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed sharpening filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 16-bit signed sharpening filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit floating-point sharpening filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit floating-point sharpening filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point sharpening filter.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit floating-point sharpening filter, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterSharpen_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterSharpen_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterSharpen_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterSharpen_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterSharpen_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterSharpen_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image sum.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppiSumGetBufferHostSize_8u_C1R to determine the minium number of bytes required.
 * @param pSum Pointer to the computed sum.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppiSum_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pSum_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp64f*)pSum_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiSum_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image sum. The result is 64-bit long long integer.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppiSumGetBufferHostSize_8u64s_C1R to determine the minium number of bytes required.
 * @param pSum Pointer to the computed sum.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_18u64s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_8u64s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_8u64s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_8u64s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppiSum_8u64s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_8u64s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64s* pSum_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp64s*)pSum_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiSum_8u64s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppiSum_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pSum_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp64f*)pSum_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiSum_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image sum.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppiSumGetBufferHostSize_16s_C1R to determine the minium number of bytes required.
 * @param pSum Pointer to the computed sum.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppiSum_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pSum_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp64f*)pSum_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiSum_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image sum.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiSumGetBufferHostSize_32f_C1R to determine the minium number of bytes required.
 * @param pSum Pointer to the computed sum.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppiSum_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pSum_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp64f*)pSum_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiSum_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image sum.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppiSumGetBufferHostSize_8u_C3R to determine the minium number of bytes required.
 * @param aSum Array that contains computed sum for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image sum.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppiSumGetBufferHostSize_16s_C3R to determine the minium number of bytes required.
 * @param aSum Array that contains computed sum for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image sum.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppiSumGetBufferHostSize_32f_C3R to determine the minium number of bytes required.
 * @param aSum Array that contains computed sum for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image sum ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppiSumGetBufferHostSize_16u_AC4R to determine the minium number of bytes required.
 * @param aSum Array that contains computed sum for each channel (alpha channel is not computed).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image sum ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppiSumGetBufferHostSize_16s_AC4R to determine the minium number of bytes required.
 * @param aSum Array that contains computed sum for each channel (alpha channel is not computed).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image sum.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiSumGetBufferHostSize_8u_C4R to determine the minium number of bytes required.
 * @param aSum Array that contains computed sum for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image sum. The result is 64-bit long long integer.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiSumGetBufferHostSize_8u64s_C4R to determine the minium number of bytes required.
 * @param aSum Array that contains computed sum for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_18u64s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jlongArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_8u64s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_8u64s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_8u64s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_8u64s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_8u64s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64s aSum_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_8u64s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image sum.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiSumGetBufferHostSize_16s_C4R to determine the minium number of bytes required.
 * @param aSum Array that contains computed sum for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image sum.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiSumGetBufferHostSize_32f_C4R to determine the minium number of bytes required.
 * @param aSum Array that contains computed sum for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_8u64s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_18u64s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_8u64s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_8u64s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_8u64s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_8u64s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_16s_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_8u64s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_18u64s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_8u64s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_8u64s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_8u64s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_8u64s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiSum_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSumGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSumGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSumGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image min. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_8u_C1R to determine the minium number of bytes required.
 * @param pMin Pointer to the computed minimum result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u* pMin_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp8u*)pMin_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMin_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image min. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_16u_C1R to determine the minium number of bytes required.
 * @param pMin Pointer to the computed minimum result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMin_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u* pMin_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16u*)pMin_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMin_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image min. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_16s_C1R to determine the minium number of bytes required.
 * @param pMin Pointer to the computed minimum result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMin_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s* pMin_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16s*)pMin_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMin_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMin_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f* pMin_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32f*)pMin_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMin_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image min. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_8u_C3R to determine the minium number of bytes required.
 * @param aMin Array that contains the computed minimum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMin_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image min. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_16u_C3R to determine the minium number of bytes required.
 * @param aMin Array that contains the computed minimum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMin_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image min. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_16s_C3R to determine the minium number of bytes required.
 * @param aMin Array that contains the computed minimum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMin_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMin_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image min. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_8u_C4R to determine the minium number of bytes required.
 * @param aMin Array that contains the computed minimum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMin_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image min. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_16u_C4R to determine the minium number of bytes required.
 * @param aMin Array that contains the computed minimum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMin_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMin_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image min. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_32f_C4R to determine the minium number of bytes required.
 * @param aMin Array that contains the computed minimum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMin_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image min ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_8u_AC4R to determine the minium number of bytes required.
 * @param aMin Array that contains the computed minimum results for each channel (alpha channel is not processed).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMin_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMin_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image min ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_16s_AC4R to determine the minium number of bytes required.
 * @param aMin Array that contains the computed minimum results for each channel (alpha channel is not processed).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMin_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image min ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinGetBufferHostSize_32f_AC4R to determine the minium number of bytes required.
 * @param aMin Array that contains the computed minimum results for each channel (alpha channel is not processed).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMin_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMin)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMin_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMin_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMin_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMin_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMin_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMin_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMin_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMin_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMin_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMin_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMin_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMin_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMin_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMin_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMin_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMin_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMin_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMin_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image MinIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinIndxGetBufferHostSize_8u_C1R to determine the minium number of bytes required.
 * @param pMin Pointer to the computed min result.
 * @param pIndexX Pointer to the X coordinate of the image min value.
 * @param pIndexY Ppointer to the Y coordinate of the image min value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMin, jintArray pIndexX, jintArray pIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMinIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexX' is null for nppiMinIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexY' is null for nppiMinIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMin=%p, pIndexX=%p, pIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin, pIndexX, pIndexY);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u* pMin_native = NULL;
    int* pIndexX_native;
    int* pIndexY_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp8u*)pMin_pointerData->getPointer(env);
    PointerData *pIndexX_pointerData = initPointerData(env, pIndexX);
    if (pIndexX_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexX_native = (int*)pIndexX_pointerData->getPointer(env);
    PointerData *pIndexY_pointerData = initPointerData(env, pIndexY);
    if (pIndexY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexY_native = (int*)pIndexY_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMin_native, pIndexX_native, pIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexX_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMin, jintArray pIndexX, jintArray pIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMinIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexX' is null for nppiMinIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexY' is null for nppiMinIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMin=%p, pIndexX=%p, pIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin, pIndexX, pIndexY);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u* pMin_native = NULL;
    int* pIndexX_native;
    int* pIndexY_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16u*)pMin_pointerData->getPointer(env);
    PointerData *pIndexX_pointerData = initPointerData(env, pIndexX);
    if (pIndexX_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexX_native = (int*)pIndexX_pointerData->getPointer(env);
    PointerData *pIndexY_pointerData = initPointerData(env, pIndexY);
    if (pIndexY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexY_native = (int*)pIndexY_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMin_native, pIndexX_native, pIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexX_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMin, jintArray pIndexX, jintArray pIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMinIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexX' is null for nppiMinIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexY' is null for nppiMinIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMin=%p, pIndexX=%p, pIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin, pIndexX, pIndexY);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s* pMin_native = NULL;
    int* pIndexX_native;
    int* pIndexY_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16s*)pMin_pointerData->getPointer(env);
    PointerData *pIndexX_pointerData = initPointerData(env, pIndexX);
    if (pIndexX_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexX_native = (int*)pIndexX_pointerData->getPointer(env);
    PointerData *pIndexY_pointerData = initPointerData(env, pIndexY);
    if (pIndexY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexY_native = (int*)pIndexY_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMin_native, pIndexX_native, pIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexX_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image MinIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinIndxGetBufferHostSize_32f_C1R to determine the minium number of bytes required.
 * @param pMin Pointer to the computed min result.
 * @param pIndexX Pointer to the X coordinate of the image min value.
 * @param pIndexY Pointer to the Y coordinate of the image min value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMin, jintArray pIndexX, jintArray pIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMinIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexX' is null for nppiMinIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexY' is null for nppiMinIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMin=%p, pIndexX=%p, pIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMin, pIndexX, pIndexY);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f* pMin_native = NULL;
    int* pIndexX_native;
    int* pIndexY_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32f*)pMin_pointerData->getPointer(env);
    PointerData *pIndexX_pointerData = initPointerData(env, pIndexX);
    if (pIndexX_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexX_native = (int*)pIndexX_pointerData->getPointer(env);
    PointerData *pIndexY_pointerData = initPointerData(env, pIndexY);
    if (pIndexY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexY_native = (int*)pIndexY_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMin_native, pIndexX_native, pIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexX_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image MinIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinIndxGetBufferHostSize_8u_C3R to determine the minium number of bytes required.
 * @param aMin Array that contains the min values.
 * @param aIndexX Array that contains the X coordinates of the image min values.
 * @param aIndexY Array that contains the Y coordinates of the image min values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMin_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMin_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image MinIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinIndxGetBufferHostSize_16s_C3R to determine the minium number of bytes required.
 * @param aMin Array that contains the min values.
 * @param aIndexX Array that contains the X coordinates of the image min values.
 * @param aIndexY Array that contains the Y coordinates of the image min values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMin_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMin_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMin_native[4];
    int aIndexX_native[4];
    int aIndexY_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMin_native[4];
    int aIndexX_native[4];
    int aIndexY_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image MinIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinIndxGetBufferHostSize_16s_C4R to determine the minium number of bytes required.
 * @param aMin Array that contains the min values.
 * @param aIndexX Array that contains the X coordinates of the image min values.
 * @param aIndexY Array that contains the Y coordinates of the image min values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMin_native[4];
    int aIndexX_native[4];
    int aIndexY_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMin_native[4];
    int aIndexX_native[4];
    int aIndexY_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMin_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image MinIndx ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinIndxGetBufferHostSize_16u_AC4R to determine the minium number of bytes required.
 * @param aMin Array that contains the min values.
 * @param aIndexX Array that contains the X coordinates of the image min values.
 * @param aIndexY Array that contains the Y coordinates of the image min values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMin_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image MinIndx ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinIndxGetBufferHostSize_16s_AC4R to determine the minium number of bytes required.
 * @param aMin Array that contains the min values.
 * @param aIndexX Array that contains the X coordinates of the image min values.
 * @param aIndexY Array that contains the Y coordinates of the image min values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMin_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image MinIndx ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMinIndxGetBufferHostSize_32f_AC4R to determine the minium number of bytes required.
 * @param aMin Array that contains the min values.
 * @param aIndexX Array that contains the X coordinates of the image min values.
 * @param aIndexY Array that contains the Y coordinates of the image min values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndx_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMin, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMinIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMinIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndx_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMin=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMin, aIndexX, aIndexY);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMin_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinIndx_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMin_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinIndxGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinIndxGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinIndxGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinIndxGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinIndxGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image Max.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_8u_C1R to determaxe the maxium number of bytes required.
 * @param pMax Pointer to the computed maximum result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u* pMax_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp8u*)pMax_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMax_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image Max.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_16u_C1R to determaxe the maxium number of bytes required.
 * @param pMax Pointer to the computed maximum result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u* pMax_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16u*)pMax_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMax_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image Max.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_16s_C1R to determaxe the maxium number of bytes required.
 * @param pMax Pointer to the computed maximum result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s* pMax_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16s*)pMax_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMax_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image Max.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_32f_C1R to determaxe the maxium number of bytes required.
 * @param pMax Pointer to the computed maximum result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f* pMax_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32f*)pMax_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMax_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image Max.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_8u_C3R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the computed maximum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMax_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image Max.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_16u_C3R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the computed maximum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMax_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image Max.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_16s_C3R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the computed maximum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMax_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image Max. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_32f_C3R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the computed maximum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMax_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image Max. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_8u_C4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the computed maximum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMax_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image Max. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_16u_C4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the computed maximum results for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMax_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMax_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMax_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image Max ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_8u_AC4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the computed maximum results for each channel (alpha channel is not processed).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMax_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image Max ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_16u_AC4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the computed maximum results for each channel (alpha channel is not processed).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMax_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image Max ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_16s_AC4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the computed maximum results for each channel (alpha channel is not processed).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMax_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image Max ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxGetBufferHostSize_32f_AC4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the computed maximum results for each channel (alpha channel is not processed).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMax_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMax)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMax_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMax_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMax_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMax_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image MaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_8u_C1R to determaxe the maxium number of bytes required.
 * @param pMax Pointer to the computed max result.
 * @param pIndexX Pointer to the X coordinate of the image max value.
 * @param pIndexY Ppointer to the Y coordinate of the image max value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMax, jintArray pIndexX, jintArray pIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexX' is null for nppiMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexY' is null for nppiMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMax=%p, pIndexX=%p, pIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax, pIndexX, pIndexY);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u* pMax_native = NULL;
    int* pIndexX_native;
    int* pIndexY_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp8u*)pMax_pointerData->getPointer(env);
    PointerData *pIndexX_pointerData = initPointerData(env, pIndexX);
    if (pIndexX_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexX_native = (int*)pIndexX_pointerData->getPointer(env);
    PointerData *pIndexY_pointerData = initPointerData(env, pIndexY);
    if (pIndexY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexY_native = (int*)pIndexY_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMax_native, pIndexX_native, pIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexX_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image MaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_16u_C1R to determaxe the maxium number of bytes required.
 * @param pMax Pointer to the computed max result.
 * @param pIndexX Pointer to the X coordinate of the image max value.
 * @param pIndexY Pointer to the Y coordinate of the image max value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMax, jintArray pIndexX, jintArray pIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexX' is null for nppiMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexY' is null for nppiMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMax=%p, pIndexX=%p, pIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax, pIndexX, pIndexY);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u* pMax_native = NULL;
    int* pIndexX_native;
    int* pIndexY_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16u*)pMax_pointerData->getPointer(env);
    PointerData *pIndexX_pointerData = initPointerData(env, pIndexX);
    if (pIndexX_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexX_native = (int*)pIndexX_pointerData->getPointer(env);
    PointerData *pIndexY_pointerData = initPointerData(env, pIndexY);
    if (pIndexY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexY_native = (int*)pIndexY_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMax_native, pIndexX_native, pIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexX_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image MaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_16s_C1R to determaxe the maxium number of bytes required.
 * @param pMax Pointer to the computed max result.
 * @param pIndexX Pointer to the X coordinate of the image max value.
 * @param pIndexY Pointer to the Y coordinate of the image max value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMax, jintArray pIndexX, jintArray pIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMaxIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexX' is null for nppiMaxIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexY' is null for nppiMaxIndx_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMax=%p, pIndexX=%p, pIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax, pIndexX, pIndexY);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s* pMax_native = NULL;
    int* pIndexX_native;
    int* pIndexY_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16s*)pMax_pointerData->getPointer(env);
    PointerData *pIndexX_pointerData = initPointerData(env, pIndexX);
    if (pIndexX_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexX_native = (int*)pIndexX_pointerData->getPointer(env);
    PointerData *pIndexY_pointerData = initPointerData(env, pIndexY);
    if (pIndexY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexY_native = (int*)pIndexY_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMax_native, pIndexX_native, pIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexX_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMax, jintArray pIndexX, jintArray pIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexX' is null for nppiMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndexY' is null for nppiMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMax=%p, pIndexX=%p, pIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMax, pIndexX, pIndexY);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f* pMax_native = NULL;
    int* pIndexX_native;
    int* pIndexY_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32f*)pMax_pointerData->getPointer(env);
    PointerData *pIndexX_pointerData = initPointerData(env, pIndexX);
    if (pIndexX_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexX_native = (int*)pIndexX_pointerData->getPointer(env);
    PointerData *pIndexY_pointerData = initPointerData(env, pIndexY);
    if (pIndexY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndexY_native = (int*)pIndexY_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMax_native, pIndexX_native, pIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexX_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndexY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image MaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_8u_C3R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the max values.
 * @param aIndexX Array that contains the X coordinates of the image max values.
 * @param aIndexY Array that contains the Y coordinates of the image max values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMax_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image MaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_16u_C3R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the max values.
 * @param aIndexX Array that contains the X coordinates of the image max values.
 * @param aIndexY Array that contains the Y coordinates of the image max values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMax_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMax_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image MaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_32f_C3R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the max values.
 * @param aIndexX Array that contains the X coordinates of the image max values.
 * @param aIndexY Array that contains the Y coordinates of the image max values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMax_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image MaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_8u_C4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the max values.
 * @param aIndexX Array that contains the X coordinates of the image max values.
 * @param aIndexY Array that contains the Y coordinates of the image max values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMax_native[4];
    int aIndexX_native[4];
    int aIndexY_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image MaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_16u_C4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the max values.
 * @param aIndexX Array that contains the X coordinates of the image max values.
 * @param aIndexY Array that contains the Y coordinates of the image max values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMax_native[4];
    int aIndexX_native[4];
    int aIndexY_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image MaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_16s_C4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the max values.
 * @param aIndexX Array that contains the X coordinates of the image max values.
 * @param aIndexY Array that contains the Y coordinates of the image max values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMax_native[4];
    int aIndexX_native[4];
    int aIndexY_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image MaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_32f_C4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the max values.
 * @param aIndexX Array that contains the X coordinates of the image max values.
 * @param aIndexY Array that contains the Y coordinates of the image max values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMax_native[4];
    int aIndexX_native[4];
    int aIndexY_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image MaxIndx ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_8u_AC4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the max values.
 * @param aIndexX Array that contains the X coordinates of the image max values.
 * @param aIndexY Array that contains the Y coordinates of the image max values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jbyteArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp8u aMax_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image MaxIndx ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_16u_AC4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the max values.
 * @param aIndexX Array that contains the X coordinates of the image max values.
 * @param aIndexY Array that contains the Y coordinates of the image max values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16u aMax_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image MaxIndx ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMaxIndxGetBufferHostSize_16s_AC4R to determaxe the maxium number of bytes required.
 * @param aMax Array that contains the max values.
 * @param aIndexX Array that contains the X coordinates of the image max values.
 * @param aIndexY Array that contains the Y coordinates of the image max values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jshortArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp16s aMax_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndx_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jfloatArray aMax, jintArray aIndexX, jintArray aIndexY)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaxIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMaxIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexX == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexX' is null for nppiMaxIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aIndexY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aIndexY' is null for nppiMaxIndx_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndx_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMax=%p, aIndexX=%p, aIndexY=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMax, aIndexX, aIndexY);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp32f aMax_native[3];
    int aIndexX_native[3];
    int aIndexY_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexX, aIndexX_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aIndexY, aIndexY_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxIndx_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMax_native, aIndexX_native, aIndexY_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexX_native, aIndexX, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aIndexY_native, aIndexY, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxIndxGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxIndxGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaxIndxGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxIndxGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaxIndxGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image MinMax.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMin Pointer to the computed minimal result.
 * @param pMax Pointer to the computed maximal result.
 * @param pDeviceBuffer Buffer to a scratch memory. 
 *        Use \ref nppiMinMaxGetBufferHostSize_8u_C1R to determine the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMinMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMinMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMin_native = NULL;
    Npp8u* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp8u*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp8u*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image MinMax.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMin Pointer to the computed minimal result.
 * @param pMax Pointer to the computed maximal result.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16u_C1R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMinMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMinMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u* pMin_native = NULL;
    Npp16u* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16u*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16u*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image MinMax.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMin Pointer to the computed minimal result.
 * @param pMax Pointer to the computed maximal result.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16s_C1R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMinMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMinMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s* pMin_native = NULL;
    Npp16s* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16s*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16s*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image MinMax.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMin Pointer to the computed minimal result.
 * @param pMax Pointer to the computed maximal result.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_32f_C1R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMinMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMinMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pMin_native = NULL;
    Npp32f* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32f*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32f*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jbyteArray aMin, jbyteArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u aMin_native[3];
    Npp8u aMax_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image MinMax.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aMin Array that contains the minima.
 * @param aMax Array that contains the maxima.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16u_C3R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jshortArray aMin, jshortArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u aMin_native[3];
    Npp16u aMax_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image MinMax.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aMin Array that contains the minima.
 * @param aMax Array that contains the maxima.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16s_C3R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jshortArray aMin, jshortArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s aMin_native[3];
    Npp16s aMax_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image MinMax.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aMin Array that contains the minima.
 * @param aMax Array that contains the maxima.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_32f_C3R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jfloatArray aMin, jfloatArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aMin_native[3];
    Npp32f aMax_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image MinMax ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aMin Array that contains the minima.
 * @param aMax Array that contains the maxima.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_8u_AC4R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jbyteArray aMin, jbyteArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u aMin_native[3];
    Npp8u aMax_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image MinMax ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aMin Array that contains the minima.
 * @param aMax Array that contains the maxima.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16u_AC4R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jshortArray aMin, jshortArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u aMin_native[3];
    Npp16u aMax_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image MinMax ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aMin Array that contains the minima.
 * @param aMax Array that contains the maxima.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16s_AC4R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jshortArray aMin, jshortArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s aMin_native[3];
    Npp16s aMax_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image MinMax ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aMin Array that contains the minima.
 * @param aMax Array that contains the maxima.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_32f_AC4R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jfloatArray aMin, jfloatArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aMin_native[3];
    Npp32f aMax_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image MinMax.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aMin Array that contains the minima.
 * @param aMax Array that contains the maxima.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_8u_C4R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jbyteArray aMin, jbyteArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u aMin_native[4];
    Npp8u aMax_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image MinMax.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aMin Array that contains the minima.
 * @param aMax Array that contains the maxima.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16u_C4R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jshortArray aMin, jshortArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u aMin_native[4];
    Npp16u aMax_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image MinMax.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aMin Array that contains the minima.
 * @param aMax Array that contains the maxima.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferHostSize_16s_C4R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jshortArray aMin, jshortArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16s aMin_native[4];
    Npp16s aMax_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jfloatArray aMin, jfloatArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f aMin_native[4];
    Npp32f aMax_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMax_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the minimal and the maximal pixel values with their X and Y coordinates of 1-channel 8-bit unsigned char image. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8u_C1R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes. 
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMinValue_native = NULL;
    Npp8u* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp8u*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp8u*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the minimal and the maximal pixel values with their X and Y coordinates of 1-channel 8-bit signed char image. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8s_C1R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes. 
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_8s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8s* pMinValue_native = NULL;
    Npp8s* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp8s*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp8s*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_8s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the minimal and the maximal pixel values with their X and Y coordinates of 1-channel 16-bit unsigned short image. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_16u_C1R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes. 
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u* pMinValue_native = NULL;
    Npp16u* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp16u*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp16u*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pMinValue_native = NULL;
    Npp32f* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp32f*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp32f*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMinMaxIndx_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_8u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMinValue_native = NULL;
    Npp8u* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp8u*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp8u*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_8u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMinMaxIndx_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_8s_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8s* pMinValue_native = NULL;
    Npp8s* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp8s*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp8s*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_8s_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 16-bit unsigned image MinMaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_16u_C1MR to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes. 
 * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
 * pMinValue = 0, pMaxValue = 0.
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMinMaxIndx_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_16u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp16u* pMinValue_native = NULL;
    Npp16u* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp16u*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp16u*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_16u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 32-bit floating point image MinMaxIndx.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_32f_C1MR to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point image is specified.
 * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
 * pMinValue = 0, pMaxValue = 0.
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMinMaxIndx_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_32f_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f* pMinValue_native = NULL;
    Npp32f* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp32f*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp32f*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_32f_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image MinMaxIndx affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8u_C3CR to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_18u_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jint nCOI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_8u_C3CR(pSrc=%p, nSrcStep=%d, oSizeROI=%p, nCOI=%d, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pMinValue_native = NULL;
    Npp8u* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp8u*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp8u*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_8u_C3CR(pSrc_native, nSrcStep_native, oSizeROI_native, nCOI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image MinMaxIndx affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8s_C3CR to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_18s_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jint nCOI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_8s_C3CR(pSrc=%p, nSrcStep=%d, oSizeROI=%p, nCOI=%d, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8s* pMinValue_native = NULL;
    Npp8s* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp8s*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp8s*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_8s_C3CR(pSrc_native, nSrcStep_native, oSizeROI_native, nCOI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image MinMaxIndx affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_16u_C3CR to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_116u_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jint nCOI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_16u_C3CR(pSrc=%p, nSrcStep=%d, oSizeROI=%p, nCOI=%d, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp16u* pMinValue_native = NULL;
    Npp16u* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp16u*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp16u*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_16u_C3CR(pSrc_native, nSrcStep_native, oSizeROI_native, nCOI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_132f_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jint nCOI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_32f_C3CR(pSrc=%p, nSrcStep=%d, oSizeROI=%p, nCOI=%d, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp32f* pMinValue_native = NULL;
    Npp32f* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp32f*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp32f*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_32f_C3CR(pSrc_native, nSrcStep_native, oSizeROI_native, nCOI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image MinMaxIndx affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8u_C3CMR to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
 * pMinValue = 0, pMaxValue = 0.
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMinMaxIndx_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_8u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pMinValue_native = NULL;
    Npp8u* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp8u*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp8u*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_8u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image MinMaxIndx affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_8s_C3CMR to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
 * pMinValue = 0, pMaxValue = 0.
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMinMaxIndx_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_8s_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8s* pMinValue_native = NULL;
    Npp8s* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp8s*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp8s*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_8s_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 16-bit unsigned image MinMaxIndx affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_16u_C3CMR to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
 * pMinValue = 0, pMaxValue = 0.
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMinMaxIndx_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_16u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp16u* pMinValue_native = NULL;
    Npp16u* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp16u*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp16u*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_16u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 32-bit floating point image MinMaxIndx affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pMinValue Pointer to the minimum value.
 * @param pMaxValue Pointer to the maximum value.
 * @param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * @param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxIndxGetBufferHostSize_32f_C3CMR to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point image is specified, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
 * pMinValue = 0, pMaxValue = 0.
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndx_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pMinValue, jobject pMaxValue, jobjectArray pMinIndex, jobjectArray pMaxIndex, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMaxIndx_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMinMaxIndx_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndx_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pMinValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinValue' is null for nppiMinMaxIndx_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxValue' is null for nppiMinMaxIndx_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndex' is null for nppiMinMaxIndx_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndex == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndex' is null for nppiMinMaxIndx_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMaxIndx_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndx_32f_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pMinValue=%p, pMaxValue=%p, pMinIndex=%p, pMaxIndex=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pMinValue, pMaxValue, pMinIndex, pMaxIndex, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp32f* pMinValue_native = NULL;
    Npp32f* pMaxValue_native = NULL;
    NppiPoint* pMinIndex_native;
    NppiPoint* pMaxIndex_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pMinValue_pointerData = initPointerData(env, pMinValue);
    if (pMinValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinValue_native = (Npp32f*)pMinValue_pointerData->getPointer(env);
    PointerData *pMaxValue_pointerData = initPointerData(env, pMaxValue);
    if (pMaxValue_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxValue_native = (Npp32f*)pMaxValue_pointerData->getPointer(env);
    PointerData *pMinIndex_pointerData = initPointerData(env, pMinIndex);
    if (pMinIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndex_native = (NppiPoint*)pMinIndex_pointerData->getPointer(env);
    PointerData *pMaxIndex_pointerData = initPointerData(env, pMaxIndex);
    if (pMaxIndex_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndex_native = (NppiPoint*)pMaxIndex_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndx_32f_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pMinValue_native, pMaxValue_native, pMinIndex_native, pMaxIndex_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pMinValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxValue_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndex_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_8s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_18s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_8s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_8s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_8s_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_16u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_32f_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_8u_C3CR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_18u_1C3CRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_8u_C3CR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_8u_C3CR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_8s_C3CR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_18s_1C3CRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_8s_C3CR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_8s_C3CR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_116u_1C3CRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_16u_C3CR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_16u_C3CR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_32f_C3CR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_132f_1C3CRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_32f_C3CR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_32f_C3CR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_8u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_8s_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMinMaxIndx_32f_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxIndxGetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxIndxGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxIndxGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxIndxGetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxIndxGetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_8u_C1R to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_16u_C1R to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_16s_C1R to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_32f_C1R to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid 
 * floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_8u_C3R to determine the minium number of bytes required.
 * @param aMean Array that contains the computed mean results.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_16u_C3R to determine the minium number of bytes required.
 * @param aMean Array that contains the computed mean results.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_16s_C3R to determine the minium number of bytes required.
 * @param aMean Array that contains the computed mean results.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_32f_C3R to determine the minium number of bytes required.
 * @param aMean Array that contains the computed mean results.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid 
 * floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_8u_C4R to determine the minium number of bytes required.
 * @param aMean Array that contains the computed mean results.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_16u_C4R to determine the minium number of bytes required.
 * @param aMean Array that contains the computed mean results.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_16s_C4R to determine the minium number of bytes required.
 * @param aMean Array that contains the computed mean results.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image Mean ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_8u_AC4R to determine the minium number of bytes required.
 * @param aMean Array that contains the computed mean results.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image Mean ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_16s_AC4R to determine the minium number of bytes required.
 * @param aMean Array that contains the computed mean results.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image Mean ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_32f_AC4R to determine the minium number of bytes required.
 * @param aMean Array that contains the computed mean results.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid 
 * floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMean' is null for nppiMean_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aMean=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aMean);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aMean_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aMean, aMean_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMean_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMean_native, aMean, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit unsigned image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_8u_C1MR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_8u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_8u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit signed image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_8s_C1MR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_8s_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_8s_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 16-bit unsigned image Mean.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_16u_C1MR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_16u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_16u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_32f_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_32f_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image Mean affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_8u_C3CMR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_8u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_8u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image Mean affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanGetBufferHostSize_8s_C3CMR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean result.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_8s_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_8s_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_16u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_16u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_32f_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_32f_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_16s_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_8u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_8s_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_16u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_32f_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_8u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_8s_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_16u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanGetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanGetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanGetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image Mean_StdDev.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_8s_C1R to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_8s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_8s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit unsigned image Mean_StdDev.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_8u_C1MR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_StdDev_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_8u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_8u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit signed image Mean_StdDev.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_8s_C1MR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_StdDev_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_8s_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_8s_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 16-bit unsigned image Mean_StdDev.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_16u_C1MR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_StdDev_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_16u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_16u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_StdDev_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_32f_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_32f_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image Mean_StdDev affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_8u_C3CR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_18u_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_8u_C3CR(pSrc=%p, nSrcStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_8u_C3CR(pSrc_native, nSrcStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image Mean_StdDev affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_8s_C3CR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_18s_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_8s_C3CR(pSrc=%p, nSrcStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_8s_C3CR(pSrc_native, nSrcStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image Mean_StdDev affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_16u_C3CR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_116u_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_16u_C3CR(pSrc=%p, nSrcStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_16u_C3CR(pSrc_native, nSrcStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image Mean_StdDev affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_32f_C3CR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid 
 * floating-point image is specified, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_132f_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_32f_C3CR(pSrc=%p, nSrcStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_32f_C3CR(pSrc_native, nSrcStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image Mean_StdDev.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_8u_C3CMR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_StdDev_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_8u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_8u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image Mean_StdDev.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_8s_C3CMR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_StdDev_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_8s_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_8s_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 16-bit unsigned image Mean_StdDev.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_16u_C3CMR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_StdDev_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_16u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_16u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 32-bit floating point image Mean_StdDev.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use \ref nppiMeanStdDevGetBufferHostSize_32f_C3CMR to determine the minium number of bytes required.
 * @param pMean Pointer to the computed mean.
 * @param pStdDev Pointer to the computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an invalid 
 * floating-point image is specified, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pDeviceBuffer, jobject pMean, jobject pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiMean_StdDev_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_32f_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_32f_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_StdDev_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_18s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_8s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_8s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_StdDev_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_StdDev_8u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_StdDev_8s_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_StdDev_32f_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_StdDev_8u_C3CR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_18u_1C3CRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_8u_C3CR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_8u_C3CR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_18s_1C3CRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_8s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_8s_C3CR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_8s_C3CR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_StdDev_16u_C3CR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_116u_1C3CRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_16u_C3CR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_16u_C3CR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_StdDev_32f_C3CR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_132f_1C3CRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_32f_C3CR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_32f_C3CR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_StdDev_8u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_StdDev_8s_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMean_StdDev_32f_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDevGetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDevGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDevGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDevGetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDevGetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_32s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_32s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_32s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image Norm_Inf ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_8u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image Norm_Inf ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_16u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit unsigned image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_8u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_8u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_8u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit signed image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_8s_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_8s_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_8s_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 16-bit unsigned image Norm_Inf.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_16u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_16u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_16u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_32f_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_32f_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image Norm_Inf affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_8u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_8u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image Norm_Inf affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_8s_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_8s_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 16-bit unsigned image Norm_Inf affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormInfGetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_16u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_16u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1Inf_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_Inf_32f_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_Inf_32f_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_32s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_132s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_32s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_32s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_8s_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_16u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_32f_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_8u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_8s_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_Inf_32f_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormInfGetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormInfGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormInfGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormInfGetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormInfGetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image Norm_L1 ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_8u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image Norm_L1 ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image Norm_L1 ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image Norm_L1 ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit unsigned image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_8u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_8u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit signed image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_8s_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_8s_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 16-bit unsigned image Norm_L1.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_16u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_16u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_32f_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_32f_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image Norm_L1 affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_8u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_8u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image Norm_L1 affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_8s_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_8s_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 16-bit unsigned image Norm_L1 affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_16u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_16u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 32-bit floating point image Norm_L1 affecting only single channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL1GetBufferHostSize_32f_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if the step of 
 * the source image cannot be divided by 4, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L1_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L1_32f_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L1_32f_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_16u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_32f_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_16s_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_8s_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L1_16u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL1GetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL1GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL1GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL1GetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL1GetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image Norm_L2 ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image Norm_L2 ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image Norm_L2 ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Three-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNorm Array that contains the norm values of Four-channels.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppiNormL2GetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jdoubleArray aNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNorm' is null for nppiNorm_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNorm_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNorm, aNorm_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNorm_native, aNorm, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit unsigned image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_8u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_8u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit signed image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_8s_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_8s_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 16-bit unsigned image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_16u_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_16u_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 32-bit floating point image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormL2GetBufferHostSize_32f_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if the step 
 * of the source image cannot be divided by 4. 
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_32f_C1MR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_32f_C1MR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppiNormL2GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_8u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_8u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image Norm_L2.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNorm Pointer to the norm value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppiNormL2GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_8s_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_8s_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_16u_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_16u_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNorm_1L2_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNorm_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNorm_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNorm_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppiNorm_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNorm_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNorm_L2_32f_C3CMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, oSizeROI, nCOI, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNorm_L2_32f_C3CMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_8s_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_16u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_8u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_16u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNorm_L2_32f_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormL2GetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormL2GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormL2GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormL2GetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormL2GetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image NormDiff_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image NormDiff_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_16s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_16s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image NormDiff_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image NormDiff_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image NormDiff_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_16s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_16s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image NormDiff_Inf ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_8u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormDiff_Inf ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_16u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image NormDiff_Inf ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_16s_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_16s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_16s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormDiff_Inf ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image NormDiff_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormDiff_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_16s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_16s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormDiff_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit unsigned images NormDiff_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_8u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_8u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_8u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_8s_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_8s_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 16-bit unsigned images NormDiff_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_16u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_16u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_16u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 32-bit floating point images NormDiff_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_32f_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_32f_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_32f_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image NormDiff_Inf affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_8u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_8u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image NormDiff_Inf affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_8s_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_8s_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 16-bit unsigned image NormDiff_Inf affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_16u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_16u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 32-bit floating point image NormDiff_Inf affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffInfGetBufferHostSize_32f_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel 
 * of interest is specified, or NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_32f_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_32f_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_8u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_8s_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_32f_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** host pointer */
/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_16s_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_8u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_8s_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_16u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiNormDiff_Inf_32f_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffInfGetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffInfGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffInfGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffInfGetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffInfGetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_16s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_16s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_16s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_16s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image NormDiff_L1 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_8u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormDiff_L1 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image NormDiff_L1 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_16s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_16s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormDiff_L1 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_16s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_16s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit unsigned image NormDiff_L1.
 * 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_8u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_8u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit signed image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_8s_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_8s_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 16-bit unsigned image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_16u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_16u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 32-bit floating point image NormDiff_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_32f_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_32f_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_32f_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image NormDiff_L1 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_8u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_8u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image NormDiff_L1 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_8s_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_8s_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 16-bit unsigned image NormDiff_L1 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_16u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_16u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 32-bit floating point image NormDiff_L1 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL1GetBufferHostSize_32f_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel 
 * of interest is specified, or NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_32f_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_32f_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @name NormDiffL1GetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_L1 primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8s_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** host pointer */
/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16s_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_8s_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_16u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1_32f_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL1GetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL1GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL1GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL1GetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL1GetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_16s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_16s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_16s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_16s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image NormDiff_L2 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_8u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormDiff_L2 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image NormDiff_L2 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_16s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_16s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormDiff_L2 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_16s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_16s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormDiff Array that contains computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormDiff' is null for nppiNormDiff_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormDiff_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormDiff, aNormDiff_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormDiff_native, aNormDiff, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit unsigned image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_8u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_8u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit signed image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_8s_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_8s_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 16-bit unsigned image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_16u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_16u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 32-bit floating point image NormDiff_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_32f_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_32f_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_32f_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image NormDiff_L2 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_8u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_8u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image NormDiff_L2 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_8s_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_8s_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 16-bit unsigned image NormDiff_L2 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_16u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_16u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 32-bit floating point image NormDiff_L2 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormDiff Pointer to the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormDiffL2GetBufferHostSize_32f_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel 
 * of interest is specified, or NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormDiff, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormDiff_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormDiff == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormDiff' is null for nppiNormDiff_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormDiff_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_32f_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormDiff=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormDiff, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormDiff_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormDiff_pointerData = initPointerData(env, pNormDiff);
    if (pNormDiff_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormDiff_native = (Npp64f*)pNormDiff_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_32f_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormDiff_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormDiff_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8s_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_32f_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_8s_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_16u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2_32f_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiffL2GetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiffL2GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormDiffL2GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiffL2GetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormDiffL2GetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_16s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_16s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_16s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_16s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image NormRel_Inf ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_8u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormRel_Inf ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_16u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image NormRel_Inf ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_16s_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_16s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_16s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormRel_Inf ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_16s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_16s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit unsigned image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_8u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_8u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_8u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit signed image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_8s_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_8s_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_8s_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 16-bit unsigned image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_16u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_16u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_16u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 32-bit floating point image NormRel_Inf.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_32f_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_32f_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_32f_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image NormRel_Inf affecting only signle channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormRel Pointer to the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_COI_ERROR if an 
 * invalid channel of interest is specified, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_8u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_8u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image NormRel_Inf affecting only signle channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormRel Pointer to the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_COI_ERROR if an invalid 
 * channel of interest is specified, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_8s_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_8s_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 16-bit unsigned image NormRel_Inf affecting only signle channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormRel Pointer to the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_COI_ERROR if an invalid 
 * channel of interest is specified, or NPP_DIVISOR_ERROR if the infinity
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_16u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_16u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 32-bit floating point image NormRel_Inf affecting only signle channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormRel Pointer to the computed relative error for the infinity norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelInfGetBufferHostSize_32f_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_COI_ERROR if an invalid channel 
 * of interest is specified, or NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point 
 * image is specified, or NPP_DIVISOR_ERROR if the infinity norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1Inf_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_Inf_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_Inf_32f_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_Inf_32f_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_132s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_32s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // XXX This function is missing in NPP 6.5 build 14
    NppStatus jniResult_native = (NppStatus)JNPP_STATUS_INTERNAL_ERROR;

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8s_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32f_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16s_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_8u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_16u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf_32f_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelInfGetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelInfGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelInfGetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelInfGetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelInfGetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_16s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_16s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_16s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_16s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image NormRel_L1 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_8u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormRel_L1 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image NormRel_L1 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_16s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_16s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormRel_L1 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_16s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_16s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_8u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_8u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_8s_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_8s_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_16u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_16u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image NormRel_L1.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_32f_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L1
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_32f_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_32f_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image NormRel_L1 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormRel Pointer to the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid 
 * channel of interest is specified, or NPP_DIVISOR_ERROR if the L1 norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_8u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_8u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image NormRel_L1 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormRel Pointer to the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid 
 * channel of interest is specified, or NPP_DIVISOR_ERROR if the L1 norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_8s_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_8s_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 16-bit unsigned image NormRel_L1 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormRel Pointer to the computed relative error for the L1 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL1GetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid 
 * channel of interest is specified, or NPP_DIVISOR_ERROR if the L1 norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_16u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_16u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L1_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L1_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L1_32f_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L1_32f_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8s_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** host pointer */
/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** host pointer */
/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16s_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_8s_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_16u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1_32f_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL1GetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL1GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL1GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL1GetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL1GetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_16s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_16s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_16s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_16s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image NormRel_L2 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_8u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormRel_L2 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_16u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image NormRel_L2 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_16s_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_16s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_16s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormRel_L2 ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_16s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_16s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aNormRel Array that contains the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aNormRel' is null for nppiNormRel_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aNormRel_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aNormRel, aNormRel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aNormRel_native, aNormRel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit unsigned image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_8u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_8u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_8u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 8-bit signed image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_8s_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_8s_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_8s_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 16-bit unsigned image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_16u_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_16u_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_16u_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked one-channel 32-bit floating point image NormRel_L2.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pNormRel Pointer to the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_32f_C1MR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_NOT_EVEN_STEP_ERROR if 
 * an invalid floating-point image is specified, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_32f_C1MR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_32f_C1MR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit unsigned image NormRel_L2 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormRel Pointer to the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_8u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid 
 * channel of interest is specified, or NPP_DIVISOR_ERROR if the L2 norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_8u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_8u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 8-bit signed image NormRel_L2 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormRel Pointer to the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_8s_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid 
 * channel of interest is specified, or NPP_DIVISOR_ERROR if the L2 norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_8s_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_8s_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked three-channel 16-bit unsigned image NormRel_L2 affecting only single channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nCOI \ref channel_of_interest_number.
 * @param pNormRel Pointer to the computed relative error for the L2 norm of two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiNormRelL2GetBufferHostSize_16u_C3CMR to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, NPP_COI_ERROR if an 
 * invalid channel of interest is specified, or NPP_DIVISOR_ERROR if the L2
 * norm of the second image is zero.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_16u_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_16u_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRel_1L2_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject oSizeROI, jint nCOI, jobject pNormRel, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormRel_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormRel_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiNormRel_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRel_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nCOI is primitive
    if (pNormRel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNormRel' is null for nppiNormRel_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiNormRel_L2_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRel_L2_32f_C3CMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, oSizeROI=%p, nCOI=%d, pNormRel=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, oSizeROI, nCOI, pNormRel, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    NppiSize oSizeROI_native;
    int nCOI_native = 0;
    Npp64f* pNormRel_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nCOI_native = (int)nCOI;
    PointerData *pNormRel_pointerData = initPointerData(env, pNormRel);
    if (pNormRel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNormRel_native = (Npp64f*)pNormRel_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormRel_L2_32f_C3CMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, oSizeROI_native, nCOI_native, pNormRel_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    // oSizeROI is a read-only structure
    // nCOI is primitive
    if (!releasePointerData(env, pNormRel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_8u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_8u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8s_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_18s_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_8s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_8s_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_8s_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_16u_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_16u_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** host pointer */
/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_32f_C1MR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_32f_C1MR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_32f_C1MR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_16u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_16u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16s_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_16s_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_16s_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_18u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_8u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_8u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_8u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_8s_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_18s_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_8s_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_8s_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_8s_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_16u_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_116u_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_16u_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_16u_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_16u_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2_32f_C3CMR.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormRelL2GetBufferHostSize_132f_1C3CMRNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormRelL2GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiNormRelL2GetBufferHostSize_32f_C3CMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormRelL2GetBufferHostSize_32f_C3CMR(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiNormRelL2GetBufferHostSize_32f_C3CMR(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDp Pointer to the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_8u64f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_18u64f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_8u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_8u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_8u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppiDotProd_8u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_8u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_8u64f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64f*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_8u64f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDp Pointer to the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_8s64f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_18s64f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_8s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_8s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_8s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppiDotProd_8s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_8s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_8s64f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64f*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_8s64f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDp Pointer to the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_16u64f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_116u64f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_16u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_16u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_16u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppiDotProd_16u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_16u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_16u64f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64f*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_16u64f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDp Pointer to the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_16s64f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_116s64f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_16s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_16s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_16s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppiDotProd_16s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_16s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_16s64f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64f*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_16s64f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit unsigned image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDp Pointer to the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_32u64f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132u64f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppiDotProd_32u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32u64f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64f*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32u64f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDp Pointer to the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_32s64f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132s64f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppiDotProd_32s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32s64f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64f*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32s64f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDp Pointer to the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_32f64f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132f64f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32f64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32f64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32f64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppiDotProd_32f64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32f64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32f64f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64f*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32f64f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_18u64f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_8u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_8u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_8u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_8u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_8u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_8u64f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_8u64f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_8s64f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_18s64f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_8s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_8s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_8s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_8s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_8s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_8s64f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_8s64f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_16u64f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_116u64f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_16u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_16u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_16u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_16u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_16u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_16u64f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_16u64f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_16s64f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_116s64f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_16s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_16s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_16s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_16s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_16s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_16s64f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_16s64f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit unsigned image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_32u64f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132u64f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_32u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32u64f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32u64f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_32s64f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132s64f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_32s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32s64f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32s64f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_32f64f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132f64f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32f64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32f64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32f64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_32f64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32f64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32f64f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32f64f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_8u64f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_18u64f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_8u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_8u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_8u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_8u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_8u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_8u64f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_8u64f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_8s64f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_18s64f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_8s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_8s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_8s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_8s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_8s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_8s64f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_8s64f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_16u64f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_116u64f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_16u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_16u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_16u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_16u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_16u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_16u64f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_16u64f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_16s64f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_116s64f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_16s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_16s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_16s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_16s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_16s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_16s64f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_16s64f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132u64f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_32u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32u64f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32u64f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_32s64f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132s64f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_32s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32s64f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32s64f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image DotProd.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_32f64f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132f64f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32f64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32f64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32f64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_32f64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32f64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32f64f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32f64f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image DotProd ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_8u64f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_18u64f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_8u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_8u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_8u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_8u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_8u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_8u64f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_8u64f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image DotProd ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_8s64f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_18s64f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_8s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_8s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_8s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_8s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_8s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_8s64f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_8s64f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image DotProd ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed Inf-norm of differences. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_16u64f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_116u64f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_16u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_16u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_16u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_16u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_16u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_16u64f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_16u64f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image DotProd ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_16s64f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_116s64f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_16s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_16s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_16s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_16s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_16s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_16s64f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_16s64f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit unsigned image DotProd ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_32u64f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132u64f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_32u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32u64f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32u64f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed image DotProd ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_32s64f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132s64f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_32s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32s64f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32s64f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image DotProd ignoring alpha channel.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDp Array that contains the computed dot product of the two images. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiDotProdGetBufferHostSize_32f64f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProd_132f64f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jdoubleArray aDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDotProd_32f64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDotProd_32f64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProd_32f64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDp' is null for nppiDotProd_32f64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiDotProd_32f64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProd_32f64f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, aDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, aDp, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f aDp_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDp, aDp_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDotProd_32f64f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, aDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDp_native, aDp, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * @name DotProdGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the Mean_StdDev primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_8u64f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_18u64f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_8u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_8u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_8u64f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_8u64f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_8s64f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_18s64f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_8s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_8s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_8s64f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_8s64f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_16u64f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_116u64f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_16u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_16u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_16u64f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_16u64f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_116s64f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_16s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_16s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_16s64f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_16s64f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132u64f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32u64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32u64f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32u64f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** host pointer */
/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_32s64f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132s64f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32s64f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32s64f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_32f64f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132f64f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32f64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32f64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32f64f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32f64f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_8u64f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_18u64f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_8u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_8u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_8u64f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_8u64f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_18s64f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_8s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_8s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_8s64f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_8s64f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** host pointer */
/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_16u64f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_116u64f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_16u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_16u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_16u64f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_16u64f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_16s64f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_116s64f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_16s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_16s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_16s64f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_16s64f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_32u64f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132u64f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32u64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32u64f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32u64f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_32s64f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132s64f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32s64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32s64f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32s64f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_32f64f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132f64f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32f64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32f64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32f64f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32f64f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_8u64f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_18u64f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_8u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_8u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_8u64f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_8u64f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_8s64f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_18s64f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_8s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_8s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_8s64f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_8s64f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_16u64f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_116u64f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_16u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_16u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_16u64f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_16u64f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_16s64f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_116s64f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_16s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_16s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_16s64f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_16s64f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_32u64f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132u64f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32u64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32u64f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32u64f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_32s64f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132s64f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32s64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32s64f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32s64f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_32f64f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132f64f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32f64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32f64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32f64f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32f64f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_8u64f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_18u64f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_8u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_8u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_8u64f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_8u64f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** host pointer */
/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_8s64f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_18s64f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_8s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_8s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_8s64f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_8s64f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_16u64f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_116u64f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_16u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_16u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_16u64f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_16u64f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_16s64f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_116s64f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_16s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_16s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_16s64f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_16s64f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_32u64f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132u64f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32u64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32u64f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32u64f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_32s64f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132s64f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32s64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32s64f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32s64f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiDotProd_32f64f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDotProdGetBufferHostSize_132f64f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDotProdGetBufferHostSize_32f64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiDotProdGetBufferHostSize_32f64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDotProdGetBufferHostSize_32f64f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiDotProdGetBufferHostSize_32f64f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image CountInRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pCounts Pointer to the number of pixels that fall into the specified range.
 * @param nLowerBound Lower bound of the specified range.
 * @param nUpperBound Upper bound of the specified range.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiCountInRangeGetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
 * is larger than the upper bound.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRange_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jintArray pCounts, jbyte nLowerBound, jbyte nUpperBound, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCountInRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pCounts == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pCounts' is null for nppiCountInRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLowerBound is primitive
    // nUpperBound is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCountInRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRange_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pCounts=%p, nLowerBound=%c, nUpperBound=%c, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pCounts, nLowerBound, nUpperBound, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int* pCounts_native;
    Npp8u nLowerBound_native;
    Npp8u nUpperBound_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pCounts_pointerData = initPointerData(env, pCounts);
    if (pCounts_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pCounts_native = (int*)pCounts_pointerData->getPointer(env);
    nLowerBound_native = (Npp8u)nLowerBound;
    nUpperBound_native = (Npp8u)nUpperBound;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCountInRange_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pCounts_native, nLowerBound_native, nUpperBound_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pCounts_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLowerBound is primitive
    // nUpperBound is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image CountInRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pCounts Pointer to the number of pixels that fall into the specified range.
 * @param nLowerBound Lower bound of the specified range.
 * @param nUpperBound Upper bound of the specified range.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiCountInRangeGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
 * is larger than the upper bound.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRange_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jintArray pCounts, jfloat nLowerBound, jfloat nUpperBound, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCountInRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pCounts == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pCounts' is null for nppiCountInRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLowerBound is primitive
    // nUpperBound is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCountInRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRange_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pCounts=%p, nLowerBound=%f, nUpperBound=%f, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pCounts, nLowerBound, nUpperBound, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int* pCounts_native;
    Npp32f nLowerBound_native;
    Npp32f nUpperBound_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pCounts_pointerData = initPointerData(env, pCounts);
    if (pCounts_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pCounts_native = (int*)pCounts_pointerData->getPointer(env);
    nLowerBound_native = (Npp32f)nLowerBound;
    nUpperBound_native = (Npp32f)nUpperBound;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCountInRange_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pCounts_native, nLowerBound_native, nUpperBound_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pCounts_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLowerBound is primitive
    // nUpperBound is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CountInRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aCounts Array that contains the number of pixels that fall into the specified range for Three-channels.
 * @param aLowerBound Fixed size array of the lower bound of the specified range, one per channel.
 * @param aUpperBound Fixed size array of the upper bound of the specified range, one per channel.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiCountInRangeGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
 * is larger than the upper bound.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRange_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jintArray aCounts, jbyteArray aLowerBound, jbyteArray aUpperBound, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCountInRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCounts == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCounts' is null for nppiCountInRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aLowerBound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aLowerBound' is null for nppiCountInRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aUpperBound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aUpperBound' is null for nppiCountInRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCountInRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRange_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aCounts=%p, aLowerBound=%p, aUpperBound=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aCounts, aLowerBound, aUpperBound, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int aCounts_native[3];
    Npp8u aLowerBound_native[3];
    Npp8u aUpperBound_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCounts, aCounts_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aLowerBound, aLowerBound_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aUpperBound, aUpperBound_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCountInRange_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aCounts_native, aLowerBound_native, aUpperBound_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCounts_native, aCounts, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aLowerBound_native, aLowerBound, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aUpperBound_native, aUpperBound, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image CountInRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aCounts Array that contains the number of pixels that fall into the specified range for Three-channels.
 * @param aLowerBound Fixed size array of the lower bound of the specified range, one per channel.
 * @param aUpperBound Fixed size array of the upper bound of the specified range, one per channel.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiCountInRangeGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
 * is larger than the upper bound.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRange_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jintArray aCounts, jfloatArray aLowerBound, jfloatArray aUpperBound, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCountInRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCounts == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCounts' is null for nppiCountInRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aLowerBound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aLowerBound' is null for nppiCountInRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aUpperBound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aUpperBound' is null for nppiCountInRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCountInRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRange_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aCounts=%p, aLowerBound=%p, aUpperBound=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aCounts, aLowerBound, aUpperBound, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int aCounts_native[3];
    Npp32f aLowerBound_native[3];
    Npp32f aUpperBound_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCounts, aCounts_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aLowerBound, aLowerBound_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aUpperBound, aUpperBound_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCountInRange_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, aCounts_native, aLowerBound_native, aUpperBound_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCounts_native, aCounts, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aLowerBound_native, aLowerBound, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aUpperBound_native, aUpperBound, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CountInRange ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aCounts Array that contains the number of pixels that fall into the specified range for Three-channels.
 * @param aLowerBound Fixed size array of the lower bound of the specified range, one per channel.
 * @param aUpperBound Fixed size array of the upper bound of the specified range, one per channel.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiCountInRangeGetBufferHostSize_8u_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
 * is larger than the upper bound.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRange_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jintArray aCounts, jbyteArray aLowerBound, jbyteArray aUpperBound, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCountInRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCounts == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCounts' is null for nppiCountInRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aLowerBound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aLowerBound' is null for nppiCountInRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aUpperBound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aUpperBound' is null for nppiCountInRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCountInRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRange_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aCounts=%p, aLowerBound=%p, aUpperBound=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aCounts, aLowerBound, aUpperBound, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int aCounts_native[3];
    Npp8u aLowerBound_native[3];
    Npp8u aUpperBound_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCounts, aCounts_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aLowerBound, aLowerBound_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aUpperBound, aUpperBound_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCountInRange_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aCounts_native, aLowerBound_native, aUpperBound_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCounts_native, aCounts, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aLowerBound_native, aLowerBound, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aUpperBound_native, aUpperBound, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CountInRange ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aCounts Array that contains the number of pixels that fall into the specified range for Three-channels.
 * @param aLowerBound Fixed size array of the lower bound of the specified range, one per channel.
 * @param aUpperBound Fixed size array of the upper bound of the specified range, one per channel.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiCountInRangeGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
 * is larger than the upper bound.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRange_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jintArray aCounts, jfloatArray aLowerBound, jfloatArray aUpperBound, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCountInRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCounts == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCounts' is null for nppiCountInRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aLowerBound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aLowerBound' is null for nppiCountInRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aUpperBound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aUpperBound' is null for nppiCountInRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCountInRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRange_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aCounts=%p, aLowerBound=%p, aUpperBound=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aCounts, aLowerBound, aUpperBound, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    int aCounts_native[3];
    Npp32f aLowerBound_native[3];
    Npp32f aUpperBound_native[3];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCounts, aCounts_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aLowerBound, aLowerBound_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aUpperBound, aUpperBound_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCountInRange_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, aCounts_native, aLowerBound_native, aUpperBound_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aCounts_native, aCounts, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aLowerBound_native, aLowerBound, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aUpperBound_native, aUpperBound, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRangeGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRangeGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiCountInRangeGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRangeGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiCountInRangeGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiCountInRange_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRangeGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRangeGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiCountInRangeGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRangeGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiCountInRangeGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiCountInRange_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRangeGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRangeGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiCountInRangeGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRangeGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiCountInRangeGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiCountInRange_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRangeGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRangeGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiCountInRangeGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRangeGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiCountInRangeGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiCountInRange_8u_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRangeGetBufferHostSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRangeGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiCountInRangeGetBufferHostSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRangeGetBufferHostSize_8u_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiCountInRangeGetBufferHostSize_8u_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppiCountInRange_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCountInRangeGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCountInRangeGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiCountInRangeGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCountInRangeGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiCountInRangeGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_8u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_8u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_16u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_16u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_16s_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_16s_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_8u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_8u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_16u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_16u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_16s_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_16s_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_32f_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_32f_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_8u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_8u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_16u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_16u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_116s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_16s_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_16s_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image MaxEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_32f_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_32f_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image MaxEvery ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_8u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_8u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image MaxEvery ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_16u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_16u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image MaxEvery ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_16s_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_16s_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image MaxEvery ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaxEvery_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMaxEvery_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMaxEvery_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaxEvery_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaxEvery_32f_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMaxEvery_32f_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image MinEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_8u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_8u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image MinEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_16u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_16u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image MinEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_16s_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_16s_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image MinEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image MinEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_8u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_8u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image MinEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_16u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_16u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image MinEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_16s_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_16s_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image MinEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_32f_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_32f_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image MinEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_8u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_8u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image MinEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_16u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_16u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_116s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_16s_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_16s_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image MinEvery.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_32f_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_32f_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image MinEvery ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_8u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_8u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image MinEvery ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_16u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_16u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image MinEvery ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_16s_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_16s_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image MinEvery ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinEvery_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinEvery_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMinEvery_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinEvery_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinEvery_32f_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMinEvery_32f_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image Integral with 32-bit signed output.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nVal The value to add to pDst image pixels
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiIntegral_18u32s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nVal)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiIntegral_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiIntegral_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiIntegral_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nVal is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiIntegral_8u32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nVal=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nVal);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    Npp32s nVal_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nVal_native = (Npp32s)nVal;

    // Native function call
    NppStatus jniResult_native = nppiIntegral_8u32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nVal_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nVal is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image Integral with 32-bit floating point output.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nVal The value to add to pDst image pixels
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiIntegral_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jfloat nVal)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiIntegral_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiIntegral_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiIntegral_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nVal is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiIntegral_8u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nVal=%f)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nVal);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    Npp32f nVal_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nVal_native = (Npp32f)nVal;

    // Native function call
    NppStatus jniResult_native = nppiIntegral_8u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nVal_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nVal is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image SqrIntegral. 
 * Destination integral image and square integral image are 32-bit signed int.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pSqr \ref destination_image_pointer.
 * @param nSqrStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 * @param nVal The value to add to pDst image pixels
 * @param nValSqr The value to add to pSqr image pixels
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrIntegral_18u32s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject pSqr, jint nSqrStep, jobject oSrcROI, jint nVal, jint nValSqr)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrIntegral_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrIntegral_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pSqr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSqr' is null for nppiSqrIntegral_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSqrStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiSqrIntegral_8u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nVal is primitive
    // nValSqr is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrIntegral_8u32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, pSqr=%p, nSqrStep=%d, oSrcROI=%p, nVal=%d, nValSqr=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, pSqr, nSqrStep, oSrcROI, nVal, nValSqr);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp32s* pSqr_native = NULL;
    int nSqrStep_native = 0;
    NppiSize oSrcROI_native;
    Npp32s nVal_native;
    Npp32s nValSqr_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pSqr_pointerData = initPointerData(env, pSqr);
    if (pSqr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSqr_native = (Npp32s*)pSqr_pointerData->getPointer(env);
    nSqrStep_native = (int)nSqrStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nVal_native = (Npp32s)nVal;
    nValSqr_native = (Npp32s)nValSqr;

    // Native function call
    NppStatus jniResult_native = nppiSqrIntegral_8u32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, pSqr_native, nSqrStep_native, oSrcROI_native, nVal_native, nValSqr_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pSqr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSqrStep is primitive
    // oSrcROI is a read-only structure
    // nVal is primitive
    // nValSqr is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image SqrIntegral. 
 * Destination integral image is 32-bit signed int. 
 * Destination square integral image is 64-bit double floating point.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pSqr \ref destination_image_pointer.
 * @param nSqrStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 * @param nVal The value to add to pDst image pixels
 * @param nValSqr The value to add to pSqr image pixels
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrIntegral_18u32s64f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject pSqr, jint nSqrStep, jobject oSrcROI, jint nVal, jdouble nValSqr)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrIntegral_8u32s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrIntegral_8u32s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pSqr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSqr' is null for nppiSqrIntegral_8u32s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSqrStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiSqrIntegral_8u32s64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nVal is primitive
    // nValSqr is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrIntegral_8u32s64f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, pSqr=%p, nSqrStep=%d, oSrcROI=%p, nVal=%d, nValSqr=%lf)\n",
        pSrc, nSrcStep, pDst, nDstStep, pSqr, nSqrStep, oSrcROI, nVal, nValSqr);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp64f* pSqr_native = NULL;
    int nSqrStep_native = 0;
    NppiSize oSrcROI_native;
    Npp32s nVal_native;
    Npp64f nValSqr_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pSqr_pointerData = initPointerData(env, pSqr);
    if (pSqr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSqr_native = (Npp64f*)pSqr_pointerData->getPointer(env);
    nSqrStep_native = (int)nSqrStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nVal_native = (Npp32s)nVal;
    nValSqr_native = (Npp64f)nValSqr;

    // Native function call
    NppStatus jniResult_native = nppiSqrIntegral_8u32s64f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, pSqr_native, nSqrStep_native, oSrcROI_native, nVal_native, nValSqr_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pSqr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSqrStep is primitive
    // oSrcROI is a read-only structure
    // nVal is primitive
    // nValSqr is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image SqrIntegral. 
 * Destination integral image is 32-bit floating point.
 * Destination square integral image is 64-bit double floating point.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pSqr \ref destination_image_pointer.
 * @param nSqrStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 * @param nVal The value to add to pDst image pixels
 * @param nValSqr The value to add to pSqr image pixels
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrIntegral_18u32f64f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject pSqr, jint nSqrStep, jobject oSrcROI, jfloat nVal, jdouble nValSqr)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrIntegral_8u32f64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrIntegral_8u32f64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pSqr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSqr' is null for nppiSqrIntegral_8u32f64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSqrStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiSqrIntegral_8u32f64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nVal is primitive
    // nValSqr is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrIntegral_8u32f64f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, pSqr=%p, nSqrStep=%d, oSrcROI=%p, nVal=%f, nValSqr=%lf)\n",
        pSrc, nSrcStep, pDst, nDstStep, pSqr, nSqrStep, oSrcROI, nVal, nValSqr);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp64f* pSqr_native = NULL;
    int nSqrStep_native = 0;
    NppiSize oSrcROI_native;
    Npp32f nVal_native;
    Npp64f nValSqr_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pSqr_pointerData = initPointerData(env, pSqr);
    if (pSqr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSqr_native = (Npp64f*)pSqr_pointerData->getPointer(env);
    nSqrStep_native = (int)nSqrStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nVal_native = (Npp32f)nVal;
    nValSqr_native = (Npp64f)nValSqr;

    // Native function call
    NppStatus jniResult_native = nppiSqrIntegral_8u32f64f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, pSqr_native, nSqrStep_native, oSrcROI_native, nVal_native, nValSqr_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pSqr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSqrStep is primitive
    // oSrcROI is a read-only structure
    // nVal is primitive
    // nValSqr is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image RectStdDev.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSqr \ref destination_image_pointer.
 * @param nSqrStep \ref destination_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oRect rectangular window
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRectStdDev_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSqr, jint nSqrStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oRect)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRectStdDev_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSqr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSqr' is null for nppiRectStdDev_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSqrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRectStdDev_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRectStdDev_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oRect == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRect' is null for nppiRectStdDev_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRectStdDev_32f_C1R(pSrc=%p, nSrcStep=%d, pSqr=%p, nSqrStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oRect=%p)\n",
        pSrc, nSrcStep, pSqr, nSqrStep, pDst, nDstStep, oSizeROI, oRect);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp64f* pSqr_native = NULL;
    int nSqrStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiRect oRect_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSqr_pointerData = initPointerData(env, pSqr);
    if (pSqr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSqr_native = (Npp64f*)pSqr_pointerData->getPointer(env);
    nSqrStep_native = (int)nSqrStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oRect, oRect_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRectStdDev_32f_C1R(pSrc_native, nSrcStep_native, pSqr_native, nSqrStep_native, pDst_native, nDstStep_native, oSizeROI_native, oRect_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSqr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSqrStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oRect is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed image RectStdDev, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSqr \ref destination_image_pointer.
 * @param nSqrStep \ref destination_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oRect rectangular window
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRectStdDev_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSqr, jint nSqrStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oRect, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRectStdDev_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSqr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSqr' is null for nppiRectStdDev_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSqrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRectStdDev_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRectStdDev_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oRect == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRect' is null for nppiRectStdDev_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRectStdDev_32s_C1RSfs(pSrc=%p, nSrcStep=%d, pSqr=%p, nSqrStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oRect=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSqr, nSqrStep, pDst, nDstStep, oSizeROI, oRect, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSqr_native = NULL;
    int nSqrStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiRect oRect_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSqr_pointerData = initPointerData(env, pSqr);
    if (pSqr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSqr_native = (Npp32s*)pSqr_pointerData->getPointer(env);
    nSqrStep_native = (int)nSqrStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oRect, oRect_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiRectStdDev_32s_C1RSfs(pSrc_native, nSrcStep_native, pSqr_native, nSqrStep_native, pDst_native, nDstStep_native, oSizeROI_native, oRect_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSqr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSqrStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oRect is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed image RectStdDev. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSqr \ref destination_image_pointer.
 * @param nSqrStep \ref destination_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oRect rectangular window
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRectStdDev_132s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSqr, jint nSqrStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oRect)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRectStdDev_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSqr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSqr' is null for nppiRectStdDev_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSqrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRectStdDev_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRectStdDev_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oRect == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRect' is null for nppiRectStdDev_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRectStdDev_32s32f_C1R(pSrc=%p, nSrcStep=%d, pSqr=%p, nSqrStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oRect=%p)\n",
        pSrc, nSrcStep, pSqr, nSqrStep, pDst, nDstStep, oSizeROI, oRect);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp64f* pSqr_native = NULL;
    int nSqrStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiRect oRect_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSqr_pointerData = initPointerData(env, pSqr);
    if (pSqr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSqr_native = (Npp64f*)pSqr_pointerData->getPointer(env);
    nSqrStep_native = (int)nSqrStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oRect, oRect_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRectStdDev_32s32f_C1R(pSrc_native, nSrcStep_native, pSqr_native, nSqrStep_native, pDst_native, nDstStep_native, oSizeROI_native, oRect_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSqr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSqrStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oRect is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * @name HistogramEven
 * The \f$nLowerLevel\f$ (inclusive) and \f$nUpperLevel\f$ (exclusive) define the boundaries of the range, 
 * which are evenly segmented into \f$nLevel - 1\f$ bins. The computed histogram is stored in \f$pHist\f$. 
 * The levels are calculated by another primitive \ref nppiEvenLevelsHost_32s and are stored 
 * in a host pointer \f$hpLevels\f$. The number of levels is also \f$nLevel - 1\f$.
 * The histogram \f$pHist[k]\f$ is defined as the total number of pixels that fall into the range:
 * \f$hpLevels[k] <= pSrc(j, i) < hpLevels[k+1]\f$.
 * The functions require additional scratch buffer for computations.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Compute levels with even distribution.
 *
 * @param hpLevels A host pointer to array which receives the levels being
 *        computed. The array needs to be of size nLevels. 
 * @param nLevels The number of levels being computed. nLevels must be at least 2.
 * @param nLowerLevel Lower boundary value of the lowest level.
 * @param nUpperLevel Upper boundary value of the greatest level.
 * @return
 *  image_data_error_codes, or NPP_HISTO_NUMBER_OF_LEVELS_ERROR if an invalid nLevels is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEvenLevelsHost_132sNative(JNIEnv *env, jclass cls, jobject hpLevels, jint nLevels, jint nLowerLevel, jint nUpperLevel)
{
    // Null-checks for non-primitive arguments
    if (hpLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpLevels' is null for nppiEvenLevelsHost_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEvenLevelsHost_32s(hpLevels=%p, nLevels=%d, nLowerLevel=%d, nUpperLevel=%d)\n",
        hpLevels, nLevels, nLowerLevel, nUpperLevel);

    // Native variable declarations
    Npp32s* hpLevels_native = NULL;
    int nLevels_native = 0;
    Npp32s nLowerLevel_native;
    Npp32s nUpperLevel_native;

    // Obtain native variable values
    PointerData *hpLevels_pointerData = initPointerData(env, hpLevels);
    if (hpLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpLevels_native = (Npp32s*)hpLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    nLowerLevel_native = (Npp32s)nLowerLevel;
    nUpperLevel_native = (Npp32s)nUpperLevel;

    // Native function call
    NppStatus jniResult_native = nppiEvenLevelsHost_32s(hpLevels_native, nLevels_native, nLowerLevel_native, nUpperLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, hpLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned HistogramEven.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param nLevels Number of levels.
 * @param nLowerLevel Lower boundary of lowest level bin.
 * @param nUpperLevel Upper boundary of highest level bin.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_8u_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jint nLevels, jint nLowerLevel, jint nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%d, nLowerLevel=%d, nUpperLevel=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    int nLevels_native = 0;
    Npp32s nLowerLevel_native;
    Npp32s nUpperLevel_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    nLowerLevel_native = (Npp32s)nLowerLevel;
    nUpperLevel_native = (Npp32s)nUpperLevel;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned HistogramEven.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_8u_C3R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    int nLevels_native[3];
    Npp32s nLowerLevel_native[3];
    Npp32s nUpperLevel_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned HistogramEven.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_8u_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    int nLevels_native[4];
    Npp32s nLowerLevel_native[4];
    Npp32s nUpperLevel_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned HistogramEven ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_8u_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    int nLevels_native[3];
    Npp32s nLowerLevel_native[3];
    Npp32s nUpperLevel_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned HistogramEven.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param nLevels Number of levels.
 * @param nLowerLevel Lower boundary of lowest level bin.
 * @param nUpperLevel Upper boundary of highest level bin.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16u_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jint nLevels, jint nLowerLevel, jint nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%d, nLowerLevel=%d, nUpperLevel=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    int nLevels_native = 0;
    Npp32s nLowerLevel_native;
    Npp32s nUpperLevel_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    nLowerLevel_native = (Npp32s)nLowerLevel;
    nUpperLevel_native = (Npp32s)nUpperLevel;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned HistogramEven.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16u_C3R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    int nLevels_native[3];
    Npp32s nLowerLevel_native[3];
    Npp32s nUpperLevel_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned HistogramEven.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16u_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    int nLevels_native[4];
    Npp32s nLowerLevel_native[4];
    Npp32s nUpperLevel_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned HistogramEven ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16u_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    int nLevels_native[3];
    Npp32s nLowerLevel_native[3];
    Npp32s nUpperLevel_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed HistogramEven.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param nLevels Number of levels.
 * @param nLowerLevel Lower boundary of lowest level bin.
 * @param nUpperLevel Upper boundary of highest level bin.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16s_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jint nLevels, jint nLowerLevel, jint nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%d, nLowerLevel=%d, nUpperLevel=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    int nLevels_native = 0;
    Npp32s nLowerLevel_native;
    Npp32s nUpperLevel_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    nLowerLevel_native = (Npp32s)nLowerLevel;
    nUpperLevel_native = (Npp32s)nUpperLevel;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed HistogramEven.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16s_C3R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    int nLevels_native[3];
    Npp32s nLowerLevel_native[3];
    Npp32s nUpperLevel_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed HistogramEven.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16s_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    int nLevels_native[4];
    Npp32s nLowerLevel_native[4];
    Npp32s nUpperLevel_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed HistogramEven ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramEvenGetBufferSize_16s_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    int nLevels_native[3];
    Npp32s nLowerLevel_native[3];
    Npp32s nUpperLevel_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @name HistogramEvenGetBufferSize
 * Companion primitives for computing the device buffer size (in bytes) required by the HistogramEven primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_8u_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_8u_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_8u_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_8u_C3R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_8u_C3R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_8u_C3R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_8u_C4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_8u_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_8u_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_8u_AC4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_8u_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_8u_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_16u_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16u_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16u_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_16u_C3R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16u_C3R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16u_C3R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_16u_C4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16u_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16u_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_16u_AC4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16u_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16u_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_16s_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16s_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16s_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_16s_C3R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16s_C3R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16s_C3R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_16s_C4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16s_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16s_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiHistogramEven_16s_AC4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16s_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16s_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} image_histogrameven */
/**
 * <pre>
 * @defgroup image_histogramrange HistogramRange
 * Primitives for computing the histogram of an image within specified ranges.
 * @{
 *
 * </pre>
 */
/**
 * <pre>
 * @name HistogramEven
 * The histogram is computed according to the ranges provided in \f$pLevels\f$. 
 * The histogram \f$pHist[k]\f$ is defined as the total number of pixels that fall into the range:
 * \f$pLevels[k] <= pSrc(j, i) < pLevels[k+1]\f$. The number of the histogram bins is \f$nLevel - 1\f$.
 * The functions require additional scratch buffer for computations.
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param pLevels Pointer to array containing the level sizes of the bins.
        The array must be of size nLevels.
 * @param nLevels Number of levels in histogram.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_8u_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jobject pLevels, jint nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_8u_C3R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_8u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_8u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_8u_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned HistogramRange ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_8u_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param pLevels Pointer to array containing the level sizes of the bins.
        The array must be of size nLevels.
 * @param nLevels Number of levels in histogram.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16u_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jobject pLevels, jint nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16u_C3R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16u_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16u_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16u_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned HistogramRange ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16u_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param pLevels Pointer to array containing the level sizes of the bins.
        The array must be of size nLevels.
 * @param nLevels Number of levels in histogram.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16s_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jobject pLevels, jint nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16s_C3R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16s_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16s_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16s_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_16s_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param pLevels Pointer to array containing the level sizes of the bins.
        The array must be of size nLevels.
 * @param nLevels Number of levels in histogram.
 * @param pBuffer Pointer to appropriately sized (\ref nppiHistogramRangeGetBufferSize_32f_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jobject pLevels, jint nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    Npp32f* pLevels_native = NULL;
    int nLevels_native = 0;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32f*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_32f_C3R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_32f_C3R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_32f_C3R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point HistogramRange.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_32f_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    const Npp32f* pLevels_native[4];
    int nLevels_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point HistogramRange ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_32f_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @name HistogramRangeGetBufferSize
 * Companion primitives for computing the device buffer size (in bytes) required by the HistogramRange primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_8u_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_8u_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_8u_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_8u_C3R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_8u_C3R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_8u_C3R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_8u_C4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_8u_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_8u_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_8u_AC4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_8u_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_8u_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16u_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16u_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16u_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16u_C3R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16u_C3R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16u_C3R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16u_C4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16u_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16u_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16u_AC4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16u_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16u_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16s_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16s_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16s_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16s_C3R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16s_C3R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16s_C3R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16s_C4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16s_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16s_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16s_AC4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16s_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16s_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_32f_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_32f_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_32f_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_32f_C3R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_32f_C3R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_32f_C3R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_32f_C4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_32f_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_32f_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_32f_AC4R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_32f_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_32f_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} image_histogramrange */
/**
 * <pre>
 * @defgroup image_proximity Image Proximity
 * Primitives for computing the proximity measure between a source image and a template image.
 *
 * \section general_introduction General Introduction
 * There are basically two approaches to compute the proximity measure for template matching, 
 * Euclidean distance and the cross correlation.
 * -# Euclidean distance computes the sum of the squared distance (SSD) between the corresponding 
 * pixels of the source image and the template image. The smaller the distance is, the more 
 * similar the source image and the template image is around the pixel. The anchor of the template  
 * image is used during the computations, which always lies in the gemotric center of the image. 
 * Given a source image \f$pSrc\f$ (\f$W_s \times H_s\f$) and a template image \f$pTpl\f$ (\f$W_t \times H_t\f$), 
 * the Euclidean distance \f$D_{st}(c,r)\f$ between two images at pixel in row \f$r\f$ and column 
 * \f$c\f$ is computed as (\f$s\f$ stands for source image and \f$t\f$ for template image for short):
 * \f[D_{st}(c,r)=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)-pSrc(j+c-\frac{H_t}{2}, i+r-\frac{W_t}{2})]^2 \f]
 * -# Cross correlation computes the sum of the product between the corresponding pixels of the source 
 * image and the template image. The cross correlation \f$R_{st}(c,r)\f$ is calculated as:
 * \f[R_{st}(c,r)=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)\cdot pSrc(j+c-\frac{H_t}{2}, i+r-\frac{W_t}{2})] \f]
 * The larger the cross correlation value is, the more similar the source image and the template image is around the pixel. 
 * -# The cross correlation \f$R_{st}(c,r)\f$ is affected by the brightness of the images which may vary due to the lighting 
 * and exposure conditions. Therefore, NPP computes the cross correlation coefficient to circumvent this dependence. 
 * This is typically done at every step by subtracting the mean from every pixel value, i.e.,
 * \f[\tilde{R}_{st}(c,r)=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)-Mean_t]\cdot [pSrc(j+c-\frac{H_t}{2}, i+r-\frac{W_t}{2})-Mean_s] \f]
 * 
 * NPP computes the normalized values of Euclidean distance, cross correlation and the cross correlation coefficient.
 * -# The normalized Euclidean distance \f$\sigma_{st}(c,r)\f$ is defined as:
 * \f[\sigma_{st}(c,r) = \frac{D_{st}(c,r)}{\sqrt{R_{ss}(c,r)\cdot R_{tt}(\frac{H_t}{2},\frac{W_t}{2})}} \f]
 * -# The normalized cross correlation \f$\rho_{st}(c,r)\f$ is defined as:
 * \f[\rho_{st}(c,r) = \frac{R_{st}(c,r)}{\sqrt{R_{ss}(c,r)\cdot R_{tt}(\frac{H_t}{2},\frac{W_t}{2})}} \f]
 * The \f$R_{ss}(c,r)\f$ and \f$R_{tt}(\frac{H_t}{2}, \frac{W_t}{2}\f$ denote the auto correlation of the source
 * image and the template image individually. They are defined as:
 * \f[R_{ss}(c,r)=\sum_{j=c-\frac{H_t}{2}}^{c+\frac{H_t}{2}}\sum_{i=r-\frac{W_t}{2}}^{r+\frac{W_t}{2}}pSrc(j, i) \f]
 * \f[R_{tt}(\frac{H_t}{2},\frac{W_t}{2})=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}pTpl(j,i) \f]
 * -# Similarly, the normalized cross correlation coefficient \f$\gamma_{st}(c,r)\f$ is calculated as:
 * \f[\gamma_{st}(c,r) = \frac{\tilde{R}_{st}(c,r)}{\sqrt{\tilde{R}_{ss}(c,r)\cdot \tilde{R}_{tt}(\frac{H_t}{2},\frac{W_t}{2})}} \f]
 * The \f$\tilde{R}_{ss}(c,r)\f$ and \f$\tilde{R}_{tt}(\frac{H_t}{2}, \frac{W_t}{2}\f$ are defined as:
 * \f[\tilde{R}_{ss}(c,r)=\sum_{j=c-\frac{H_t}{2}}^{c+\frac{H_t}{2}}\sum_{i=r-\frac{W_t}{2}}^{r+\frac{W_t}{2}}[pSrc(j, i)-Mean_s] \f]
 * \f[\tilde{R}_{tt}(\frac{H_t}{2},\frac{W_t}{2})=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)-Mean_t] \f]
 *
 * \section category Categorizations
 * The Euclidean distance and the cross correlation are categorized into three types, full, same, and valid. 
 * -# Full mode indicates that the anchor of the template image starts from the outside of the source image, 
 * assuming the out-of-boundary pixels are zeor-padded. The size of the destination
 * image is \f$(W_s + W_t - 1) \times (H_s + H_t - 1)\f$.
 * -# Same mode means that the anchor of the template image starts from the top left pixel
 * of the source image. All the out-of-boundary pixels are also zero-padded.
 * The size of the destination image is the same as the source one, i.e., \f$W_s \times H_s\f$.
 * -# Valid mode indicates that there are no out-of-boudnary readings from the source image.
 * The anchor of the template image starts from the inside of the source image. The 
 * size of the destination image is \f$(W_s - W_t + 1) \times (H_s - H_t + 1)\f$.
 * 
 * @{
 * </pre>
 */
/**
 * <pre>
 * @defgroup sqrdistancefullnorm SqrDistanceFull_Norm
 * Primitives for computing the normalized Euclidean distance between two images with full mode.
 * @{
 * </pre>
 */
/**
 * <pre>
 * @name SqrDistanceFull_Norm
 * The functions compute the \f$\sigma_{st}(c,r)\f$ in \ref general_introduction
 * with full mode (see \ref category).
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image SqrDistanceFull_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8u_C1RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8u_C1RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image SqrDistanceFull_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8u_C3RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8u_C3RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceFull_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8u_C4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8u_C4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceFull_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8u_AC4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image SqrDistanceFull_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceFull_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8s32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8s32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8s32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8s32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8s32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8s32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image SqrDistanceFull_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_8s32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_8s32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_16u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_16u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_16u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_16u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image SqrDistanceFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_16u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_16u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image SqrDistanceFull_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceFull_1Norm_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceFull_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceFull_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceFull_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceFull_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceFull_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceFull_Norm_16u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceFull_Norm_16u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} SqrDistanceFull_Norm */
/**
 * <pre>
 * @defgroup sqrdistancesamenorm SqrDistanceSame_Norm
 * Primitives for computing the normalized Euclidean distance between two images with same mode.
 * @{
 * </pre>
 */
/**
 * <pre>
 * @name SqrDistanceSame_Norm
 * The functions compute the \f$\sigma_{st}(c,r)\f$ in \ref general_introduction 
 * with same mode (see \ref category).
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image SqrDistanceSame_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8u_C1RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8u_C1RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image SqrDistanceSame_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8u_C3RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8u_C3RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceSame_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8u_C4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8u_C4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceSame_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8u_AC4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image SqrDistanceSame_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceSame_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8s32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8s32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8s32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8s32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8s32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8s32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image SqrDistanceSame_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_8s32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_8s32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_16u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_16u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_16u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_16u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image SqrDistanceSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_16u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_16u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image SqrDistanceSame_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceSame_1Norm_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceSame_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceSame_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceSame_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceSame_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceSame_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceSame_Norm_16u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceSame_Norm_16u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} SqrDistanceFull_Norm */
/**
 * <pre>
 * @defgroup sqrdistancevalidnorm SqrDistanceValid_Norm
 * Primitives for computing the normalized Euclidean distance between two images with valid mode.
 * @{
 * </pre>
 */
/**
 * <pre>
 * @name SqrDistanceValid_Norm
 * The functions compute the \f$\sigma_{st}(c,r)\f$ in \ref general_introduction 
 * with valid mode (see \ref category).
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image SqrDistanceValid_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8u_C1RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8u_C1RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image SqrDistanceValid_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8u_C3RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8u_C3RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceValid_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8u_C4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8u_C4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceValid_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8u_AC4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image SqrDistanceValid_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image SqrDistanceValid_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8s32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8s32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8s32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8s32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8s32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8s32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image SqrDistanceValid_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_8s32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_8s32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_16u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_16u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_16u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_16u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image SqrDistanceValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_16u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_16u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image SqrDistanceValid_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrDistanceValid_1Norm_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrDistanceValid_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiSqrDistanceValid_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiSqrDistanceValid_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiSqrDistanceValid_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrDistanceValid_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrDistanceValid_Norm_16u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiSqrDistanceValid_Norm_16u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} SqrDistanceValid_Norm*/
/**
 * <pre>
 * @defgroup crosscorrfullnorm CrossCorrFull_Norm
 * Primitives for computing the normalized cross correlation between two images with full mode.
 *
 * @{
 * </pre>
 */
/**
 * <pre>
 * @name CrossCorrFull_Norm
 * The functions compute the \f$\rho_{st}(c,r)\f$ in \ref general_introduction 
 * with full mode (see \ref category).
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrFull_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8u_C1RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8u_C1RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrFull_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8u_C3RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8u_C3RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrFull_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8u_C4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8u_C4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrFull_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8u_AC4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrFull_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrFull_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8s32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8s32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8s32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8s32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8s32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8s32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrFull_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_8s32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_8s32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_16u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_16u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_16u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_16u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrFull_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_16u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_16u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrFull_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1Norm_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_Norm_16u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_Norm_16u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} CrossCorrFull_Norm */
/**
 * <pre>
 * @defgroup crosscorrsamenorm CrossCorrSame_Norm
 * Primitives for computing the normalized cross correlation between two images with same mode.
 *
 * @{
 * </pre>
 */
/**
 * <pre>
 * @name CrossCorrSame_Norm
 * The functions compute the \f$\rho_{st}(c,r)\f$ in \ref general_introduction 
 * with same mode (see \ref category).
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrSame_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8u_C1RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8u_C1RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrSame_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8u_C3RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8u_C3RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrSame_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8u_C4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8u_C4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrSame_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8u_AC4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrSame_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrSame_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8s32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8s32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8s32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8s32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8s32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8s32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrSame_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_8s32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_8s32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_16u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_16u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_16u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_16u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrSame_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_16u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_16u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrSame_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1Norm_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_Norm_16u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_Norm_16u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} CrossCorrSame_Norm*/
/**
 * <pre>
 * @defgroup crosscorrvalidnorm CrossCorrValid_Norm
 * Primitives for computing the normalized cross correlation between two images with valid mode.
 *
 * @{
 * </pre>
 */
/**
 * <pre>
 * @name CrossCorrValid_Norm
 * The functions compute the \f$\rho_{st}(c,r)\f$ in \ref general_introduction 
 * with valid mode (see \ref category).
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrValid_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8u_C1RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8u_C1RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrValid_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8u_C3RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8u_C3RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrValid_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8u_C4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8u_C4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrValid_Norm ignoring alpha channel, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8u_AC4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrValid_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrValid_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8s32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8s32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8s32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8s32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8s32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8s32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrValid_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_8s32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_8s32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_16u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_16u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_16u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_16u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrValid_Norm. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_16u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_16u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrValid_Norm ignoring alpha channel. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1Norm_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_Norm_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_Norm_16u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_Norm_16u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} CrossCorrValid_Norm*/
/**
 * <pre>
 * @defgroup crosscorrvalid CrossCorrValid
 * Primitives for computing the cross correlation between two images with valid mode.
 *
 * @{
 * </pre>
 */
/**
 * <pre>
 * @name CrossCorrValid
 * The functions compute the \f$R_{st}(c,r)\f$ in \ref general_introduction 
 * with valid mode (see \ref category).
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 32-bit floating point images CrossCorrValid.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned images CrossCorrValid.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_8u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_8u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed images CrossCorrValid.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_8s32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_8s32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned images CrossCorrValid.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_16u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_16u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} CrossCorrValid*/
/**
 * <pre>
 * @defgroup crosscorrfullnormlevel CrossCorrFull_NormLevel
 * Primitives for computing the normalized cross correlation coefficient between two images with full mode.
 *
 * @{
 * </pre>
 */
/**
 * <pre>
 * @name CrossCorrFull_NormLevel
 * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction 
 * with full mode (see \ref category). The functions require additional scratch buffer for computations.
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8u_C1RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8u_C1RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8u_C1RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8u_C3RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8u_C3RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8u_C3RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8u_C4RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8u_C4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8u_C4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrFull_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8u_AC4RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8u_AC4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrFull_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8u32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8u32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8u32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrFull_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8u32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8s32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8s32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8s32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8s32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8s32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8s32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8s32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8s32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8s32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrFull_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_8s32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_8s32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_8s32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_16u32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_16u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_16u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_16u32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_16u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_16u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrFull_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_16u32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_16u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_16u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrFull_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiFullNormLevelGetBufferHostSize_16u32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrFull_1NormLevel_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrFull_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrFull_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrFull_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrFull_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrFull_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrFull_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrFull_NormLevel_16u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrFull_NormLevel_16u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @name FullNormLevelGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrFull_NormLevel primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Buffer size (in bytes) for nppiCrossCorrFull_NormLevel_8u_C1RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8u_C1RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8u_C1RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u_C3RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8u_C3RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8u_C3RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u_C4RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8u_C4RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8u_C4RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u_AC4RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8u_AC4RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8u_AC4RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8u32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8u32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8u32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8u32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8u32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8u32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8u32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8u32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8u32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8s32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8s32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8s32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8s32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8s32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8s32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8s32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8s32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8s32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_8s32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_8s32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_8s32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_16u32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_16u32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_16u32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_16u32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_16u32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_16u32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_16u32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_16u32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_16u32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrFull_NormLevel_16u32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFullNormLevelGetBufferHostSize_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFullNormLevelGetBufferHostSize_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiFullNormLevelGetBufferHostSize_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFullNormLevelGetBufferHostSize_16u32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiFullNormLevelGetBufferHostSize_16u32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} CrossCorrFull_NormLevel*/
/**
 * <pre>
 * @defgroup crosscorrsamenormlevel CrossCorrSame_NormLevel
 * Primitives for computing the normalized cross correlation coefficient between two images with same mode.
 *
 * @{
 * </pre>
 */
/**
 * <pre>
 * @name CrossCorrSame_NormLevel
 * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction 
 * with same mode (see \ref category). The functions require additional scratch buffer for computations.
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8u_C1RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8u_C1RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8u_C1RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8u_C3RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8u_C3RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8u_C3RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8u_C4RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8u_C4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8u_C4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrSame_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8u_AC4RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8u_AC4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrSame_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8u32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8u32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8u32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrSame_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8u32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8s32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8s32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8s32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8s32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8s32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8s32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8s32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8s32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8s32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrSame_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_8s32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_8s32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_8s32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_16u32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_16u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_16u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_16u32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_16u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_16u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrSame_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_16u32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_16u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_16u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrSame_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiSameNormLevelGetBufferHostSize_16u32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrSame_1NormLevel_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrSame_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrSame_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrSame_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrSame_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrSame_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrSame_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrSame_NormLevel_16u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrSame_NormLevel_16u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @name SameNormLevelGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrSame_NormLevel primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Buffer size (in bytes) for nppiCrossCorrSame_NormLevel_8u_C1RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8u_C1RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8u_C1RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u_C3RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8u_C3RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8u_C3RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u_C4RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8u_C4RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8u_C4RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u_AC4RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8u_AC4RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8u_AC4RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8u32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8u32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8u32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8u32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8u32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8u32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8u32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8u32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8u32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8s32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8s32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8s32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8s32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8s32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8s32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8s32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8s32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8s32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_8s32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_8s32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_8s32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_16u32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_16u32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_16u32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_16u32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_16u32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_16u32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_16u32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_16u32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_16u32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrSame_NormLevel_16u32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSameNormLevelGetBufferHostSize_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSameNormLevelGetBufferHostSize_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiSameNormLevelGetBufferHostSize_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSameNormLevelGetBufferHostSize_16u32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiSameNormLevelGetBufferHostSize_16u32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} CrossCorrSame_NormLevel*/
/**
 * <pre>
 * @defgroup crosscorrvalidnormlevel CrossCorrValid_NormLevel
 * Primitives for computing the normalized cross correlation coefficient between two images with valid mode.
 *
 * @{
 * </pre>
 */
/**
 * <pre>
 * @name CrossCorrValid_NormLevel
 * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction 
 * with valid mode (see \ref category). The functions require additional scratch buffer for computations.
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8u_C1RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8u_C1RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8u_C1RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8u_C3RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8u_C3RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8u_C3RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8u_C4RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8u_C4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8u_C4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrValid_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8u_AC4RSfs to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8u_AC4RSfs(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image CrossCorrValid_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp32f* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp32f*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit unsigned image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8u32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8u32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8u32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image CrossCorrValid_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8u32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8s32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8s32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8s32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8s32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8s32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8s32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8s32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8s32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8s32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image CrossCorrValid_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_8s32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_8s32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp8s* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp8s*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_8s32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_16u32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_16u32f_C1R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_16u32f_C1R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_16u32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_16u32f_C3R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_16u32f_C3R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrValid_NormLevel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_16u32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_16u32f_C4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_16u32f_C4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image CrossCorrValid_NormLevel ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcRoiSize \ref roi_specification.
 * @param pTpl Pointer to the template image.
 * @param nTplStep Number of bytes between successive rows in the template image.
 * @param oTplRoiSize \ref roi_specification.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiValidNormLevelGetBufferHostSize_16u32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCrossCorrValid_1NormLevel_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcRoiSize, jobject pTpl, jint nTplStep, jobject oTplRoiSize, jobject pDst, jint nDstStep, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCrossCorrValid_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcRoiSize' is null for nppiCrossCorrValid_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTpl == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTpl' is null for nppiCrossCorrValid_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTplStep is primitive
    if (oTplRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oTplRoiSize' is null for nppiCrossCorrValid_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCrossCorrValid_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiCrossCorrValid_NormLevel_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCrossCorrValid_NormLevel_16u32f_AC4R(pSrc=%p, nSrcStep=%d, oSrcRoiSize=%p, pTpl=%p, nTplStep=%d, oTplRoiSize=%p, pDst=%p, nDstStep=%d, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSrcRoiSize, pTpl, nTplStep, oTplRoiSize, pDst, nDstStep, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcRoiSize_native;
    Npp16u* pTpl_native = NULL;
    int nTplStep_native = 0;
    NppiSize oTplRoiSize_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcRoiSize, oSrcRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTpl_pointerData = initPointerData(env, pTpl);
    if (pTpl_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTpl_native = (Npp16u*)pTpl_pointerData->getPointer(env);
    nTplStep_native = (int)nTplStep;
    if (!initNative(env, oTplRoiSize, oTplRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiCrossCorrValid_NormLevel_16u32f_AC4R(pSrc_native, nSrcStep_native, oSrcRoiSize_native, pTpl_native, nTplStep_native, oTplRoiSize_native, pDst_native, nDstStep_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcRoiSize is a read-only structure
    if (!releasePointerData(env, pTpl_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nTplStep is primitive
    // oTplRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @name ValidNormLevelGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrValid_NormLevel primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Buffer size (in bytes) for nppiCrossCorrValid_NormLevel_8u_C1RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8u_C1RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8u_C1RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u_C3RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8u_C3RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8u_C3RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u_C4RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8u_C4RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8u_C4RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u_AC4RSfs.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8u_AC4RSfs(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8u_AC4RSfs(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8u32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8u32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8u32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8u32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18u32f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8u32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8u32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8u32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8u32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8u32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8s32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18s32f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8s32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8s32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8s32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18s32f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8s32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8s32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8s32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8s32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18s32f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8s32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8s32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8s32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_8s32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_18s32f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_8s32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_8s32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_8s32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_16u32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_16u32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_16u32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_16u32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_16u32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_16u32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_16u32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_116u32f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_16u32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_16u32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_16u32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiCrossCorrValid_NormLevel_16u32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiValidNormLevelGetBufferHostSize_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiValidNormLevelGetBufferHostSize_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiValidNormLevelGetBufferHostSize_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiValidNormLevelGetBufferHostSize_16u32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiValidNormLevelGetBufferHostSize_16u32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/** @} CrossCorrValid_NormLevel*/
/** @} image_proximity */
/**
 * <pre>
 * @defgroup image_quality_index Image Quality Index
 * Primitives for computing the image quality index of two images.
 * @{
 * </pre>
 */
/**
 * <pre>
 * @name QualityIndex
 * Given two images \f$M\f$ and \f$N\f$ (both \f$W \times H\f$), the
 * mathematical formula to calculate the image quality index \f$Q\f$ between them is expressed as:
 * \f[Q = \frac{4\sigma_{MN}\tilde{M}\tilde{N}}{[(\tilde{M}^2)+(\tilde{N}^2)][(\sigma_M)^2+(\sigma_N)^2]} \f]
 * where
 * \f[\tilde{M} = \frac{1}{W\cdot H}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}M(j,i)\f]
 * \f[\tilde{N} = \frac{1}{W\cdot H}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}N(j,i)\f]
 * \f[\sigma_{M} = \sqrt{\frac{1}{W\cdot H-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}[M(j,i)-\tilde{M}]^2}\f]
 * \f[\sigma_{N} = \sqrt{\frac{1}{W\cdot H-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}[N(j,i)-\tilde{N}]^2}\f]
 * \f[\sigma_{MN} = \frac{1}{W\cdot H-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}[M(j,i)-\tilde{M}][N(j,i)-\tilde{N}]\f]
 * The functions require additional scratch buffer for computations.
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image QualityIndex.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oRoiSize \ref roi_specification.
 * @param pDst Pointer to the quality index.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiQualityIndexGetBufferHostSize_8u32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of 
 * either image are constant numberse.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndex_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oRoiSize, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiQualityIndex_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiQualityIndex_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRoiSize' is null for nppiQualityIndex_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiQualityIndex_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiQualityIndex_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndex_8u32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oRoiSize=%p, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oRoiSize_native;
    Npp32f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oRoiSize, oRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQualityIndex_8u32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oRoiSize_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image QualityIndex.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oRoiSize \ref roi_specification.
 * @param pDst Pointer to the quality index.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiQualityIndexGetBufferHostSize_16u32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of 
 * either image are constant numberse.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndex_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oRoiSize, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiQualityIndex_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiQualityIndex_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRoiSize' is null for nppiQualityIndex_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiQualityIndex_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiQualityIndex_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndex_16u32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oRoiSize=%p, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oRoiSize_native;
    Npp32f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oRoiSize, oRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQualityIndex_16u32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oRoiSize_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image QualityIndex.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oRoiSize \ref roi_specification.
 * @param pDst Pointer to the quality index.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiQualityIndexGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of 
 * either image are constant numberse.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndex_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oRoiSize, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiQualityIndex_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiQualityIndex_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRoiSize' is null for nppiQualityIndex_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiQualityIndex_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiQualityIndex_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndex_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oRoiSize=%p, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oRoiSize_native;
    Npp32f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oRoiSize, oRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQualityIndex_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oRoiSize_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image QualityIndex.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oRoiSize \ref roi_specification.
 * @param pDst Pointer to the quality index.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiQualityIndexGetBufferHostSize_8u32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of 
 * either image are constant numberse.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndex_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oRoiSize, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiQualityIndex_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiQualityIndex_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRoiSize' is null for nppiQualityIndex_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiQualityIndex_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiQualityIndex_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndex_8u32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oRoiSize=%p, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oRoiSize_native;
    Npp32f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oRoiSize, oRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQualityIndex_8u32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oRoiSize_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image QualityIndex.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oRoiSize \ref roi_specification.
 * @param pDst Pointer to the quality index.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiQualityIndexGetBufferHostSize_16u32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of 
 * either image are constant numberse.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndex_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oRoiSize, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiQualityIndex_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiQualityIndex_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRoiSize' is null for nppiQualityIndex_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiQualityIndex_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiQualityIndex_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndex_16u32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oRoiSize=%p, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oRoiSize_native;
    Npp32f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oRoiSize, oRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQualityIndex_16u32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oRoiSize_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image QualityIndex.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oRoiSize \ref roi_specification.
 * @param pDst Pointer to the quality index.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiQualityIndexGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of 
 * either image are constant numberse.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndex_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oRoiSize, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiQualityIndex_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiQualityIndex_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRoiSize' is null for nppiQualityIndex_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiQualityIndex_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiQualityIndex_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndex_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oRoiSize=%p, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oRoiSize_native;
    Npp32f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oRoiSize, oRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQualityIndex_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oRoiSize_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image QualityIndex.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oRoiSize \ref roi_specification.
 * @param pDst Pointer to the quality index.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiQualityIndexGetBufferHostSize_8u32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of 
 * either image are constant numberse.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndex_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oRoiSize, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiQualityIndex_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiQualityIndex_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRoiSize' is null for nppiQualityIndex_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiQualityIndex_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiQualityIndex_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndex_8u32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oRoiSize=%p, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oRoiSize_native;
    Npp32f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oRoiSize, oRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQualityIndex_8u32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oRoiSize_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image QualityIndex.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oRoiSize \ref roi_specification.
 * @param pDst Pointer to the quality index.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiQualityIndexGetBufferHostSize_16u32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of 
 * either image are constant numberse.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndex_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oRoiSize, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiQualityIndex_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiQualityIndex_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRoiSize' is null for nppiQualityIndex_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiQualityIndex_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiQualityIndex_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndex_16u32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oRoiSize=%p, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oRoiSize_native;
    Npp32f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oRoiSize, oRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQualityIndex_16u32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oRoiSize_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image QualityIndex.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oRoiSize \ref roi_specification.
 * @param pDst Pointer to the quality index.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiQualityIndexGetBufferHostSize_32f_AC4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of 
 * either image are constant numberse.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndex_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oRoiSize, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiQualityIndex_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiQualityIndex_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oRoiSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oRoiSize' is null for nppiQualityIndex_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiQualityIndex_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiQualityIndex_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndex_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oRoiSize=%p, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oRoiSize, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oRoiSize_native;
    Npp32f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oRoiSize, oRoiSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQualityIndex_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oRoiSize_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oRoiSize is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @name QualityIndexGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the QualityIndex primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiQualityIndex_8u32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndexGetBufferHostSize_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiQualityIndexGetBufferHostSize_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiQualityIndexGetBufferHostSize_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndexGetBufferHostSize_8u32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiQualityIndexGetBufferHostSize_8u32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiQualityIndex_16u32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndexGetBufferHostSize_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiQualityIndexGetBufferHostSize_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiQualityIndexGetBufferHostSize_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndexGetBufferHostSize_16u32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiQualityIndexGetBufferHostSize_16u32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiQualityIndex_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndexGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiQualityIndexGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiQualityIndexGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndexGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiQualityIndexGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiQualityIndex_8u32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndexGetBufferHostSize_18u32f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiQualityIndexGetBufferHostSize_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiQualityIndexGetBufferHostSize_8u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndexGetBufferHostSize_8u32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiQualityIndexGetBufferHostSize_8u32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiQualityIndex_16u32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndexGetBufferHostSize_116u32f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiQualityIndexGetBufferHostSize_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiQualityIndexGetBufferHostSize_16u32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndexGetBufferHostSize_16u32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiQualityIndexGetBufferHostSize_16u32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiQualityIndex_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndexGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiQualityIndexGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiQualityIndexGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndexGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiQualityIndexGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiQualityIndex_8u32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndexGetBufferHostSize_18u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiQualityIndexGetBufferHostSize_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiQualityIndexGetBufferHostSize_8u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndexGetBufferHostSize_8u32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiQualityIndexGetBufferHostSize_8u32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiQualityIndex_16u32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndexGetBufferHostSize_116u32f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiQualityIndexGetBufferHostSize_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiQualityIndexGetBufferHostSize_16u32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndexGetBufferHostSize_16u32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiQualityIndexGetBufferHostSize_16u32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size (in bytes) for \ref nppiQualityIndex_32f_AC4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQualityIndexGetBufferHostSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiQualityIndexGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiQualityIndexGetBufferHostSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQualityIndexGetBufferHostSize_32f_AC4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiQualityIndexGetBufferHostSize_32f_AC4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @}  */
/** @} image_quality_index */
/**
 * <pre>
 * @defgroup image_maximum_error MaximumError
 * Primitives for computing the maximum error between two images.
 * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$, 
 * the maximum error is defined as the largest absolute difference between pixels of two images.
 * If the image is in complex format, the absolute value of the complex number is provided.
 * 
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_8s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_8s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16sc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16sc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32sc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32sc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 64-bit floating point image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_64f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_64f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 8-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_18u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_8u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_8u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 8-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_18s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_8s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_8s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit signed complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16sc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16sc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit signed complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32sc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32sc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit floating point image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132f_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32f_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32f_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit floating point complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132fc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32fc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32fc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 64-bit floating point image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_164f_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_64f_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_64f_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_18s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_8s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_8s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16sc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16sc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32sc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32sc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 64-bit floating point image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_164f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_64f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_64f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_18s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_8s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_8s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_116sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_16sc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_16sc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit unsigned image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32sc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32sc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point complex image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 64-bit floating point image Maximum_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumError_164f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumError_64f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumError_64f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @name NormDiffInfGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_8s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_18s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_8s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_8s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16sc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16sc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16sc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32sc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32sc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32sc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32fc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32fc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32fc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_64f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_164f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_64f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_64f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_8u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_18u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_8u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_8u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_8s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_18s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_8s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_8s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16sc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116sc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16sc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16sc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32sc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132sc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32sc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32sc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32f_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132f_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32f_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32f_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32fc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132fc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32fc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32fc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_64f_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_164f_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_64f_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_64f_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_8s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_18s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_8s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_8s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16sc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116sc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16sc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16sc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32sc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132sc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32sc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32sc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32fc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32fc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32fc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_64f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_164f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_64f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_64f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_8s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_18s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_8s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_8s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_16sc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_116sc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_16sc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_16sc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32sc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132sc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32sc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32sc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_32fc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_32fc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_32fc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumError_64f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumErrorGetBufferHostSize_164f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumErrorGetBufferHostSize_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumErrorGetBufferHostSize_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumErrorGetBufferHostSize_64f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumErrorGetBufferHostSize_64f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @defgroup image_average_error AverageError
 * Primitives for computing the average error between two images.
 * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$, 
 * the average error is defined as:
 * \f[Average Error = \frac{1}{W\cdot H\cdot N}\sum_{n=0}^{N-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}\left|pSrc1(j,i) - pSrc2(j,i)\right|\f]
 * where N stands for the number of channels. If the image is in complex format, the absolute value is used for computation.
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_8s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_8s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16sc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16sc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32sc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32sc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 64-bit floating point image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_64f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_64f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 8-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_18u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_8u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_8u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 8-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_18s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_8s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_8s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit signed complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16sc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16sc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit signed complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32sc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32sc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit floating point image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132f_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32f_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32f_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit floating point complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132fc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32fc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32fc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 64-bit floating point image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_164f_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_64f_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_64f_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_18s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_8s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_8s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16sc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16sc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32sc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32sc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 64-bit floating point image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_164f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_64f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_64f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_18s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_8s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_8s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_116sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_16sc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_16sc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit unsigned image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32sc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32sc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point complex image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 64-bit floating point image Average_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageError_164f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageError_64f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageError_64f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @name NormDiffInfGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Buffer size for \ref nppiAverageError_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_8s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_18s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_8s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_8s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16sc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16sc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16sc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32sc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32sc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32sc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32fc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32fc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32fc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_64f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_164f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_64f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_64f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_8u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_18u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_8u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_8u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_8s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_18s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_8s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_8s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16sc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116sc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16sc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16sc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32sc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132sc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32sc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32sc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32f_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132f_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32f_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32f_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32fc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132fc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32fc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32fc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_64f_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_164f_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_64f_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_64f_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_8s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_18s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_8s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_8s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16sc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116sc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16sc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16sc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32sc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132sc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32sc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32sc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32fc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32fc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32fc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_64f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_164f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_64f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_64f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_8s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_18s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_8s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_8s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_16sc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_116sc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_16sc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_16sc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32sc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132sc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32sc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32sc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_32fc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_32fc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_32fc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageError_64f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageErrorGetBufferHostSize_164f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageErrorGetBufferHostSize_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageErrorGetBufferHostSize_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageErrorGetBufferHostSize_64f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageErrorGetBufferHostSize_64f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @defgroup image_maximum_relative_error MaximumRelativeError
 * Primitives for computing the maximum relative error between two images.
 * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$, 
 * the maximum relative error is defined as:
 * \f[MaximumRelativeError = max{\frac{\left|pSrc1(j,i) - pSrc2(j,i)\right|}{max(\left|pSrc1(j,i)\right|, \left|pSrc2(j,i)\right|)}}\f]
 * If the image is in complex format, the absolute value is used for computation.
 * For multiple channles, the maximum relative error of all the channles is returned.
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_8s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_8s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16sc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16sc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32sc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32sc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 64-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_64f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_64f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 8-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_18u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_8u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_8u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 8-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_18s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_8s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_8s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16sc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16sc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32sc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32sc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132f_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32f_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32f_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit floating point complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132fc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32fc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32fc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 64-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_164f_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_64f_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_64f_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_18s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_8s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_8s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16sc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16sc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32sc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32sc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 64-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_164f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_64f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_64f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_18s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_8s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_8s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_116sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_16sc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_16sc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32sc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32sc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 64-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiMaximumRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeError_164f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMaximumRelativeError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMaximumRelativeError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiMaximumRelativeError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMaximumRelativeError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeError_64f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeError_64f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @name NormDiffInfGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_8s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_18s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_8s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_8s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16sc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16sc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16sc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32sc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32sc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32sc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32fc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32fc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32fc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_64f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_164f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_64f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_64f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_8u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_18u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_8u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_8u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_8s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_18s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_8s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_8s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16sc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116sc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16sc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16sc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32sc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132sc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32sc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32sc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32f_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132f_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32f_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32f_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32fc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132fc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32fc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32fc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_64f_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_164f_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_64f_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_64f_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_8s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_18s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_8s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_8s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16sc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116sc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16sc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16sc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32sc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132sc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32sc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32sc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32fc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32fc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32fc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_64f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_164f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_64f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_64f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_8s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_18s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_8s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_8s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_16sc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_116sc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_16sc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_16sc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32sc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132sc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32sc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32sc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_32fc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_32fc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_32fc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiMaximumRelativeError_64f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMaximumRelativeErrorGetBufferHostSize_164f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMaximumRelativeErrorGetBufferHostSize_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMaximumRelativeErrorGetBufferHostSize_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMaximumRelativeErrorGetBufferHostSize_64f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMaximumRelativeErrorGetBufferHostSize_64f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @defgroup image_average_relative_error AverageRelativeError
 * Primitives for computing the average relative error between two images.
 * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$, 
 * the maximum relative error is defined as:
 * \f[AverageRelativeError = \frac{1}{W\cdot H\cdot N}\sum_{n=0}^{N-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}\frac{\left|pSrc1(j,i) - pSrc2(j,i)\right|}{max(\left|pSrc1(j,i)\right|, \left|pSrc2(j,i)\right|)}\f]
 * where N is the number of channels.
 * If the image is in complex format, the absolute value is used for computation.
 * @{
 * </pre>
 */
/**
 * <pre>
 * One-channel 8-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 8-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_8s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_8s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 16-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16sc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16sc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32sc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32sc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 32-bit floating point complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One-channel 64-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C1R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_64f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_64f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 8-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_18u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_8u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_8u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 8-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_18s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_8s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_8s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16sc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16sc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132u_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32u_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32u_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32s_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32s_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32sc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32sc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132f_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32f_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32f_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit floating point complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132fc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32fc_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32fc_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 64-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C2R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_164f_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_64f_C2R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_64f_C2R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_18s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_8s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_8s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16sc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16sc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32sc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32sc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating point complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 64-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C3R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_164f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_64f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_64f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_8u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_18s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_8s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_8s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_116sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_16sc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_16sc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit unsigned image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16u_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_16s_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32sc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32sc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating point complex image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error (absolute value). 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 64-bit floating point image MaximumRelative_Error.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pError Pointer to the computed error. 
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * Use \ref nppiAverageRelativeErrorGetBufferHostSize_32f_C4R to compute the required size (in bytes).
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_EVEN_STEP_ERROR if an 
 * invalid floating-point image is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeError_164f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject oSizeROI, jobject pError, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAverageRelativeError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAverageRelativeError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pError == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pError' is null for nppiAverageRelativeError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiAverageRelativeError_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeError_64f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, oSizeROI=%p, pError=%p, pDeviceBuffer=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, oSizeROI, pError, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp64f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pError_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pError_pointerData = initPointerData(env, pError);
    if (pError_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pError_native = (Npp64f*)pError_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeError_64f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, oSizeROI_native, pError_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pError_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @} */
/**
 * <pre>
 * @name NormDiffInfGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
 * @{
 * </pre>
 */
/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_8s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_18s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_8s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_8s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16sc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16sc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16sc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32s_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32s_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32s_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32sc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32sc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32sc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32fc_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32fc_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32fc_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_64f_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_164f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_64f_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_64f_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_8u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_18u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_8u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_8u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_8u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_8s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_18s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_8s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_8s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16sc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116sc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16sc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16sc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32u_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132u_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32u_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32u_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32s_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132s_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32s_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32s_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32sc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132sc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32sc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32sc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32f_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132f_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32f_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32f_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32fc_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132fc_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32fc_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32fc_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_64f_C2R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_164f_1C2RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_64f_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_64f_C2R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_64f_C2R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_8u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_8u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_8u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_8s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_18s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_8s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_8s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16sc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116sc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16sc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16sc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32u_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132u_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32u_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32u_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32s_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132s_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32s_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32s_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32sc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132sc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32sc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32sc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32fc_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32fc_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32fc_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_64f_C3R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_164f_1C3RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_64f_C3R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_64f_C3R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_8u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_8s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_18s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_8s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_8s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_16sc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_116sc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_16sc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_16sc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32u_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32s_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32s_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32s_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32sc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132sc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32sc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32sc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_32fc_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_32fc_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_32fc_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Buffer size for \ref nppiAverageRelativeError_64f_C4R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAverageRelativeErrorGetBufferHostSize_164f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAverageRelativeErrorGetBufferHostSize_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiAverageRelativeErrorGetBufferHostSize_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAverageRelativeErrorGetBufferHostSize_64f_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiAverageRelativeErrorGetBufferHostSize_64f_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMagnitude_132fc32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMagnitude_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMagnitude_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMagnitude_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMagnitude_32fc32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMagnitude_32fc32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex to 32-bit floating point squared magnitude.
 * 
 * Converts complex-number pixel image to single channel image computing
 * the result pixels as the squared magnitude of the complex values.
 * 
 * The squared magnitude is an itermediate result of magnitude computation and
 * can thus be computed faster than actual magnitude. If magnitudes are required
 * for sorting/comparing only, using this function instead of nppiMagnitude_32fc32f_C1R
 * can be a worthwhile performance optimization.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMagnitudeSqr_132fc32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMagnitudeSqr_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMagnitudeSqr_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMagnitudeSqr_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMagnitudeSqr_32fc32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMagnitudeSqr_32fc32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * @param oSrcROI Region of interest in the source image.
 * @param pDstRect User supplied host memory pointer to an NppiRect structure that will be filled in by this function with the region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetResizeRectNative(JNIEnv *env, jclass cls, jobject oSrcROI, jobjectArray pDstRect, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiGetResizeRect");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDstRect == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstRect' is null for nppiGetResizeRect");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetResizeRect(oSrcROI=%p, pDstRect=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        oSrcROI, pDstRect, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    NppiRect oSrcROI_native;
    NppiRect* pDstRect_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDstRect_pointerData = initPointerData(env, pDstRect);
    if (pDstRect_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstRect_native = (NppiRect*)pDstRect_pointerData->getPointer(env);
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiGetResizeRect(oSrcROI_native, pDstRect_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDstRect_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image resize not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of interpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image resize not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of interpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed image resize not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of interpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    const Npp16s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_116s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_16s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    const Npp16s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_16s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image resize not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of interpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 64-bit floating point image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_64f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_64f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_164f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_64f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_64f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 64-bit floating point image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_164f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_64f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_64f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 64-bit floating point image resize not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of interpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_164f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_64f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_64f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 64-bit floating point planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_164f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_64f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    const Npp64f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_64f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 64-bit floating point planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param nXShift Source pixel shift in x-direction.
 * @param nYShift Source pixel shift in y-direction.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResizeSqrPixel_164f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jdouble nXFactor, jdouble nYFactor, jdouble nXShift, jdouble nYShift, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResizeSqrPixel_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResizeSqrPixel_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResizeSqrPixel_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResizeSqrPixel_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiResizeSqrPixel_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResizeSqrPixel_64f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nXFactor=%lf, nYFactor=%lf, nXShift=%lf, nYShift=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nXFactor, nYFactor, nXShift, nYShift, eInterpolation);

    // Native variable declarations
    const Npp64f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    double nXShift_native = 0.0;
    double nYShift_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    nXShift_native = (double)nXShift;
    nYShift_native = (double)nYShift;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResizeSqrPixel_64f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nXFactor_native, nYFactor_native, nXShift_native, nYShift_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // nXShift is primitive
    // nYShift is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image resize not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of interpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed.
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image resize not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of interpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image resize not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of interpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point planar image resize.
 *
 * @param pSrc \ref source_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_planar_image_pointer_array (host memory array containing device memory image plane pointers).
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image.
 * @param nXFactor Factor by which x dimension is changed. 
 * @param nYFactor Factor by which y dimension is changed. 
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref resize_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject dstROISize, jdouble nXFactor, jdouble nYFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, nXFactor=%lf, nYFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, nXFactor, nYFactor, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double nXFactor_native = 0.0;
    double nYFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nXFactor_native = (double)nXFactor;
    nYFactor_native = (double)nYFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, nXFactor_native, nYFactor_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // nXFactor is primitive
    // nYFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 8-bit unsigned image remap.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image remap.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image remap not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of interpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar image remap.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobjectArray pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned planar image remap.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobjectArray pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned image remap.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image remap.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned planar image remap.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobjectArray pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned planar image remap.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobjectArray pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed image remap.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed image remap.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed image remap.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed image remap not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed planar image remap.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobjectArray pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    const Npp16s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16s* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed planar image remap.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_116s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobjectArray pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_16s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_16s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    const Npp16s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp16s* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_16s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit floating point image remap.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image remap.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image remap not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit floating point planar image remap.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobjectArray pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobjectArray pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp32f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp32f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp32f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 64-bit floating point image remap.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_64f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp64f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp64f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp64f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_64f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_164f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_64f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp64f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp64f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp64f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_64f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 64-bit floating point image remap.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_164f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_64f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp64f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp64f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp64f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_64f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 64-bit floating point image remap not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_164f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_64f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp64f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp64f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp64f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_64f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 64-bit floating point planar image remap.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_164f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobjectArray pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_64f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    const Npp64f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp64f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp64f* pDst_native[3];
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp64f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp64f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_64f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 64-bit floating point planar image remap.
 *
 * @param pSrc \ref source_planar_image_pointer_array.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image.
 * @param oSrcROI Region of interest in the source image.
 * @param pXMap Device memory pointer to 2D image array of X coordinate values to be used when sampling source image. 
 * @param nXMapStep pXMap image array line step in bytes.
 * @param pYMap Device memory pointer to 2D image array of Y coordinate values to be used when sampling source image. 
 * @param nYMapStep pYMap image array line step in bytes.
 * @param pDst \ref destination_planar_image_pointer_array.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Region of interest size in the destination image.
 * @param eInterpolation The type of eInterpolation to perform resampling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref remap_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRemap_164f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pXMap, jint nXMapStep, jobject pYMap, jint nYMapStep, jobjectArray pDst, jint nDstStep, jobject oDstSizeROI, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRemap_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRemap_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRemap_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pXMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pXMap' is null for nppiRemap_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nXMapStep is primitive
    if (pYMap == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pYMap' is null for nppiRemap_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nYMapStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRemap_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiRemap_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRemap_64f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pXMap=%p, nXMapStep=%d, pYMap=%p, nYMapStep=%d, pDst=%p, nDstStep=%d, oDstSizeROI=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pXMap, nXMapStep, pYMap, nYMapStep, pDst, nDstStep, oDstSizeROI, eInterpolation);

    // Native variable declarations
    const Npp64f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pXMap_native = NULL;
    int nXMapStep_native = 0;
    Npp64f* pYMap_native = NULL;
    int nYMapStep_native = 0;
    Npp64f* pDst_native[4];
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pXMap_pointerData = initPointerData(env, pXMap);
    if (pXMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pXMap_native = (Npp64f*)pXMap_pointerData->getPointer(env);
    nXMapStep_native = (int)nXMapStep;
    PointerData *pYMap_pointerData = initPointerData(env, pYMap);
    if (pYMap_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pYMap_native = (Npp64f*)pYMap_pointerData->getPointer(env);
    nYMapStep_native = (int)nYMapStep;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRemap_64f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pXMap_native, nXMapStep_native, pYMap_native, nYMapStep_native, pDst_native, nDstStep_native, oDstSizeROI_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pXMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nXMapStep is primitive
    if (!releasePointerData(env, pYMap_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nYMapStep is primitive
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetRotateQuadNative(JNIEnv *env, jclass cls, jobject oSrcROI, jobjectArray aQuad, jdouble nAngle, jdouble nShiftX, jdouble nShiftY)
{
    // Null-checks for non-primitive arguments
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiGetRotateQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aQuad' is null for nppiGetRotateQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetRotateQuad(oSrcROI=%p, aQuad=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf)\n",
        oSrcROI, aQuad, nAngle, nShiftX, nShiftY);

    // Native variable declarations
    NppiRect oSrcROI_native;
    double aQuad_native[4][2];
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;

    // Obtain native variable values
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aQuad, aQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;

    // Native function call
    NppStatus jniResult_native = nppiGetRotateQuad(oSrcROI_native, aQuad_native, nAngle_native, nShiftX_native, nShiftY_native);

    // Write back native variable values
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aQuad_native, aQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetRotateBoundNative(JNIEnv *env, jclass cls, jobject oSrcROI, jobjectArray aBoundingBox, jdouble nAngle, jdouble nShiftX, jdouble nShiftY)
{
    // Null-checks for non-primitive arguments
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiGetRotateBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aBoundingBox == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aBoundingBox' is null for nppiGetRotateBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetRotateBound(oSrcROI=%p, aBoundingBox=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf)\n",
        oSrcROI, aBoundingBox, nAngle, nShiftX, nShiftY);

    // Native variable declarations
    NppiRect oSrcROI_native;
    double aBoundingBox_native[2][2];
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;

    // Obtain native variable values
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aBoundingBox, aBoundingBox_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;

    // Native function call
    NppStatus jniResult_native = nppiGetRotateBound(oSrcROI_native, aBoundingBox_native, nAngle_native, nShiftX_native, nShiftY_native);

    // Write back native variable values
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aBoundingBox_native, aBoundingBox)) return JNPP_STATUS_INTERNAL_ERROR;
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image rotate ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image rotate ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit float image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit float image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned in place image mirror.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned in place image mirror.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_C4IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image mirror not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit unsigned in place image mirror.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_C1IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_C3IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_C4IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned in place image mirror not affecting alpha.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 16-bit signed in place image mirror.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit signed image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16s_C4IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16s_C4IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed image mirror not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed in place image mirror not affecting alpha.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit signed in place image mirror.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_C1IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit signed in place image mirror.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_C3IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit signed in place image mirror.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_C4IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_C4IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit image mirror not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_AC4IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 1 channel 32-bit float in place image mirror.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit float image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit float image mirror not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit float in place image mirror not affecting alpha.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref mirror_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMirror_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oROI=%p, flip=%d)\n",
        pSrcDst, nSrcDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes affine transform coefficients based on source ROI and destination quadrilateral.
 *
 * The function computes the coefficients of an affine transformation that maps the
 * given source ROI (axis aligned rectangle with integer coordinates) to a quadrilateral
 * in the destination image.
 *
 * An affine transform in 2D is fully determined by the mapping of just three vertices.
 * This function's API allows for passing a complete quadrilateral effectively making the 
 * prolem overdetermined. What this means in practice is, that for certain quadrilaterals it is
 * not possible to find an affine transform that would map all four corners of the source
 * ROI to the four vertices of that quadrilateral.
 *
 * The function circumvents this problem by only looking at the first three vertices of
 * the destination image quadrilateral to determine the affine transformation's coefficients.
 * If the destination quadrilateral is indeed one that cannot be mapped using an affine
 * transformation the functions informs the user of this situation by returning a 
 * ::NPP_AFFINE_QUAD_INCORRECT_WARNING.
 *
 * @param oSrcROI The source ROI. This rectangle needs to be at least one pixel wide and
 *          high. If either width or hight are less than one an ::NPP_RECT_ERROR is returned.
 * @param aQuad The destination quadrilateral.
 * @param aCoeffs The resulting affine transform coefficients.
 * @return
 *  Error codes:
 *         - ::NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *           has zero or negative value
 *         - ::NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the oSrcROI and source image is less than or
 *           equal to 1
 *         - ::NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - ::NPP_AFFINE_QUAD_INCORRECT_WARNING Indicates a warning when quad
 *           does not conform to the transform properties. Fourth vertex is
 *           ignored, internally computed coordinates are used instead
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetAffineTransformNative(JNIEnv *env, jclass cls, jobject oSrcROI, jobjectArray aQuad, jobjectArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiGetAffineTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aQuad' is null for nppiGetAffineTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiGetAffineTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetAffineTransform(oSrcROI=%p, aQuad=%p, aCoeffs=%p)\n",
        oSrcROI, aQuad, aCoeffs);

    // Native variable declarations
    NppiRect oSrcROI_native;
    double aQuad_native[4][2];
    double aCoeffs_native[2][3];

    // Obtain native variable values
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aQuad, aQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetAffineTransform(oSrcROI_native, aQuad_native, aCoeffs_native);

    // Write back native variable values
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aQuad_native, aQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetAffineQuadNative(JNIEnv *env, jclass cls, jobject oSrcROI, jobjectArray aQuad, jobjectArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiGetAffineQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aQuad' is null for nppiGetAffineQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiGetAffineQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetAffineQuad(oSrcROI=%p, aQuad=%p, aCoeffs=%p)\n",
        oSrcROI, aQuad, aCoeffs);

    // Native variable declarations
    NppiRect oSrcROI_native;
    double aQuad_native[4][2];
    double aCoeffs_native[2][3];

    // Obtain native variable values
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aQuad, aQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetAffineQuad(oSrcROI_native, aQuad_native, aCoeffs_native);

    // Write back native variable values
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aQuad_native, aQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetAffineBoundNative(JNIEnv *env, jclass cls, jobject oSrcROI, jobjectArray aBound, jobjectArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiGetAffineBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aBound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aBound' is null for nppiGetAffineBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiGetAffineBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetAffineBound(oSrcROI=%p, aBound=%p, aCoeffs=%p)\n",
        oSrcROI, aBound, aCoeffs);

    // Native variable declarations
    NppiRect oSrcROI_native;
    double aBound_native[2][2];
    double aCoeffs_native[2][3];

    // Obtain native variable values
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aBound, aBound_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetAffineBound(oSrcROI_native, aBound_native, aCoeffs_native);

    // Write back native variable values
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aBound_native, aBound)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 8-bit unsigned affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 8-bit unsigned affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned affine warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 16-bit unsigned affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit signed affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed affine warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 32-bit signed affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit signed affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating-point affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point affine warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 32-bit floating-point affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit floating-point affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 64-bit floating-point affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_164f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_64f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_64f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_64f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 64-bit floating-point affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_164f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_64f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_64f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_64f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 64-bit floating-point affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_164f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_64f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_64f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_64f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 64-bit floating-point affine warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_164f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_64f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_64f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_64f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 64-bit floating-point affine warp.
 * 
 * @param aSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_164f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiWarpAffine_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiWarpAffine_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_64f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_64f_P3R(aSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        aSrc, oSrcSize, nSrcStep, oSrcROI, aDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp64f* aSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* aDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDst, aDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_64f_P3R(aSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 64-bit floating-point affine warp.
 * 
 * @param aSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_164f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiWarpAffine_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffine_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiWarpAffine_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffine_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffine_64f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_64f_P4R(aSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        aSrc, oSrcSize, nSrcStep, oSrcROI, aDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp64f* aSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp64f* aDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDst, aDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_64f_P4R(aSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned integer backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer backwards affine warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 8-bit unsigned integer backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned integer backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer backwards affine warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit signed integer backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed integer backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed integer backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit signed integer backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating-point backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 32-bit floating-point backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit floating-point backwards affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned integer quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 8-bit unsigned integer quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 8-bit unsigned integer quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer quad-based affine warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 16-bit unsigned integer quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 16-bit unsigned integer quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed integer quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed integer quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed integer quad-based affine warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 32-bit signed integer quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit signed integer quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating-point quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point quad-based affine warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 32-bit floating-point quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit floating-point quad-based affine warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref affine_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates perspective transform coefficients given source rectangular ROI
 * and its destination quadrangle projection
 *
 * @param oSrcROI Source ROI
 * @param quad Destination quadrangle
 * @param aCoeffs Perspective transform coefficients
 * @return
 *  Error codes:
 *         - ::NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *           has zero or negative value
 *         - ::NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the oSrcROI and source image is less than or
 *           equal to 1
 *         - ::NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetPerspectiveTransformNative(JNIEnv *env, jclass cls, jobject oSrcROI, jobjectArray quad, jobjectArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiGetPerspectiveTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (quad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'quad' is null for nppiGetPerspectiveTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiGetPerspectiveTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetPerspectiveTransform(oSrcROI=%p, quad=%p, aCoeffs=%p)\n",
        oSrcROI, quad, aCoeffs);

    // Native variable declarations
    NppiRect oSrcROI_native;
    double quad_native[4][2];
    double aCoeffs_native[3][3];

    // Obtain native variable values
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, quad, quad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetPerspectiveTransform(oSrcROI_native, quad_native, aCoeffs_native);

    // Write back native variable values
    // oSrcROI is a read-only structure
    if (!releaseNative(env, quad_native, quad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates perspective transform projection of given source rectangular
 * ROI
 *
 * @param oSrcROI Source ROI
 * @param quad Destination quadrangle
 * @param aCoeffs Perspective transform coefficients
 * @return
 *  Error codes:
 *         - ::NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *           has zero or negative value
 *         - ::NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the oSrcROI and source image is less than or
 *           equal to 1
 *         - ::NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetPerspectiveQuadNative(JNIEnv *env, jclass cls, jobject oSrcROI, jobjectArray quad, jobjectArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiGetPerspectiveQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (quad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'quad' is null for nppiGetPerspectiveQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiGetPerspectiveQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetPerspectiveQuad(oSrcROI=%p, quad=%p, aCoeffs=%p)\n",
        oSrcROI, quad, aCoeffs);

    // Native variable declarations
    NppiRect oSrcROI_native;
    double quad_native[4][2];
    double aCoeffs_native[3][3];

    // Obtain native variable values
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, quad, quad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetPerspectiveQuad(oSrcROI_native, quad_native, aCoeffs_native);

    // Write back native variable values
    // oSrcROI is a read-only structure
    if (!releaseNative(env, quad_native, quad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates bounding box of the perspective transform projection of the
 * given source rectangular ROI
 *
 * @param oSrcROI Source ROI
 * @param bound Bounding box of the transformed source ROI
 * @param aCoeffs Perspective transform coefficients
 * @return
 *  Error codes:
 *         - ::NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *           has zero or negative value
 *         - ::NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the oSrcROI and source image is less than or
 *           equal to 1
 *         - ::NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetPerspectiveBoundNative(JNIEnv *env, jclass cls, jobject oSrcROI, jobjectArray bound, jobjectArray aCoeffs)
{
    // Null-checks for non-primitive arguments
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiGetPerspectiveBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (bound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'bound' is null for nppiGetPerspectiveBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiGetPerspectiveBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetPerspectiveBound(oSrcROI=%p, bound=%p, aCoeffs=%p)\n",
        oSrcROI, bound, aCoeffs);

    // Native variable declarations
    NppiRect oSrcROI_native;
    double bound_native[2][2];
    double aCoeffs_native[3][3];

    // Obtain native variable values
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, bound, bound_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetPerspectiveBound(oSrcROI_native, bound_native, aCoeffs_native);

    // Write back native variable values
    // oSrcROI is a read-only structure
    if (!releaseNative(env, bound_native, bound)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 8-bit unsigned integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer perspective warp, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 8-bit unsigned integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 8-bit unsigned integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 16-bit unsigned integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit signed integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed integer perspective warp, igoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 32-bit signed integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit signed integer perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating-point perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point perspective warp, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 32-bit floating-point perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit floating-point perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 8-bit unsigned integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 8-bit unsigned integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 8-bit unsigned integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 16-bit unsigned integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 16-bit unsigned integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit signed integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed integer backwards perspective warp, ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 32-bit signed integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit signed integer backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point backwards perspective warp, ignorning alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 32-bit floating-point backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit floating-point backwards perspective warp.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aCoeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aCoeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aCoeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aCoeffs' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aCoeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, aCoeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aCoeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aCoeffs, aCoeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, aCoeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aCoeffs_native, aCoeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 8-bit unsigned integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned integer quad-based perspective warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 8-bit unsigned integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 8-bit unsigned integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 16-bit unsigned integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned integer quad-based perspective warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 16-bit unsigned integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 16-bit unsigned integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit signed integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed integer quad-based perspective warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit signed integer quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel 32-bit floating-point quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating-point quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point quad-based perspective warp, ignoring alpha channel.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobject pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel planar 32-bit floating-point quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel planar 32-bit floating-point quad-based perspective warp.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcROI Source ROI
 * @param aSrcQuad Source quad.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Destination ROI
 * @param aDstQuad Destination quad.
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref perspective_transform_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobjectArray aSrcQuad, jobjectArray pDst, jint nDstStep, jobject oDstROI, jobjectArray aDstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcQuad' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstQuad' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, aSrcQuad=%p, pDst=%p, nDstStep=%d, oDstROI=%p, aDstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, aSrcQuad, pDst, nDstStep, oDstROI, aDstQuad, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    double aSrcQuad_native[4][2];
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double aDstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcQuad, aSrcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstQuad, aDstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, aSrcQuad_native, pDst_native, nDstStep_native, oDstROI_native, aDstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aSrcQuad_native, aSrcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // oDstROI is a read-only structure
    if (!releaseNative(env, aDstQuad_native, aDstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Apply quality factor to raw 8-bit quantization table.
 *
 * This is effectively and in-place method that modifies a given raw
 * quantization table based on a quality factor.
 * Note that this method is a host method and that the pointer to the
 * raw quantization table is a host pointer.
 *
 * @param hpQuantRawTable Raw quantization table.
 * @param nQualityFactor Quality factor for the table. Range is [1:100].
 * @return
 *  Error code:
 *      ::NPP_NULL_POINTER_ERROR is returned if hpQuantRawTable is 0.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQuantFwdRawTableInit_1JPEG_18uNative(JNIEnv *env, jclass cls, jobject hpQuantRawTable, jint nQualityFactor)
{
    // Null-checks for non-primitive arguments
    if (hpQuantRawTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpQuantRawTable' is null for nppiQuantFwdRawTableInit_JPEG_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nQualityFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQuantFwdRawTableInit_JPEG_8u(hpQuantRawTable=%p, nQualityFactor=%d)\n",
        hpQuantRawTable, nQualityFactor);

    // Native variable declarations
    Npp8u* hpQuantRawTable_native = NULL;
    int nQualityFactor_native = 0;

    // Obtain native variable values
    PointerData *hpQuantRawTable_pointerData = initPointerData(env, hpQuantRawTable);
    if (hpQuantRawTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpQuantRawTable_native = (Npp8u*)hpQuantRawTable_pointerData->getPointer(env);
    nQualityFactor_native = (int)nQualityFactor;

    // Native function call
    NppStatus jniResult_native = nppiQuantFwdRawTableInit_JPEG_8u(hpQuantRawTable_native, nQualityFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, hpQuantRawTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nQualityFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQuantFwdTableInit_1JPEG_18u16uNative(JNIEnv *env, jclass cls, jobject hpQuantRawTable, jobject hpQuantFwdRawTable)
{
    // Null-checks for non-primitive arguments
    if (hpQuantRawTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpQuantRawTable' is null for nppiQuantFwdTableInit_JPEG_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpQuantFwdRawTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpQuantFwdRawTable' is null for nppiQuantFwdTableInit_JPEG_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQuantFwdTableInit_JPEG_8u16u(hpQuantRawTable=%p, hpQuantFwdRawTable=%p)\n",
        hpQuantRawTable, hpQuantFwdRawTable);

    // Native variable declarations
    Npp8u* hpQuantRawTable_native = NULL;
    Npp16u* hpQuantFwdRawTable_native = NULL;

    // Obtain native variable values
    PointerData *hpQuantRawTable_pointerData = initPointerData(env, hpQuantRawTable);
    if (hpQuantRawTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpQuantRawTable_native = (Npp8u*)hpQuantRawTable_pointerData->getPointer(env);
    PointerData *hpQuantFwdRawTable_pointerData = initPointerData(env, hpQuantFwdRawTable);
    if (hpQuantFwdRawTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpQuantFwdRawTable_native = (Npp16u*)hpQuantFwdRawTable_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQuantFwdTableInit_JPEG_8u16u(hpQuantRawTable_native, hpQuantFwdRawTable_native);

    // Write back native variable values
    if (!releasePointerData(env, hpQuantRawTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, hpQuantFwdRawTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Initializes a quantization table for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R().
 *      The nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R() method uses a quantization table
 * in a 16-bit format allowing for faster processing. In addition it converts the 
 * data order from zigzag layout to original row-order layout. Typically raw
 * quantization tables are stored in zigzag format.
 *
 * This method is a host method and consumes and produces host data. I.e. the pointers
 * passed to this function must be host pointers. The resulting table needs to be
 * transferred to device memory in order to be used with nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R()
 * function.
 *
 * @param hpQuantRawTable Raw quantization table.
 * @param hpQuantFwdRawTable Inverse quantization table.
 * @return
 *  ::NPP_NULL_POINTER_ERROR pQuantRawTable or pQuantFwdRawTable is0.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQuantInvTableInit_1JPEG_18u16uNative(JNIEnv *env, jclass cls, jobject hpQuantRawTable, jobject hpQuantFwdRawTable)
{
    // Null-checks for non-primitive arguments
    if (hpQuantRawTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpQuantRawTable' is null for nppiQuantInvTableInit_JPEG_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpQuantFwdRawTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpQuantFwdRawTable' is null for nppiQuantInvTableInit_JPEG_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQuantInvTableInit_JPEG_8u16u(hpQuantRawTable=%p, hpQuantFwdRawTable=%p)\n",
        hpQuantRawTable, hpQuantFwdRawTable);

    // Native variable declarations
    Npp8u* hpQuantRawTable_native = NULL;
    Npp16u* hpQuantFwdRawTable_native = NULL;

    // Obtain native variable values
    PointerData *hpQuantRawTable_pointerData = initPointerData(env, hpQuantRawTable);
    if (hpQuantRawTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpQuantRawTable_native = (Npp8u*)hpQuantRawTable_pointerData->getPointer(env);
    PointerData *hpQuantFwdRawTable_pointerData = initPointerData(env, hpQuantFwdRawTable);
    if (hpQuantFwdRawTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpQuantFwdRawTable_native = (Npp16u*)hpQuantFwdRawTable_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQuantInvTableInit_JPEG_8u16u(hpQuantRawTable_native, hpQuantFwdRawTable_native);

    // Write back native variable values
    if (!releasePointerData(env, hpQuantRawTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, hpQuantFwdRawTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDCTQuantFwd8x8LS_1JPEG_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject pQuantFwdTable, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pQuantFwdTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pQuantFwdTable' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, pQuantFwdTable=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, pQuantFwdTable, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp16u* pQuantFwdTable_native = NULL;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pQuantFwdTable_pointerData = initPointerData(env, pQuantFwdTable);
    if (pQuantFwdTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pQuantFwdTable_native = (Npp16u*)pQuantFwdTable_pointerData->getPointer(env);
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, pQuantFwdTable_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pQuantFwdTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDCTQuantInv8x8LS_1JPEG_116s8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject pQuantInvTable, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pQuantInvTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pQuantInvTable' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, pQuantInvTable=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, pQuantInvTable, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp16u* pQuantInvTable_native = NULL;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pQuantInvTable_pointerData = initPointerData(env, pQuantInvTable);
    if (pQuantInvTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pQuantInvTable_native = (Npp16u*)pQuantInvTable_pointerData->getPointer(env);
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, pQuantInvTable_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pQuantInvTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Initializes DCT state structure and allocates additional resources.
 *
 * @see nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEW(), nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEW.
 * 
 * @param ppState Pointer to pointer to DCT state structure. 
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDCTInitAllocNative(JNIEnv *env, jclass cls, jobject ppState)
{
    // Null-checks for non-primitive arguments
    if (ppState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'ppState' is null for nppiDCTInitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDCTInitAlloc(ppState=%p)\n",
        ppState);

    // Native variable declarations
    NppiDCTState* ppState_native;

    // Obtain native variable values
    // ppState is write-only

    // Native function call
    NppStatus jniResult_native = nppiDCTInitAlloc(&ppState_native);

    // Write back native variable values
    setNativePointerValue(env, ppState, (jlong)ppState_native);

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Frees the additional resources of the DCT state structure.
 *
 * @see nppiDCTInitAlloc
 * 
 * @param pState Pointer to DCT state structure. 
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pState
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDCTFreeNative(JNIEnv *env, jclass cls, jobject pState)
{
    // Null-checks for non-primitive arguments
    if (pState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pState' is null for nppiDCTFree");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDCTFree(pState=%p)\n",
        pState);

    // Native variable declarations
    NppiDCTState* pState_native;

    // Obtain native variable values
    pState_native = (NppiDCTState*)getNativePointerValue(env, pState);

    // Native function call
    NppStatus jniResult_native = nppiDCTFree(pState_native);

    // Write back native variable values
    // pState is a read-only native pointer

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDCTQuantFwd8x8LS_1JPEG_18u16s_1C1R_1NEWNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject pQuantizationTable, jobject oSizeROI, jobject pState)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEW");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEW");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pQuantizationTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pQuantizationTable' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEW");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEW");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pState' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEW");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEW(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, pQuantizationTable=%p, oSizeROI=%p, pState=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, pQuantizationTable, oSizeROI, pState);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pQuantizationTable_native = NULL;
    NppiSize oSizeROI_native;
    NppiDCTState* pState_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pQuantizationTable_pointerData = initPointerData(env, pQuantizationTable);
    if (pQuantizationTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pQuantizationTable_native = (Npp8u*)pQuantizationTable_pointerData->getPointer(env);
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    pState_native = (NppiDCTState*)getNativePointerValue(env, pState);

    // Native function call
    NppStatus jniResult_native = nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R_NEW(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, pQuantizationTable_native, oSizeROI_native, pState_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pQuantizationTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure
    // pState is a read-only native pointer

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse DCT, de-quantization and level shift part of the JPEG decoding.
 * Input is expected in 64x1 macro blocks and output is expected to be in 8x8
 * macro blocks. The new version of the primitive takes the ROI in image pixel size and
 * works with DCT coefficients that are in zig-zag order.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep Image width in pixels x 8 x sizeof(Npp16s).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pQuantizationTable Quantization Table in zig-zag order.
 * @param oSizeROI \ref roi_specification.
 * @param pState Pointer to DCT state structure. This structure must be
 *          initialized allocated and initialized using nppiDCTInitAlloc().  
 * @return
 *  Error codes:
 *         - ::NPP_SIZE_ERROR For negative input height/width or not a multiple of
 *           8 width/height.
 *         - ::NPP_STEP_ERROR If input image width is not multiple of 8 or does not
 *           match ROI.
 *         - ::NPP_NULL_POINTER_ERROR If the destination pointer is 0.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDCTQuantInv8x8LS_1JPEG_116s8u_1C1R_1NEWNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject pQuantizationTable, jobject oSizeROI, jobject pState)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEW");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEW");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pQuantizationTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pQuantizationTable' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEW");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEW");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pState' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEW");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEW(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, pQuantizationTable=%p, oSizeROI=%p, pState=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, pQuantizationTable, oSizeROI, pState);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp8u* pQuantizationTable_native = NULL;
    NppiSize oSizeROI_native;
    NppiDCTState* pState_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pQuantizationTable_pointerData = initPointerData(env, pQuantizationTable);
    if (pQuantizationTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pQuantizationTable_native = (Npp8u*)pQuantizationTable_pointerData->getPointer(env);
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    pState_native = (NppiDCTState*)getNativePointerValue(env, pState);

    // Native function call
    NppStatus jniResult_native = nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R_NEW(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, pQuantizationTable_native, oSizeROI_native, pState_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pQuantizationTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure
    // pState is a read-only native pointer

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Returns the length of the NppiDecodeHuffmanSpec structure.
 * @param pSize Pointer to a variable that will receive the length of the NppiDecodeHuffmanSpec structure.
 * @return
 *  Error codes:
 *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
 * *
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDecodeHuffmanSpecGetBufSize_1JPEGNative(JNIEnv *env, jclass cls, jintArray pSize)
{
    // Null-checks for non-primitive arguments
    if (pSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSize' is null for nppiDecodeHuffmanSpecGetBufSize_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDecodeHuffmanSpecGetBufSize_JPEG(pSize=%p)\n",
        pSize);

    // Native variable declarations
    int* pSize_native;

    // Obtain native variable values
    PointerData *pSize_pointerData = initPointerData(env, pSize);
    if (pSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSize_native = (int*)pSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDecodeHuffmanSpecGetBufSize_JPEG(pSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Creates a Huffman table in a format that is suitable for the decoder on the host.
 * @param pRawHuffmanTable Huffman table formated as specified in the JPEG standard.
 * @param eTableType Enum specifying type of table (nppiDCTable or nppiACTable).
 * @param pHuffmanSpec Pointer to the Huffman table for the decoder
 * @return
 *  Error codes:
 *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
 * *
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDecodeHuffmanSpecInitHost_1JPEGNative(JNIEnv *env, jclass cls, jobject pRawHuffmanTable, jint eTableType, jobject pHuffmanSpec)
{
    // Null-checks for non-primitive arguments
    if (pRawHuffmanTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRawHuffmanTable' is null for nppiDecodeHuffmanSpecInitHost_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eTableType is primitive
    if (pHuffmanSpec == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHuffmanSpec' is null for nppiDecodeHuffmanSpecInitHost_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDecodeHuffmanSpecInitHost_JPEG(pRawHuffmanTable=%p, eTableType=%d, pHuffmanSpec=%p)\n",
        pRawHuffmanTable, eTableType, pHuffmanSpec);

    // Native variable declarations
    Npp8u* pRawHuffmanTable_native = NULL;
    NppiHuffmanTableType eTableType_native;
    NppiDecodeHuffmanSpec* pHuffmanSpec_native;

    // Obtain native variable values
    PointerData *pRawHuffmanTable_pointerData = initPointerData(env, pRawHuffmanTable);
    if (pRawHuffmanTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRawHuffmanTable_native = (Npp8u*)pRawHuffmanTable_pointerData->getPointer(env);
    eTableType_native = (NppiHuffmanTableType)eTableType;
    PointerData *pHuffmanSpec_pointerData = initPointerData(env, pHuffmanSpec);
    if (pHuffmanSpec_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHuffmanSpec_native = (NppiDecodeHuffmanSpec*)pHuffmanSpec_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDecodeHuffmanSpecInitHost_JPEG(pRawHuffmanTable_native, eTableType_native, pHuffmanSpec_native);

    // Write back native variable values
    if (!releasePointerData(env, pRawHuffmanTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // eTableType is primitive
    if (!releasePointerData(env, pHuffmanSpec_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Allocates memory and creates a Huffman table in a format that is suitable for the decoder on the host.
 * @param pRawHuffmanTable Huffman table formated as specified in the JPEG standard.
 * @param eTableType Enum specifying type of table (nppiDCTable or nppiACTable).
 * @param ppHuffmanSpec Pointer to returned pointer to the Huffman table for the decoder
 * @return
 *  Error codes:
 *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
 * *
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDecodeHuffmanSpecInitAllocHost_1JPEGNative(JNIEnv *env, jclass cls, jobject pRawHuffmanTable, jint eTableType, jobject ppHuffmanSpec)
{
    // Null-checks for non-primitive arguments
    if (pRawHuffmanTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRawHuffmanTable' is null for nppiDecodeHuffmanSpecInitAllocHost_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eTableType is primitive
    if (ppHuffmanSpec == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'ppHuffmanSpec' is null for nppiDecodeHuffmanSpecInitAllocHost_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDecodeHuffmanSpecInitAllocHost_JPEG(pRawHuffmanTable=%p, eTableType=%d, ppHuffmanSpec=%p)\n",
        pRawHuffmanTable, eTableType, ppHuffmanSpec);

    // Native variable declarations
    Npp8u* pRawHuffmanTable_native = NULL;
    NppiHuffmanTableType eTableType_native;
    NppiDecodeHuffmanSpec* ppHuffmanSpec_native;

    // Obtain native variable values
    PointerData *pRawHuffmanTable_pointerData = initPointerData(env, pRawHuffmanTable);
    if (pRawHuffmanTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRawHuffmanTable_native = (Npp8u*)pRawHuffmanTable_pointerData->getPointer(env);
    eTableType_native = (NppiHuffmanTableType)eTableType;
    // ppHuffmanSpec is write-only

    // Native function call
    NppStatus jniResult_native = nppiDecodeHuffmanSpecInitAllocHost_JPEG(pRawHuffmanTable_native, eTableType_native, &ppHuffmanSpec_native);

    // Write back native variable values
    if (!releasePointerData(env, pRawHuffmanTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // eTableType is primitive
    setNativePointerValue(env, ppHuffmanSpec, (jlong)ppHuffmanSpec_native);

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDecodeHuffmanSpecFreeHost_1JPEGNative(JNIEnv *env, jclass cls, jobject pHuffmanSpec)
{
    // Null-checks for non-primitive arguments
    if (pHuffmanSpec == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHuffmanSpec' is null for nppiDecodeHuffmanSpecFreeHost_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDecodeHuffmanSpecFreeHost_JPEG(pHuffmanSpec=%p)\n",
        pHuffmanSpec);

    // Native variable declarations
    NppiDecodeHuffmanSpec* pHuffmanSpec_native;

    // Obtain native variable values
    PointerData *pHuffmanSpec_pointerData = initPointerData(env, pHuffmanSpec);
    if (pHuffmanSpec_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHuffmanSpec_native = (NppiDecodeHuffmanSpec*)pHuffmanSpec_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiDecodeHuffmanSpecFreeHost_JPEG(pHuffmanSpec_native);

    // Write back native variable values
    if (!releasePointerData(env, pHuffmanSpec_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Huffman Decoding of the JPEG decoding on the host.
 * Input is expected in byte stuffed huffman encoded JPEG scan and output is expected to be 64x1 macro blocks.
 *
 * @param pSrc Byte-stuffed huffman encoded JPEG scan.
 * @param nLength Byte length of the input.
 * @param restartInterval Restart Interval, see JPEG standard.
 * @param Ss Start Coefficient, see JPEG standard.
 * @param Se End Coefficient, see JPEG standard.
 * @param Ah Bit Approximation High, see JPEG standard.
 * @param Al Bit Approximation Low, see JPEG standard.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pHuffmanTableDC DC Huffman table.
 * @param pHuffmanTableAC AC Huffman table.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  Error codes:
 *         - ::NPP_SIZE_ERROR For negative input height/width or not a multiple of
 *           8 width/height.
 *         - ::NPP_STEP_ERROR If input image width is not multiple of 8 or does not
 *           match ROI.
 *         - ::NPP_NULL_POINTER_ERROR If the destination pointer is 0.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDecodeHuffmanScanHost_1JPEG_18u16s_1P1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jint restartInterval, jint Ss, jint Se, jint Ah, jint Al, jobject pDst, jint nDstStep, jobject pHuffmanTableDC, jobject pHuffmanTableAC, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDecodeHuffmanScanHost_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // restartInterval is primitive
    // Ss is primitive
    // Se is primitive
    // Ah is primitive
    // Al is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDecodeHuffmanScanHost_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pHuffmanTableDC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHuffmanTableDC' is null for nppiDecodeHuffmanScanHost_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHuffmanTableAC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHuffmanTableAC' is null for nppiDecodeHuffmanScanHost_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDecodeHuffmanScanHost_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDecodeHuffmanScanHost_JPEG_8u16s_P1R(pSrc=%p, nLength=%d, restartInterval=%d, Ss=%d, Se=%d, Ah=%d, Al=%d, pDst=%p, nDstStep=%d, pHuffmanTableDC=%p, pHuffmanTableAC=%p, oSizeROI=%p)\n",
        pSrc, nLength, restartInterval, Ss, Se, Ah, Al, pDst, nDstStep, pHuffmanTableDC, pHuffmanTableAC, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nLength_native;
    Npp32s restartInterval_native;
    Npp32s Ss_native;
    Npp32s Se_native;
    Npp32s Ah_native;
    Npp32s Al_native;
    Npp16s* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiDecodeHuffmanSpec* pHuffmanTableDC_native;
    NppiDecodeHuffmanSpec* pHuffmanTableAC_native;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nLength_native = (Npp32s)nLength;
    restartInterval_native = (Npp32s)restartInterval;
    Ss_native = (Npp32s)Ss;
    Se_native = (Npp32s)Se;
    Ah_native = (Npp32s)Ah;
    Al_native = (Npp32s)Al;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    PointerData *pHuffmanTableDC_pointerData = initPointerData(env, pHuffmanTableDC);
    if (pHuffmanTableDC_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHuffmanTableDC_native = (NppiDecodeHuffmanSpec*)pHuffmanTableDC_pointerData->getPointer(env);
    PointerData *pHuffmanTableAC_pointerData = initPointerData(env, pHuffmanTableAC);
    if (pHuffmanTableAC_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHuffmanTableAC_native = (NppiDecodeHuffmanSpec*)pHuffmanTableAC_pointerData->getPointer(env);
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDecodeHuffmanScanHost_JPEG_8u16s_P1R(pSrc_native, nLength_native, restartInterval_native, Ss_native, Se_native, Ah_native, Al_native, pDst_native, nDstStep_native, pHuffmanTableDC_native, pHuffmanTableAC_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // restartInterval is primitive
    // Ss is primitive
    // Se is primitive
    // Ah is primitive
    // Al is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pHuffmanTableDC_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pHuffmanTableAC_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDecodeHuffmanScanHost_1JPEG_18u16s_1P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jint nRestartInterval, jint nSs, jint nSe, jint nAh, jint nAl, jobjectArray apDst, jintArray aDstStep, jobjectArray apHuffmanDCTable, jobjectArray apHuffmanACTable, jobjectArray aSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDecodeHuffmanScanHost_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nRestartInterval is primitive
    // nSs is primitive
    // nSe is primitive
    // nAh is primitive
    // nAl is primitive
    if (apDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'apDst' is null for nppiDecodeHuffmanScanHost_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstStep' is null for nppiDecodeHuffmanScanHost_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (apHuffmanDCTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'apHuffmanDCTable' is null for nppiDecodeHuffmanScanHost_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (apHuffmanACTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'apHuffmanACTable' is null for nppiDecodeHuffmanScanHost_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSizeROI' is null for nppiDecodeHuffmanScanHost_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDecodeHuffmanScanHost_JPEG_8u16s_P3R(pSrc=%p, nLength=%d, nRestartInterval=%d, nSs=%d, nSe=%d, nAh=%d, nAl=%d, apDst=%p, aDstStep=%p, apHuffmanDCTable=%p, apHuffmanACTable=%p, aSizeROI=%p)\n",
        pSrc, nLength, nRestartInterval, nSs, nSe, nAh, nAl, apDst, aDstStep, apHuffmanDCTable, apHuffmanACTable, aSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nLength_native;
    Npp32s nRestartInterval_native;
    Npp32s nSs_native;
    Npp32s nSe_native;
    Npp32s nAh_native;
    Npp32s nAl_native;
    Npp16s* apDst_native[3];
    Npp32s aDstStep_native[3];
    NppiDecodeHuffmanSpec* apHuffmanDCTable_native[3];
    NppiDecodeHuffmanSpec* apHuffmanACTable_native[3];
    NppiSize aSizeROI_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nLength_native = (Npp32s)nLength;
    nRestartInterval_native = (Npp32s)nRestartInterval;
    nSs_native = (Npp32s)nSs;
    nSe_native = (Npp32s)nSe;
    nAh_native = (Npp32s)nAh;
    nAl_native = (Npp32s)nAl;
    if (!initNative(env, apDst, apDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstStep, aDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, apHuffmanDCTable, apHuffmanDCTable_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, apHuffmanACTable, apHuffmanACTable_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSizeROI, aSizeROI_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDecodeHuffmanScanHost_JPEG_8u16s_P3R(pSrc_native, nLength_native, nRestartInterval_native, nSs_native, nSe_native, nAh_native, nAl_native, apDst_native, aDstStep_native, apHuffmanDCTable_native, apHuffmanACTable_native, aSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nRestartInterval is primitive
    // nSs is primitive
    // nSe is primitive
    // nAh is primitive
    // nAl is primitive
    // apDst is an array of native pointers
    if (!releaseNative(env, aDstStep_native, aDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // apHuffmanDCTable is an array of native pointers
    // apHuffmanACTable is an array of native pointers
    // aSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Returns the length of the NppiEncodeHuffmanSpec structure.
 * @param pSize Pointer to a variable that will receive the length of the NppiEncodeHuffmanSpec structure.
 * @return
 *  Error codes:
 *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
 * *
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEncodeHuffmanSpecGetBufSize_1JPEGNative(JNIEnv *env, jclass cls, jintArray pSize)
{
    // Null-checks for non-primitive arguments
    if (pSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSize' is null for nppiEncodeHuffmanSpecGetBufSize_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEncodeHuffmanSpecGetBufSize_JPEG(pSize=%p)\n",
        pSize);

    // Native variable declarations
    int* pSize_native;

    // Obtain native variable values
    PointerData *pSize_pointerData = initPointerData(env, pSize);
    if (pSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSize_native = (int*)pSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiEncodeHuffmanSpecGetBufSize_JPEG(pSize_native);

    // Write back native variable values
    if (!releasePointerData(env, pSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Creates a Huffman table in a format that is suitable for the encoder.
 * @param pRawHuffmanTable Huffman table formated as specified in the JPEG standard.
 * @param eTableType Enum specifying type of table (nppiDCTable or nppiACTable).
 * @param pHuffmanSpec Pointer to the Huffman table for the decoder
 * @return
 *  Error codes:
 *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
 * *
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEncodeHuffmanSpecInit_1JPEGNative(JNIEnv *env, jclass cls, jobject pRawHuffmanTable, jint eTableType, jobject pHuffmanSpec)
{
    // Null-checks for non-primitive arguments
    if (pRawHuffmanTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRawHuffmanTable' is null for nppiEncodeHuffmanSpecInit_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eTableType is primitive
    if (pHuffmanSpec == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHuffmanSpec' is null for nppiEncodeHuffmanSpecInit_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEncodeHuffmanSpecInit_JPEG(pRawHuffmanTable=%p, eTableType=%d, pHuffmanSpec=%p)\n",
        pRawHuffmanTable, eTableType, pHuffmanSpec);

    // Native variable declarations
    Npp8u* pRawHuffmanTable_native = NULL;
    NppiHuffmanTableType eTableType_native;
    NppiEncodeHuffmanSpec* pHuffmanSpec_native;

    // Obtain native variable values
    PointerData *pRawHuffmanTable_pointerData = initPointerData(env, pRawHuffmanTable);
    if (pRawHuffmanTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRawHuffmanTable_native = (Npp8u*)pRawHuffmanTable_pointerData->getPointer(env);
    eTableType_native = (NppiHuffmanTableType)eTableType;
    pHuffmanSpec_native = (NppiEncodeHuffmanSpec*)getNativePointerValue(env, pHuffmanSpec);

    // Native function call
    NppStatus jniResult_native = nppiEncodeHuffmanSpecInit_JPEG(pRawHuffmanTable_native, eTableType_native, pHuffmanSpec_native);

    // Write back native variable values
    if (!releasePointerData(env, pRawHuffmanTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // eTableType is primitive
    // pHuffmanSpec is a read-only native pointer

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Allocates memory and creates a Huffman table in a format that is suitable for the encoder.
 * @param pRawHuffmanTable Huffman table formated as specified in the JPEG standard.
 * @param eTableType Enum specifying type of table (nppiDCTable or nppiACTable).
 * @param ppHuffmanSpec Pointer to returned pointer to the Huffman table for the encoder
 * @return
 *  Error codes:
 *         - ::NPP_NULL_POINTER_ERROR If one of the pointers is 0.
 * *
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEncodeHuffmanSpecInitAlloc_1JPEGNative(JNIEnv *env, jclass cls, jobject pRawHuffmanTable, jint eTableType, jobject ppHuffmanSpec)
{
    // Null-checks for non-primitive arguments
    if (pRawHuffmanTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRawHuffmanTable' is null for nppiEncodeHuffmanSpecInitAlloc_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eTableType is primitive
    if (ppHuffmanSpec == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'ppHuffmanSpec' is null for nppiEncodeHuffmanSpecInitAlloc_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEncodeHuffmanSpecInitAlloc_JPEG(pRawHuffmanTable=%p, eTableType=%d, ppHuffmanSpec=%p)\n",
        pRawHuffmanTable, eTableType, ppHuffmanSpec);

    // Native variable declarations
    Npp8u* pRawHuffmanTable_native = NULL;
    NppiHuffmanTableType eTableType_native;
    NppiEncodeHuffmanSpec* ppHuffmanSpec_native;

    // Obtain native variable values
    PointerData *pRawHuffmanTable_pointerData = initPointerData(env, pRawHuffmanTable);
    if (pRawHuffmanTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRawHuffmanTable_native = (Npp8u*)pRawHuffmanTable_pointerData->getPointer(env);
    eTableType_native = (NppiHuffmanTableType)eTableType;
    // ppHuffmanSpec is write-only

    // Native function call
    NppStatus jniResult_native = nppiEncodeHuffmanSpecInitAlloc_JPEG(pRawHuffmanTable_native, eTableType_native, &ppHuffmanSpec_native);

    // Write back native variable values
    if (!releasePointerData(env, pRawHuffmanTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // eTableType is primitive
    setNativePointerValue(env, ppHuffmanSpec, (jlong)ppHuffmanSpec_native);

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEncodeHuffmanSpecFree_1JPEGNative(JNIEnv *env, jclass cls, jobject pHuffmanSpec)
{
    // Null-checks for non-primitive arguments
    if (pHuffmanSpec == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHuffmanSpec' is null for nppiEncodeHuffmanSpecFree_JPEG");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEncodeHuffmanSpecFree_JPEG(pHuffmanSpec=%p)\n",
        pHuffmanSpec);

    // Native variable declarations
    NppiEncodeHuffmanSpec* pHuffmanSpec_native;

    // Obtain native variable values
    pHuffmanSpec_native = (NppiEncodeHuffmanSpec*)getNativePointerValue(env, pHuffmanSpec);

    // Native function call
    NppStatus jniResult_native = nppiEncodeHuffmanSpecFree_JPEG(pHuffmanSpec_native);

    // Write back native variable values
    // pHuffmanSpec is a read-only native pointer

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Huffman Encoding of the JPEG Encoding.
 * Input is expected to be 64x1 macro blocks and output is expected as byte stuffed huffman encoded JPEG scan.
 *
 * @param pSrc \ref destination_image_pointer.
 * @param nSrcStep \ref destination_image_line_step.
 * @param nRestartInterval Restart Interval, see JPEG standard. Currently only values <=0 are supported.
 * @param nSs Start Coefficient, see JPEG standard.
 * @param nSe End Coefficient, see JPEG standard.
 * @param nAh Bit Approximation High, see JPEG standard.
 * @param nAl Bit Approximation Low, see JPEG standard.
 * @param pDst Byte-stuffed huffman encoded JPEG scan.
 * @param nLength Byte length of the huffman encoded JPEG scan.
 * @param pHuffmanTableDC DC Huffman table.
 * @param pHuffmanTableAC AC Huffman table.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  Error codes:
 *         - ::NPP_SIZE_ERROR For negative input height/width or not a multiple of
 *           8 width/height.
 *         - ::NPP_STEP_ERROR If input image width is not multiple of 8 or does not
 *           match ROI.
 *         - ::NPP_NULL_POINTER_ERROR If the destination pointer is 0.
 *         - ::NPP_NOT_SUFFICIENT_COMPUTE_CAPABILITY If the device has compute capability < 2.0. 
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEncodeHuffmanScan_1JPEG_18u16s_1P1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jint restartInterval, jint Ss, jint Se, jint Ah, jint Al, jobject pDst, jobject nLength, jobject pHuffmanTableDC, jobject pHuffmanTableAC, jobject oSizeROI, jobject pTempStorage)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    // restartInterval is primitive
    // Ss is primitive
    // Se is primitive
    // Ah is primitive
    // Al is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLength == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLength' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHuffmanTableDC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHuffmanTableDC' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHuffmanTableAC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHuffmanTableAC' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTempStorage == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTempStorage' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEncodeHuffmanScan_JPEG_8u16s_P1R(pSrc=%p, nSrcStep=%d, restartInterval=%d, Ss=%d, Se=%d, Ah=%d, Al=%d, pDst=%p, nLength=%p, pHuffmanTableDC=%p, pHuffmanTableAC=%p, oSizeROI=%p, pTempStorage=%p)\n",
        pSrc, nSrcStep, restartInterval, Ss, Se, Ah, Al, pDst, nLength, pHuffmanTableDC, pHuffmanTableAC, oSizeROI, pTempStorage);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32s restartInterval_native;
    Npp32s Ss_native;
    Npp32s Se_native;
    Npp32s Ah_native;
    Npp32s Al_native;
    Npp8u* pDst_native = NULL;
    Npp32s* nLength_native = NULL;
    NppiEncodeHuffmanSpec* pHuffmanTableDC_native;
    NppiEncodeHuffmanSpec* pHuffmanTableAC_native;
    NppiSize oSizeROI_native;
    Npp8u* pTempStorage_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    restartInterval_native = (Npp32s)restartInterval;
    Ss_native = (Npp32s)Ss;
    Se_native = (Npp32s)Se;
    Ah_native = (Npp32s)Ah;
    Al_native = (Npp32s)Al;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    PointerData *nLength_pointerData = initPointerData(env, nLength);
    if (nLength_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nLength_native = (Npp32s*)nLength_pointerData->getPointer(env);
    pHuffmanTableDC_native = (NppiEncodeHuffmanSpec*)getNativePointerValue(env, pHuffmanTableDC);
    pHuffmanTableAC_native = (NppiEncodeHuffmanSpec*)getNativePointerValue(env, pHuffmanTableAC);
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTempStorage_pointerData = initPointerData(env, pTempStorage);
    if (pTempStorage_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTempStorage_native = (Npp8u*)pTempStorage_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiEncodeHuffmanScan_JPEG_8u16s_P1R(pSrc_native, nSrcStep_native, restartInterval_native, Ss_native, Se_native, Ah_native, Al_native, pDst_native, nLength_native, pHuffmanTableDC_native, pHuffmanTableAC_native, oSizeROI_native, pTempStorage_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // restartInterval is primitive
    // Ss is primitive
    // Se is primitive
    // Ah is primitive
    // Al is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, nLength_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // pHuffmanTableDC is a read-only native pointer
    // pHuffmanTableAC is a read-only native pointer
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pTempStorage_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Huffman Encoding of the JPEG Encoding.
 * Input is expected to be 64x1 macro blocks and output is expected as byte stuffed huffman encoded JPEG scan.
 *
 * @param apSrc \ref destination_image_pointer.
 * @param aSrcStep \ref destination_image_line_step.
 * @param nRestartInterval Restart Interval, see JPEG standard. Currently only values <=0 are supported.
 * @param nSs Start Coefficient, see JPEG standard.
 * @param nSe End Coefficient, see JPEG standard.
 * @param nAh Bit Approximation High, see JPEG standard.
 * @param nAl Bit Approximation Low, see JPEG standard.
 * @param pDst Byte-stuffed huffman encoded JPEG scan.
 * @param nLength Byte length of the huffman encoded JPEG scan.
 * @param apHuffmanTableDC DC Huffman tables.
 * @param apHuffmanTableAC AC Huffman tables.
 * @param aSizeROI \ref roi_specification.
 * @return
 *  Error codes:
 *         - ::NPP_SIZE_ERROR For negative input height/width or not a multiple of
 *           8 width/height.
 *         - ::NPP_STEP_ERROR If input image width is not multiple of 8 or does not
 *           match ROI.
 *         - ::NPP_NULL_POINTER_ERROR If the destination pointer is 0.
 *         - ::NPP_NOT_SUFFICIENT_COMPUTE_CAPABILITY If the device has compute capability < 2.0.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEncodeHuffmanScan_1JPEG_18u16s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray apSrc, jintArray aSrcStep, jint nRestartInterval, jint nSs, jint nSe, jint nAh, jint nAl, jobject pDst, jobject nLength, jobjectArray apHuffmanDCTable, jobjectArray apHuffmanACTable, jobjectArray aSizeROI, jobject pTempStorage)
{
    // Null-checks for non-primitive arguments
    if (apSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'apSrc' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcStep' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nRestartInterval is primitive
    // nSs is primitive
    // nSe is primitive
    // nAh is primitive
    // nAl is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLength == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLength' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (apHuffmanDCTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'apHuffmanDCTable' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (apHuffmanACTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'apHuffmanACTable' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSizeROI' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTempStorage == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTempStorage' is null for nppiEncodeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEncodeHuffmanScan_JPEG_8u16s_P3R(apSrc=%p, aSrcStep=%p, nRestartInterval=%d, nSs=%d, nSe=%d, nAh=%d, nAl=%d, pDst=%p, nLength=%p, apHuffmanDCTable=%p, apHuffmanACTable=%p, aSizeROI=%p, pTempStorage=%p)\n",
        apSrc, aSrcStep, nRestartInterval, nSs, nSe, nAh, nAl, pDst, nLength, apHuffmanDCTable, apHuffmanACTable, aSizeROI, pTempStorage);

    // Native variable declarations
    Npp16s* apSrc_native[3];
    Npp32s aSrcStep_native[3];
    Npp32s nRestartInterval_native;
    Npp32s nSs_native;
    Npp32s nSe_native;
    Npp32s nAh_native;
    Npp32s nAl_native;
    Npp8u* pDst_native = NULL;
    Npp32s* nLength_native = NULL;
    NppiEncodeHuffmanSpec* apHuffmanDCTable_native[3];
    NppiEncodeHuffmanSpec* apHuffmanACTable_native[3];
    NppiSize aSizeROI_native[3];
    Npp8u* pTempStorage_native = NULL;

    // Obtain native variable values
    if (!initNative(env, apSrc, apSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcStep, aSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nRestartInterval_native = (Npp32s)nRestartInterval;
    nSs_native = (Npp32s)nSs;
    nSe_native = (Npp32s)nSe;
    nAh_native = (Npp32s)nAh;
    nAl_native = (Npp32s)nAl;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    PointerData *nLength_pointerData = initPointerData(env, nLength);
    if (nLength_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    nLength_native = (Npp32s*)nLength_pointerData->getPointer(env);
    if (!initNative(env, apHuffmanDCTable, apHuffmanDCTable_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, apHuffmanACTable, apHuffmanACTable_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSizeROI, aSizeROI_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTempStorage_pointerData = initPointerData(env, pTempStorage);
    if (pTempStorage_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTempStorage_native = (Npp8u*)pTempStorage_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiEncodeHuffmanScan_JPEG_8u16s_P3R(apSrc_native, aSrcStep_native, nRestartInterval_native, nSs_native, nSe_native, nAh_native, nAl_native, pDst_native, nLength_native, apHuffmanDCTable_native, apHuffmanACTable_native, aSizeROI_native, pTempStorage_native);

    // Write back native variable values
    // apSrc is an array of native pointers
    if (!releaseNative(env, aSrcStep_native, aSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // nRestartInterval is primitive
    // nSs is primitive
    // nSe is primitive
    // nAh is primitive
    // nAl is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, nLength_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // apHuffmanDCTable is an array of native pointers
    // apHuffmanACTable is an array of native pointers
    // aSizeROI is a read-only structure
    if (!releasePointerData(env, pTempStorage_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEncodeOptimizeHuffmanScan_1JPEG_18u16s_1P1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jint nRestartInterval, jint nSs, jint nSe, jint nAh, jint nAl, jobject pDst, jobject pLength, jobject hpCodesDC, jobject hpTableDC, jobject hpCodesAC, jobject hpTableAC, jobject aHuffmanDCTable, jobject aHuffmanACTable, jobject oSizeROI, jobject pTempStorage)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    // nRestartInterval is primitive
    // nSs is primitive
    // nSe is primitive
    // nAh is primitive
    // nAl is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLength == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLength' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpCodesDC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpCodesDC' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpTableDC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpTableDC' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpCodesAC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpCodesAC' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpTableAC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpTableAC' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aHuffmanDCTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aHuffmanDCTable' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aHuffmanACTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aHuffmanACTable' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTempStorage == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTempStorage' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R(pSrc=%p, nSrcStep=%d, nRestartInterval=%d, nSs=%d, nSe=%d, nAh=%d, nAl=%d, pDst=%p, pLength=%p, hpCodesDC=%p, hpTableDC=%p, hpCodesAC=%p, hpTableAC=%p, aHuffmanDCTable=%p, aHuffmanACTable=%p, oSizeROI=%p, pTempStorage=%p)\n",
        pSrc, nSrcStep, nRestartInterval, nSs, nSe, nAh, nAl, pDst, pLength, hpCodesDC, hpTableDC, hpCodesAC, hpTableAC, aHuffmanDCTable, aHuffmanACTable, oSizeROI, pTempStorage);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32s nRestartInterval_native;
    Npp32s nSs_native;
    Npp32s nSe_native;
    Npp32s nAh_native;
    Npp32s nAl_native;
    Npp8u* pDst_native = NULL;
    Npp32s* pLength_native = NULL;
    Npp8u* hpCodesDC_native = NULL;
    Npp8u* hpTableDC_native = NULL;
    Npp8u* hpCodesAC_native = NULL;
    Npp8u* hpTableAC_native = NULL;
    NppiEncodeHuffmanSpec* aHuffmanDCTable_native;
    NppiEncodeHuffmanSpec* aHuffmanACTable_native;
    NppiSize oSizeROI_native;
    Npp8u* pTempStorage_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    nRestartInterval_native = (Npp32s)nRestartInterval;
    nSs_native = (Npp32s)nSs;
    nSe_native = (Npp32s)nSe;
    nAh_native = (Npp32s)nAh;
    nAl_native = (Npp32s)nAl;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    PointerData *pLength_pointerData = initPointerData(env, pLength);
    if (pLength_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLength_native = (Npp32s*)pLength_pointerData->getPointer(env);
    PointerData *hpCodesDC_pointerData = initPointerData(env, hpCodesDC);
    if (hpCodesDC_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpCodesDC_native = (Npp8u*)hpCodesDC_pointerData->getPointer(env);
    PointerData *hpTableDC_pointerData = initPointerData(env, hpTableDC);
    if (hpTableDC_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpTableDC_native = (Npp8u*)hpTableDC_pointerData->getPointer(env);
    PointerData *hpCodesAC_pointerData = initPointerData(env, hpCodesAC);
    if (hpCodesAC_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpCodesAC_native = (Npp8u*)hpCodesAC_pointerData->getPointer(env);
    PointerData *hpTableAC_pointerData = initPointerData(env, hpTableAC);
    if (hpTableAC_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpTableAC_native = (Npp8u*)hpTableAC_pointerData->getPointer(env);
    aHuffmanDCTable_native = (NppiEncodeHuffmanSpec*)getNativePointerValue(env, aHuffmanDCTable);
    aHuffmanACTable_native = (NppiEncodeHuffmanSpec*)getNativePointerValue(env, aHuffmanACTable);
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTempStorage_pointerData = initPointerData(env, pTempStorage);
    if (pTempStorage_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTempStorage_native = (Npp8u*)pTempStorage_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P1R(pSrc_native, nSrcStep_native, nRestartInterval_native, nSs_native, nSe_native, nAh_native, nAl_native, pDst_native, pLength_native, hpCodesDC_native, hpTableDC_native, hpCodesAC_native, hpTableAC_native, aHuffmanDCTable_native, aHuffmanACTable_native, oSizeROI_native, pTempStorage_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // nRestartInterval is primitive
    // nSs is primitive
    // nSe is primitive
    // nAh is primitive
    // nAl is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLength_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, hpCodesDC_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, hpTableDC_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, hpCodesAC_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, hpTableAC_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // aHuffmanDCTable is a read-only native pointer
    // aHuffmanACTable is a read-only native pointer
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pTempStorage_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Optimize Huffman Encoding of the JPEG Encoding.
 * Input is expected to be 64x1 macro blocks and output is expected as byte stuffed huffman encoded JPEG scan.
 *
 * @param apSrc \ref destination_image_pointer.
 * @param aSrcStep \ref destination_image_line_step.
 * @param nRestartInterval Restart Interval, see JPEG standard. Currently only values <=0 are supported.
 * @param nSs Start Coefficient, see JPEG standard.
 * @param nSe End Coefficient, see JPEG standard.
 * @param nAh Bit Approximation High, see JPEG standard.
 * @param nAl Bit Approximation Low, see JPEG standard.
 * @param pDst Byte-stuffed huffman encoded JPEG scan.
 * @param pLength Pointer to the byte length of the huffman encoded JPEG scan.
 * @param hpCodesDC Host pointer to the code of the huffman tree for DC component.
 * @param hpTableDC Host pointer to the table of the huffman tree for DC component.
 * @param hpCodesAC Host pointer to the code of the huffman tree for AC component.
 * @param hpTableAC Host pointer to the table of the huffman tree for AC component.
 * @param apHuffmanTableDC DC Huffman tables.
 * @param apHuffmanTableAC AC Huffman tables.
 * @param aSizeROI \ref roi_specification.
 * @return
 *  Error codes:
 *         - ::NPP_SIZE_ERROR For negative input height/width or not a multiple of
 *           8 width/height.
 *         - ::NPP_STEP_ERROR If input image width is not multiple of 8 or does not
 *           match ROI.
 *         - ::NPP_NULL_POINTER_ERROR If the destination pointer is 0.
 *         - ::NPP_NOT_SUFFICIENT_COMPUTE_CAPABILITY If the device has compute capability < 2.0.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEncodeOptimizeHuffmanScan_1JPEG_18u16s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray aSrcStep, jint nRestartInterval, jint nSs, jint nSe, jint nAh, jint nAl, jobject pDst, jobject pLength, jobjectArray hpCodesDC, jobjectArray hpTableDC, jobjectArray hpCodesAC, jobjectArray hpTableAC, jobjectArray aHuffmanDCTable, jobjectArray aHuffmanACTable, jobjectArray oSizeROI, jobject pTempStorage)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcStep' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nRestartInterval is primitive
    // nSs is primitive
    // nSe is primitive
    // nAh is primitive
    // nAl is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLength == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLength' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpCodesDC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpCodesDC' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpTableDC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpTableDC' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpCodesAC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpCodesAC' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpTableAC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpTableAC' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aHuffmanDCTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aHuffmanDCTable' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aHuffmanACTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aHuffmanACTable' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTempStorage == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTempStorage' is null for nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R(pSrc=%p, aSrcStep=%p, nRestartInterval=%d, nSs=%d, nSe=%d, nAh=%d, nAl=%d, pDst=%p, pLength=%p, hpCodesDC=%p, hpTableDC=%p, hpCodesAC=%p, hpTableAC=%p, aHuffmanDCTable=%p, aHuffmanACTable=%p, oSizeROI=%p, pTempStorage=%p)\n",
        pSrc, aSrcStep, nRestartInterval, nSs, nSe, nAh, nAl, pDst, pLength, hpCodesDC, hpTableDC, hpCodesAC, hpTableAC, aHuffmanDCTable, aHuffmanACTable, oSizeROI, pTempStorage);

    // Native variable declarations
    Npp16s* pSrc_native[3];
    Npp32s aSrcStep_native[3];
    Npp32s nRestartInterval_native;
    Npp32s nSs_native;
    Npp32s nSe_native;
    Npp32s nAh_native;
    Npp32s nAl_native;
    Npp8u* pDst_native = NULL;
    Npp32s* pLength_native = NULL;
    Npp8u* hpCodesDC_native[3];
    Npp8u* hpTableDC_native[3];
    Npp8u* hpCodesAC_native[3];
    Npp8u* hpTableAC_native[3];
    NppiEncodeHuffmanSpec* aHuffmanDCTable_native[3];
    NppiEncodeHuffmanSpec* aHuffmanACTable_native[3];
    NppiSize oSizeROI_native[3];
    Npp8u* pTempStorage_native = NULL;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aSrcStep, aSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nRestartInterval_native = (Npp32s)nRestartInterval;
    nSs_native = (Npp32s)nSs;
    nSe_native = (Npp32s)nSe;
    nAh_native = (Npp32s)nAh;
    nAl_native = (Npp32s)nAl;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    PointerData *pLength_pointerData = initPointerData(env, pLength);
    if (pLength_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLength_native = (Npp32s*)pLength_pointerData->getPointer(env);
    if (!initNative(env, hpCodesDC, hpCodesDC_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, hpTableDC, hpTableDC_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, hpCodesAC, hpCodesAC_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, hpTableAC, hpTableAC_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aHuffmanDCTable, aHuffmanDCTable_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aHuffmanACTable, aHuffmanACTable_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pTempStorage_pointerData = initPointerData(env, pTempStorage);
    if (pTempStorage_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTempStorage_native = (Npp8u*)pTempStorage_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiEncodeOptimizeHuffmanScan_JPEG_8u16s_P3R(pSrc_native, aSrcStep_native, nRestartInterval_native, nSs_native, nSe_native, nAh_native, nAl_native, pDst_native, pLength_native, hpCodesDC_native, hpTableDC_native, hpCodesAC_native, hpTableAC_native, aHuffmanDCTable_native, aHuffmanACTable_native, oSizeROI_native, pTempStorage_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    if (!releaseNative(env, aSrcStep_native, aSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // nRestartInterval is primitive
    // nSs is primitive
    // nSe is primitive
    // nAh is primitive
    // nAl is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLength_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpCodesDC is an array of native pointers
    // hpTableDC is an array of native pointers
    // hpCodesAC is an array of native pointers
    // hpTableAC is an array of native pointers
    // aHuffmanDCTable is an array of native pointers
    // aHuffmanACTable is an array of native pointers
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pTempStorage_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates the size of the temporary buffer for baseline Huffman encoding.
 *
 * @see nppiEncodeHuffmanScan_JPEG_8u16s_P1R(), nppiEncodeHuffmanScan_JPEG_8u16s_P3R().
 * 
 * @param oSize Image Dimension.
 * @param pBufSize Pointer to variable that returns the size of the
 *        temporary buffer. 
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEncodeHuffmanGetSizeNative(JNIEnv *env, jclass cls, jobject oSize, jint nChannels, jintArray pBufSize)
{
    // Null-checks for non-primitive arguments
    if (oSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSize' is null for nppiEncodeHuffmanGetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nChannels is primitive
    if (pBufSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBufSize' is null for nppiEncodeHuffmanGetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEncodeHuffmanGetSize(oSize=%p, nChannels=%d, pBufSize=%p)\n",
        oSize, nChannels, pBufSize);

    // Native variable declarations
    NppiSize oSize_native;
    int nChannels_native = 0;
    int* pBufSize_native;

    // Obtain native variable values
    if (!initNative(env, oSize, oSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nChannels_native = (int)nChannels;
    PointerData *pBufSize_pointerData = initPointerData(env, pBufSize);
    if (pBufSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBufSize_native = (int*)pBufSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiEncodeHuffmanGetSize(oSize_native, nChannels_native, pBufSize_native);

    // Write back native variable values
    // oSize is a read-only structure
    // nChannels is primitive
    if (!releasePointerData(env, pBufSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEncodeOptimizeHuffmanGetSizeNative(JNIEnv *env, jclass cls, jobject oSize, jint nChannels, jintArray pBufSize)
{
    // Null-checks for non-primitive arguments
    if (oSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSize' is null for nppiEncodeOptimizeHuffmanGetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nChannels is primitive
    if (pBufSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBufSize' is null for nppiEncodeOptimizeHuffmanGetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEncodeOptimizeHuffmanGetSize(oSize=%p, nChannels=%d, pBufSize=%p)\n",
        oSize, nChannels, pBufSize);

    // Native variable declarations
    NppiSize oSize_native;
    int nChannels_native = 0;
    int* pBufSize_native;

    // Obtain native variable values
    if (!initNative(env, oSize, oSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nChannels_native = (int)nChannels;
    PointerData *pBufSize_pointerData = initPointerData(env, pBufSize);
    if (pBufSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBufSize_native = (int*)pBufSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiEncodeOptimizeHuffmanGetSize(oSize_native, nChannels_native, pBufSize_native);

    // Write back native variable values
    // oSize is a read-only structure
    // nChannels is primitive
    if (!releasePointerData(env, pBufSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates the size of the temporary buffer for graph-cut with 4 neighborhood labeling.
 *
 * @see nppiGraphcutInitAlloc(), nppiGraphcut_32s8u().
 * 
 * @param oSize Graph size.
 * @param pBufSize Pointer to variable that returns the size of the
 *        temporary buffer. 
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcutGetSizeNative(JNIEnv *env, jclass cls, jobject oSize, jintArray pBufSize)
{
    // Null-checks for non-primitive arguments
    if (oSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSize' is null for nppiGraphcutGetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBufSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBufSize' is null for nppiGraphcutGetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcutGetSize(oSize=%p, pBufSize=%p)\n",
        oSize, pBufSize);

    // Native variable declarations
    NppiSize oSize_native;
    int* pBufSize_native;

    // Obtain native variable values
    if (!initNative(env, oSize, oSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBufSize_pointerData = initPointerData(env, pBufSize);
    if (pBufSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBufSize_native = (int*)pBufSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiGraphcutGetSize(oSize_native, pBufSize_native);

    // Write back native variable values
    // oSize is a read-only structure
    if (!releasePointerData(env, pBufSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates the size of the temporary buffer for graph-cut with 8 neighborhood labeling.
 *
 * @see nppiGraphcut8InitAlloc(), nppiGraphcut8_32s8u().
 * 
 * @param oSize Graph size.
 * @param pBufSize Pointer to variable that returns the size of the
 *        temporary buffer. 
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcut8GetSizeNative(JNIEnv *env, jclass cls, jobject oSize, jintArray pBufSize)
{
    // Null-checks for non-primitive arguments
    if (oSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSize' is null for nppiGraphcut8GetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBufSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBufSize' is null for nppiGraphcut8GetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcut8GetSize(oSize=%p, pBufSize=%p)\n",
        oSize, pBufSize);

    // Native variable declarations
    NppiSize oSize_native;
    int* pBufSize_native;

    // Obtain native variable values
    if (!initNative(env, oSize, oSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBufSize_pointerData = initPointerData(env, pBufSize);
    if (pBufSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBufSize_native = (int*)pBufSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiGraphcut8GetSize(oSize_native, pBufSize_native);

    // Write back native variable values
    // oSize is a read-only structure
    if (!releasePointerData(env, pBufSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Initializes graph-cut state structure and allocates additional resources for graph-cut with 8 neighborhood labeling.
 *
 * @see nppiGraphcut_32s8u(), nppiGraphcutGetSize().
 * 
 * @param oSize Graph size
 * @param ppState Pointer to pointer to graph-cut state structure. 
 * @param pDeviceMem pDeviceMem to the sufficient amount of device memory. The CUDA runtime or NPP memory allocators
 *      must be used to allocate this memory. The minimum amount of device memory required
 *      to run graph-cut on a for a specific image size is computed by nppiGraphcutGetSize().
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcutInitAllocNative(JNIEnv *env, jclass cls, jobject oSize, jobject ppState, jobject pDeviceMem)
{
    // Null-checks for non-primitive arguments
    if (oSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSize' is null for nppiGraphcutInitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (ppState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'ppState' is null for nppiGraphcutInitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceMem == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceMem' is null for nppiGraphcutInitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcutInitAlloc(oSize=%p, ppState=%p, pDeviceMem=%p)\n",
        oSize, ppState, pDeviceMem);

    // Native variable declarations
    NppiSize oSize_native;
    NppiGraphcutState* ppState_native;
    Npp8u* pDeviceMem_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSize, oSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // ppState is write-only
    PointerData *pDeviceMem_pointerData = initPointerData(env, pDeviceMem);
    if (pDeviceMem_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceMem_native = (Npp8u*)pDeviceMem_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiGraphcutInitAlloc(oSize_native, &ppState_native, pDeviceMem_native);

    // Write back native variable values
    // oSize is a read-only structure
    setNativePointerValue(env, ppState, (jlong)ppState_native);
    if (!releasePointerData(env, pDeviceMem_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcut8InitAllocNative(JNIEnv *env, jclass cls, jobject oSize, jobject ppState, jobject pDeviceMem)
{
    // Null-checks for non-primitive arguments
    if (oSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSize' is null for nppiGraphcut8InitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (ppState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'ppState' is null for nppiGraphcut8InitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceMem == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceMem' is null for nppiGraphcut8InitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcut8InitAlloc(oSize=%p, ppState=%p, pDeviceMem=%p)\n",
        oSize, ppState, pDeviceMem);

    // Native variable declarations
    NppiSize oSize_native;
    NppiGraphcutState** ppState_native;
    Npp8u* pDeviceMem_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSize, oSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *ppState_pointerData = initPointerData(env, ppState);
    if (ppState_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    ppState_native = (NppiGraphcutState**)ppState_pointerData->getPointer(env);
    PointerData *pDeviceMem_pointerData = initPointerData(env, pDeviceMem);
    if (pDeviceMem_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceMem_native = (Npp8u*)pDeviceMem_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiGraphcut8InitAlloc(oSize_native, ppState_native, pDeviceMem_native);

    // Write back native variable values
    // oSize is a read-only structure
    if (!releasePointerData(env, ppState_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceMem_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Frees the additional resources of the graph-cut state structure.
 *
 * @see nppiGraphcutInitAlloc
 * @see nppiGraphcut8InitAlloc
 * 
 * @param pState Pointer to graph-cut state structure. 
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pState
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcutFreeNative(JNIEnv *env, jclass cls, jobject pState)
{
    // Null-checks for non-primitive arguments
    if (pState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pState' is null for nppiGraphcutFree");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcutFree(pState=%p)\n",
        pState);

    // Native variable declarations
    NppiGraphcutState* pState_native;

    // Obtain native variable values
    pState_native = (NppiGraphcutState*)getNativePointerValue(env, pState);

    // Native function call
    NppStatus jniResult_native = nppiGraphcutFree(pState_native);

    // Write back native variable values
    // pState is a read-only native pointer

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Graphcut of a flow network (32bit signed integer edge capacities). The
 * function computes the minimal cut (graphcut) of a 2D regular 4-connected
 * graph. 
 * The inputs are the capacities of the horizontal (in transposed form),
 * vertical and terminal (source and sink) edges. The capacities to source and
 * sink 
 * are stored as capacity differences in the terminals array 
 * ( terminals(x) = source(x) - sink(x) ). The implementation assumes that the
 * edge capacities 
 * for boundary edges that would connect to nodes outside the specified domain
 * are set to 0 (for example left(0,*) == 0). If this is not fulfilled the
 * computed labeling may be wrong!
 * The computed binary labeling is encoded as unsigned 8bit values (0 and >0).
 *
 * @see nppiGraphcutInitAlloc(), nppiGraphcutFree(), nppiGraphcutGetSize().
 *
 * @param pTerminals Pointer to differences of terminal edge capacities
 *        (terminal(x) = source(x) - sink(x))
 * @param pLeftTransposed Pointer to transposed left edge capacities
 *        (left(0,*) must be 0)
 * @param pRightTransposed Pointer to transposed right edge capacities
 *        (right(width-1,*) must be 0)
 * @param pTop Pointer to top edge capacities (top(*,0) must be 0)
 * @param pBottom Pointer to bottom edge capacities (bottom(*,height-1)
 *        must be 0)
 * @param nStep Step in bytes between any pair of sequential rows of edge
 *        capacities
 * @param nTransposedStep Step in bytes between any pair of sequential
 *        rows of tranposed edge capacities
 * @param size Graph size
 * @param pLabel Pointer to destination label image 
 * @param nLabelStep Step in bytes between any pair of sequential rows of
 *        label image
 * @param pState Pointer to graph-cut state structure. This structure must be
 *          initialized allocated and initialized using nppiGraphcutInitAlloc().
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcut_132s8uNative(JNIEnv *env, jclass cls, jobject pTerminals, jobject pLeftTransposed, jobject pRightTransposed, jobject pTop, jobject pBottom, jint nStep, jint nTransposedStep, jobject size, jobject pLabel, jint nLabelStep, jobject pState)
{
    // Null-checks for non-primitive arguments
    if (pTerminals == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTerminals' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLeftTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLeftTransposed' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pRightTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRightTransposed' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTop == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTop' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottom == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottom' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nStep is primitive
    // nTransposedStep is primitive
    if (size == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'size' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLabel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLabel' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLabelStep is primitive
    if (pState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pState' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcut_32s8u(pTerminals=%p, pLeftTransposed=%p, pRightTransposed=%p, pTop=%p, pBottom=%p, nStep=%d, nTransposedStep=%d, size=%p, pLabel=%p, nLabelStep=%d, pState=%p)\n",
        pTerminals, pLeftTransposed, pRightTransposed, pTop, pBottom, nStep, nTransposedStep, size, pLabel, nLabelStep, pState);

    // Native variable declarations
    Npp32s* pTerminals_native = NULL;
    Npp32s* pLeftTransposed_native = NULL;
    Npp32s* pRightTransposed_native = NULL;
    Npp32s* pTop_native = NULL;
    Npp32s* pBottom_native = NULL;
    int nStep_native = 0;
    int nTransposedStep_native = 0;
    NppiSize size_native;
    Npp8u* pLabel_native = NULL;
    int nLabelStep_native = 0;
    NppiGraphcutState* pState_native;

    // Obtain native variable values
    PointerData *pTerminals_pointerData = initPointerData(env, pTerminals);
    if (pTerminals_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTerminals_native = (Npp32s*)pTerminals_pointerData->getPointer(env);
    PointerData *pLeftTransposed_pointerData = initPointerData(env, pLeftTransposed);
    if (pLeftTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLeftTransposed_native = (Npp32s*)pLeftTransposed_pointerData->getPointer(env);
    PointerData *pRightTransposed_pointerData = initPointerData(env, pRightTransposed);
    if (pRightTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRightTransposed_native = (Npp32s*)pRightTransposed_pointerData->getPointer(env);
    PointerData *pTop_pointerData = initPointerData(env, pTop);
    if (pTop_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTop_native = (Npp32s*)pTop_pointerData->getPointer(env);
    PointerData *pBottom_pointerData = initPointerData(env, pBottom);
    if (pBottom_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottom_native = (Npp32s*)pBottom_pointerData->getPointer(env);
    nStep_native = (int)nStep;
    nTransposedStep_native = (int)nTransposedStep;
    if (!initNative(env, size, size_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pLabel_pointerData = initPointerData(env, pLabel);
    if (pLabel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLabel_native = (Npp8u*)pLabel_pointerData->getPointer(env);
    nLabelStep_native = (int)nLabelStep;
    pState_native = (NppiGraphcutState*)getNativePointerValue(env, pState);

    // Native function call
    NppStatus jniResult_native = nppiGraphcut_32s8u(pTerminals_native, pLeftTransposed_native, pRightTransposed_native, pTop_native, pBottom_native, nStep_native, nTransposedStep_native, size_native, pLabel_native, nLabelStep_native, pState_native);

    // Write back native variable values
    if (!releasePointerData(env, pTerminals_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLeftTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pRightTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTop_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottom_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nStep is primitive
    // nTransposedStep is primitive
    // size is a read-only structure
    if (!releasePointerData(env, pLabel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLabelStep is primitive
    // pState is a read-only native pointer

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Graphcut of a flow network (32bit signed integer edge capacities). The
 * function computes the minimal cut (graphcut) of a 2D regular 8-connected
 * graph. 
 * The inputs are the capacities of the horizontal (in transposed form),
 * vertical and terminal (source and sink) edges. The capacities to source and
 * sink 
 * are stored as capacity differences in the terminals array 
 * ( terminals(x) = source(x) - sink(x) ). The implementation assumes that the
 * edge capacities 
 * for boundary edges that would connect to nodes outside the specified domain
 * are set to 0 (for example left(0,*) == 0). If this is not fulfilled the
 * computed labeling may be wrong!
 * The computed binary labeling is encoded as unsigned 8bit values (0 and >0).
 *
 * @see nppiGraphcut8InitAlloc(), nppiGraphcutFree(), nppiGraphcut8GetSize().
 *
 * @param pTerminals Pointer to differences of terminal edge capacities
 *        (terminal(x) = source(x) - sink(x))
 * @param pLeftTransposed Pointer to transposed left edge capacities
 *        (left(0,*) must be 0)
 * @param pRightTransposed Pointer to transposed right edge capacities
 *        (right(width-1,*) must be 0)
 * @param pTop Pointer to top edge capacities (top(*,0) must be 0)
 * @param pTopLeft Pointer to top left edge capacities (topleft(*,0) 
 *        & topleft(0,*) must be 0)
 * @param pTopRight Pointer to top right edge capacities (topright(*,0)
 *        & topright(width-1,*) must be 0)
 * @param pBottom Pointer to bottom edge capacities (bottom(*,height-1)
 *        must be 0)
 * @param pBottomLeft Pointer to bottom left edge capacities 
 *        (bottomleft(*,height-1) && bottomleft(0,*) must be 0)
 * @param pBottomRight Pointer to bottom right edge capacities 
 *        (bottomright(*,height-1) && bottomright(width-1,*) must be 0)
 * @param nStep Step in bytes between any pair of sequential rows of edge
 *        capacities
 * @param nTransposedStep Step in bytes between any pair of sequential
 *        rows of tranposed edge capacities
 * @param size Graph size
 * @param pLabel Pointer to destination label image 
 * @param nLabelStep Step in bytes between any pair of sequential rows of
 *        label image
 * @param pState Pointer to graph-cut state structure. This structure must be
 *          initialized allocated and initialized using nppiGraphcut8InitAlloc().
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcut8_132s8uNative(JNIEnv *env, jclass cls, jobject pTerminals, jobject pLeftTransposed, jobject pRightTransposed, jobject pTop, jobject pTopLeft, jobject pTopRight, jobject pBottom, jobject pBottomLeft, jobject pBottomRight, jint nStep, jint nTransposedStep, jobject size, jobject pLabel, jint nLabelStep, jobject pState)
{
    // Null-checks for non-primitive arguments
    if (pTerminals == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTerminals' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLeftTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLeftTransposed' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pRightTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRightTransposed' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTop == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTop' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTopLeft == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTopLeft' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTopRight == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTopRight' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottom == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottom' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottomLeft == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottomLeft' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottomRight == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottomRight' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nStep is primitive
    // nTransposedStep is primitive
    if (size == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'size' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLabel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLabel' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLabelStep is primitive
    if (pState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pState' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcut8_32s8u(pTerminals=%p, pLeftTransposed=%p, pRightTransposed=%p, pTop=%p, pTopLeft=%p, pTopRight=%p, pBottom=%p, pBottomLeft=%p, pBottomRight=%p, nStep=%d, nTransposedStep=%d, size=%p, pLabel=%p, nLabelStep=%d, pState=%p)\n",
        pTerminals, pLeftTransposed, pRightTransposed, pTop, pTopLeft, pTopRight, pBottom, pBottomLeft, pBottomRight, nStep, nTransposedStep, size, pLabel, nLabelStep, pState);

    // Native variable declarations
    Npp32s* pTerminals_native = NULL;
    Npp32s* pLeftTransposed_native = NULL;
    Npp32s* pRightTransposed_native = NULL;
    Npp32s* pTop_native = NULL;
    Npp32s* pTopLeft_native = NULL;
    Npp32s* pTopRight_native = NULL;
    Npp32s* pBottom_native = NULL;
    Npp32s* pBottomLeft_native = NULL;
    Npp32s* pBottomRight_native = NULL;
    int nStep_native = 0;
    int nTransposedStep_native = 0;
    NppiSize size_native;
    Npp8u* pLabel_native = NULL;
    int nLabelStep_native = 0;
    NppiGraphcutState* pState_native;

    // Obtain native variable values
    PointerData *pTerminals_pointerData = initPointerData(env, pTerminals);
    if (pTerminals_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTerminals_native = (Npp32s*)pTerminals_pointerData->getPointer(env);
    PointerData *pLeftTransposed_pointerData = initPointerData(env, pLeftTransposed);
    if (pLeftTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLeftTransposed_native = (Npp32s*)pLeftTransposed_pointerData->getPointer(env);
    PointerData *pRightTransposed_pointerData = initPointerData(env, pRightTransposed);
    if (pRightTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRightTransposed_native = (Npp32s*)pRightTransposed_pointerData->getPointer(env);
    PointerData *pTop_pointerData = initPointerData(env, pTop);
    if (pTop_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTop_native = (Npp32s*)pTop_pointerData->getPointer(env);
    PointerData *pTopLeft_pointerData = initPointerData(env, pTopLeft);
    if (pTopLeft_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTopLeft_native = (Npp32s*)pTopLeft_pointerData->getPointer(env);
    PointerData *pTopRight_pointerData = initPointerData(env, pTopRight);
    if (pTopRight_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTopRight_native = (Npp32s*)pTopRight_pointerData->getPointer(env);
    PointerData *pBottom_pointerData = initPointerData(env, pBottom);
    if (pBottom_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottom_native = (Npp32s*)pBottom_pointerData->getPointer(env);
    PointerData *pBottomLeft_pointerData = initPointerData(env, pBottomLeft);
    if (pBottomLeft_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottomLeft_native = (Npp32s*)pBottomLeft_pointerData->getPointer(env);
    PointerData *pBottomRight_pointerData = initPointerData(env, pBottomRight);
    if (pBottomRight_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottomRight_native = (Npp32s*)pBottomRight_pointerData->getPointer(env);
    nStep_native = (int)nStep;
    nTransposedStep_native = (int)nTransposedStep;
    if (!initNative(env, size, size_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pLabel_pointerData = initPointerData(env, pLabel);
    if (pLabel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLabel_native = (Npp8u*)pLabel_pointerData->getPointer(env);
    nLabelStep_native = (int)nLabelStep;
    pState_native = (NppiGraphcutState*)getNativePointerValue(env, pState);

    // Native function call
    NppStatus jniResult_native = nppiGraphcut8_32s8u(pTerminals_native, pLeftTransposed_native, pRightTransposed_native, pTop_native, pTopLeft_native, pTopRight_native, pBottom_native, pBottomLeft_native, pBottomRight_native, nStep_native, nTransposedStep_native, size_native, pLabel_native, nLabelStep_native, pState_native);

    // Write back native variable values
    if (!releasePointerData(env, pTerminals_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLeftTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pRightTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTop_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTopLeft_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTopRight_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottom_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottomLeft_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottomRight_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nStep is primitive
    // nTransposedStep is primitive
    // size is a read-only structure
    if (!releasePointerData(env, pLabel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLabelStep is primitive
    // pState is a read-only native pointer

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcut_132f8uNative(JNIEnv *env, jclass cls, jobject pTerminals, jobject pLeftTransposed, jobject pRightTransposed, jobject pTop, jobject pBottom, jint nStep, jint nTransposedStep, jobject size, jobject pLabel, jint nLabelStep, jobject pState)
{
    // Null-checks for non-primitive arguments
    if (pTerminals == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTerminals' is null for nppiGraphcut_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLeftTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLeftTransposed' is null for nppiGraphcut_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pRightTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRightTransposed' is null for nppiGraphcut_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTop == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTop' is null for nppiGraphcut_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottom == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottom' is null for nppiGraphcut_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nStep is primitive
    // nTransposedStep is primitive
    if (size == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'size' is null for nppiGraphcut_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLabel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLabel' is null for nppiGraphcut_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLabelStep is primitive
    if (pState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pState' is null for nppiGraphcut_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcut_32f8u(pTerminals=%p, pLeftTransposed=%p, pRightTransposed=%p, pTop=%p, pBottom=%p, nStep=%d, nTransposedStep=%d, size=%p, pLabel=%p, nLabelStep=%d, pState=%p)\n",
        pTerminals, pLeftTransposed, pRightTransposed, pTop, pBottom, nStep, nTransposedStep, size, pLabel, nLabelStep, pState);

    // Native variable declarations
    Npp32f* pTerminals_native = NULL;
    Npp32f* pLeftTransposed_native = NULL;
    Npp32f* pRightTransposed_native = NULL;
    Npp32f* pTop_native = NULL;
    Npp32f* pBottom_native = NULL;
    int nStep_native = 0;
    int nTransposedStep_native = 0;
    NppiSize size_native;
    Npp8u* pLabel_native = NULL;
    int nLabelStep_native = 0;
    NppiGraphcutState* pState_native;

    // Obtain native variable values
    PointerData *pTerminals_pointerData = initPointerData(env, pTerminals);
    if (pTerminals_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTerminals_native = (Npp32f*)pTerminals_pointerData->getPointer(env);
    PointerData *pLeftTransposed_pointerData = initPointerData(env, pLeftTransposed);
    if (pLeftTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLeftTransposed_native = (Npp32f*)pLeftTransposed_pointerData->getPointer(env);
    PointerData *pRightTransposed_pointerData = initPointerData(env, pRightTransposed);
    if (pRightTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRightTransposed_native = (Npp32f*)pRightTransposed_pointerData->getPointer(env);
    PointerData *pTop_pointerData = initPointerData(env, pTop);
    if (pTop_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTop_native = (Npp32f*)pTop_pointerData->getPointer(env);
    PointerData *pBottom_pointerData = initPointerData(env, pBottom);
    if (pBottom_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottom_native = (Npp32f*)pBottom_pointerData->getPointer(env);
    nStep_native = (int)nStep;
    nTransposedStep_native = (int)nTransposedStep;
    if (!initNative(env, size, size_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pLabel_pointerData = initPointerData(env, pLabel);
    if (pLabel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLabel_native = (Npp8u*)pLabel_pointerData->getPointer(env);
    nLabelStep_native = (int)nLabelStep;
    pState_native = (NppiGraphcutState*)getNativePointerValue(env, pState);

    // Native function call
    NppStatus jniResult_native = nppiGraphcut_32f8u(pTerminals_native, pLeftTransposed_native, pRightTransposed_native, pTop_native, pBottom_native, nStep_native, nTransposedStep_native, size_native, pLabel_native, nLabelStep_native, pState_native);

    // Write back native variable values
    if (!releasePointerData(env, pTerminals_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLeftTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pRightTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTop_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottom_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nStep is primitive
    // nTransposedStep is primitive
    // size is a read-only structure
    if (!releasePointerData(env, pLabel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLabelStep is primitive
    // pState is a read-only native pointer

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Graphcut of a flow network (32bit float edge capacities). The
 * function computes the minimal cut (graphcut) of a 2D regular 8-connected
 * graph. 
 * The inputs are the capacities of the horizontal (in transposed form),
 * vertical and terminal (source and sink) edges. The capacities to source and
 * sink 
 * are stored as capacity differences in the terminals array 
 * ( terminals(x) = source(x) - sink(x) ). The implementation assumes that the
 * edge capacities 
 * for boundary edges that would connect to nodes outside the specified domain
 * are set to 0 (for example left(0,*) == 0). If this is not fulfilled the
 * computed labeling may be wrong!
 * The computed binary labeling is encoded as unsigned 8bit values (0 and >0).
 *
 * @see nppiGraphcut8InitAlloc(), nppiGraphcutFree(), nppiGraphcut8GetSize().
 *
 * @param pTerminals Pointer to differences of terminal edge capacities
 *        (terminal(x) = source(x) - sink(x))
 * @param pLeftTransposed Pointer to transposed left edge capacities
 *        (left(0,*) must be 0)
 * @param pRightTransposed Pointer to transposed right edge capacities
 *        (right(width-1,*) must be 0)
 * @param pTop Pointer to top edge capacities (top(*,0) must be 0)
 * @param pTopLeft Pointer to top left edge capacities (topleft(*,0) 
 *        & topleft(0,*) must be 0)
 * @param pTopRight Pointer to top right edge capacities (topright(*,0)
 *        & topright(width-1,*) must be 0)
 * @param pBottom Pointer to bottom edge capacities (bottom(*,height-1)
 *        must be 0)
 * @param pBottomLeft Pointer to bottom left edge capacities 
 *        (bottomleft(*,height-1) && bottomleft(0,*) must be 0)
 * @param pBottomRight Pointer to bottom right edge capacities 
 *        (bottomright(*,height-1) && bottomright(width-1,*) must be 0)
 * @param nStep Step in bytes between any pair of sequential rows of edge
 *        capacities
 * @param nTransposedStep Step in bytes between any pair of sequential
 *        rows of tranposed edge capacities
 * @param size Graph size
 * @param pLabel Pointer to destination label image 
 * @param nLabelStep Step in bytes between any pair of sequential rows of
 *        label image
 * @param pState Pointer to graph-cut state structure. This structure must be
 *          initialized allocated and initialized using nppiGraphcut8InitAlloc().
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcut8_132f8uNative(JNIEnv *env, jclass cls, jobject pTerminals, jobject pLeftTransposed, jobject pRightTransposed, jobject pTop, jobject pTopLeft, jobject pTopRight, jobject pBottom, jobject pBottomLeft, jobject pBottomRight, jint nStep, jint nTransposedStep, jobject size, jobject pLabel, jint nLabelStep, jobject pState)
{
    // Null-checks for non-primitive arguments
    if (pTerminals == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTerminals' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLeftTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLeftTransposed' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pRightTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRightTransposed' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTop == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTop' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTopLeft == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTopLeft' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTopRight == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTopRight' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottom == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottom' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottomLeft == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottomLeft' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottomRight == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottomRight' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nStep is primitive
    // nTransposedStep is primitive
    if (size == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'size' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLabel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLabel' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLabelStep is primitive
    if (pState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pState' is null for nppiGraphcut8_32f8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcut8_32f8u(pTerminals=%p, pLeftTransposed=%p, pRightTransposed=%p, pTop=%p, pTopLeft=%p, pTopRight=%p, pBottom=%p, pBottomLeft=%p, pBottomRight=%p, nStep=%d, nTransposedStep=%d, size=%p, pLabel=%p, nLabelStep=%d, pState=%p)\n",
        pTerminals, pLeftTransposed, pRightTransposed, pTop, pTopLeft, pTopRight, pBottom, pBottomLeft, pBottomRight, nStep, nTransposedStep, size, pLabel, nLabelStep, pState);

    // Native variable declarations
    Npp32f* pTerminals_native = NULL;
    Npp32f* pLeftTransposed_native = NULL;
    Npp32f* pRightTransposed_native = NULL;
    Npp32f* pTop_native = NULL;
    Npp32f* pTopLeft_native = NULL;
    Npp32f* pTopRight_native = NULL;
    Npp32f* pBottom_native = NULL;
    Npp32f* pBottomLeft_native = NULL;
    Npp32f* pBottomRight_native = NULL;
    int nStep_native = 0;
    int nTransposedStep_native = 0;
    NppiSize size_native;
    Npp8u* pLabel_native = NULL;
    int nLabelStep_native = 0;
    NppiGraphcutState* pState_native;

    // Obtain native variable values
    PointerData *pTerminals_pointerData = initPointerData(env, pTerminals);
    if (pTerminals_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTerminals_native = (Npp32f*)pTerminals_pointerData->getPointer(env);
    PointerData *pLeftTransposed_pointerData = initPointerData(env, pLeftTransposed);
    if (pLeftTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLeftTransposed_native = (Npp32f*)pLeftTransposed_pointerData->getPointer(env);
    PointerData *pRightTransposed_pointerData = initPointerData(env, pRightTransposed);
    if (pRightTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRightTransposed_native = (Npp32f*)pRightTransposed_pointerData->getPointer(env);
    PointerData *pTop_pointerData = initPointerData(env, pTop);
    if (pTop_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTop_native = (Npp32f*)pTop_pointerData->getPointer(env);
    PointerData *pTopLeft_pointerData = initPointerData(env, pTopLeft);
    if (pTopLeft_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTopLeft_native = (Npp32f*)pTopLeft_pointerData->getPointer(env);
    PointerData *pTopRight_pointerData = initPointerData(env, pTopRight);
    if (pTopRight_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTopRight_native = (Npp32f*)pTopRight_pointerData->getPointer(env);
    PointerData *pBottom_pointerData = initPointerData(env, pBottom);
    if (pBottom_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottom_native = (Npp32f*)pBottom_pointerData->getPointer(env);
    PointerData *pBottomLeft_pointerData = initPointerData(env, pBottomLeft);
    if (pBottomLeft_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottomLeft_native = (Npp32f*)pBottomLeft_pointerData->getPointer(env);
    PointerData *pBottomRight_pointerData = initPointerData(env, pBottomRight);
    if (pBottomRight_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottomRight_native = (Npp32f*)pBottomRight_pointerData->getPointer(env);
    nStep_native = (int)nStep;
    nTransposedStep_native = (int)nTransposedStep;
    if (!initNative(env, size, size_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pLabel_pointerData = initPointerData(env, pLabel);
    if (pLabel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLabel_native = (Npp8u*)pLabel_pointerData->getPointer(env);
    nLabelStep_native = (int)nLabelStep;
    pState_native = (NppiGraphcutState*)getNativePointerValue(env, pState);

    // Native function call
    NppStatus jniResult_native = nppiGraphcut8_32f8u(pTerminals_native, pLeftTransposed_native, pRightTransposed_native, pTop_native, pTopLeft_native, pTopRight_native, pBottom_native, pBottomLeft_native, pBottomRight_native, nStep_native, nTransposedStep_native, size_native, pLabel_native, nLabelStep_native, pState_native);

    // Write back native variable values
    if (!releasePointerData(env, pTerminals_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLeftTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pRightTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTop_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTopLeft_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTopRight_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottom_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottomLeft_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottomRight_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nStep is primitive
    // nTransposedStep is primitive
    // size is a read-only structure
    if (!releasePointerData(env, pLabel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLabelStep is primitive
    // pState is a read-only native pointer

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

